@@ -31,8 +31,12 @@
 #include <gtkmm/button.h>
 #include <gtkmm/combobox.h>
 #include <gtkmm/comboboxtext.h>
+#include <gtkmm/textview.h>
 #include <gtkmm/treemodel.h>
+#include <gtkmm/treeview.h>
+#include <gtkmm/treestore.h>
 #include <gtkmm/liststore.h>
+#include <gtkmm/scrolledwindow.h>
 
 #include "ardour/plugin.h"
 #include "ardour/types.h"
@@ -64,14 +68,14 @@ public:
 		MidiBus,
 		VCAMaster,
 	};
-	TypeWanted type_wanted() const;
+	TypeWanted type_wanted();
 
 	ARDOUR::ChanCount channels ();
+	uint32_t channel_count ();
 	int count ();
 
 	std::string name_template () const;
 	bool name_template_is_default () const;
-	std::string track_template ();
 	ARDOUR::PluginInfoPtr requested_instrument ();
 
 	ARDOUR::TrackMode mode();
@@ -80,13 +84,22 @@ public:
 	RouteDialogs::InsertAt insert_at();
 	bool use_strict_io();
 
+	std::string get_template_path();
+
+	void reset_name_edited () { name_edited_by_user = false; }
+
 private:
 	Gtk::Entry name_template_entry;
-	Gtk::ComboBoxText track_bus_combo;
 	Gtk::Adjustment routes_adjustment;
 	Gtk::SpinButton routes_spinner;
 	Gtk::ComboBoxText channel_combo;
 	Gtk::Label configuration_label;
+	Gtk::Label manual_label;
+	Gtk::Label add_label;
+	Gtk::Label name_label;
+	Gtk::Label group_label;
+	Gtk::Label insert_label;
+	Gtk::Label strict_io_label;
 	Gtk::Label mode_label;
 	Gtk::Label instrument_label;
 	Gtk::ComboBoxText mode_combo;
@@ -95,8 +108,6 @@ private:
 	Gtk::ComboBoxText insert_at_combo;
 	Gtk::ComboBoxText strict_io_combo;
 
-	std::vector<ARDOUR::TemplateInfo> route_templates;
-
 	void track_type_chosen ();
 	void refill_channel_setups ();
 	void refill_route_groups ();
@@ -108,6 +119,31 @@ private:
 	bool route_separator (const Glib::RefPtr<Gtk::TreeModel> &m, const Gtk::TreeModel::iterator &i);
 	void maybe_update_name_template_entry ();
 
+	struct TrackTemplateColumns : public Gtk::TreeModel::ColumnRecord {
+		TrackTemplateColumns () {
+			add (name);
+			add (path);
+			add (description);
+			add (created_with);
+		}
+
+		Gtk::TreeModelColumn<std::string> name;
+		Gtk::TreeModelColumn<std::string> path;
+		Gtk::TreeModelColumn<std::string> description;
+		Gtk::TreeModelColumn<std::string> created_with;
+	};
+
+	TrackTemplateColumns track_template_columns;
+
+    Glib::RefPtr<Gtk::TreeStore>  trk_template_model;
+    Gtk::TreeView                 trk_template_chooser;
+
+	void trk_template_row_selected ();
+
+	Gtk::TextView trk_template_desc;
+	Gtk::Frame    trk_template_outer_frame;
+	Gtk::Frame    trk_template_desc_frame;
+
 	void reset_template_option_visibility ();
 	void new_group_dialog_finished (int, RouteGroupDialog*);
 	void on_show ();
@@ -115,13 +151,13 @@ private:
 
 	struct ChannelSetup {
 		std::string name;
-		std::string template_path;
 		uint32_t    channels;
 	};
 
 	typedef std::vector<ChannelSetup> ChannelSetups;
 	ChannelSetups channel_setups;
 
+	static std::vector<std::pair<std::string, std::string> > builtin_types;
 	static std::vector<std::string> channel_combo_strings;
 	static std::vector<std::string> bus_mode_strings;
 
