@@ -127,6 +127,7 @@ class MainClock;
 class Mixer_UI;
 class PublicEditor;
 class SaveAsDialog;
+class SaveTemplateDialog;
 class SessionDialog;
 class SessionOptionEditorWindow;
 class Splash;
@@ -282,54 +283,21 @@ public:
 	void flush_videotimeline_cache (bool localcacheonly=false);
 	void export_video (bool range = false);
 
-	void session_add_vca (std::string const &, uint32_t);
-
 	void session_add_audio_route (bool, int32_t, int32_t, ARDOUR::TrackMode, ARDOUR::RouteGroup *, uint32_t, std::string const &, bool, ARDOUR::PresentationInfo::order_t order);
-	void session_add_audio_track (
-		int input_channels,
-		int32_t output_channels,
-		ARDOUR::TrackMode mode,
-		ARDOUR::RouteGroup* route_group,
-		uint32_t how_many,
-		std::string const & name_template,
-		bool strict_io,
-		ARDOUR::PresentationInfo::order_t order
-		) {
-		session_add_audio_route (true, input_channels, output_channels, mode, route_group, how_many, name_template, strict_io, order);
-	}
 
-	void session_add_audio_bus (
-			int input_channels,
-			int32_t output_channels,
-			ARDOUR::RouteGroup* route_group,
-			uint32_t how_many,
-			std::string const & name_template,
-			bool strict_io,
-			ARDOUR::PresentationInfo::order_t order
-			) {
-		session_add_audio_route (false, input_channels, output_channels, ARDOUR::Normal, route_group, how_many, name_template, strict_io, order);
-	}
-
-	void session_add_midi_track (
-			ARDOUR::RouteGroup* route_group,
-			uint32_t how_many,
-			std::string const & name_template,
-			bool strict_io,
-			ARDOUR::PluginInfoPtr instrument,
-			ARDOUR::Plugin::PresetRecord* preset,
-			ARDOUR::PresentationInfo::order_t order
-		) {
-		session_add_midi_route (true, route_group, how_many, name_template, strict_io, instrument, preset, order);
-	}
-
-	void session_add_mixed_track (const ARDOUR::ChanCount&, const ARDOUR::ChanCount&, ARDOUR::RouteGroup*, uint32_t, std::string const &, bool,
+	void session_add_mixed_track (const ARDOUR::ChanCount&, const ARDOUR::ChanCount&, ARDOUR::RouteGroup*,
+	                              uint32_t, std::string const &, bool strict_io,
 	                              ARDOUR::PluginInfoPtr, ARDOUR::Plugin::PresetRecord* pset,
 	                              ARDOUR::PresentationInfo::order_t order);
-	void session_add_midi_bus (ARDOUR::RouteGroup*, uint32_t, std::string const &, bool, ARDOUR::PluginInfoPtr, ARDOUR::Plugin::PresetRecord* pset,
+
+	void session_add_midi_bus (ARDOUR::RouteGroup*, uint32_t, std::string const &, bool strict_io,
+	                           ARDOUR::PluginInfoPtr, ARDOUR::Plugin::PresetRecord* pset,
 	                           ARDOUR::PresentationInfo::order_t order);
+
 	void session_add_midi_route (bool, ARDOUR::RouteGroup *, uint32_t, std::string const &, bool,
 	                             ARDOUR::PluginInfoPtr, ARDOUR::Plugin::PresetRecord*,
 	                             ARDOUR::PresentationInfo::order_t order);
+
 	void display_insufficient_ports_message ();
 
 	void attach_to_engine ();
@@ -380,6 +348,8 @@ public:
 
 	ARDOUR::PresentationInfo::order_t translate_order (RouteDialogs::InsertAt);
 
+	std::map<std::string, std::string> route_setup_info (const std::string& script_path);
+
 protected:
 	friend class PublicEditor;
 
@@ -652,11 +622,12 @@ private:
 
 	void open_session ();
 	void open_recent_session ();
-	bool process_save_template_prompter (ArdourWidgets::Prompter& prompter);
+	void save_template_dialog_response (int response, SaveTemplateDialog* d);
 	void save_template ();
 	void manage_templates ();
 
 	void meta_session_setup (const std::string& script_path);
+	void meta_route_setup (const std::string& script_path);
 
 	void edit_metadata ();
 	void import_metadata ();
