@@ -29,15 +29,15 @@
 #include <cmath>
 
 #include <sigc++/bind.h>
+#include <gtkmm/settings.h>
+
 #include "canvas/canvas.h"
 
 #include "pbd/error.h"
 #include "pbd/basename.h"
 #include "pbd/fastlog.h"
 
-#include "gtkmm2ext/cairocell.h"
 #include "gtkmm2ext/utils.h"
-#include "gtkmm2ext/click_box.h"
 #include "gtkmm2ext/window_title.h"
 
 #include "ardour/profile.h"
@@ -52,13 +52,11 @@
 #include "main_clock.h"
 #include "mixer_ui.h"
 #include "utils.h"
-#include "theme_manager.h"
+#include "time_info_box.h"
 #include "midi_tracer.h"
-#include "shuttle_control.h"
 #include "global_port_matrix.h"
 #include "location_ui.h"
 #include "rc_option_editor.h"
-#include "time_info_box.h"
 
 #include "pbd/i18n.h"
 
@@ -66,11 +64,11 @@ using namespace std;
 using namespace ARDOUR;
 using namespace PBD;
 using namespace Gtkmm2ext;
+using namespace ArdourWidgets;
 using namespace Gtk;
 using namespace Glib;
 using namespace ARDOUR_UI_UTILS;
 
-
 void
 ARDOUR_UI::setup_tooltips ()
 {
@@ -85,9 +83,9 @@ ARDOUR_UI::setup_tooltips ()
 	set_tip (auto_loop_button, _("Play loop range"));
 	set_tip (midi_panic_button, _("MIDI Panic\nSend note off and reset controller messages on all MIDI channels"));
 	set_tip (auto_return_button, _("Return to last playback start when stopped"));
-	set_tip (follow_edits_button, _("Playhead follows range selections and edits"));
-	set_tip (auto_input_button, _("Be sensible about input monitoring"));
-	set_tip (click_button, _("Enable/Disable audio click"));
+	set_tip (follow_edits_button, _("Playhead follows Range tool clicks, and Range selections"));
+	set_tip (auto_input_button, _("Track Input Monitoring automatically follows transport state"));
+	parameter_changed("click-gain");
 	set_tip (solo_alert_button, _("When active, something is soloed.\nClick to de-solo everything"));
 	set_tip (auditioning_alert_button, _("When active, auditioning is taking place.\nClick to stop the audition"));
 	set_tip (feedback_alert_button, _("When active, there is a feedback loop."));
@@ -179,52 +177,106 @@ bool drag_failed (const Glib::RefPtr<Gdk::DragContext>& context, DragResult resu
 	return false;
 }
 
+void
+ARDOUR_UI::repack_transport_hbox ()
+{
+	if (time_info_box) {
+		if (time_info_box->get_parent()) {
+			transport_hbox.remove (*time_info_box);
+		}
+		if (UIConfiguration::instance().get_show_toolbar_selclock ()) {
+			transport_hbox.pack_start (*time_info_box, false, false);
+			time_info_box->show();
+		}
+	}
+
+	if (mini_timeline.get_parent()) {
+		transport_hbox.remove (mini_timeline);
+	}
+	if (UIConfiguration::instance().get_show_mini_timeline ()) {
+		transport_hbox.pack_start (mini_timeline, true, true);
+		mini_timeline.show();
+	}
+
+	if (editor_meter) {
+		if (meter_box.get_parent()) {
+			transport_hbox.remove (meter_box);
+			transport_hbox.remove (editor_meter_peak_display);
+		}
+
+		if (UIConfiguration::instance().get_show_editor_meter()) {
+			transport_hbox.pack_end (editor_meter_peak_display, false, false);
+			transport_hbox.pack_end (meter_box, false, false);
+			meter_box.show();
+			editor_meter_peak_display.show();
+		}
+	}
+
+	bool show_mon = UIConfiguration::instance().get_show_toolbar_monitoring ();
+	if (show_mon) {
+		monitor_in_button.show ();
+		monitor_disk_button.show ();
+		auto_input_button.show ();
+		monitoring_spacer.show ();
+	} else {
+		monitor_in_button.hide ();
+		monitor_disk_button.hide ();
+		auto_input_button.hide ();
+		monitoring_spacer.hide ();
+	}
+
+	bool show_rec = UIConfiguration::instance().get_show_toolbar_recpunch ();
+	if (show_rec) {
+		punch_label.show ();
+		layered_label.show ();
+		punch_in_button.show ();
+		punch_out_button.show ();
+		layered_button.show ();
+		recpunch_spacer.show ();
+	} else {
+		punch_label.hide ();
+		layered_label.hide ();
+		punch_in_button.hide ();
+		punch_out_button.hide ();
+		layered_button.hide ();
+		recpunch_spacer.hide ();
+	}
+
+}
+
+void
+ARDOUR_UI::update_clock_visibility ()
+{
+	if (ARDOUR::Profile->get_small_screen()) {
+		return;
+	}
+	if (UIConfiguration::instance().get_show_secondary_clock ()) {
+		secondary_clock->show();
+		secondary_clock->left_btn()->show();
+		secondary_clock->right_btn()->show();
+		if (secondary_clock_spacer) {
+			secondary_clock_spacer->show();
+		}
+	} else {
+		secondary_clock->hide();
+		secondary_clock->left_btn()->hide();
+		secondary_clock->right_btn()->hide();
+		if (secondary_clock_spacer) {
+			secondary_clock_spacer->hide();
+		}
+	}
+}
+
 void
 ARDOUR_UI::setup_transport ()
 {
 	RefPtr<Action> act;
-
-	transport_hbox.set_border_width (PX_SCALE(3));
-	transport_hbox.set_spacing (PX_SCALE(3));
-
-	transport_base.set_name ("TransportBase");
-	transport_base.add (transport_hbox);
-
-	transport_frame.set_shadow_type (SHADOW_OUT);
-	transport_frame.set_name ("BaseFrame");
-	transport_frame.add (transport_base);
-
-	auto_return_button.set_text(_("Auto Return"));
-
-	follow_edits_button.set_text(_("Follow Edits"));
-
-//	auto_input_button.set_text (_("Auto Input"));
-
-	click_button.set_icon (ArdourIcon::TransportMetronom);
+	/* setup actions */
 
 	act = ActionManager::get_action ("Transport", "ToggleClick");
 	click_button.set_related_action (act);
 	click_button.signal_button_press_event().connect (sigc::mem_fun (*this, &ARDOUR_UI::click_button_clicked), false);
-
-	auto_return_button.set_name ("transport option button");
-	follow_edits_button.set_name ("transport option button");
-	auto_input_button.set_name ("transport option button");
-
-	/* these have to provide a clear indication of active state */
-
-	click_button.set_name ("transport button");
-	sync_button.set_name ("transport active option button");
-
-	stop_button.set_active (true);
-
-	goto_start_button.set_icon (ArdourIcon::TransportStart);
-	goto_end_button.set_icon (ArdourIcon::TransportEnd);
-	roll_button.set_icon (ArdourIcon::TransportPlay);
-	stop_button.set_icon (ArdourIcon::TransportStop);
-	play_selection_button.set_icon (ArdourIcon::TransportRange);
-	auto_loop_button.set_icon (ArdourIcon::TransportLoop);
-	rec_button.set_icon (ArdourIcon::RecButton);
-	midi_panic_button.set_icon (ArdourIcon::TransportPanic);
+	click_button.signal_scroll_event().connect (sigc::mem_fun (*this, &ARDOUR_UI::click_button_scroll), false);
 
 	act = ActionManager::get_action (X_("Transport"), X_("Stop"));
 	stop_button.set_related_action (act);
@@ -243,16 +295,26 @@ ARDOUR_UI::setup_transport ()
 	act = ActionManager::get_action (X_("MIDI"), X_("panic"));
 	midi_panic_button.set_related_action (act);
 	act = ActionManager::get_action (X_("Transport"), X_("ToggleExternalSync"));
+
 	sync_button.set_related_action (act);
+	sync_button.signal_button_press_event().connect (sigc::mem_fun (*this, &ARDOUR_UI::sync_button_clicked), false);
 
-	/* clocks, etc. */
+	sync_button.set_sizing_text (S_("LogestSync|M-Clk"));
 
-	ARDOUR_UI::Clock.connect (sigc::mem_fun (primary_clock, &AudioClock::set));
-	ARDOUR_UI::Clock.connect (sigc::mem_fun (secondary_clock, &AudioClock::set));
+	/* CANNOT sigc::bind these to clicked or toggled, must use pressed or released */
+	act = ActionManager::get_action (X_("Main"), X_("cancel-solo"));
+	solo_alert_button.set_related_action (act);
+	auditioning_alert_button.signal_clicked.connect (sigc::mem_fun(*this,&ARDOUR_UI::audition_alert_clicked));
+	error_alert_button.signal_button_release_event().connect (sigc::mem_fun(*this,&ARDOUR_UI::error_alert_press), false);
+	act = ActionManager::get_action (X_("Editor"), X_("toggle-log-window"));
+	error_alert_button.set_related_action(act);
+	error_alert_button.set_fallthrough_to_parent(true);
 
-	primary_clock->ValueChanged.connect (sigc::mem_fun(*this, &ARDOUR_UI::primary_clock_value_changed));
-	secondary_clock->ValueChanged.connect (sigc::mem_fun(*this, &ARDOUR_UI::secondary_clock_value_changed));
-	big_clock->ValueChanged.connect (sigc::mem_fun(*this, &ARDOUR_UI::big_clock_value_changed));
+	layered_button.signal_clicked.connect (sigc::mem_fun(*this,&ARDOUR_UI::layered_button_clicked));
+
+	editor_visibility_button.set_related_action (ActionManager::get_action (X_("Common"), X_("change-editor-visibility")));
+	mixer_visibility_button.set_related_action (ActionManager::get_action (X_("Common"), X_("change-mixer-visibility")));
+	prefs_visibility_button.set_related_action (ActionManager::get_action (X_("Common"), X_("change-preferences-visibility")));
 
 	act = ActionManager::get_action ("Transport", "ToggleAutoReturn");
 	auto_return_button.set_related_action (act);
@@ -261,145 +323,23 @@ ARDOUR_UI::setup_transport ()
 	act = ActionManager::get_action ("Transport", "ToggleAutoInput");
 	auto_input_button.set_related_action (act);
 
-	/* alerts */
+	act = ActionManager::get_action ("Transport", "TogglePunchIn");
+	punch_in_button.set_related_action (act);
+	act = ActionManager::get_action ("Transport", "TogglePunchOut");
+	punch_out_button.set_related_action (act);
 
-	/* CANNOT sigc::bind these to clicked or toggled, must use pressed or released */
+	act = ActionManager::get_action ("Transport", "SessionMonitorIn");
+	monitor_in_button.set_related_action (act);
+	act = ActionManager::get_action ("Transport", "SessionMonitorDisk");
+	monitor_disk_button.set_related_action (act);
 
-	solo_alert_button.set_name ("rude solo");
-	act = ActionManager::get_action (X_("Main"), X_("cancel-solo"));
-	solo_alert_button.set_related_action (act);
-	auditioning_alert_button.set_name ("rude audition");
-	auditioning_alert_button.signal_button_press_event().connect (sigc::mem_fun(*this,&ARDOUR_UI::audition_alert_press), false);
-	feedback_alert_button.set_name ("feedback alert");
-	feedback_alert_button.signal_button_press_event().connect (sigc::mem_fun (*this, &ARDOUR_UI::feedback_alert_press), false);
-	error_alert_button.set_name ("error alert");
-	error_alert_button.signal_button_release_event().connect (sigc::mem_fun(*this,&ARDOUR_UI::error_alert_press), false);
-	act = ActionManager::get_action (X_("Editor"), X_("toggle-log-window"));
-	error_alert_button.set_related_action(act);
-	error_alert_button.set_fallthrough_to_parent(true);
+	/* connect signals */
+	ARDOUR_UI::Clock.connect (sigc::mem_fun (primary_clock, &AudioClock::set));
+	ARDOUR_UI::Clock.connect (sigc::mem_fun (secondary_clock, &AudioClock::set));
 
-	alert_box.set_homogeneous (true);
-	alert_box.set_spacing (PX_SCALE(2));
-	alert_box.pack_start (solo_alert_button, true, true);
-	alert_box.pack_start (auditioning_alert_button, true, true);
-	alert_box.pack_start (feedback_alert_button, true, true);
-
-	/* all transport buttons should be the same size vertically and
-	 * horizontally
-	 */
-
-	Glib::RefPtr<SizeGroup> transport_button_size_group = SizeGroup::create (SIZE_GROUP_BOTH);
-	transport_button_size_group->add_widget (goto_start_button);
-	transport_button_size_group->add_widget (goto_end_button);
-	transport_button_size_group->add_widget (auto_loop_button);
-	transport_button_size_group->add_widget (rec_button);
-	transport_button_size_group->add_widget (play_selection_button);
-	transport_button_size_group->add_widget (roll_button);
-	transport_button_size_group->add_widget (stop_button);
-
-	/* the icon for this has an odd aspect ratio, so fatten up the button */
-	midi_panic_button.set_size_request (PX_SCALE(25), -1);
-	goto_start_button.set_size_request (PX_SCALE(28), PX_SCALE(44));
-	click_button.set_size_request (PX_SCALE(32), PX_SCALE(44));
-
-
-	HBox* tbox1 = manage (new HBox);
-	HBox* tbox2 = manage (new HBox);
-	HBox* tbox = manage (new HBox);
-
-	VBox* vbox1 = manage (new VBox);
-	VBox* vbox2 = manage (new VBox);
-
-	Alignment* a1 = manage (new Alignment);
-	Alignment* a2 = manage (new Alignment);
-
-	tbox1->set_spacing (PX_SCALE(2));
-	tbox2->set_spacing (PX_SCALE(2));
-	tbox->set_spacing (PX_SCALE(2));
-
-	if (!Profile->get_trx()) {
-		tbox1->pack_start (midi_panic_button, true, true, 5);
-		tbox1->pack_start (click_button, true, true, 5);
-	}
-
-	tbox1->pack_start (goto_start_button, true, true);
-	tbox1->pack_start (goto_end_button, true, true);
-	tbox1->pack_start (auto_loop_button, true, true);
-
-	if (!Profile->get_trx()) {
-		tbox2->pack_start (play_selection_button, true, true);
-	}
-	tbox2->pack_start (roll_button, true, true);
-	tbox2->pack_start (stop_button, true, true);
-	tbox2->pack_start (rec_button, true, true, 5);
-
-	vbox1->pack_start (*tbox1, true, true);
-	vbox2->pack_start (*tbox2, true, true);
-
-	a1->add (*vbox1);
-	a1->set (0.5, 0.5, 0.0, 1.0);
-	a2->add (*vbox2);
-	a2->set (0.5, 0.5, 0.0, 1.0);
-
-	tbox->pack_start (*a1, false, false);
-	tbox->pack_start (*a2, false, false);
-
-	HBox* clock_box = manage (new HBox);
-
-	clock_box->pack_start (*primary_clock, false, false);
-	if (!ARDOUR::Profile->get_small_screen() && !ARDOUR::Profile->get_trx()) {
-		clock_box->pack_start (*secondary_clock, false, false);
-	}
-	clock_box->set_spacing (PX_SCALE(3));
-
-	shuttle_box = manage (new ShuttleControl);
-	shuttle_box->show ();
-
-	VBox* transport_vbox = manage (new VBox);
-	transport_vbox->set_name ("TransportBase");
-	transport_vbox->set_border_width (0);
-	transport_vbox->set_spacing (PX_SCALE(3));
-	transport_vbox->pack_start (*tbox, true, true, 0);
-
-	if (!Profile->get_trx()) {
-		transport_vbox->pack_start (*shuttle_box, false, false, 0);
-	}
-
-	time_info_box = manage (new TimeInfoBox);
-
-	transport_hbox.pack_start (*transport_vbox, false, true);
-
-	/* transport related toggle controls */
-
-	VBox* auto_box = manage (new VBox);
-	auto_box->set_homogeneous (true);
-	auto_box->set_spacing (PX_SCALE(2));
-	auto_box->pack_start (sync_button, true, true);
-	if (!ARDOUR::Profile->get_trx()) {
-		auto_box->pack_start (follow_edits_button, true, true);
-		auto_box->pack_start (auto_return_button, true, true);
-	}
-
-	if (!ARDOUR::Profile->get_trx()) {
-		transport_hbox.pack_start (*auto_box, false, false);
-	}
-	transport_hbox.pack_start (*clock_box, true, true);
-
-	if (ARDOUR::Profile->get_trx()) {
-		transport_hbox.pack_start (*auto_box, false, false);
-	}
-
-	if (!ARDOUR::Profile->get_trx()) {
-		transport_hbox.pack_start (*time_info_box, false, false);
-	}
-
-	if (!ARDOUR::Profile->get_trx()) {
-		transport_hbox.pack_start (alert_box, false, false);
-		transport_hbox.pack_start (meter_box, false, false);
-		transport_hbox.pack_start (editor_meter_peak_display, false, false);
-	}
-
-	Gtk::VBox*   window_button_box = manage (new Gtk::VBox);
+	primary_clock->ValueChanged.connect (sigc::mem_fun(*this, &ARDOUR_UI::primary_clock_value_changed));
+	secondary_clock->ValueChanged.connect (sigc::mem_fun(*this, &ARDOUR_UI::secondary_clock_value_changed));
+	big_clock->ValueChanged.connect (sigc::mem_fun(*this, &ARDOUR_UI::big_clock_value_changed));
 
 	editor_visibility_button.signal_drag_failed().connect (sigc::bind (sigc::ptr_fun (drag_failed), editor));
 	mixer_visibility_button.signal_drag_failed().connect (sigc::bind (sigc::ptr_fun (drag_failed), mixer));
@@ -411,37 +351,291 @@ ARDOUR_UI::setup_transport ()
 	mixer_visibility_button.signal_button_press_event().connect (sigc::bind (sigc::mem_fun (*this, &ARDOUR_UI::tabbable_visibility_button_press), X_("mixer")), false);
 	prefs_visibility_button.signal_button_press_event().connect (sigc::bind (sigc::mem_fun (*this, &ARDOUR_UI::tabbable_visibility_button_press), X_("preferences")), false);
 
-	editor_visibility_button.set_related_action (ActionManager::get_action (X_("Common"), X_("change-editor-visibility")));
+	/* setup widget style/name */
+
+	auto_return_button.set_name ("transport option button");
+	follow_edits_button.set_name ("transport option button");
+
+	solo_alert_button.set_name ("rude solo");
+	auditioning_alert_button.set_name ("rude audition");
+	feedback_alert_button.set_name ("feedback alert");
+	error_alert_button.set_name ("error alert");
+
+	solo_alert_button.set_elements (ArdourButton::Element(ArdourButton::Body|ArdourButton::Text));
+	auditioning_alert_button.set_elements (ArdourButton::Element(ArdourButton::Body|ArdourButton::Text));
+	feedback_alert_button.set_elements (ArdourButton::Element(ArdourButton::Body|ArdourButton::Text));
+
+	solo_alert_button.set_layout_font (UIConfiguration::instance().get_SmallerFont());
+	auditioning_alert_button.set_layout_font (UIConfiguration::instance().get_SmallerFont());
+	feedback_alert_button.set_layout_font (UIConfiguration::instance().get_SmallerFont());
+
 	editor_visibility_button.set_name (X_("page switch button"));
-	mixer_visibility_button.set_related_action (ActionManager::get_action (X_("Common"), X_("change-mixer-visibility")));
 	mixer_visibility_button.set_name (X_("page switch button"));
-	prefs_visibility_button.set_related_action (ActionManager::get_action (X_("Common"), X_("change-preferences-visibility")));
 	prefs_visibility_button.set_name (X_("page switch button"));
 
+	punch_in_button.set_name ("punch button");
+	punch_out_button.set_name ("punch button");
+	layered_button.set_name (("layered button"));
+
+	monitor_in_button.set_name ("monitor button");
+	monitor_disk_button.set_name ("monitor button");
+	auto_input_button.set_name ("transport option button");
+
+	click_button.set_name ("transport button");
+	sync_button.set_name ("transport active option button");
+
+	/* and widget text */
+	auto_return_button.set_text(_("Auto Return"));
+	follow_edits_button.set_text(_("Follow Range"));
+	punch_in_button.set_text (_("In"));
+	punch_out_button.set_text (_("Out"));
+	layered_button.set_text (_("Non-Layered"));
+
+	monitor_in_button.set_text (_("All In"));
+	monitor_disk_button.set_text (_("All Disk"));
+	auto_input_button.set_text (_("Auto-Input"));
+
+	punch_label.set_text (_("Punch:"));
+	layered_label.set_text (_("Rec:"));
+
+	/* and tooltips */
+
 	Gtkmm2ext::UI::instance()->set_tip (editor_visibility_button,
 	                                    string_compose (_("Drag this tab to the desktop to show %1 in its own window\n\n"
-	                                                      "To put the window back, use the Window > %1 > Attach menu action"), editor->name()));
+	                                                      "To re-attach the window, use the Window > %1 > Attach menu action"), editor->name()));
 
 	Gtkmm2ext::UI::instance()->set_tip (mixer_visibility_button,
 	                                    string_compose (_("Drag this tab to the desktop to show %1 in its own window\n\n"
-	                                                      "To put the window back, use the Window > %1 > Attach menu action"), mixer->name()));
+	                                                      "To re-attach the window, use the Window > %1 > Attach menu action"), mixer->name()));
 
 	Gtkmm2ext::UI::instance()->set_tip (prefs_visibility_button,
 	                                    string_compose (_("Drag this tab to the desktop to show %1 in its own window\n\n"
-	                                                      "To put the window back, use the Window > %1 > Attach menu action"), rc_option_editor->name()));
+	                                                      "To re-attach the window, use the Window > %1 > Attach menu action"), rc_option_editor->name()));
 
-	window_button_box->pack_start (editor_visibility_button, true, false);
-	window_button_box->pack_start (mixer_visibility_button, true, false);
-	window_button_box->pack_start (prefs_visibility_button, true, false);
+	Gtkmm2ext::UI::instance()->set_tip (punch_in_button, _("Start recording at auto-punch start"));
+	Gtkmm2ext::UI::instance()->set_tip (punch_out_button, _("Stop recording at auto-punch end"));
 
-	transport_hbox.pack_end (*window_button_box, false, false);
-	transport_hbox.pack_end (action_script_table, false, false);
+	Gtkmm2ext::UI::instance()->set_tip (monitor_in_button, _("Force all tracks to monitor Input, unless they are explicitly set to monitor Disk"));
+	Gtkmm2ext::UI::instance()->set_tip (monitor_disk_button, _("Force all tracks to monitor Disk playback, unless they are explicitly set to Input"));
 
+	/* setup icons */
+
+	click_button.set_icon (ArdourIcon::TransportMetronom);
+	goto_start_button.set_icon (ArdourIcon::TransportStart);
+	goto_end_button.set_icon (ArdourIcon::TransportEnd);
+	roll_button.set_icon (ArdourIcon::TransportPlay);
+	stop_button.set_icon (ArdourIcon::TransportStop);
+	play_selection_button.set_icon (ArdourIcon::TransportRange);
+	auto_loop_button.set_icon (ArdourIcon::TransportLoop);
+	rec_button.set_icon (ArdourIcon::RecButton);
+	midi_panic_button.set_icon (ArdourIcon::TransportPanic);
+
+	/* transport control size-group */
+
+	Glib::RefPtr<SizeGroup> transport_button_size_group = SizeGroup::create (SIZE_GROUP_BOTH);
+	transport_button_size_group->add_widget (goto_start_button);
+	transport_button_size_group->add_widget (goto_end_button);
+	transport_button_size_group->add_widget (auto_loop_button);
+	transport_button_size_group->add_widget (rec_button);
+	transport_button_size_group->add_widget (play_selection_button);
+	transport_button_size_group->add_widget (roll_button);
+	transport_button_size_group->add_widget (stop_button);
+
+	Glib::RefPtr<SizeGroup> punch_button_size_group = SizeGroup::create (Gtk::SIZE_GROUP_HORIZONTAL);
+	punch_button_size_group->add_widget (punch_in_button);
+	punch_button_size_group->add_widget (punch_out_button);
+
+	Glib::RefPtr<SizeGroup> monitor_button_size_group = SizeGroup::create (Gtk::SIZE_GROUP_HORIZONTAL);
+	monitor_button_size_group->add_widget (monitor_in_button);
+	monitor_button_size_group->add_widget (monitor_disk_button);
+
+	/* and now the layout... */
+
+	/* top level packing */
+	transport_table.set_spacings (0);
+	transport_table.set_row_spacings (4);
+	transport_table.set_border_width (2);
+
+	transport_frame.set_name ("TransportFrame");
+	transport_frame.set_shadow_type (Gtk::SHADOW_NONE);
+
+	/* An event box to hold the table. We use this because we want specific
+	   control over the background color, and without this event box,
+	   nothing inside the transport_frame actually draws a background. We
+	   would therefore end up seeing the background of the parent widget,
+	   which is probably some default color. Adding the EventBox adds a
+	   widget that will draw the background, using a style based on
+	   the parent, "TransportFrame".
+	*/
+	Gtk::EventBox* ebox = manage (new Gtk::EventBox);
+	transport_frame.add (*ebox);
+	ebox->add (transport_table);
+
+	/* transport controls sub-group */
+	click_button.set_size_request (PX_SCALE(20), PX_SCALE(20));
+
+	HBox* tbox = manage (new HBox);
+	tbox->set_spacing (PX_SCALE(2));
+
+	tbox->pack_start (midi_panic_button, true, true, 0);
+	tbox->pack_start (click_button, true, true, 0);
+	tbox->pack_start (goto_start_button, true, true);
+	tbox->pack_start (goto_end_button, true, true);
+	tbox->pack_start (auto_loop_button, true, true);
+	tbox->pack_start (play_selection_button, true, true);
+
+	tbox->pack_start (roll_button, true, true);
+	tbox->pack_start (stop_button, true, true);
+	tbox->pack_start (rec_button, true, true, 3);
+
+	/* alert box sub-group */
+	VBox* alert_box = manage (new VBox);
+	alert_box->set_homogeneous (true);
+	alert_box->set_spacing (1);
+	alert_box->set_border_width (0);
+	alert_box->pack_start (solo_alert_button, true, true);
+	alert_box->pack_start (auditioning_alert_button, true, true);
+	alert_box->pack_start (feedback_alert_button, true, true);
+
+	/* clock button size groups */
+	Glib::RefPtr<SizeGroup> button_height_size_group = SizeGroup::create (Gtk::SIZE_GROUP_VERTICAL);
+	button_height_size_group->add_widget (follow_edits_button);
+	button_height_size_group->add_widget (*primary_clock->left_btn());
+	button_height_size_group->add_widget (*primary_clock->right_btn());
+	button_height_size_group->add_widget (*secondary_clock->left_btn());
+	button_height_size_group->add_widget (*secondary_clock->right_btn());
+
+	button_height_size_group->add_widget (stop_button);
+//	button_height_size_group->add_widget (sync_button);
+	button_height_size_group->add_widget (auto_return_button);
+
+	//tab selections
+	button_height_size_group->add_widget (editor_visibility_button);
+	button_height_size_group->add_widget (mixer_visibility_button);
+
+	//punch section
+	button_height_size_group->add_widget (punch_in_button);
+	button_height_size_group->add_widget (punch_out_button);
+	button_height_size_group->add_widget (layered_button);
+
+	//input monitoring section
+	button_height_size_group->add_widget (monitor_in_button);
+	button_height_size_group->add_widget (monitor_disk_button);
+	button_height_size_group->add_widget (auto_input_button);
+
+	Glib::RefPtr<SizeGroup> clock1_size_group = SizeGroup::create (SIZE_GROUP_HORIZONTAL);
+	clock1_size_group->add_widget (*primary_clock->left_btn());
+	clock1_size_group->add_widget (*primary_clock->right_btn());
+
+	Glib::RefPtr<SizeGroup> clock2_size_group = SizeGroup::create (SIZE_GROUP_HORIZONTAL);
+	clock2_size_group->add_widget (*secondary_clock->left_btn());
+	clock2_size_group->add_widget (*secondary_clock->right_btn());
+
+	/* sub-layout for Sync | Shuttle (grow) */
+	HBox* ssbox = manage (new HBox);
+	ssbox->set_spacing (PX_SCALE(2));
+	ssbox->pack_start (sync_button, false, false, 0);
+	ssbox->pack_start (shuttle_box, true, true, 0);
+	ssbox->pack_start (*shuttle_box.info_button(), false, false, 0);
+
+
+	/* and the main table layout */
+	int vpadding = 1;
+	int hpadding = 2;
+	int col = 0;
+#define TCOL col, col + 1
+
+	transport_table.attach (*tbox, TCOL, 0, 1 , SHRINK, SHRINK, 0, 0);
+	transport_table.attach (*ssbox, TCOL, 1, 2 , FILL, SHRINK, 0, 0);
+	++col;
+
+	transport_table.attach (*(manage (new ArdourVSpacer ())), TCOL, 0, 2 , SHRINK, EXPAND|FILL, 3, 0);
+	++col;
+
+	transport_table.attach (punch_label, TCOL, 0, 1 , FILL, SHRINK, 3, 0);
+	transport_table.attach (layered_label, TCOL, 1, 2 , FILL, SHRINK, 3, 0);
+	++col;
+
+	transport_table.attach (punch_in_button,  col,      col + 1, 0, 1 , FILL, SHRINK, hpadding, vpadding);
+	transport_table.attach (punch_space,      col + 1,  col + 2, 0, 1 , FILL, SHRINK, 0, vpadding);
+	transport_table.attach (punch_out_button, col + 2,  col + 3, 0, 1 , FILL, SHRINK, hpadding, vpadding);
+	transport_table.attach (layered_button,   col,      col + 3, 1, 2 , FILL, SHRINK, hpadding, vpadding);
+	col += 3;
+
+	transport_table.attach (recpunch_spacer, TCOL, 0, 2 , SHRINK, EXPAND|FILL, 3, 0);
+	++col;
+
+	transport_table.attach (auto_input_button,   col,     col + 3, 0, 1 , FILL, SHRINK, hpadding, vpadding);
+	transport_table.attach (monitor_in_button,   col,     col + 1, 1, 2 , FILL, SHRINK, hpadding, vpadding);
+	transport_table.attach (mon_space,           col + 1, col + 2, 1, 2 , FILL, SHRINK, 2, vpadding);
+	transport_table.attach (monitor_disk_button, col + 2, col + 3, 1, 2 , FILL, SHRINK, hpadding, vpadding);
+	col += 3;
+
+	transport_table.attach (monitoring_spacer, TCOL, 0, 2 , SHRINK, EXPAND|FILL, 3, 0);
+	++col;
+
+	transport_table.attach (follow_edits_button, TCOL, 0, 1 , FILL, SHRINK, hpadding, vpadding);
+	transport_table.attach (auto_return_button,  TCOL, 1, 2 , FILL, SHRINK, hpadding, vpadding);
+	++col;
+
+	transport_table.attach (*(manage (new ArdourVSpacer ())), TCOL, 0, 2 , SHRINK, EXPAND|FILL, 3, 0);
+	++col;
+
+	transport_table.attach (*primary_clock,              col,     col + 2, 0, 1 , FILL, SHRINK, hpadding, 0);
+	transport_table.attach (*primary_clock->left_btn(),  col,     col + 1, 1, 2 , FILL, SHRINK, hpadding, 0);
+	transport_table.attach (*primary_clock->right_btn(), col + 1, col + 2, 1, 2 , FILL, SHRINK, hpadding, 0);
+	col += 2;
+
+	transport_table.attach (*(manage (new ArdourVSpacer ())), TCOL, 0, 2 , SHRINK, EXPAND|FILL, 3, 0);
+	++col;
+
+	if (!ARDOUR::Profile->get_small_screen()) {
+		transport_table.attach (*secondary_clock,              col,     col + 2, 0, 1 , FILL, SHRINK, hpadding, 0);
+		transport_table.attach (*secondary_clock->left_btn(),  col,     col + 1, 1, 2 , FILL, SHRINK, hpadding, 0);
+		transport_table.attach (*secondary_clock->right_btn(), col + 1, col + 2, 1, 2 , FILL, SHRINK, hpadding, 0);
+		secondary_clock->set_no_show_all (true);
+		secondary_clock->left_btn()->set_no_show_all (true);
+		secondary_clock->right_btn()->set_no_show_all (true);
+		col += 2;
+
+		secondary_clock_spacer = manage (new ArdourVSpacer ());
+		transport_table.attach (*secondary_clock_spacer, TCOL, 0, 2 , SHRINK, EXPAND|FILL, 3, 0);
+		++col;
+	}
+
+	transport_table.attach (*alert_box, TCOL, 0, 2, SHRINK, EXPAND|FILL, hpadding, 0);
+	++col;
+
+	transport_table.attach (*(manage (new ArdourVSpacer ())), TCOL, 0, 2 , SHRINK, EXPAND|FILL, 3, 0);
+	++col;
+
+	/* editor-meter, mini-timeline and selection clock are options in the transport_hbox */
+	transport_hbox.set_spacing (3);
+	transport_table.attach (transport_hbox, TCOL, 0, 2, EXPAND|FILL, EXPAND|FILL, hpadding, 0);
+	++col;
+
+	/* lua script action buttons */
+	transport_table.attach (action_script_table, TCOL, 0, 2, SHRINK, EXPAND|FILL, 1, 0);
+	++col;
+
+	transport_table.attach (editor_visibility_button, TCOL, 0, 1 , FILL, SHRINK, hpadding, vpadding);
+	transport_table.attach (mixer_visibility_button,  TCOL, 1, 2 , FILL, SHRINK, hpadding, vpadding);
+	++col;
+
+	repack_transport_hbox ();
+	update_clock_visibility ();
 	/* desensitize */
 
+	feedback_alert_button.set_sensitive (false);
+	feedback_alert_button.set_visual_state (Gtkmm2ext::NoVisualState);
+	auditioning_alert_button.set_sensitive (false);
+	auditioning_alert_button.set_visual_state (Gtkmm2ext::NoVisualState);
+
+	stop_button.set_active (true);
 	set_transport_sensitivity (false);
 }
 #undef PX_SCALE
+#undef TCOL
 
 void
 ARDOUR_UI::soloing_changed (bool onoff)
@@ -455,6 +649,10 @@ void
 ARDOUR_UI::_auditioning_changed (bool onoff)
 {
 	auditioning_alert_button.set_active (onoff);
+	auditioning_alert_button.set_sensitive (onoff);
+	if (!onoff) {
+		auditioning_alert_button.set_visual_state (Gtkmm2ext::NoVisualState);
+	}
 	set_transport_sensitivity (!onoff);
 }
 
@@ -464,19 +662,12 @@ ARDOUR_UI::auditioning_changed (bool onoff)
 	UI::instance()->call_slot (MISSING_INVALIDATOR, boost::bind (&ARDOUR_UI::_auditioning_changed, this, onoff));
 }
 
-bool
-ARDOUR_UI::audition_alert_press (GdkEventButton*)
+void
+ARDOUR_UI::audition_alert_clicked ()
 {
 	if (_session) {
 		_session->cancel_audition();
 	}
-	return true;
-}
-
-bool
-ARDOUR_UI::feedback_alert_press (GdkEventButton *)
-{
-	return true;
 }
 
 bool
@@ -499,6 +690,14 @@ ARDOUR_UI::error_alert_press (GdkEventButton* ev)
 	return !do_toggle;
 }
 
+void
+ARDOUR_UI::layered_button_clicked ()
+{
+	if (_session) {
+		_session->config.set_layered_record_mode (!_session->config.get_layered_record_mode ());
+	}
+}
+
 void
 ARDOUR_UI::solo_blink (bool onoff)
 {
@@ -596,13 +795,21 @@ ARDOUR_UI::error_blink (bool onoff)
 	}
 }
 
-
+void
+ARDOUR_UI::set_loop_sensitivity ()
+{
+	if (!_session || _session->config.get_external_sync()) {
+		auto_loop_button.set_sensitive (false);
+	} else {
+		auto_loop_button.set_sensitive (_session && _session->locations()->auto_loop_location());
+	}
+}
 
 void
 ARDOUR_UI::set_transport_sensitivity (bool yn)
 {
 	ActionManager::set_sensitive (ActionManager::transport_sensitive_actions, yn);
-	shuttle_box->set_sensitive (yn);
+	shuttle_box.set_sensitive (yn);
 }
 
 void
@@ -635,7 +842,7 @@ ARDOUR_UI::show_ui_prefs ()
 {
 	if (rc_option_editor) {
 		show_tabbable (rc_option_editor);
-		rc_option_editor->set_current_page (_("GUI"));
+		rc_option_editor->set_current_page (_("Appearance"));
 	}
 }
 
@@ -648,7 +855,43 @@ ARDOUR_UI::click_button_clicked (GdkEventButton* ev)
 	}
 
 	show_tabbable (rc_option_editor);
-	rc_option_editor->set_current_page (_("Misc/Click"));
+	rc_option_editor->set_current_page (_("Metronome"));
+	return true;
+}
+
+bool
+ARDOUR_UI::click_button_scroll (GdkEventScroll* ev)
+{
+	gain_t gain = Config->get_click_gain();
+	float gain_db = accurate_coefficient_to_dB (gain);
+
+	switch (ev->direction) {
+		case GDK_SCROLL_UP:
+		case GDK_SCROLL_LEFT:
+			gain_db += 1;
+			break;
+		case GDK_SCROLL_DOWN:
+		case GDK_SCROLL_RIGHT:
+			gain_db -= 1;
+			break;
+	}
+	gain_db = std::max (-60.f, gain_db);
+	gain = dB_to_coefficient (gain_db);
+	gain = std::min (gain, Config->get_max_gain());
+	Config->set_click_gain (gain);
+	return true;
+}
+
+bool
+ARDOUR_UI::sync_button_clicked (GdkEventButton* ev)
+{
+	if (ev->button != 3) {
+		/* this handler is just for button-3 clicks */
+		return false;
+	}
+
+	show_tabbable (rc_option_editor);
+	rc_option_editor->set_current_page (_("Sync"));
 	return true;
 }
 
