@@ -26,10 +26,11 @@
 #include <gtkmm/style.h>
 #include <sigc++/bind.h>
 
-#include "gtkmm2ext/cairocell.h"
 #include "gtkmm2ext/utils.h"
 #include "gtkmm2ext/rgb_macros.h"
 
+#include "widgets/tooltips.h"
+
 #include "ardour/profile.h"
 #include "ardour/lmath.h"
 #include "ardour/session.h"
@@ -39,6 +40,7 @@
 
 #include "ardour_ui.h"
 #include "audio_clock.h"
+#include "enums_convert.h"
 #include "gui_thread.h"
 #include "keyboard.h"
 #include "ui_config.h"
@@ -48,6 +50,7 @@
 
 using namespace ARDOUR;
 using namespace ARDOUR_UI_UTILS;
+using namespace ArdourWidgets;
 using namespace PBD;
 using namespace Gtk;
 using namespace std;
@@ -56,14 +59,9 @@ using Gtkmm2ext::Keyboard;
 
 sigc::signal<void> AudioClock::ModeChanged;
 vector<AudioClock*> AudioClock::clocks;
-const double AudioClock::info_font_scale_factor = 0.68;
-const double AudioClock::separator_height = 0.0;
-const double AudioClock::x_leading_padding = 6.0;
 
 #define BBT_BAR_CHAR "|"
 #define BBT_SCANF_FORMAT "%" PRIu32 "%*c%" PRIu32 "%*c%" PRIu32
-#define INFO_FONT_SIZE ((int)lrint(font_size * info_font_scale_factor))
-#define TXTSPAN "<span font-family=\"Sans\" foreground=\"white\">"
 
 AudioClock::AudioClock (const string& clock_name, bool transient, const string& widget_name,
 			bool allow_edit, bool follows_playhead, bool duration, bool with_info,
@@ -80,6 +78,8 @@ AudioClock::AudioClock (const string& clock_name, bool transient, const string&
 	, _edit_by_click_field (false)
 	, _negative_allowed (false)
 	, edit_is_negative (false)
+	, _limit_pos (INT64_MAX - 1)
+	, _with_info (with_info)
 	, editing_attr (0)
 	, foreground_attr (0)
 	, first_height (0)
@@ -87,9 +87,6 @@ AudioClock::AudioClock (const string& clock_name, bool transient, const string&
 	, style_resets_first (true)
 	, layout_height (0)
 	, layout_width (0)
-	, info_height (0)
-	, upper_height (0)
-	, mode_based_info_ratio (1.0)
 	, corner_radius (4)
 	, font_size (10240)
 	, editing (false)
@@ -107,11 +104,6 @@ AudioClock::AudioClock (const string& clock_name, bool transient, const string&
 	_layout = Pango::Layout::create (get_pango_context());
 	_layout->set_attributes (normal_attributes);
 
-	if (with_info) {
-		_left_layout = Pango::Layout::create (get_pango_context());
-		_right_layout = Pango::Layout::create (get_pango_context());
-	}
-
 	set_widget_name (widget_name);
 
 	_mode = BBT; /* lie to force mode switch */
@@ -122,6 +114,12 @@ AudioClock::AudioClock (const string& clock_name, bool transient, const string&
 		clocks.push_back (this);
 	}
 
+	_left_btn.set_sizing_text (_("0000000000000"));
+	// NB right_btn is in a size-group
+
+	_left_btn.set_layout_font (UIConfiguration::instance().get_SmallFont());
+	_right_btn.set_layout_font (UIConfiguration::instance().get_SmallFont());
+
 	UIConfiguration::instance().ColorsChanged.connect (sigc::mem_fun (*this, &AudioClock::set_colors));
 	UIConfiguration::instance().DPIReset.connect (sigc::mem_fun (*this, &AudioClock::dpi_reset));
 }
@@ -175,18 +173,6 @@ AudioClock::set_font (Pango::FontDescription font)
 
 	normal_attributes.change (*font_attr);
 	editing_attributes.change (*font_attr);
-
-	/* now a smaller version of the same font */
-
-	delete font_attr;
-	font.set_size (INFO_FONT_SIZE);
-	font.set_weight (Pango::WEIGHT_NORMAL);
-	font_attr = new Pango::AttrFontDesc (Pango::Attribute::create_attr_font_desc (font));
-
-	info_attributes.change (*font_attr);
-
-	/* and an even smaller one */
-
 	delete font_attr;
 
 	/* get the figure width for the font. This doesn't have to super
@@ -270,7 +256,6 @@ AudioClock::set_colors ()
 	editing_attr = new Pango::AttrColor (Pango::Attribute::create_attr_foreground (r, g, b));
 
 	normal_attributes.change (*foreground_attr);
-	info_attributes.change (*foreground_attr);
 	editing_attributes.change (*foreground_attr);
 	editing_attributes.change (*editing_attr);
 
@@ -293,20 +278,17 @@ AudioClock::set_scale (double x, double y)
 }
 
 void
-AudioClock::render (cairo_t* cr, cairo_rectangle_t*)
+AudioClock::render (Cairo::RefPtr<Cairo::Context> const& ctx, cairo_rectangle_t*)
 {
+	cairo_t* cr = ctx->cobj();
 	/* main layout: rounded rect, plus the text */
 
 	if (_need_bg) {
 		cairo_set_source_rgba (cr, bg_r, bg_g, bg_b, bg_a);
 		if (corner_radius) {
-			if (_left_layout) {
-				Gtkmm2ext::rounded_top_half_rectangle (cr, 0, 0, get_width(), upper_height, corner_radius);
-			} else {
-				Gtkmm2ext::rounded_rectangle (cr, 0, 0, get_width(), upper_height, corner_radius);
-			}
+			Gtkmm2ext::rounded_rectangle (cr, 0, 0, get_width(), get_height(), corner_radius);
 		} else {
-			cairo_rectangle (cr, 0, 0, get_width(), upper_height);
+			cairo_rectangle (cr, 0, 0, get_width(), get_height());
 		}
 		cairo_fill (cr);
 	}
@@ -314,7 +296,7 @@ AudioClock::render (cairo_t* cr, cairo_rectangle_t*)
 	double lw = layout_width * xscale;
 	double lh = layout_height * yscale;
 
-	cairo_move_to (cr, (get_width() - lw) / 2.0, (upper_height - lh) / 2.0);
+	cairo_move_to (cr, (get_width() - lw) / 2.0, (get_height() - lh) / 2.0);
 
 	if (xscale != 1.0 || yscale != 1.0) {
 		cairo_save (cr);
@@ -327,81 +309,6 @@ AudioClock::render (cairo_t* cr, cairo_rectangle_t*)
 		cairo_restore (cr);
 	}
 
-	if (_left_layout) {
-
-		double h = get_height() - upper_height - separator_height;
-
-		if (_need_bg) {
-			cairo_set_source_rgba (cr, bg_r, bg_g, bg_b, bg_a);
-		}
-
-		if (mode_based_info_ratio != 1.0) {
-
-			double left_rect_width = get_left_rect_width();
-
-			if (_need_bg) {
-				if (corner_radius) {
-					Gtkmm2ext::rounded_bottom_half_rectangle (cr, 0, upper_height + separator_height,
-							left_rect_width + (separator_height == 0 ? corner_radius : 0),
-							h, corner_radius);
-				} else {
-					cairo_rectangle (cr, 0, upper_height + separator_height, left_rect_width, h);
-				}
-				cairo_fill (cr);
-			}
-
-			cairo_move_to (cr, x_leading_padding, upper_height + separator_height + ((h - info_height)/2.0));
-			pango_cairo_show_layout (cr, _left_layout->gobj());
-
-			if (_need_bg) {
-				if (corner_radius) {
-					Gtkmm2ext::rounded_bottom_half_rectangle (cr, left_rect_width + separator_height,
-							upper_height + separator_height,
-							get_width() - separator_height - left_rect_width,
-							h, corner_radius);
-				} else {
-					cairo_rectangle (cr, left_rect_width + separator_height, upper_height + separator_height,
-							 get_width() - separator_height - left_rect_width, h);
-				}
-				cairo_fill (cr);
-			}
-
-
-			if (_right_layout->get_alignment() == Pango::ALIGN_RIGHT) {
-				/* right-align does not work per se beacuse layout width is unset.
-				 * Using _right_layout->set_width([value >=0]) would also enable
-				 * word-wrapping which is not wanted here.
-				 * The solution is to custom align the layout depending on its size.
-				 * if it is larger than the available space it will be cropped on the
-				 * right edge rather than override text on the left side.
-				 */
-				int x, rw, rh;
-				_right_layout->get_pixel_size(rw, rh);
-				x = get_width() - rw - separator_height - x_leading_padding;
-				if (x < x_leading_padding + left_rect_width + separator_height) {
-					/* rather cut off the right end than overlap with the text on the left */
-					x = x_leading_padding + left_rect_width + separator_height;
-				}
-				cairo_move_to (cr, x, upper_height + separator_height + ((h - info_height)/2.0));
-			} else {
-				cairo_move_to (cr, x_leading_padding + left_rect_width + separator_height, upper_height + separator_height + ((h - info_height)/2.0));
-			}
-			pango_cairo_show_layout (cr, _right_layout->gobj());
-
-		} else {
-			/* no info to display, or just one */
-
-			if (_need_bg) {
-				if (corner_radius) {
-					Gtkmm2ext::rounded_bottom_half_rectangle (cr, 0, upper_height + separator_height, get_width(), h, corner_radius);
-				} else {
-					cairo_rectangle (cr, 0, upper_height + separator_height, get_width(), h);
-				}
-				cairo_fill (cr);
-			}
-		}
-	}
-
 	if (editing) {
 		if (!insert_map.empty()) {
 
@@ -421,10 +328,10 @@ AudioClock::render (cairo_t* cr, cairo_rectangle_t*)
 
 				cairo_set_source_rgba (cr, cursor_r, cursor_g, cursor_b, cursor_a);
 				cairo_rectangle (cr,
-						 min (get_width() - 2.0,
-						      (double) xcenter + cursor.get_x()/PANGO_SCALE + em_width),
-						 (upper_height - layout_height)/2.0,
-						 2.0, cursor.get_height()/PANGO_SCALE);
+				                 min (get_width() - 2.0,
+				                 (double) xcenter + cursor.get_x()/PANGO_SCALE + em_width),
+				                 (get_height() - layout_height)/2.0,
+				                 2.0, cursor.get_height()/PANGO_SCALE);
 				cairo_fill (cr);
 			} else {
 				/* we've entered all possible digits, no cursor */
@@ -435,26 +342,14 @@ AudioClock::render (cairo_t* cr, cairo_rectangle_t*)
 				cairo_set_source_rgba (cr, cursor_r, cursor_g, cursor_b, cursor_a);
 				cairo_rectangle (cr,
 						 (get_width()/2.0),
-						 (upper_height - layout_height)/2.0,
-						 2.0, upper_height);
+						 (get_height() - layout_height)/2.0,
+						 2.0, get_height());
 				cairo_fill (cr);
 			}
 		}
 	}
 }
 
-void
-AudioClock::on_size_allocate (Gtk::Allocation& alloc)
-{
-	CairoWidget::on_size_allocate (alloc);
-
-	if (_left_layout) {
-		upper_height = (get_height()/2.0) - 1.0;
-	} else {
-		upper_height = get_height();
-	}
-}
-
 void
 AudioClock::set_clock_dimensions (Gtk::Requisition& req)
 {
@@ -502,42 +397,6 @@ AudioClock::on_size_request (Gtk::Requisition* req)
 	/* now tackle height, for which we need to know the height of the lower
 	 * layout
 	 */
-
-	if (_left_layout) {
-
-		Glib::RefPtr<Pango::Layout> tmp;
-		Glib::RefPtr<Gtk::Style> style = get_style ();
-		Pango::FontDescription font;
-		int w;
-
-		tmp = Pango::Layout::create (get_pango_context());
-
-		if (!is_realized()) {
-			font = get_font_for_style (get_name());
-		} else {
-			font = style->get_font();
-		}
-
-		tmp->set_font_description (font);
-
-		font.set_size (INFO_FONT_SIZE);
-		font.set_weight (Pango::WEIGHT_NORMAL);
-		tmp->set_font_description (font);
-
-		/* we only care about height, so put as much stuff in here
-		   as possible that might change the height.
-		*/
-		tmp->set_text ("qyhH|"); /* one ascender, one descender */
-
-		tmp->get_pixel_size (w, info_height);
-
-		/* silly extra padding that seems necessary to correct the info
-		 * that pango just gave us. I have no idea why.
-		 */
-
-		req->height += info_height;
-		req->height += separator_height;
-	}
 }
 
 void
@@ -918,12 +777,6 @@ AudioClock::session_property_changed (const PropertyChange&)
 	set (last_when, true);
 }
 
-void
-AudioClock::metric_position_changed ()
-{
-	set (last_when, true);
-}
-
 void
 AudioClock::session_configuration_changed (std::string p)
 {
@@ -960,7 +813,7 @@ AudioClock::session_configuration_changed (std::string p)
 void
 AudioClock::set (framepos_t when, bool force, framecnt_t offset)
 {
- 	if ((!force && !is_visible()) || _session == 0) {
+	if ((!force && !is_visible()) || _session == 0) {
 		return;
 	}
 
@@ -968,6 +821,12 @@ AudioClock::set (framepos_t when, bool force, framecnt_t offset)
 		when = when - offset;
 	}
 
+	if (when > _limit_pos) {
+		when = _limit_pos;
+	} else if (when < -_limit_pos) {
+		when = -_limit_pos;
+	}
+
 	if (when == last_when && !force) {
 #if 0 // XXX return if no change and no change forced. verify Aug/2014
 		if (_mode != Timecode && _mode != MinSec) {
@@ -989,27 +848,21 @@ AudioClock::set (framepos_t when, bool force, framecnt_t offset)
 #endif
 	}
 
+	bool btn_en = false;
+
 	if (!editing) {
-		if (_right_layout) {
-			_right_layout->set_alignment(Pango::ALIGN_LEFT);
-		}
 
 		switch (_mode) {
 		case Timecode:
-			if (_right_layout) {
-				_right_layout->set_alignment(Pango::ALIGN_RIGHT);
-			}
 			set_timecode (when, force);
 			break;
 
 		case BBT:
 			set_bbt (when, offset, force);
+			btn_en = true;
 			break;
 
 		case MinSec:
-			if (_right_layout) {
-				_right_layout->set_alignment(Pango::ALIGN_RIGHT);
-			}
 			set_minsec (when, force);
 			break;
 
@@ -1019,6 +872,28 @@ AudioClock::set (framepos_t when, bool force, framecnt_t offset)
 		}
 	}
 
+	if (_with_info) {
+		_left_btn.set_sensitive (btn_en);
+		_right_btn.set_sensitive (btn_en);
+		_left_btn.set_visual_state (Gtkmm2ext::NoVisualState);
+		_right_btn.set_visual_state (Gtkmm2ext::NoVisualState);
+		if (btn_en) {
+			_left_btn.set_elements (ArdourButton::Element(ArdourButton::Edge|ArdourButton::Body|ArdourButton::Text));
+			_right_btn.set_elements (ArdourButton::Element(ArdourButton::Edge|ArdourButton::Body|ArdourButton::Text));
+			_left_btn.set_alignment (.5, .5);
+			_right_btn.set_alignment (.5, .5);
+			set_tooltip (_left_btn, _("Change current tempo"));
+			set_tooltip (_right_btn, _("Change current time signature"));
+		} else {
+			_left_btn.set_elements (ArdourButton::Text);
+			_right_btn.set_elements (ArdourButton::Text);
+			_left_btn.set_alignment (0, .5);
+			_right_btn.set_alignment (1, .5);
+			set_tooltip (_left_btn, _(""));
+			set_tooltip (_right_btn, _(""));
+		}
+	}
+
 	queue_draw ();
 	last_when = when;
 }
@@ -1026,7 +901,7 @@ AudioClock::set (framepos_t when, bool force, framecnt_t offset)
 void
 AudioClock::set_slave_info ()
 {
-	if (!_left_layout || !_right_layout) {
+	if (!_with_info) {
 		return;
 	}
 
@@ -1037,20 +912,16 @@ AudioClock::set_slave_info ()
 
 		switch (sync_src) {
 		case Engine:
-			_left_layout->set_markup (string_compose ("<span size=\"%1\">" TXTSPAN "%2</span></span>",
-						INFO_FONT_SIZE, sync_source_to_string(sync_src, true)));
-			_right_layout->set_text ("");
+			_left_btn.set_text (sync_source_to_string (sync_src, true), true);
+			_right_btn.set_text ("", true);
 			break;
 		case MIDIClock:
 			if (slave) {
-				_left_layout->set_markup (string_compose ("<span size=\"%1\">" TXTSPAN "%2</span></span>",
-							INFO_FONT_SIZE, sync_source_to_string(sync_src, true)));
-				_right_layout->set_markup (string_compose ("<span size=\"%1\">" TXTSPAN "%2</span></span>",
-							INFO_FONT_SIZE, slave->approximate_current_delta()));
+				_left_btn.set_text (sync_source_to_string (sync_src, true), true);
+				_right_btn.set_text (slave->approximate_current_delta (), true);
 			} else {
-				_left_layout->set_markup (string_compose ("<span size=\"%1\">" TXTSPAN "%2</span></span>",
-							INFO_FONT_SIZE, _("--pending--")));
-				_right_layout->set_text ("");
+				_left_btn.set_text (_("--pending--"), true);
+				_right_btn.set_text ("", true);
 			}
 			break;
 		case LTC:
@@ -1060,23 +931,41 @@ AudioClock::set_slave_info ()
 				TimecodeSlave* tcslave;
 				if ((tcslave = dynamic_cast<TimecodeSlave*>(_session->slave())) != 0) {
 					matching = (tcslave->apparent_timecode_format() == _session->config.get_timecode_format());
-					_left_layout->set_markup (string_compose ("<span size=\"%1\">" TXTSPAN "%2</span><span foreground=\"%3\">%4</span></span>",
-										  INFO_FONT_SIZE, sync_source_to_string(sync_src, true)[0], (matching?"green":"red"),
-										  dynamic_cast<TimecodeSlave*>(slave)->approximate_current_position()));
-					_right_layout->set_markup (string_compose ("<span size=\"%1\">" TXTSPAN "%2</span></span>",
-										   INFO_FONT_SIZE, slave->approximate_current_delta()));
+					_left_btn.set_text (string_compose ("%1<span face=\"monospace\" foreground=\"%3\">%2</span>",
+								sync_source_to_string(sync_src, true)[0],
+								dynamic_cast<TimecodeSlave*>(slave)->approximate_current_position (),
+								matching ? "#66ff66" : "#ff3333"
+								), true);
+					_right_btn.set_text (slave->approximate_current_delta (), true);
 				}
 			} else {
-				_left_layout->set_markup (string_compose ("<span size=\"%1\">" TXTSPAN "%2</span></span>",
-							INFO_FONT_SIZE, _("--pending--")));
-				_right_layout->set_text ("");
+				_left_btn.set_text (_("--pending--"), true);
+				_right_btn.set_text ("", true);
 			}
 			break;
 		}
 	} else {
-		_left_layout->set_markup (string_compose ("<span size=\"%1\">" TXTSPAN "INT/%2</span></span>",
-					INFO_FONT_SIZE, sync_source_to_string(sync_src, true)));
-		_right_layout->set_text ("");
+		_left_btn.set_text (string_compose ("%1/%2",
+					_("INT"), sync_source_to_string(sync_src, true)), true);
+		_right_btn.set_text ("", true);
+	}
+}
+
+void
+AudioClock::set_out_of_bounds (bool negative)
+{
+	if (is_duration) {
+		if (negative) {
+			_layout->set_text (" >>> -- <<< ");
+		} else {
+			_layout->set_text (" >>> ++ <<< ");
+		}
+	} else {
+		if (negative) {
+			_layout->set_text (" <<<<<<<<<< ");
+		} else {
+			_layout->set_text (" >>>>>>>>>> ");
+		}
 	}
 }
 
@@ -1088,12 +977,8 @@ AudioClock::set_frames (framepos_t when, bool /*force*/)
 
 	if (_off) {
 		_layout->set_text (" ----------");
-
-		if (_left_layout) {
-			_left_layout->set_text ("");
-			_right_layout->set_text ("");
-		}
-
+		_left_btn.set_text ("", true);
+		_right_btn.set_text ("", true);
 		return;
 	}
 
@@ -1102,15 +987,17 @@ AudioClock::set_frames (framepos_t when, bool /*force*/)
 		negative = true;
 	}
 
-	if (negative) {
+	if (when >= _limit_pos) {
+		set_out_of_bounds (negative);
+	} else if (negative) {
 		snprintf (buf, sizeof (buf), "-%10" PRId64, when);
+		_layout->set_text (buf);
 	} else {
 		snprintf (buf, sizeof (buf), " %10" PRId64, when);
+		_layout->set_text (buf);
 	}
 
-	_layout->set_text (buf);
-
-	if (_left_layout) {
+	if (_with_info) {
 		framecnt_t rate = _session->frame_rate();
 
 		if (fmod (rate, 100.0) == 0.0) {
@@ -1119,18 +1006,15 @@ AudioClock::set_frames (framepos_t when, bool /*force*/)
 			sprintf (buf, "%" PRId64 "Hz", rate);
 		}
 
-		_left_layout->set_markup (string_compose ("<span size=\"%1\">" TXTSPAN "%2 </span><span foreground=\"green\">%3</span></span>",
-				INFO_FONT_SIZE, _("SR"), buf));
+		_left_btn.set_text (string_compose ("%1 %2", _("SR"), buf), true);
 
 		float vid_pullup = _session->config.get_video_pullup();
 
 		if (vid_pullup == 0.0) {
-			_right_layout->set_markup (string_compose ("<span size=\"%1\">" TXTSPAN "%2 </span><span foreground=\"green\">off</span></span>",
-					INFO_FONT_SIZE, _("Pull")));
+			_right_btn.set_text ("", true);
 		} else {
 			sprintf (buf, _("%+.4f%%"), vid_pullup);
-			_right_layout->set_markup (string_compose ("<span size=\"%1\">" TXTSPAN "%2 </span><span foreground=\"green\">%3</span></span>",
-					INFO_FONT_SIZE, _("Pull"), buf));
+			_right_btn.set_text (string_compose ("%1 %2", _("Pull"), buf), true);
 		}
 	}
 }
@@ -1176,18 +1060,19 @@ AudioClock::set_minsec (framepos_t when, bool /*force*/)
 
 	if (_off) {
 		_layout->set_text (" --:--:--.---");
-
-		if (_left_layout) {
-			_left_layout->set_text ("");
-			_right_layout->set_text ("");
-		}
+		_left_btn.set_text ("", true);
+		_right_btn.set_text ("", true);
 
 		return;
 	}
 
-	print_minsec (when, buf, sizeof (buf), _session->frame_rate());
+	if (when >= _limit_pos || when <= -_limit_pos) {
+		set_out_of_bounds (when < 0);
+	} else {
+		print_minsec (when, buf, sizeof (buf), _session->frame_rate());
+		_layout->set_text (buf);
+	}
 
-	_layout->set_text (buf);
 	set_slave_info();
 }
 
@@ -1199,11 +1084,8 @@ AudioClock::set_timecode (framepos_t when, bool /*force*/)
 
 	if (_off) {
 		_layout->set_text (" --:--:--:--");
-		if (_left_layout) {
-			_left_layout->set_text ("");
-			_right_layout->set_text ("");
-		}
-
+		_left_btn.set_text ("", true);
+		_right_btn.set_text ("", true);
 		return;
 	}
 
@@ -1211,6 +1093,11 @@ AudioClock::set_timecode (framepos_t when, bool /*force*/)
 		when = -when;
 		negative = true;
 	}
+	if (when >= _limit_pos) {
+		set_out_of_bounds (negative);
+		set_slave_info();
+		return;
+	}
 
 	if (is_duration) {
 		_session->timecode_duration (when, TC);
@@ -1232,12 +1119,10 @@ AudioClock::set_bbt (framepos_t when, framecnt_t offset, bool /*force*/)
 	Timecode::BBT_Time BBT;
 	bool negative = false;
 
-	if (_off) {
+	if (_off || when >= _limit_pos || when < -_limit_pos) {
 		_layout->set_text (" ---|--|----");
-		if (_left_layout) {
-			_left_layout->set_text ("");
-			_right_layout->set_text ("");
-		}
+		_left_btn.set_text ("", true);
+		_right_btn.set_text ("", true);
 		return;
 	}
 
@@ -1305,7 +1190,7 @@ AudioClock::set_bbt (framepos_t when, framecnt_t offset, bool /*force*/)
 
 	_layout->set_text (buf);
 
-	if (_right_layout) {
+	if (_with_info) {
 		framepos_t pos;
 
 		if (bbt_reference_time < 0) {
@@ -1316,14 +1201,19 @@ AudioClock::set_bbt (framepos_t when, framecnt_t offset, bool /*force*/)
 
 		TempoMetric m (_session->tempo_map().metric_at (pos));
 
-		snprintf (buf, sizeof(buf), "%-5.3f/%f", _session->tempo_map().tempo_at_frame (pos).note_types_per_minute(), m.tempo().note_type());
-		/* XXX this doesn't fit inside the container. */
-		_left_layout->set_markup (string_compose ("<span size=\"%1\">" TXTSPAN "%3</span> <span foreground=\"green\">%2</span></span>",
-							  INFO_FONT_SIZE, buf, _("Tempo")));
+		if (m.tempo().note_type() == 4) {
+			snprintf (buf, sizeof(buf), "\u2669 = %.3f", _session->tempo_map().tempo_at_frame (pos).note_types_per_minute());
+			_left_btn.set_text (string_compose ("%1", buf), true);
+		} else if (m.tempo().note_type() == 8) {
+			snprintf (buf, sizeof(buf), "\u266a = %.3f", _session->tempo_map().tempo_at_frame (pos).note_types_per_minute());
+			_left_btn.set_text (string_compose ("%1", buf), true);
+		} else {
+			snprintf (buf, sizeof(buf), "%.1f = %.3f", m.tempo().note_type(), _session->tempo_map().tempo_at_frame (pos).note_types_per_minute());
+			_left_btn.set_text (string_compose ("%1: %2", S_("Tempo|T"), buf), true);
+		}
 
 		snprintf (buf, sizeof(buf), "%g/%g", m.meter().divisions_per_bar(), m.meter().note_divisor());
-		_right_layout->set_markup (string_compose ("<span size=\"%1\">" TXTSPAN "%3</span> <span foreground=\"green\">%2</span></span>",
-							   INFO_FONT_SIZE, buf, _("Meter")));
+		_right_btn.set_text (string_compose ("%1: %2", S_("TimeSignature|TS"), buf), true);
 	}
 }
 
@@ -1334,24 +1224,30 @@ AudioClock::set_session (Session *s)
 
 	if (_session) {
 
+		int64_t limit_sec = UIConfiguration::instance().get_clock_display_limit ();
+		if (limit_sec > 0) {
+			_limit_pos = (framecnt_t) floor ((double)(limit_sec * _session->frame_rate()));
+		}
+
+		Config->ParameterChanged.connect (_session_connections, invalidator (*this), boost::bind (&AudioClock::session_configuration_changed, this, _1), gui_context());
 		_session->config.ParameterChanged.connect (_session_connections, invalidator (*this), boost::bind (&AudioClock::session_configuration_changed, this, _1), gui_context());
 		_session->tempo_map().PropertyChanged.connect (_session_connections, invalidator (*this), boost::bind (&AudioClock::session_property_changed, this, _1), gui_context());
-		_session->tempo_map().MetricPositionChanged.connect (_session_connections, invalidator (*this), boost::bind (&AudioClock::metric_position_changed, this), gui_context());
+		_session->tempo_map().MetricPositionChanged.connect (_session_connections, invalidator (*this), boost::bind (&AudioClock::session_property_changed, this, _1), gui_context());
 
-		XMLProperty const * prop;
 		XMLNode* node = _session->extra_xml (X_("ClockModes"));
-		AudioClock::Mode amode;
 
 		if (node) {
 			for (XMLNodeList::const_iterator i = node->children().begin(); i != node->children().end(); ++i) {
-				if ((prop = (*i)->property (X_("name"))) && prop->value() == _name) {
+				std::string name;
+				if ((*i)->get_property (X_("name"), name) && name == _name) {
 
-					if ((prop = (*i)->property (X_("mode"))) != 0) {
-						amode = AudioClock::Mode (string_2_enum (prop->value(), amode));
+					AudioClock::Mode amode;
+					if ((*i)->get_property (X_("mode"), amode)) {
 						set_mode (amode, true);
 					}
-					if ((prop = (*i)->property (X_("on"))) != 0) {
-						set_off (!string_is_affirmative (prop->value()));
+					bool on;
+					if ((*i)->get_property (X_("on"), on)) {
+						set_off (!on);
 					}
 					break;
 				}
@@ -1625,6 +1521,11 @@ AudioClock::index_to_field (int index) const
 bool
 AudioClock::on_button_press_event (GdkEventButton *ev)
 {
+	if (!_session || _session->actively_recording()) {
+		/* swallow event, do nothing */
+		return true;
+	}
+
 	switch (ev->button) {
 	case 1:
 		if (editable && !_off) {
@@ -1638,7 +1539,7 @@ AudioClock::on_button_press_event (GdkEventButton *ev)
 			 */
 			int xcenter = (get_width() - layout_width) /2;
 
-			y = ev->y - ((upper_height - layout_height)/2);
+			y = ev->y - ((get_height() - layout_height)/2);
 			x = ev->x - xcenter;
 
 			if (!_layout->xy_to_index (x * PANGO_SCALE, y * PANGO_SCALE, index, trailing)) {
@@ -1668,6 +1569,11 @@ AudioClock::on_button_press_event (GdkEventButton *ev)
 bool
 AudioClock::on_button_release_event (GdkEventButton *ev)
 {
+	if (!_session || _session->actively_recording()) {
+		/* swallow event, do nothing */
+		return true;
+	}
+
 	if (editable && !_off) {
 		if (dragging) {
 			gdk_pointer_ungrab (GDK_CURRENT_TIME);
@@ -1684,7 +1590,7 @@ AudioClock::on_button_release_event (GdkEventButton *ev)
 						int xcenter = (get_width() - layout_width) /2;
 						int index = 0;
 						int trailing;
-						int y = ev->y - ((upper_height - layout_height)/2);
+						int y = ev->y - ((get_height() - layout_height)/2);
 						int x = ev->x - xcenter;
 						Field f;
 
@@ -1741,7 +1647,7 @@ AudioClock::on_scroll_event (GdkEventScroll *ev)
 	int index;
 	int trailing;
 
-	if (editing || _session == 0 || !editable || _off) {
+	if (editing || _session == 0 || !editable || _off || _session->actively_recording())  {
 		return false;
 	}
 
@@ -1753,7 +1659,7 @@ AudioClock::on_scroll_event (GdkEventScroll *ev)
 	 */
 
 	int xcenter = (get_width() - layout_width) /2;
-	y = ev->y - ((upper_height - layout_height)/2);
+	y = ev->y - ((get_height() - layout_height)/2);
 	x = ev->x - xcenter;
 
 	if (!_layout->xy_to_index (x * PANGO_SCALE, y * PANGO_SCALE, index, trailing)) {
@@ -1805,7 +1711,7 @@ AudioClock::on_scroll_event (GdkEventScroll *ev)
 bool
 AudioClock::on_motion_notify_event (GdkEventMotion *ev)
 {
-	if (editing || _session == 0 || !dragging) {
+	if (editing || _session == 0 || !dragging || _session->actively_recording()) {
 		return false;
 	}
 
@@ -1817,8 +1723,7 @@ AudioClock::on_motion_notify_event (GdkEventMotion *ev)
 
 
 	if (Keyboard::modifier_state_contains (ev->state,
-					       Keyboard::PrimaryModifier|Keyboard::SecondaryModifier)) {
-
+	                                       Keyboard::PrimaryModifier|Keyboard::SecondaryModifier)) {
 		pixel_frame_scale_factor = 0.025f;
 	}
 
@@ -1841,9 +1746,9 @@ AudioClock::on_motion_notify_event (GdkEventMotion *ev)
 			set ((framepos_t) floor (pos - drag_accum * frames), false); // minus because up is negative in GTK
 		} else {
 			set (0 , false);
- 		}
+		}
 
-	       	drag_accum= 0;
+		drag_accum= 0;
 		ValueChanged();	 /* EMIT_SIGNAL */
 	}
 
@@ -1976,7 +1881,7 @@ AudioClock::timecode_validate_edit (const string&)
 	char ignored[2];
 
 	if (sscanf (_layout->get_text().c_str(), "%[- _]%" PRId32 ":%" PRId32 ":%" PRId32 "%[:;]%" PRId32,
-		    ignored, &hours, &TC.minutes, &TC.seconds, ignored, &TC.frames) != 6) {
+	            ignored, &hours, &TC.minutes, &TC.seconds, ignored, &TC.frames) != 6) {
 		return false;
 	}
 
@@ -2091,10 +1996,10 @@ AudioClock::frames_from_bbt_string (framepos_t pos, const string& str) const
 	if (is_duration) {
 		any.bbt.bars++;
 		any.bbt.beats++;
-                return _session->any_duration_to_frames (pos, any);
+		return _session->any_duration_to_frames (pos, any);
 	} else {
-                return _session->convert_to_frames (any);
-        }
+		return _session->convert_to_frames (any);
+	}
 }
 
 
@@ -2200,22 +2105,8 @@ AudioClock::set_mode (Mode m, bool noemit)
 	Gtk::Requisition req;
 	set_clock_dimensions (req);
 
-	if (_left_layout) {
-
-		_left_layout->set_attributes (info_attributes);
-		_right_layout->set_attributes (info_attributes);
-		/* adjust info_height according to font size */
-		int ignored;
-		_left_layout->set_text (" 1234567890");
-		_left_layout->get_pixel_size (ignored, info_height);
-
-		_left_layout->set_text ("");
-		_right_layout->set_text ("");
-	}
-
 	switch (_mode) {
 	case Timecode:
-		mode_based_info_ratio = 0.6;
 		insert_map.push_back (11);
 		insert_map.push_back (10);
 		insert_map.push_back (8);
@@ -2227,7 +2118,6 @@ AudioClock::set_mode (Mode m, bool noemit)
 		break;
 
 	case BBT:
-		mode_based_info_ratio = 0.5;
 		insert_map.push_back (11);
 		insert_map.push_back (10);
 		insert_map.push_back (9);
@@ -2240,7 +2130,6 @@ AudioClock::set_mode (Mode m, bool noemit)
 		break;
 
 	case MinSec:
-		mode_based_info_ratio = 0.6;
 		insert_map.push_back (12);
 		insert_map.push_back (11);
 		insert_map.push_back (10);
@@ -2253,7 +2142,6 @@ AudioClock::set_mode (Mode m, bool noemit)
 		break;
 
 	case Frames:
-		mode_based_info_ratio = 0.45;
 		break;
 	}
 
