@@ -25,8 +25,6 @@
 
 #include <gtkmm.h>
 
-#include <gtkmm2ext/gtk_ui.h>
-
 #include "ardour/playlist.h"
 #include "ardour/audioregion.h"
 #include "ardour/audiosource.h"
@@ -38,6 +36,10 @@
 
 #include "evoral/Curve.hpp"
 
+#include "gtkmm2ext/gtk_ui.h"
+#include "gtkmm2ext/utils.h"
+#include "gtkmm2ext/colors.h"
+
 #include "canvas/rectangle.h"
 #include "canvas/polygon.h"
 #include "canvas/poly_line.h"
@@ -45,12 +47,13 @@
 #include "canvas/text.h"
 #include "canvas/xfade_curve.h"
 #include "canvas/debug.h"
-#include "canvas/utils.h"
-#include "canvas/colors.h"
+
+#include "waveview/debug.h"
 
 #include "streamview.h"
 #include "audio_region_view.h"
 #include "audio_time_axis.h"
+#include "enums_convert.h"
 #include "public_editor.h"
 #include "audio_region_editor.h"
 #include "audio_streamview.h"
@@ -183,7 +186,7 @@ AudioRegionView::init (bool wfd)
 	// needs to be created first, RegionView::init() calls set_height()
 	pending_peak_data = new ArdourCanvas::Rectangle (group);
 	CANVAS_DEBUG_NAME (pending_peak_data, string_compose ("pending peak rectangle for %1", region()->name()));
-	pending_peak_data->set_outline_color (ArdourCanvas::rgba_to_color (0, 0, 0, 0.0));
+	pending_peak_data->set_outline_color (Gtkmm2ext::rgba_to_color (0, 0, 0, 0.0));
 	pending_peak_data->set_pattern (pending_peak_pattern);
 	pending_peak_data->set_data ("regionview", this);
 	pending_peak_data->hide ();
@@ -197,28 +200,28 @@ AudioRegionView::init (bool wfd)
 	if (!_recregion) {
 		fade_in_handle = new ArdourCanvas::Rectangle (group);
 		CANVAS_DEBUG_NAME (fade_in_handle, string_compose ("fade in handle for %1", region()->name()));
-		fade_in_handle->set_outline_color (ArdourCanvas::rgba_to_color (0, 0, 0, 1.0));
+		fade_in_handle->set_outline_color (Gtkmm2ext::rgba_to_color (0, 0, 0, 1.0));
 		fade_in_handle->set_fill_color (UIConfiguration::instance().color ("inactive fade handle"));
 		fade_in_handle->set_data ("regionview", this);
 		fade_in_handle->hide ();
 
 		fade_out_handle = new ArdourCanvas::Rectangle (group);
 		CANVAS_DEBUG_NAME (fade_out_handle, string_compose ("fade out handle for %1", region()->name()));
-		fade_out_handle->set_outline_color (ArdourCanvas::rgba_to_color (0, 0, 0, 1.0));
+		fade_out_handle->set_outline_color (Gtkmm2ext::rgba_to_color (0, 0, 0, 1.0));
 		fade_out_handle->set_fill_color (UIConfiguration::instance().color ("inactive fade handle"));
 		fade_out_handle->set_data ("regionview", this);
 		fade_out_handle->hide ();
 
 		fade_in_trim_handle = new ArdourCanvas::Rectangle (group);
 		CANVAS_DEBUG_NAME (fade_in_handle, string_compose ("fade in trim handle for %1", region()->name()));
-		fade_in_trim_handle->set_outline_color (ArdourCanvas::rgba_to_color (0, 0, 0, 1.0));
+		fade_in_trim_handle->set_outline_color (Gtkmm2ext::rgba_to_color (0, 0, 0, 1.0));
 		fade_in_trim_handle->set_fill_color (UIConfiguration::instance().color ("inactive fade handle"));
 		fade_in_trim_handle->set_data ("regionview", this);
 		fade_in_trim_handle->hide ();
 
 		fade_out_trim_handle = new ArdourCanvas::Rectangle (group);
 		CANVAS_DEBUG_NAME (fade_out_handle, string_compose ("fade out trim handle for %1", region()->name()));
-		fade_out_trim_handle->set_outline_color (ArdourCanvas::rgba_to_color (0, 0, 0, 1.0));
+		fade_out_trim_handle->set_outline_color (Gtkmm2ext::rgba_to_color (0, 0, 0, 1.0));
 		fade_out_trim_handle->set_fill_color (UIConfiguration::instance().color ("inactive fade handle"));
 		fade_out_trim_handle->set_data ("regionview", this);
 		fade_out_trim_handle->hide ();
@@ -378,6 +381,7 @@ AudioRegionView::region_scale_amplitude_changed ()
 	for (uint32_t n = 0; n < waves.size(); ++n) {
 		waves[n]->gain_changed ();
 	}
+	region_renamed ();
 }
 
 void
@@ -393,6 +397,14 @@ AudioRegionView::region_renamed ()
 		str = string ("!") + str;
 	}
 
+
+	boost::shared_ptr<AudioRegion> ar (audio_region());
+	if (ar->scale_amplitude() != 1.0) {
+		char tmp[32];
+		snprintf (tmp, 32, " (%.1fdB)", accurate_coefficient_to_dB (ar->scale_amplitude()));
+		str += tmp;
+	}
+
 	set_item_name (str, this);
 	set_name_text (str);
 }
@@ -417,7 +429,7 @@ AudioRegionView::region_resized (const PropertyChange& what_changed)
 		for (vector<GhostRegion*>::iterator i = ghosts.begin(); i != ghosts.end(); ++i) {
 			if ((agr = dynamic_cast<AudioGhostRegion*>(*i)) != 0) {
 
-				for (vector<WaveView*>::iterator w = agr->waves.begin(); w != agr->waves.end(); ++w) {
+				for (vector<ArdourWaveView::WaveView*>::iterator w = agr->waves.begin(); w != agr->waves.end(); ++w) {
 					(*w)->region_resized ();
 				}
 			}
@@ -878,7 +890,7 @@ AudioRegionView::redraw_end_xfade ()
 
 void
 AudioRegionView::redraw_end_xfade_to (boost::shared_ptr<AudioRegion> ar, framecnt_t width, Points& points, double effective_height,
-				      double rect_edge, double rect_width)
+                                      double rect_edge, double rect_width)
 {
 	if (points.size() < 2) {
 		return;
@@ -1129,7 +1141,7 @@ AudioRegionView::delete_waves ()
 	}
 	_data_ready_connections.clear ();
 
-	for (vector<WaveView*>::iterator w = waves.begin(); w != waves.end(); ++w) {
+	for (vector<ArdourWaveView::WaveView*>::iterator w = waves.begin(); w != waves.end(); ++w) {
 		group->remove(*w);
 	}
 	waves.clear();
@@ -1227,7 +1239,7 @@ AudioRegionView::create_one_wave (uint32_t which, bool /*direct*/)
 
 	gdouble yoff = which * ht;
 
-	WaveView *wave = new WaveView (group, audio_region ());
+	ArdourWaveView::WaveView *wave = new ArdourWaveView::WaveView (group, audio_region ());
 	CANVAS_DEBUG_NAME (wave, string_compose ("wave view for chn %1 of %2", which, get_item_name()));
 
 	wave->set_channel (which);
@@ -1242,15 +1254,15 @@ AudioRegionView::create_one_wave (uint32_t which, bool /*direct*/)
 
 	switch (UIConfiguration::instance().get_waveform_shape()) {
 	case Rectified:
-		wave->set_shape (WaveView::Rectified);
+		wave->set_shape (ArdourWaveView::WaveView::Rectified);
 		break;
 	default:
-		wave->set_shape (WaveView::Normal);
+		wave->set_shape (ArdourWaveView::WaveView::Normal);
 	}
 
 	wave->set_logscaled (UIConfiguration::instance().get_waveform_scale() == Logarithmic);
 
-	vector<ArdourCanvas::WaveView*> v;
+	vector<ArdourWaveView::WaveView*> v;
 	v.push_back (wave);
 	set_some_waveform_colors (v);
 
@@ -1289,11 +1301,10 @@ AudioRegionView::create_one_wave (uint32_t which, bool /*direct*/)
 		pending_peak_data->hide ();
 
 		/* Restore stacked coverage */
-		std::string str = trackview.gui_property ("layer-display");
-		if (!str.empty()) {
-			LayerDisplay layer_display;
-			update_coverage_frames (LayerDisplay (string_2_enum (str, layer_display)));
-		}
+		LayerDisplay layer_display;
+		if (trackview.get_gui_property ("layer-display", layer_display)) {
+			update_coverage_frames (layer_display);
+	  }
 	}
 
 	/* channel wave created, don't hook into peaks ready anymore */
@@ -1338,11 +1349,10 @@ AudioRegionView::add_gain_point_event (ArdourCanvas::Item *item, GdkEvent *ev, b
 	/* don't create points that can't be seen */
 
 	update_envelope_visibility ();
-
 	framepos_t rpos = region ()->position ();
-	framepos_t fx = trackview.editor().pixel_to_sample (mx) + rpos;
-	trackview.editor ().snap_to_with_modifier (fx, ev);
-	fx -= rpos;
+	MusicFrame snap_pos (trackview.editor().pixel_to_sample (mx) + rpos, 0);
+	trackview.editor ().snap_to_with_modifier (snap_pos, ev);
+	framepos_t fx = snap_pos.frame - rpos;
 
 	if (fx > _region->length()) {
 		return;
@@ -1418,7 +1428,7 @@ AudioRegionView::add_ghost (TimeAxisView& tv)
 			break;
 		}
 
-		WaveView *wave = new WaveView (ghost->group, audio_region());
+		ArdourWaveView::WaveView *wave = new ArdourWaveView::WaveView (ghost->group, audio_region());
 		CANVAS_DEBUG_NAME (wave, string_compose ("ghost wave for %1", get_item_name()));
 
 		wave->set_channel (n);
@@ -1538,12 +1548,12 @@ AudioRegionView::set_waveform_colors ()
 }
 
 void
-AudioRegionView::set_some_waveform_colors (vector<ArdourCanvas::WaveView*>& waves_to_color)
+AudioRegionView::set_some_waveform_colors (vector<ArdourWaveView::WaveView*>& waves_to_color)
 {
-	ArdourCanvas::Color fill;
-	ArdourCanvas::Color outline;
-	ArdourCanvas::Color clip = UIConfiguration::instance().color ("clipped waveform");
-	ArdourCanvas::Color zero = UIConfiguration::instance().color ("zero line");
+	Gtkmm2ext::Color fill;
+	Gtkmm2ext::Color outline;
+	Gtkmm2ext::Color clip = UIConfiguration::instance().color ("clipped waveform");
+	Gtkmm2ext::Color zero = UIConfiguration::instance().color ("zero line");
 
 	if (_selected) {
 		if (_region->muted()) {
@@ -1570,7 +1580,7 @@ AudioRegionView::set_some_waveform_colors (vector<ArdourCanvas::WaveView*>& wave
 		}
 	}
 
-        for (vector<ArdourCanvas::WaveView*>::iterator w = waves_to_color.begin(); w != waves_to_color.end(); ++w) {
+	for (vector<ArdourWaveView::WaveView*>::iterator w = waves_to_color.begin(); w != waves_to_color.end(); ++w) {
 		(*w)->set_fill_color (fill);
 		(*w)->set_outline_color (outline);
 		(*w)->set_clip_color (clip);
@@ -1615,17 +1625,14 @@ AudioRegionView::set_fade_visibility (bool yn)
 void
 AudioRegionView::update_coverage_frames (LayerDisplay d)
 {
-	if (d != Stacked) {
-		/* don't do coverage frames unless we're in stacked mode */
-		return;
-	}
-
 	RegionView::update_coverage_frames (d);
 
-	if (fade_in_handle)       { fade_in_handle->raise_to_top (); }
-	if (fade_out_handle)      { fade_out_handle->raise_to_top (); }
-	if (fade_in_trim_handle)  { fade_in_trim_handle->raise_to_top (); }
-	if (fade_out_trim_handle) { fade_out_trim_handle->raise_to_top (); }
+	if (d == Stacked) {
+		if (fade_in_handle)       { fade_in_handle->raise_to_top (); }
+		if (fade_out_handle)      { fade_out_handle->raise_to_top (); }
+		if (fade_in_trim_handle)  { fade_in_trim_handle->raise_to_top (); }
+		if (fade_out_trim_handle) { fade_out_trim_handle->raise_to_top (); }
+	}
 }
 
 void
@@ -1715,11 +1722,11 @@ AudioRegionView::update_transient(float /*old_pos*/, float new_pos)
 		float* pos = (float*) (*l).second->get_data ("position");
 
 		if (rint(new_pos) == rint(*pos)) {
-				framepos_t position = _region->position();
-		    framepos_t old_frame = (*l).first;
-		    framepos_t new_frame = trackview.editor().pixel_to_sample (new_pos) + position;
-		    _region->update_transient (old_frame, new_frame);
-		    break;
+			framepos_t position = _region->position();
+			framepos_t old_frame = (*l).first;
+			framepos_t new_frame = trackview.editor().pixel_to_sample (new_pos) + position;
+			_region->update_transient (old_frame, new_frame);
+			break;
 		}
 	}
 }
@@ -1737,8 +1744,8 @@ AudioRegionView::remove_transient (float pos)
 	for (l = feature_lines.begin(); l != feature_lines.end(); ++l) {
 		float *line_pos = (float*) (*l).second->get_data ("position");
 		if (rint(pos) == rint(*line_pos)) {
-		    _region->remove_transient ((*l).first);
-		    break;
+			_region->remove_transient ((*l).first);
+			break;
 		}
 	}
 }
