@@ -581,6 +581,59 @@ Editor::autoscroll_active () const
 	return autoscroll_connection.connected ();
 }
 
+std::pair <framepos_t,framepos_t>
+Editor::session_gui_extents ( bool use_extra ) const
+{
+	if (!_session) {
+		return std::pair <framepos_t,framepos_t>(max_framepos,0);
+	}
+	
+	framecnt_t session_extent_start = _session->current_start_frame();
+	framecnt_t session_extent_end = _session->current_end_frame();
+
+	//calculate the extents of all regions in every playlist
+	//NOTE:  we should listen to playlists, and cache these values so we don't calculate them every time.
+	{
+		boost::shared_ptr<RouteList> rl = _session->get_routes();
+		for (RouteList::iterator r = rl->begin(); r != rl->end(); ++r) {
+			boost::shared_ptr<Track> tr = boost::dynamic_pointer_cast<Track> (*r);
+			if (tr) {
+				boost::shared_ptr<Playlist> pl = tr->playlist();
+				if ( pl && !pl->all_regions_empty() ) {
+					pair<framepos_t, framepos_t> e;
+					e = pl->get_extent();
+					if (e.first < session_extent_start) {
+						session_extent_start = e.first;
+					}
+					if (e.second > session_extent_end) {
+						session_extent_end = e.second;
+					}
+				}
+			}
+		}
+	}
+
+	//ToDo: also incorporate automation regions (in case the session has no audio/midi but is just used for automating plugins or the like)
+
+	//add additional time to the ui extents ( user-defined in config )
+	if (use_extra) {
+		framecnt_t const extra = UIConfiguration::instance().get_extra_ui_extents_time() * 60 * _session->nominal_frame_rate();
+		session_extent_end += extra;
+		session_extent_start -= extra;
+	}
+			
+	//range-check
+	if (session_extent_end > max_framepos) {
+		session_extent_end = max_framepos;
+	}
+	if (session_extent_start < 0) {
+		session_extent_start = 0;
+	}
+	
+	std::pair <framepos_t,framepos_t> ret (session_extent_start, session_extent_end);
+	return ret;
+}
+
 bool
 Editor::autoscroll_canvas ()
 {
@@ -612,6 +665,8 @@ Editor::autoscroll_canvas ()
 			dx += 10 + (2 * (autoscroll_cnt/2));
 
 			dx = pixel_to_sample (dx);
+			
+			dx *= UIConfiguration::instance().get_draggable_playhead_speed();
 
 			if (leftmost_frame < max_framepos - dx) {
 				new_frame = leftmost_frame + dx;
@@ -628,6 +683,8 @@ Editor::autoscroll_canvas ()
 
 			dx = pixel_to_sample (dx);
 
+			dx *= UIConfiguration::instance().get_draggable_playhead_speed();
+
 			if (leftmost_frame >= dx) {
 				new_frame = leftmost_frame - dx;
 			} else {
