@@ -171,7 +171,7 @@ Editor::track_canvas_button_press_event (GdkEventButton *event)
 		begin_reversible_selection_op (X_("Clear Selection Click (track canvas)"));
 		selection->clear ();
 		commit_reversible_selection_op();
-        }
+	}
 	return false;
 }
 
@@ -179,10 +179,10 @@ bool
 Editor::track_canvas_button_release_event (GdkEventButton *event)
 {
 	if (!Keyboard::is_context_menu_event (event)) {
-                if (_drags->active ()) {
-                        _drags->end_grab ((GdkEvent*) event);
-                }
-        }
+		if (_drags->active ()) {
+			_drags->end_grab ((GdkEvent*) event);
+		}
+	}
 	return false;
 }
 
@@ -198,6 +198,10 @@ Editor::track_canvas_motion_notify_event (GdkEventMotion */*event*/)
 bool
 Editor::typed_event (ArdourCanvas::Item* item, GdkEvent *event, ItemType type)
 {
+	if (!session () || session()->loading () || session()->deletion_in_progress ()) {
+		return false;
+	}
+
 	gint ret = FALSE;
 
 	switch (event->type) {
@@ -532,7 +536,6 @@ Editor::canvas_fade_in_handle_event (GdkEvent *event, ArdourCanvas::Item* item,
 
 	case GDK_BUTTON_RELEASE:
 		ret = button_release_handler (item, event, trim ? FadeInTrimHandleItem : FadeInHandleItem);
-		maybe_locate_with_edit_preroll ( rv->region()->position() );
 		break;
 
 	case GDK_MOTION_NOTIFY:
@@ -616,7 +619,6 @@ Editor::canvas_fade_out_handle_event (GdkEvent *event, ArdourCanvas::Item* item,
 
 	case GDK_BUTTON_RELEASE:
 		ret = button_release_handler (item, event, trim ? FadeOutTrimHandleItem : FadeOutHandleItem);
-		maybe_locate_with_edit_preroll ( rv->region()->last_frame() - rv->get_fade_out_shape_width() );
 		break;
 
 	case GDK_MOTION_NOTIFY:
@@ -639,9 +641,9 @@ Editor::canvas_fade_out_handle_event (GdkEvent *event, ArdourCanvas::Item* item,
 }
 
 struct DescendingRegionLayerSorter {
-    bool operator()(boost::shared_ptr<Region> a, boost::shared_ptr<Region> b) {
-	    return a->layer() > b->layer();
-    }
+	bool operator()(boost::shared_ptr<Region> a, boost::shared_ptr<Region> b) {
+		return a->layer() > b->layer();
+	}
 };
 
 bool
@@ -1008,7 +1010,7 @@ Editor::canvas_videotl_bar_event (GdkEvent *event, ArdourCanvas::Item* item)
 }
 
 bool
-Editor::canvas_tempo_marker_event (GdkEvent *event, ArdourCanvas::Item* item, TempoMarker* /*marker*/)
+Editor::canvas_tempo_marker_event (GdkEvent *event, ArdourCanvas::Item* item, TempoMarker* marker)
 {
 	return typed_event (item, event, TempoMarkerItem);
 }
@@ -1078,18 +1080,6 @@ Editor::canvas_ruler_event (GdkEvent *event, ArdourCanvas::Item* item, ItemType
 		return handled;
 	}
 
-	switch (event->type) {
-	case GDK_BUTTON_PRESS:
-		if (UIConfiguration::instance ().get_use_time_rulers_to_zoom_with_vertical_drag () &&
-		    Keyboard::no_modifier_keys_pressed (&event->button) && event->button.button == 1) {
-			_drags->set(new RulerZoomDrag(this, item), event);
-			return true;
-		}
-		break;
-	default:
-		break;
-	}
-
 	return typed_event (item, event, type);
 }
 
@@ -1212,8 +1202,8 @@ Editor::track_canvas_drag_motion (Glib::RefPtr<Gdk::DragContext> const& context,
 
 			if (tv.first == 0
 			    && (
-			        boost::dynamic_pointer_cast<AudioRegion> (region) != 0 ||
-			        boost::dynamic_pointer_cast<MidiRegion> (region) != 0
+				boost::dynamic_pointer_cast<AudioRegion> (region) != 0 ||
+				boost::dynamic_pointer_cast<MidiRegion> (region) != 0
 			       )
 			   )
 			{
@@ -1261,9 +1251,9 @@ Editor::track_canvas_drag_motion (Glib::RefPtr<Gdk::DragContext> const& context,
 
 void
 Editor::drop_regions (const Glib::RefPtr<Gdk::DragContext>& /*context*/,
-		      int x, int y,
-		      const SelectionData& /*data*/,
-		      guint /*info*/, guint /*time*/)
+                      int x, int y,
+                      const SelectionData& /*data*/,
+                      guint /*info*/, guint /*time*/)
 {
 	GdkEvent event;
 	double px;
@@ -1293,14 +1283,16 @@ Editor::drop_regions (const Glib::RefPtr<Gdk::DragContext>& /*context*/,
 				}
 				list<boost::shared_ptr<AudioTrack> > audio_tracks;
 				audio_tracks = session()->new_audio_track (region->n_channels(), output_chan, 0, 1, region->name(), PresentationInfo::max_order);
-				rtav = dynamic_cast<RouteTimeAxisView*> (axis_view_from_stripable (audio_tracks.front()));
+				rtav = dynamic_cast<RouteTimeAxisView*> (time_axis_view_from_stripable (audio_tracks.front()));
 			} else if (boost::dynamic_pointer_cast<MidiRegion> (region)) {
 				ChanCount one_midi_port (DataType::MIDI, 1);
 				list<boost::shared_ptr<MidiTrack> > midi_tracks;
-				midi_tracks = session()->new_midi_track (one_midi_port, one_midi_port, boost::shared_ptr<ARDOUR::PluginInfo>(),
+				midi_tracks = session()->new_midi_track (one_midi_port, one_midi_port,
+				                                         Config->get_strict_io () || Profile->get_mixbus (),
+				                                         boost::shared_ptr<ARDOUR::PluginInfo>(),
 				                                         (ARDOUR::Plugin::PresetRecord*) 0,
 				                                         (ARDOUR::RouteGroup*) 0, 1, region->name(), PresentationInfo::max_order);
-				rtav = dynamic_cast<RouteTimeAxisView*> (axis_view_from_stripable (midi_tracks.front()));
+				rtav = dynamic_cast<RouteTimeAxisView*> (time_axis_view_from_stripable (midi_tracks.front()));
 			} else {
 				return;
 			}
@@ -1316,7 +1308,7 @@ Editor::drop_regions (const Glib::RefPtr<Gdk::DragContext>& /*context*/,
 		if ((boost::dynamic_pointer_cast<AudioRegion> (region_copy) != 0 && dynamic_cast<AudioTimeAxisView*> (rtav) != 0) ||
 		    (boost::dynamic_pointer_cast<MidiRegion> (region_copy) != 0 && dynamic_cast<MidiTimeAxisView*> (rtav) != 0)) {
 			_drags->set (new RegionInsertDrag (this, region_copy, rtav, pos), &event);
-			_drags->end_grab (0);
+			_drags->end_grab (&event);
 		}
 	}
 }
