@@ -65,6 +65,8 @@ private:
 	bool on_enter_notify_event (GdkEventCrossing*);
 	bool on_leave_notify_event (GdkEventCrossing*);
 
+	void reset_to_extents ();
+
 	void centre_on_click (GdkEventButton *);
 	void render (Cairo::RefPtr<Cairo::Context> const&, cairo_rectangle_t*);
 	void render_region (RegionView*, cairo_t*, double) const;
@@ -81,7 +83,7 @@ private:
 	bool suspending_editor_updates () const;
 	double playhead_frame_to_position (framepos_t) const;
 	framepos_t position_to_playhead_frame_to_position (double pos) const;
-	void set_overlays_dirty (int, int, int, int);
+	void set_overlays_dirty_rect (int, int, int, int);
 
 	void summary_zoom_step (  int steps );
 
