@@ -75,11 +75,16 @@ Editor::remove_metric_marks ()
 	}
 	tempo_curves.clear ();
 }
-
+struct CurveComparator {
+	bool operator() (TempoCurve const * a, TempoCurve const * b) {
+		return a->tempo().frame() < b->tempo().frame();
+	}
+};
 void
 Editor::draw_metric_marks (const Metrics& metrics)
 {
 	char buf[64];
+	TempoSection* prev_ts = 0;
 	double max_tempo = 0.0;
 	double min_tempo = DBL_MAX;
 
@@ -87,7 +92,7 @@ Editor::draw_metric_marks (const Metrics& metrics)
 
 	for (Metrics::const_iterator i = metrics.begin(); i != metrics.end(); ++i) {
 		const MeterSection *ms;
-		const TempoSection *ts;
+		TempoSection *ts;
 
 		if ((ms = dynamic_cast<const MeterSection*>(*i)) != 0) {
 			snprintf (buf, sizeof(buf), "%g/%g", ms->divisions_per_bar(), ms->note_divisor ());
@@ -98,29 +103,35 @@ Editor::draw_metric_marks (const Metrics& metrics)
 				metric_marks.push_back (new MeterMarker (*this, *meter_group, UIConfiguration::instance().color ("meter marker"), buf,
 									 *(const_cast<MeterSection*>(ms))));
 			}
-		} else if ((ts = dynamic_cast<const TempoSection*>(*i)) != 0) {
-			if (UIConfiguration::instance().get_allow_non_quarter_pulse()) {
-				snprintf (buf, sizeof (buf), "%.3f/%.0f", ts->note_types_per_minute(), ts->note_type());
-			} else {
-				snprintf (buf, sizeof (buf), "%.3f", ts->note_types_per_minute());
-			}
+		} else if ((ts = dynamic_cast<TempoSection*>(*i)) != 0) {
 
 			max_tempo = max (max_tempo, ts->note_types_per_minute());
+			max_tempo = max (max_tempo, ts->end_note_types_per_minute());
 			min_tempo = min (min_tempo, ts->note_types_per_minute());
+			min_tempo = min (min_tempo, ts->end_note_types_per_minute());
+			uint32_t const tc_color = UIConfiguration::instance().color ("tempo curve");
 
-			tempo_curves.push_back (new TempoCurve (*this, *tempo_group, UIConfiguration::instance().color ("tempo curve"),
+			tempo_curves.push_back (new TempoCurve (*this, *tempo_group, tc_color,
 								*(const_cast<TempoSection*>(ts)), ts->frame(), false));
+
+			const std::string tname (X_(""));
 			if (ts->position_lock_style() == MusicTime) {
-				metric_marks.push_back (new TempoMarker (*this, *tempo_group, UIConfiguration::instance().color ("tempo marker music"), buf,
+				metric_marks.push_back (new TempoMarker (*this, *tempo_group, UIConfiguration::instance().color ("tempo marker music"), tname,
 								 *(const_cast<TempoSection*>(ts))));
 			} else {
-				metric_marks.push_back (new TempoMarker (*this, *tempo_group, UIConfiguration::instance().color ("tempo marker"), buf,
+				metric_marks.push_back (new TempoMarker (*this, *tempo_group, UIConfiguration::instance().color ("tempo marker"), tname,
 								 *(const_cast<TempoSection*>(ts))));
 			}
-
+			if (prev_ts && abs (prev_ts->end_note_types_per_minute() - ts->note_types_per_minute()) < 1.0) {
+				metric_marks.back()->set_points_color (UIConfiguration::instance().color ("tempo marker music"));
+			} else {
+				metric_marks.back()->set_points_color (UIConfiguration::instance().color ("tempo marker"));
+			}
+			prev_ts = ts;
 		}
 
 	}
+	tempo_curves.sort (CurveComparator());
 
 	const double min_tempo_range = 5.0;
 	const double tempo_delta = fabs (max_tempo - min_tempo);
@@ -140,6 +151,13 @@ Editor::draw_metric_marks (const Metrics& metrics)
 		} else {
 			(*x)->set_position ((*x)->tempo().frame(), UINT32_MAX);
 		}
+
+		if (!(*x)->tempo().active()) {
+			(*x)->hide();
+		} else {
+			(*x)->show();
+		}
+
 		++x;
 	}
 
@@ -163,7 +181,7 @@ Editor::tempo_map_changed (const PropertyChange& /*ignored*/)
 	ENSURE_GUI_THREAD (*this, &Editor::tempo_map_changed, ignored);
 
 	if (tempo_lines) {
-		tempo_lines->tempo_map_changed();
+		tempo_lines->tempo_map_changed(_session->tempo_map().music_origin());
 	}
 
 	compute_bbt_ruler_scale (leftmost_frame, leftmost_frame + current_page_samples());
@@ -176,14 +194,8 @@ Editor::tempo_map_changed (const PropertyChange& /*ignored*/)
 	update_tempo_based_rulers ();
 }
 
-struct CurveComparator {
-	bool operator() (TempoCurve const * a, TempoCurve const * b) {
-		return a->position() < b->position();
-	}
-};
-
 void
-Editor::marker_position_changed ()
+Editor::tempometric_position_changed (const PropertyChange& /*ignored*/)
 {
 	if (!_session) {
 		return;
@@ -192,33 +204,36 @@ Editor::marker_position_changed ()
 	ENSURE_GUI_THREAD (*this, &Editor::tempo_map_changed);
 
 	if (tempo_lines) {
-		tempo_lines->tempo_map_changed();
+		tempo_lines->tempo_map_changed(_session->tempo_map().music_origin());
 	}
 
+	TempoSection* prev_ts = 0;
 	double max_tempo = 0.0;
 	double min_tempo = DBL_MAX;
 
 	for (Marks::iterator x = metric_marks.begin(); x != metric_marks.end(); ++x) {
 		TempoMarker* tempo_marker;
 		MeterMarker* meter_marker;
-		const TempoSection *ts;
+		TempoSection *ts;
 		const MeterSection *ms;
 
 		if ((tempo_marker = dynamic_cast<TempoMarker*> (*x)) != 0) {
 			if ((ts = &tempo_marker->tempo()) != 0) {
-				tempo_marker->set_position (ts->frame ());
-				char buf[64];
 
-				if (UIConfiguration::instance().get_allow_non_quarter_pulse()) {
-					snprintf (buf, sizeof (buf), "%.3f/%.0f", ts->note_types_per_minute(), ts->note_type());
+				tempo_marker->set_position (ts->frame ());
+
+				if (prev_ts && abs (prev_ts->end_note_types_per_minute() - ts->note_types_per_minute()) < 1.0) {
+					tempo_marker->set_points_color (UIConfiguration::instance().color ("tempo marker music"));
 				} else {
-					snprintf (buf, sizeof (buf), "%.3f", ts->note_types_per_minute());
+					tempo_marker->set_points_color (UIConfiguration::instance().color ("tempo marker"));
 				}
 
-				tempo_marker->set_name (buf);
-
 				max_tempo = max (max_tempo, ts->note_types_per_minute());
+				max_tempo = max (max_tempo, ts->end_note_types_per_minute());
 				min_tempo = min (min_tempo, ts->note_types_per_minute());
+				min_tempo = min (min_tempo, ts->end_note_types_per_minute());
+
+				prev_ts = ts;
 			}
 		}
 		if ((meter_marker = dynamic_cast<MeterMarker*> (*x)) != 0) {
@@ -248,6 +263,13 @@ Editor::marker_position_changed ()
 		} else {
 			(*x)->set_position ((*x)->tempo().frame(), UINT32_MAX);
 		}
+
+		if (!(*x)->tempo().active()) {
+			(*x)->hide();
+		} else {
+			(*x)->show();
+		}
+
 		++x;
 	}
 
@@ -277,7 +299,10 @@ Editor::redisplay_tempo (bool immediate_redraw)
 	}
 
 	if (immediate_redraw) {
-		compute_bbt_ruler_scale (leftmost_frame, leftmost_frame + current_page_samples());
+
+//only recalculate bbt_ruler_scale on a zoom or snap-change; not every redraw; if a case is found where this is necessary, uncomment this line.
+//		compute_bbt_ruler_scale (leftmost_frame, leftmost_frame + current_page_samples());
+
 		std::vector<TempoMap::BBTPoint> grid;
 
 		if (bbt_ruler_scale != bbt_show_many) {
@@ -291,6 +316,24 @@ Editor::redisplay_tempo (bool immediate_redraw)
 		Glib::signal_idle().connect (sigc::bind_return (sigc::bind (sigc::mem_fun (*this, &Editor::redisplay_tempo), true), false));
 	}
 }
+void
+Editor::tempo_curve_selected (TempoSection* ts, bool yn)
+{
+	if (ts == 0) {
+		return;
+	}
+
+	for (Curves::iterator x = tempo_curves.begin(); x != tempo_curves.end(); ++x) {
+		if (&(*x)->tempo() == ts) {
+			if (yn) {
+				(*x)->set_color_rgba (UIConfiguration::instance().color ("location marker"));
+			} else {
+				(*x)->set_color_rgba (UIConfiguration::instance().color ("tempo curve"));
+			}
+			break;
+		}
+	}
+}
 
 /* computes a grid starting a beat before and ending a beat after leftmost and rightmost respectively */
 void
@@ -351,7 +394,7 @@ Editor::draw_measures (std::vector<ARDOUR::TempoMap::BBTPoint>& grid)
 	}
 
 	if (tempo_lines == 0) {
-		tempo_lines = new TempoLines (time_line_group, ArdourCanvas::LineSet::Vertical);
+		tempo_lines = new TempoLines (time_line_group, ArdourCanvas::LineSet::Vertical, new BeatsFramesConverter (_session->tempo_map(), _session->tempo_map().music_origin()));
 	}
 
 	const unsigned divisions = get_grid_beat_divisions(leftmost_frame);
@@ -373,7 +416,7 @@ Editor::mouse_add_new_tempo_event (framepos_t frame)
 	if (pulse > 0.0) {
 		XMLNode &before = map.get_state();
 		/* add music-locked ramped (?) tempo using the bpm/note type at frame*/
-		map.add_tempo (map.tempo_at_frame (frame), pulse, 0, TempoSection::Ramp, MusicTime);
+		map.add_tempo (map.tempo_at_frame (frame), pulse, 0, MusicTime);
 
 		XMLNode &after = map.get_state();
 		_session->add_command(new MementoCommand<TempoMap>(map, &before, &after));
@@ -409,15 +452,14 @@ Editor::mouse_add_new_meter_event (framepos_t frame)
 	Timecode::BBT_Time requested;
 	meter_dialog.get_bbt_time (requested);
 
-	const double beat = map.beat_at_bbt (requested);
-
+	const double al_frame = map.frame_at_bbt (requested);
 	begin_reversible_command (_("add meter mark"));
-        XMLNode &before = map.get_state();
+	XMLNode &before = map.get_state();
 
 	if (meter_dialog.get_lock_style() == MusicTime) {
-		map.add_meter (Meter (bpb, note_type), beat, requested, MusicTime);
+		map.add_meter (Meter (bpb, note_type), requested, 0, MusicTime);
 	} else {
-		map.add_meter (Meter (bpb, note_type), beat, requested, AudioTime);
+		map.add_meter (Meter (bpb, note_type), requested, al_frame, AudioTime);
 	}
 
 	_session->add_command(new MementoCommand<TempoMap>(map, &before, &map.get_state()));
@@ -442,7 +484,7 @@ Editor::remove_tempo_marker (ArdourCanvas::Item* item)
 		abort(); /*NOTREACHED*/
 	}
 
-	if (tempo_marker->tempo().movable()) {
+	if (!tempo_marker->tempo().locked_to_meter() && tempo_marker->tempo().active()) {
 		Glib::signal_idle().connect (sigc::bind (sigc::mem_fun(*this, &Editor::real_remove_tempo_marker), &tempo_marker->tempo()));
 	}
 }
@@ -467,13 +509,13 @@ Editor::edit_meter_section (MeterSection* section)
 
 	Timecode::BBT_Time when;
 	meter_dialog.get_bbt_time (when);
-
+	const framepos_t frame = _session->tempo_map().frame_at_bbt (when);
 	const PositionLockStyle pls = (meter_dialog.get_lock_style() == AudioTime) ? AudioTime : MusicTime;
 
 	begin_reversible_command (_("replace meter mark"));
-        XMLNode &before = _session->tempo_map().get_state();
+	XMLNode &before = _session->tempo_map().get_state();
 
-	_session->tempo_map().replace_meter (*section, meter, when, pls);
+	_session->tempo_map().replace_meter (*section, meter, when, frame, pls);
 
 	XMLNode &after = _session->tempo_map().get_state();
 	_session->add_command(new MementoCommand<TempoMap>(_session->tempo_map(), &before, &after));
@@ -493,24 +535,23 @@ Editor::edit_tempo_section (TempoSection* section)
 	}
 
 	double bpm = tempo_dialog.get_bpm ();
+	double end_bpm = tempo_dialog.get_end_bpm ();
 	double nt = tempo_dialog.get_note_type ();
 	bpm = max (0.01, bpm);
-	const Tempo tempo (bpm, nt);
+	const Tempo tempo (bpm, nt, end_bpm);
 
 	Timecode::BBT_Time when;
 	tempo_dialog.get_bbt_time (when);
 
-	const TempoSection::Type ttype (tempo_dialog.get_tempo_type());
-
 	begin_reversible_command (_("replace tempo mark"));
 	XMLNode &before = _session->tempo_map().get_state();
 
 	if (tempo_dialog.get_lock_style() == AudioTime) {
 		framepos_t const f = _session->tempo_map().predict_tempo_position (section, when).second;
-		_session->tempo_map().replace_tempo (*section, tempo, 0.0, f, ttype, AudioTime);
+		_session->tempo_map().replace_tempo (*section, tempo, 0.0, f, AudioTime);
 	} else {
 		double const p = _session->tempo_map().predict_tempo_position (section, when).first;
-		_session->tempo_map().replace_tempo (*section, tempo, p, 0, ttype, MusicTime);
+		_session->tempo_map().replace_tempo (*section, tempo, p, 0, MusicTime);
 	}
 
 	XMLNode &after = _session->tempo_map().get_state();
@@ -559,7 +600,7 @@ Editor::remove_meter_marker (ArdourCanvas::Item* item)
 		abort(); /*NOTREACHED*/
 	}
 
-	if (meter_marker->meter().movable()) {
+	if (!meter_marker->meter().initial()) {
 	  Glib::signal_idle().connect (sigc::bind (sigc::mem_fun(*this, &Editor::real_remove_meter_marker), &meter_marker->meter()));
 	}
 }
