@@ -19,6 +19,12 @@
 
 #include <limits.h>
 
+#include <pangomm.h>
+
+#include <gtkmm/alignment.h>
+#include <gdkmm/color.h>
+#include <gtkmm/style.h>
+
 #include "ardour/amp.h"
 #include "ardour/logmeter.h"
 #include "ardour/route_group.h"
@@ -26,12 +32,11 @@
 #include "ardour/dB.h"
 #include "ardour/utils.h"
 
-#include <pangomm.h>
-#include <gtkmm/style.h>
-#include <gdkmm/color.h>
-#include <gtkmm2ext/utils.h>
-#include <gtkmm2ext/fastmeter.h>
-#include <gtkmm2ext/gtk_ui.h>
+#include "gtkmm2ext/utils.h"
+#include "gtkmm2ext/gtk_ui.h"
+
+#include "widgets/tooltips.h"
+
 #include "pbd/fastlog.h"
 #include "pbd/stacktrace.h"
 
@@ -42,7 +47,6 @@
 #include "utils.h"
 #include "meter_patterns.h"
 #include "timers.h"
-#include "tooltips.h"
 #include "ui_config.h"
 
 #include "ardour/session.h"
@@ -55,7 +59,7 @@
 #include "pbd/i18n.h"
 
 using namespace ARDOUR;
-using namespace ARDOUR_UI_UTILS;
+using namespace ArdourWidgets;
 using namespace PBD;
 using namespace Gtkmm2ext;
 using namespace Gtk;
@@ -90,8 +94,8 @@ GainMeterBase::GainMeterBase (Session* s, bool horizontal, int fader_length, int
 	                   1.0,  // upper
 	                   dB_coeff_step(Config->get_max_gain()) / 10.0,  // step increment
 	                   dB_coeff_step(Config->get_max_gain()))  // page increment
-	, gain_automation_style_button ("")
 	, gain_automation_state_button ("")
+	, meter_point_button (_("pre"))
 	, gain_astate_propagate (false)
 	, _data_type (DataType::AUDIO)
 {
@@ -119,7 +123,7 @@ GainMeterBase::GainMeterBase (Session* s, bool horizontal, int fader_length, int
 	meter_metric_area.signal_button_press_event().connect (sigc::mem_fun (*this, &GainMeterBase::level_meter_button_press));
 	meter_metric_area.add_events (Gdk::BUTTON_PRESS_MASK);
 
-	gain_slider->set_tweaks (PixFader::Tweaks(PixFader::NoButtonForward | PixFader::NoVerticalScroll));
+	gain_slider->set_tweaks (ArdourFader::Tweaks(ArdourFader::NoButtonForward | ArdourFader::NoVerticalScroll));
 	gain_slider->StartGesture.connect (sigc::mem_fun (*this, &GainMeter::amp_start_touch));
 	gain_slider->StopGesture.connect (sigc::mem_fun (*this, &GainMeter::amp_stop_touch));
 	gain_slider->set_name ("GainFader");
@@ -146,26 +150,48 @@ GainMeterBase::GainMeterBase (Session* s, bool horizontal, int fader_length, int
 	peak_display.unset_flags (Gtk::CAN_FOCUS);
 	peak_display.set_editable (false);
 
-	gain_automation_style_button.set_name ("mixer strip button");
 	gain_automation_state_button.set_name ("mixer strip button");
 
 	set_tooltip (gain_automation_state_button, _("Fader automation mode"));
-	set_tooltip (gain_automation_style_button, _("Fader automation type"));
 
-	gain_automation_style_button.unset_flags (Gtk::CAN_FOCUS);
 	gain_automation_state_button.unset_flags (Gtk::CAN_FOCUS);
 
 	gain_automation_state_button.set_size_request(15, 15);
-	gain_automation_style_button.set_size_request(15, 15);
-
-	gain_astyle_menu.items().push_back (MenuElem (_("Trim")));
-	gain_astyle_menu.items().push_back (MenuElem (_("Abs")));
 
 	gain_astate_menu.set_name ("ArdourContextMenu");
 	gain_astate_menu.set_reserve_toggle_size(false);
-	gain_astyle_menu.set_name ("ArdourContextMenu");
+
+	meter_point_button.set_name ("mixer strip button");
+
+	set_tooltip (&meter_point_button, _("Metering point"));
+
+	meter_point_button.unset_flags (Gtk::CAN_FOCUS);
+
+	meter_point_button.set_size_request(15, 15);
+
+	meter_point_menu.set_name ("ArdourContextMenu");
+	meter_point_menu.set_reserve_toggle_size(false);
+
+	meter_point_menu.items().clear ();
+	meter_point_menu.items().push_back (MenuElem(_("Input"),
+	 sigc::bind (sigc::mem_fun (*this,
+	 &GainMeterBase::meter_point_clicked), (MeterPoint) MeterInput)));
+	meter_point_menu.items().push_back (MenuElem(_("Pre Fader"),
+	 sigc::bind (sigc::mem_fun (*this,
+	 &GainMeterBase::meter_point_clicked), (MeterPoint) MeterPreFader)));
+	meter_point_menu.items().push_back (MenuElem(_("Post Fader"),
+	 sigc::bind (sigc::mem_fun (*this,
+	 &GainMeterBase::meter_point_clicked), (MeterPoint) MeterPostFader)));
+	meter_point_menu.items().push_back (MenuElem(_("Output"),
+	 sigc::bind (sigc::mem_fun (*this,
+	 &GainMeterBase::meter_point_clicked), (MeterPoint) MeterOutput)));
+	meter_point_menu.items().push_back (MenuElem(_("Custom"),
+	 sigc::bind (sigc::mem_fun (*this,
+	 &GainMeterBase::meter_point_clicked), (MeterPoint) MeterCustom)));
+	meter_point_button.signal_button_press_event().connect (sigc::mem_fun (*this, &GainMeter::meter_press), false);
 
 	gain_adjustment.signal_value_changed().connect (sigc::mem_fun(*this, &GainMeterBase::fader_moved));
+	peak_display.signal_button_press_event().connect (sigc::mem_fun(*this, &GainMeterBase::peak_button_press), false);
 	peak_display.signal_button_release_event().connect (sigc::mem_fun(*this, &GainMeterBase::peak_button_release), false);
 	gain_display.signal_key_press_event().connect (sigc::mem_fun(*this, &GainMeterBase::gain_key_press), false);
 
@@ -237,12 +263,10 @@ GainMeterBase::set_controls (boost::shared_ptr<Route> r,
 		gain_astate_menu.items().push_back (MenuElem (_("Touch"),
 							      sigc::bind (sigc::mem_fun (*this, &GainMeterBase::set_gain_astate), (AutoState) ARDOUR::Touch)));
 
-		connections.push_back (gain_automation_style_button.signal_button_press_event().connect (sigc::mem_fun(*this, &GainMeterBase::gain_automation_style_button_event), false));
 		connections.push_back (gain_automation_state_button.signal_button_press_event().connect (sigc::mem_fun(*this, &GainMeterBase::gain_automation_state_button_event), false));
 		connections.push_back (ChangeGainAutomationState.connect (sigc::mem_fun(*this, &GainMeterBase::set_gain_astate)));
 
 		_control->alist()->automation_state_changed.connect (model_connections, invalidator (*this), boost::bind (&GainMeter::gain_automation_state_changed, this), gui_context());
-		_control->alist()->automation_style_changed.connect (model_connections, invalidator (*this), boost::bind (&GainMeter::gain_automation_style_changed, this), gui_context());
 
 		gain_automation_state_changed ();
 	}
@@ -268,7 +292,12 @@ GainMeterBase::set_gain_astate (AutoState as)
 		ChangeGainAutomationState (as);
 		return;
 	}
-	_amp->set_parameter_automation_state (Evoral::Parameter (GainAutomation), as);
+	if (_amp) {
+		_amp->set_parameter_automation_state (Evoral::Parameter (GainAutomation), as);
+	} else if (_control) {
+		_control->set_automation_state (as);
+		_session->set_dirty ();
+	}
 }
 
 void
@@ -290,7 +319,7 @@ GainMeterBase::setup_gain_adjustment ()
 		gain_adjustment.set_upper (GAIN_COEFF_UNITY);
 		gain_adjustment.set_step_increment (dB_coeff_step(Config->get_max_gain()) / 10.0);
 		gain_adjustment.set_page_increment (dB_coeff_step(Config->get_max_gain()));
-		gain_slider->set_default_value (gain_to_slider_position (GAIN_COEFF_UNITY));
+		gain_slider->set_default_value (gain_to_slider_position_with_max (GAIN_COEFF_UNITY, Config->get_max_gain()));
 	} else {
 		_data_type = DataType::MIDI;
 		gain_adjustment.set_lower (0.0);
@@ -354,7 +383,7 @@ GainMeterBase::setup_meters (int len)
 void
 GainMeterBase::set_type (MeterType t)
 {
-	level_meter->set_type(t);
+	level_meter->set_meter_type(t);
 }
 
 void
@@ -388,7 +417,7 @@ GainMeter::set_type (MeterType t)
 bool
 GainMeterBase::gain_key_press (GdkEventKey* ev)
 {
-	if (key_is_legal_for_numeric_entry (ev->keyval)) {
+	if (ARDOUR_UI_UTILS::key_is_legal_for_numeric_entry (ev->keyval)) {
 		/* drop through to normal handling */
 		return false;
 	}
@@ -396,6 +425,12 @@ GainMeterBase::gain_key_press (GdkEventKey* ev)
 	return true;
 }
 
+bool
+GainMeterBase::peak_button_press (GdkEventButton* ev)
+{
+	return true;
+}
+
 bool
 GainMeterBase::peak_button_release (GdkEventButton* ev)
 {
@@ -417,6 +452,10 @@ GainMeterBase::peak_button_release (GdkEventButton* ev)
 void
 GainMeterBase::reset_peak_display ()
 {
+	if (!_route) {
+		// catch "reset all" for VCAs
+		return;
+	}
 	_meter->reset_max();
 	level_meter->clear_meters();
 	max_peak = minus_infinity ();
@@ -547,7 +586,14 @@ GainMeterBase::fader_moved ()
 			value = gain_adjustment.get_value();
 		}
 
-		_control->set_value (value, Controllable::UseGroup);
+		// XXX hack allow to override group
+		// (this breaks group'ed  shift+click reset)
+		if (Keyboard::the_keyboard().key_is_down (GDK_Shift_R)
+				|| Keyboard::the_keyboard().key_is_down (GDK_Shift_L)) {
+			_control->set_value (value, Controllable::InverseGroup);
+		} else {
+			_control->set_value (value, Controllable::UseGroup);
+		}
 	}
 
 	show_gain ();
@@ -580,7 +626,8 @@ GainMeterBase::effective_gain_display ()
 void
 GainMeterBase::gain_changed ()
 {
-	Gtkmm2ext::UI::instance()->call_slot (invalidator (*this), boost::bind (&GainMeterBase::effective_gain_display, this));
+	ENSURE_GUI_THREAD (*this, &GainMeterBase::gain_automation_state_changed);
+	effective_gain_display ();
 }
 
 void
@@ -604,114 +651,44 @@ void
 GainMeterBase::update_gain_sensitive ()
 {
 	bool x = !(_control->alist()->automation_state() & Play);
-	static_cast<Gtkmm2ext::SliderController*>(gain_slider)->set_sensitive (x);
-}
-
-static MeterPoint
-next_meter_point (MeterPoint mp)
-{
-	switch (mp) {
-	case MeterInput:
-		return MeterPreFader;
-		break;
-
-	case MeterPreFader:
-		return MeterPostFader;
-		break;
-
-	case MeterPostFader:
-		return MeterOutput;
-		break;
-
-	case MeterOutput:
-		return MeterCustom;
-		break;
-
-	case MeterCustom:
-		return MeterInput;
-		break;
-	}
-
-	abort(); /*NOTREACHED*/
-	return MeterInput;
+	static_cast<ArdourWidgets::SliderController*>(gain_slider)->set_sensitive (x);
 }
 
 gint
 GainMeterBase::meter_press(GdkEventButton* ev)
 {
-	wait_for_release = false;
-
 	if (!_route) {
-		return FALSE;
+		return false;
 	}
-
 	if (!ignore_toggle) {
-
-		if (Keyboard::is_context_menu_event (ev)) {
-
-			// no menu at this time.
-
-		} else {
-
-			if (Keyboard::is_button2_event(ev)) {
-
-				// Primary-button2 click is the midi binding click
-				// button2-click is "momentary"
-
-				if (!Keyboard::modifier_state_equals (ev->state, Keyboard::ModifierMask (Keyboard::PrimaryModifier))) {
-					wait_for_release = true;
-					old_meter_point = _route->meter_point ();
-				}
-			}
-
-			if (_route && (ev->button == 1 || Keyboard::is_button2_event (ev))) {
-
+		switch (ev->button) {
+			case 1:
 				if (Keyboard::modifier_state_equals (ev->state, Keyboard::ModifierMask (Keyboard::PrimaryModifier|Keyboard::TertiaryModifier))) {
 
 					/* Primary+Tertiary-click applies change to all routes */
 
-					_session->foreach_route (this, &GainMeterBase::set_meter_point, next_meter_point (_route->meter_point()));
-
+					meter_point_change_target = MeterPointChangeAll;
 
 				} else if (Keyboard::modifier_state_equals (ev->state, Keyboard::PrimaryModifier)) {
 
-					/* Primary-click: solo mix group.
-					   NOTE: Primary-button2 is MIDI learn.
-					*/
+					/* Primary-click: apply change to all routes in group */
 
-					if (ev->button == 1) {
-						set_route_group_meter_point (*_route, next_meter_point (_route->meter_point()));
-					}
+					meter_point_change_target = MeterPointChangeGroup;
 
 				} else {
 
 					/* click: change just this route */
 
-					// XXX no undo yet
-
-					_route->set_meter_point (next_meter_point (_route->meter_point()));
+					meter_point_change_target = MeterPointChangeSingle;
 				}
-			}
+				Gtkmm2ext::anchored_menu_popup(&meter_point_menu,
+			  	                             &meter_point_button,
+			    	                           "", 1, ev->time);
+				break;
+			default:
+				break;
 		}
 	}
-
-	return true;
-
-}
-
-gint
-GainMeterBase::meter_release(GdkEventButton*)
-{
-	if (!ignore_toggle) {
-		if (wait_for_release) {
-			wait_for_release = false;
-
-			if (_route) {
-				set_meter_point (*_route, old_meter_point);
-			}
-		}
-	}
-
 	return true;
 }
 
@@ -734,10 +711,20 @@ GainMeterBase::set_route_group_meter_point (Route& route, MeterPoint mp)
 }
 
 void
-GainMeterBase::meter_point_clicked ()
+GainMeterBase::meter_point_clicked (MeterPoint mp)
 {
 	if (_route) {
-		/* WHAT? */
+		switch (meter_point_change_target) {
+			case MeterPointChangeAll:
+				_session->foreach_route (this, &GainMeterBase::set_meter_point, mp);
+				break;
+			case MeterPointChangeGroup:
+				set_route_group_meter_point (*_route, mp);
+				break;
+			case MeterPointChangeSingle:
+				_route->set_meter_point (mp);
+				break;
+		}
 	}
 }
 
@@ -750,7 +737,8 @@ GainMeterBase::amp_start_touch ()
 void
 GainMeterBase::amp_stop_touch ()
 {
-	_control->stop_touch (false, _control->session().transport_frame());
+	_control->stop_touch (_control->session().transport_frame());
+	effective_gain_display ();
 }
 
 gint
@@ -774,22 +762,6 @@ GainMeterBase::gain_automation_state_button_event (GdkEventButton *ev)
 	return TRUE;
 }
 
-gint
-GainMeterBase::gain_automation_style_button_event (GdkEventButton *ev)
-{
-	if (ev->type == GDK_BUTTON_RELEASE) {
-		return TRUE;
-	}
-
-	switch (ev->button) {
-	case 1:
-		gain_astyle_menu.popup (1, ev->time);
-		break;
-	default:
-		break;
-	}
-	return TRUE;
-}
 
 string
 GainMeterBase::astate_string (AutoState state)
@@ -826,43 +798,6 @@ GainMeterBase::_astate_string (AutoState state, bool shrt)
 	return sstr;
 }
 
-string
-GainMeterBase::astyle_string (AutoStyle style)
-{
-	return _astyle_string (style, false);
-}
-
-string
-GainMeterBase::short_astyle_string (AutoStyle style)
-{
-	return _astyle_string (style, true);
-}
-
-string
-GainMeterBase::_astyle_string (AutoStyle style, bool shrt)
-{
-	if (style & Trim) {
-		return _("Trim");
-	} else {
-	        /* XXX it might different in different languages */
-
-		return (shrt ? _("Abs") : _("Abs"));
-	}
-}
-
-void
-GainMeterBase::gain_automation_style_changed ()
-{
-	switch (_width) {
-	case Wide:
-		gain_automation_style_button.set_text (astyle_string(_control->alist()->automation_style()));
-		break;
-	case Narrow:
-		gain_automation_style_button.set_text  (short_astyle_string(_control->alist()->automation_style()));
-		break;
-	}
-}
-
 void
 GainMeterBase::gain_automation_state_changed ()
 {
@@ -888,14 +823,6 @@ GainMeterBase::gain_automation_state_changed ()
 	update_gain_sensitive ();
 
 	gain_watching.disconnect();
-
-	if (automation_watch_required) {
-		/* start watching automation so that things move */
-		gain_watching = Timers::rapid_connect (sigc::mem_fun (*this, &GainMeterBase::effective_gain_display));
-	} else {
-		/* update once to get the correct value shown as we re-enter off/manual mode */
-		effective_gain_display();
-	}
 }
 
 const ChanCount
@@ -974,17 +901,13 @@ GainMeter::GainMeter (Session* s, int fader_length)
 	meter_metric_area.set_name ("AudioTrackMetrics");
 	meter_metric_area.set_size_request(PX_SCALE(24, 24), -1);
 
-	gain_automation_style_button.set_name ("mixer strip button");
 	gain_automation_state_button.set_name ("mixer strip button");
 
 	set_tooltip (gain_automation_state_button, _("Fader automation mode"));
-	set_tooltip (gain_automation_style_button, _("Fader automation type"));
 
-	gain_automation_style_button.unset_flags (Gtk::CAN_FOCUS);
 	gain_automation_state_button.unset_flags (Gtk::CAN_FOCUS);
 
 	gain_automation_state_button.set_size_request (PX_SCALE(12, 15), PX_SCALE(12, 15));
-	gain_automation_style_button.set_size_request (PX_SCALE(12, 15), PX_SCALE(12, 15));
 
 	fader_vbox.set_spacing (0);
 	fader_vbox.pack_start (*gain_slider, true, true);
