@@ -34,11 +34,19 @@
 #include "pbd/xml++.h"
 #include "pbd/failed_constructor.h"
 
+#include "evoral/midi_events.h"
+#include "evoral/PatchChange.hpp"
+
+#include "midi++/midnam_patch.h"
+
+#include "ardour/midi_patch_manager.h"
+#include "ardour/midi_track.h"
 #include "ardour/plugin.h"
 #include "ardour/plugin_insert.h"
 #include "ardour/session.h"
 #include "ardour/value_as_string.h"
 
+#include "gtkmm2ext/menu_elems.h"
 #include "gtkmm2ext/utils.h"
 #include "gtkmm2ext/doi.h"
 
@@ -70,6 +78,11 @@ GenericPluginUI::GenericPluginUI (boost::shared_ptr<PluginInsert> pi, bool scrol
 	: PlugUIBase (pi)
 	, automation_menu (0)
 	, is_scrollable(scrollable)
+	, _plugin_pianokeyboard_expander (_("MIDI Keyboard"))
+	, _piano (0)
+	, _pianomm (0)
+	, _piano_velocity (*manage (new Adjustment (100, 1, 127, 1, 16)))
+	, _piano_channel (*manage (new Adjustment (0, 1, 16, 1, 1)))
 {
 	set_name ("PluginEditor");
 	set_border_width (10);
@@ -80,9 +93,7 @@ GenericPluginUI::GenericPluginUI (boost::shared_ptr<PluginInsert> pi, bool scrol
 
 	HBox* constraint_hbox = manage (new HBox);
 	HBox* smaller_hbox = manage (new HBox);
-	HBox* automation_hbox = manage (new HBox);
 	smaller_hbox->set_spacing (4);
-	automation_hbox->set_spacing (6);
 	Label* combo_label = manage (new Label (_("<span size=\"large\">Presets</span>")));
 	combo_label->set_use_markup (true);
 
@@ -96,7 +107,9 @@ GenericPluginUI::GenericPluginUI (boost::shared_ptr<PluginInsert> pi, bool scrol
 	smaller_hbox->pack_start (add_button, false, false);
 	smaller_hbox->pack_start (save_button, false, false);
 	smaller_hbox->pack_start (delete_button, false, false);
-	smaller_hbox->pack_start (reset_button, false, false, 4);
+	if (pi->controls().size() > 0) {
+		smaller_hbox->pack_start (reset_button, false, false, 4);
+	}
 	smaller_hbox->pack_start (bypass_button, false, true, 4);
 
 	automation_manual_all_button.set_text(_("Manual"));
@@ -108,27 +121,58 @@ GenericPluginUI::GenericPluginUI (boost::shared_ptr<PluginInsert> pi, bool scrol
 	automation_touch_all_button.set_text(_("Touch"));
 	automation_touch_all_button.set_name (X_("generic button"));
 
-	Label* l = manage (new Label (_("All Automation")));
-	l->set_alignment (1.0, 0.5);
-	automation_hbox->pack_start (*l, true, true);
-	automation_hbox->pack_start (automation_manual_all_button, false, false);
-	automation_hbox->pack_start (automation_play_all_button, false, false);
-	automation_hbox->pack_start (automation_write_all_button, false, false);
-	automation_hbox->pack_start (automation_touch_all_button, false, false);
-
 	constraint_hbox->set_spacing (5);
 	constraint_hbox->set_homogeneous (false);
 
 	VBox* v1_box = manage (new VBox);
 	VBox* v2_box = manage (new VBox);
-	pack_end (plugin_analysis_expander, false, false);
+	if (pi->is_instrument ()) {
+		_piano = (PianoKeyboard*)piano_keyboard_new();
+		_pianomm = Glib::wrap((GtkWidget*)_piano);
+		_pianomm->set_flags(Gtk::CAN_FOCUS);
+		_pianomm->add_events(Gdk::KEY_PRESS_MASK|Gdk::KEY_RELEASE_MASK);
+
+		g_signal_connect (G_OBJECT (_piano), "note-on", G_CALLBACK (GenericPluginUI::_note_on_event_handler), this);
+		g_signal_connect (G_OBJECT (_piano), "note-off", G_CALLBACK (GenericPluginUI::_note_off_event_handler), this);
+
+		HBox* box = manage (new HBox);
+		box->pack_start (*manage (new Label (_("Channel:"))), false, false);
+		box->pack_start (_piano_channel, false, false);
+		box->pack_start (*manage (new Label (_("Velocity:"))), false, false);
+		box->pack_start (_piano_velocity, false, false);
+
+		Box* box2 = manage (new HBox ());
+		box2->pack_start (*box, true, false);
+
+		_pianobox.set_spacing (4);
+		_pianobox.pack_start (*box2, true, true);
+		_pianobox.pack_start (*_pianomm, true, true);
+
+		_plugin_pianokeyboard_expander.set_expanded(false);
+		_plugin_pianokeyboard_expander.property_expanded().signal_changed().connect( sigc::mem_fun(*this, &GenericPluginUI::toggle_pianokeyboard));
+
+		pack_end (_plugin_pianokeyboard_expander, false, false);
+	} else {
+		pack_end (plugin_analysis_expander, false, false);
+	}
 	if (!plugin->get_docs().empty()) {
 		pack_end (description_expander, false, false);
 	}
 
 	v1_box->set_spacing (6);
 	v1_box->pack_start (*smaller_hbox, false, true);
-	v1_box->pack_start (*automation_hbox, false, true);
+	if (pi->controls().size() > 0) {
+		HBox* automation_hbox = manage (new HBox);
+		automation_hbox->set_spacing (6);
+		Label* l = manage (new Label (_("All Automation")));
+		l->set_alignment (1.0, 0.5);
+		automation_hbox->pack_start (*l, true, true);
+		automation_hbox->pack_start (automation_manual_all_button, false, false);
+		automation_hbox->pack_start (automation_play_all_button, false, false);
+		automation_hbox->pack_start (automation_write_all_button, false, false);
+		automation_hbox->pack_start (automation_touch_all_button, false, false);
+		v1_box->pack_start (*automation_hbox, false, true);
+	}
 	v2_box->pack_start (focus_button, false, true);
 
 	main_contents.pack_start (settings_box, false, false);
@@ -138,29 +182,31 @@ GenericPluginUI::GenericPluginUI (boost::shared_ptr<PluginInsert> pi, bool scrol
 
 	main_contents.pack_start (*constraint_hbox, false, false);
 
-	if (is_scrollable) {
-		Gtk::ScrolledWindow *scroller = manage (new Gtk::ScrolledWindow());
-		scroller->set_policy (Gtk::POLICY_NEVER, Gtk::POLICY_AUTOMATIC);
-		scroller->set_name ("PluginEditor");
-		scroller->add (hpacker);
-		main_contents.pack_start (*scroller, true, true);
-	} else {
-		main_contents.pack_start (hpacker, false, false);
-	}
-
 	pi->ActiveChanged.connect (active_connection, invalidator (*this), boost::bind (&GenericPluginUI::processor_active_changed, this, boost::weak_ptr<Processor>(pi)), gui_context());
 
 	bypass_button.set_active (!pi->enabled());
 
+	/* ScrolledWindow will wrap hpacker in a Viewport */
+	scroller.add (hpacker);
+	Viewport* view = static_cast<Viewport*>(scroller.get_child());
+	view->set_shadow_type(Gtk::SHADOW_NONE);
+
+	main_contents.pack_start (scroller, true, true);
+
 	prefheight = 0;
 	build ();
 
-	/* Listen for property changes that are not notified normally because
-	 * AutomationControl has only support for numeric values currently.
-	 * The only case is Variant::PATH for now */
-	plugin->PropertyChanged.connect(*this, invalidator(*this),
-	                                boost::bind(&GenericPluginUI::path_property_changed, this, _1, _2),
-	                                gui_context());
+	if (insert->plugin()->has_midnam() && insert->plugin()->knows_bank_patch()) {
+		build_midi_table ();
+	}
+
+	if (is_scrollable) {
+		scroller.set_policy (Gtk::POLICY_NEVER, Gtk::POLICY_AUTOMATIC);
+		scroller.set_name ("PluginEditor");
+	} else {
+		scroller.signal_size_request().connect (sigc::mem_fun(*this, &GenericPluginUI::scroller_size_request));
+		scroller.set_policy (Gtk::POLICY_AUTOMATIC, Gtk::POLICY_NEVER);
+	}
 
 	main_contents.show ();
 }
@@ -170,6 +216,40 @@ GenericPluginUI::~GenericPluginUI ()
 	if (output_controls.size() > 0) {
 		screen_update_connection.disconnect();
 	}
+	delete _pianomm;
+}
+
+void
+GenericPluginUI::scroller_size_request (Gtk::Requisition* a)
+{
+	Glib::RefPtr<Gdk::Screen> screen = get_screen();
+	if (!screen)
+		screen = Gdk::Screen::get_default();
+
+	int maximum_width;
+	{
+		Gdk::Rectangle monitor;
+		const int monitor_num = screen->get_monitor_at_window (get_window ());
+		screen->get_monitor_geometry (
+				(monitor_num < 0) ? 0 : monitor_num,
+				monitor);
+
+		maximum_width = monitor.get_width() * 0.9;
+	}
+
+	GtkRequisition request = hpacker.size_request();
+
+	if (request.width > maximum_width) {
+		for (vector<ControlUI*>::const_iterator cuip = input_controls.begin();
+		                                        cuip != input_controls.end();
+		                                        ++cuip) {
+			if (!(*cuip)->short_autostate)
+				set_short_autostate(*cuip, true);
+		}
+		request = hpacker.size_request();
+	}
+
+	a->width = min(request.width, maximum_width);
 }
 
 // Some functions for calculating the 'similarity' of two plugin
@@ -304,6 +384,14 @@ GenericPluginUI::build ()
 		control_uis.push_back(cui);
 	}
 	if (!descs.empty()) {
+		/* Listen for property changes that are not notified normally because
+		 * AutomationControl has only support for numeric values currently.
+		 * The only case is Variant::PATH for now */
+		plugin->PropertyChanged.connect(*this, invalidator(*this),
+				boost::bind(&GenericPluginUI::path_property_changed, this, _1, _2),
+				gui_context());
+
+		/* and query current property value */
 		plugin->announce_property_values();
 	}
 
@@ -361,10 +449,11 @@ GenericPluginUI::automatic_layout (const std::vector<ControlUI*>& control_uis)
 	button_table->set_homogeneous (false);
 	button_table->set_row_spacings (2);
 	button_table->set_col_spacings (2);
+	button_table->set_border_width (5);
+
 	output_table->set_homogeneous (true);
 	output_table->set_row_spacings (2);
 	output_table->set_col_spacings (2);
-	button_table->set_border_width (5);
 	output_table->set_border_width (5);
 
 
@@ -557,6 +646,145 @@ GenericPluginUI::custom_layout (const std::vector<ControlUI*>& control_uis)
 	}
 }
 
+void
+GenericPluginUI::build_midi_table ()
+{
+	Gtk::Table* pgm_table = manage (new Gtk::Table (8, 5));
+
+	pgm_table->set_homogeneous (false);
+	pgm_table->set_row_spacings (2);
+	pgm_table->set_col_spacings (2);
+	pgm_table->set_border_width (5);
+	pgm_table->set_col_spacing (2, 10);
+
+	Frame* frame = manage (new Frame);
+	frame->set_name ("BaseFrame");
+	if (dynamic_cast<MidiTrack*> (insert->owner())) {
+		frame->set_label (_("MIDI Progams (sent to track)"));
+	} else {
+		frame->set_label (_("MIDI Progams (volatile)"));
+	}
+	frame->add (*pgm_table);
+	hpacker.pack_start (*frame, false, false);
+
+	for (uint8_t chn = 0; chn < 16; ++chn) {
+		int col = 3 * (chn / 8);
+		int row = chn % 8;
+		ArdourDropdown* cui = manage (new ArdourWidgets::ArdourDropdown ());
+		cui->set_sizing_text ("Stereo Grand Piano");
+		cui->set_text_ellipsize (Pango::ELLIPSIZE_END);
+		cui->set_layout_ellipsize_width (PANGO_SCALE * 112 * UIConfiguration::instance ().get_ui_scale ());
+		midi_pgmsel.push_back (cui);
+		pgm_table->attach (*manage (new Label (string_compose ("C%1:", (int)(chn + 1)), ALIGN_RIGHT)), col, col + 1, row, row+1, FILL, SHRINK);
+		pgm_table->attach (*cui, col + 1, col + 2, row, row+1, SHRINK, SHRINK);
+	}
+
+	insert->plugin ()->read_midnam();
+
+	midi_refill_patches ();
+
+	insert->plugin()->BankPatchChange.connect (
+			midi_connections, invalidator (*this),
+			boost::bind (&GenericPluginUI::midi_bank_patch_change, this, _1),
+			gui_context());
+
+	insert->plugin()->UpdatedMidnam.connect (
+			midi_connections, invalidator (*this),
+			boost::bind (&GenericPluginUI::midi_refill_patches, this),
+			gui_context());
+}
+
+void
+GenericPluginUI::midi_refill_patches ()
+{
+	assert (midi_pgmsel.size() == 16);
+
+	pgm_names.clear ();
+
+	const std::string model = insert->plugin ()->midnam_model ();
+	std::string mode;
+	const std::list<std::string> device_modes = MIDI::Name::MidiPatchManager::instance().custom_device_mode_names_by_model (model);
+	if (device_modes.size() > 0) {
+		mode = device_modes.front();
+	}
+
+	for (uint8_t chn = 0; chn < 16; ++chn) {
+		midi_pgmsel[chn]->clear_items ();
+		boost::shared_ptr<MIDI::Name::ChannelNameSet> cns =
+			MIDI::Name::MidiPatchManager::instance().find_channel_name_set (model, mode, chn);
+
+		if (cns) {
+			using namespace Menu_Helpers;
+			using namespace Gtkmm2ext;
+
+			for (MIDI::Name::ChannelNameSet::PatchBanks::const_iterator i = cns->patch_banks().begin(); i != cns->patch_banks().end(); ++i) {
+				const MIDI::Name::PatchNameList& patches = (*i)->patch_name_list ();
+				for (MIDI::Name::PatchNameList::const_iterator j = patches.begin(); j != patches.end(); ++j) {
+					const std::string pgm = (*j)->name ();
+					MIDI::Name::PatchPrimaryKey const& key = (*j)->patch_primary_key ();
+					assert ((*i)->number () == key.bank());
+					const uint32_t bp = (key.bank() << 7) | key.program();
+					midi_pgmsel[chn]->AddMenuElem (MenuElemNoMnemonic (pgm, sigc::bind (sigc::mem_fun (*this, &GenericPluginUI::midi_bank_patch_select), chn, bp)));
+					pgm_names[bp] = pgm;
+				}
+			}
+		}
+
+		midi_bank_patch_change (chn);
+	}
+}
+
+void
+GenericPluginUI::midi_bank_patch_change (uint8_t chn)
+{
+	assert (chn < 16 && midi_pgmsel.size() == 16);
+	uint32_t bankpgm = insert->plugin()->bank_patch (chn);
+	if (bankpgm == UINT32_MAX) {
+		midi_pgmsel[chn]->set_text (_("--Unset--"));
+	} else {
+		int bank = bankpgm >> 7;
+		int pgm = bankpgm & 127;
+		if (pgm_names.find (bankpgm) != pgm_names.end ()) {
+			midi_pgmsel[chn]->set_text (pgm_names[bankpgm]);
+		} else {
+			midi_pgmsel[chn]->set_text (string_compose ("Bank %1,%2 Pgm %3",
+						(bank >> 7) + 1, (bank & 127) + 1, pgm +1));
+		}
+	}
+}
+
+void
+GenericPluginUI::midi_bank_patch_select (uint8_t chn, uint32_t bankpgm)
+{
+	int bank = bankpgm >> 7;
+	int pgm = bankpgm & 127;
+	MidiTrack* mt = dynamic_cast<MidiTrack*> (insert->owner());
+	if (mt) {
+		/* send to track */
+		boost::shared_ptr<AutomationControl> bank_msb = mt->automation_control(Evoral::Parameter (MidiCCAutomation, chn, MIDI_CTL_MSB_BANK), true);
+		boost::shared_ptr<AutomationControl> bank_lsb = mt->automation_control(Evoral::Parameter (MidiCCAutomation, chn, MIDI_CTL_LSB_BANK), true);
+		boost::shared_ptr<AutomationControl> program = mt->automation_control(Evoral::Parameter (MidiPgmChangeAutomation, chn), true);
+
+		bank_msb->set_value (bank >> 7, PBD::Controllable::NoGroup);
+		bank_lsb->set_value (bank & 127, PBD::Controllable::NoGroup);
+		program->set_value (pgm, PBD::Controllable::NoGroup);
+	} else {
+		uint8_t event[3];
+		event[0] = (MIDI_CMD_CONTROL | chn);
+		event[1] = 0x00;
+		event[2] = bank >> 7;
+		insert->write_immediate_event (3, event);
+
+		event[1] = 0x20;
+		event[2] = bank & 127;
+		insert->write_immediate_event (3, event);
+
+		event[0] = (MIDI_CMD_PGM_CHANGE | chn);
+		event[1] = pgm;
+		insert->write_immediate_event (2, event);
+	}
+}
+
 GenericPluginUI::ControlUI::ControlUI (const Evoral::Parameter& p)
 	: param(p)
 	, automate_button (X_("")) // force creation of a label
@@ -573,13 +801,6 @@ GenericPluginUI::ControlUI::ControlUI (const Evoral::Parameter& p)
 	automate_button.set_name ("plugin automation state button");
 	set_tooltip (automate_button, _("Automation control"));
 
-	/* XXX translators: use a string here that will be at least as long
-	   as the longest automation label (see ::automation_state_changed()
-	   below). be sure to include a descender.
-	*/
-
-	automate_button.set_sizing_text(_("Mgnual"));
-
 	ignore_change = false;
 	update_pending = false;
 	button = false;
@@ -595,6 +816,21 @@ GenericPluginUI::ControlUI::~ControlUI()
 	}
 }
 
+void
+GenericPluginUI::set_short_autostate (ControlUI* cui, bool value)
+{
+	cui->short_autostate = value;
+	if (value) {
+		cui->automate_button.set_sizing_text("M");
+	} else {
+		/* XXX translators: use a string here that will be at least as long
+		   as the longest automation label (see ::automation_state_changed()
+		   below). be sure to include a descender. */
+		cui->automate_button.set_sizing_text(_("Mgnual"));
+	}
+	automation_state_changed(cui);
+}
+
 void
 GenericPluginUI::automation_state_changed (ControlUI* cui)
 {
@@ -607,7 +843,7 @@ GenericPluginUI::automation_state_changed (ControlUI* cui)
 
 	cui->automate_button.set_active((state != ARDOUR::Off));
 
-	if (cui->knobtable) {
+	if (cui->short_autostate) {
 		cui->automate_button.set_text (
 				GainMeterBase::astate_string (state));
 		return;
@@ -678,6 +914,7 @@ GenericPluginUI::build_control_ui (const Evoral::Parameter&             param,
 	control_ui->label.set_alignment (0.0, 0.5);
 	control_ui->label.set_name ("PluginParameterLabel");
 	control_ui->set_spacing (5);
+	set_short_autostate(control_ui, false);
 
 	if (is_input) {
 
@@ -797,7 +1034,7 @@ GenericPluginUI::build_control_ui (const Evoral::Parameter&             param,
 		}
 
 		if (use_knob) {
-			control_ui->automate_button.set_sizing_text("M");
+			set_short_autostate(control_ui, true);
 
 			control_ui->label.set_alignment (0.5, 0.5);
 			control_ui->knobtable = manage (new Table());
@@ -1127,3 +1364,69 @@ GenericPluginUI::path_property_changed (uint32_t key, const Variant& value)
 		std::cerr << "warning: property change for property with no control" << std::endl;
 	}
 }
+
+void
+GenericPluginUI::toggle_pianokeyboard ()
+{
+	if (_plugin_pianokeyboard_expander.get_expanded()) {
+		_plugin_pianokeyboard_expander.add (_pianobox);
+		_pianobox.show_all ();
+	} else {
+		const int child_height = _plugin_pianokeyboard_expander.get_child ()->get_height ();
+		_plugin_pianokeyboard_expander.get_child ()->hide ();
+		_plugin_pianokeyboard_expander.remove ();
+
+		Gtk::Window *toplevel = (Gtk::Window*) _plugin_pianokeyboard_expander.get_ancestor (GTK_TYPE_WINDOW);
+		if (toplevel) {
+			Gtk::Requisition wr;
+			toplevel->get_size (wr.width, wr.height);
+			wr.height -= child_height;
+			toplevel->resize (wr.width, wr.height);
+		}
+	}
+}
+
+void
+GenericPluginUI::_note_on_event_handler(GtkWidget*, int note, gpointer arg)
+{
+	((GenericPluginUI*)arg)->note_on_event_handler(note);
+}
+
+void
+GenericPluginUI::_note_off_event_handler(GtkWidget*, int note, gpointer arg)
+{
+	((GenericPluginUI*)arg)->note_off_event_handler(note);
+}
+
+void
+GenericPluginUI::note_on_event_handler (int note)
+{
+	MidiTrack* mt = dynamic_cast<MidiTrack*> (insert->owner());
+	_pianomm->grab_focus ();
+	uint8_t channel = _piano_channel.get_value_as_int () - 1;
+	uint8_t event[3];
+	event[0] = (MIDI_CMD_NOTE_ON | channel);
+	event[1] = note;
+	event[2] = _piano_velocity.get_value_as_int ();
+	if (mt) {
+		mt->write_immediate_event (3, event);
+	} else {
+		insert->write_immediate_event (3, event);
+	}
+}
+
+void
+GenericPluginUI::note_off_event_handler (int note)
+{
+	MidiTrack* mt = dynamic_cast<MidiTrack*> (insert->owner());
+	uint8_t channel = _piano_channel.get_value_as_int () - 1;
+	uint8_t event[3];
+	event[0] = (MIDI_CMD_NOTE_OFF | channel);
+	event[1] = note;
+	event[2] = 0;
+	if (mt) {
+		mt->write_immediate_event (3, event);
+	} else {
+		insert->write_immediate_event (3, event);
+	}
+}
