@@ -86,9 +86,8 @@ GroupTabs::set_session (Session* s)
 void
 GroupTabs::on_size_request (Gtk::Requisition *req)
 {
-	/* Use a dummy, small width and the actual height that we want */
-	req->width = 16;
-	req->height = 16;
+	req->width = std::max (16.f, rintf (16.f * UIConfiguration::instance().get_ui_scale()));
+	req->height = std::max (16.f, rintf (16.f * UIConfiguration::instance().get_ui_scale()));
 }
 
 bool
@@ -144,10 +143,10 @@ GroupTabs::on_button_press_event (GdkEventButton* ev)
 
 		RouteGroup* g = t ? t->group : 0;
 
-		if (Keyboard::modifier_state_equals (ev->state, Keyboard::PrimaryModifier) && g) {
-			/* edit */
-			RouteGroupDialog d (g, false);
-			d.present ();
+		if (Keyboard::modifier_state_equals (ev->state, Keyboard::TertiaryModifier) && g) {
+			remove_group (g);
+		} else if (Keyboard::modifier_state_equals (ev->state, Keyboard::PrimaryModifier) && g) {
+			edit_group (g);
 		} else {
 			Menu* m = get_menu (g, true);
 			if (m) {
@@ -241,8 +240,9 @@ GroupTabs::on_button_release_event (GdkEventButton*)
 }
 
 void
-GroupTabs::render (cairo_t* cr, cairo_rectangle_t*)
+GroupTabs::render (Cairo::RefPtr<Cairo::Context> const& ctx, cairo_rectangle_t*)
 {
+	cairo_t* cr = ctx->cobj();
 	if (_dragging == 0) {
 		_tabs = compute_tabs ();
 	}
@@ -306,7 +306,7 @@ GroupTabs::add_new_from_items (Menu_Helpers::MenuList& items)
 	using namespace Menu_Helpers;
 	Menu *new_from;
 
-	new_from = new Menu;
+	new_from = manage (new Menu);
 	{
 		MenuList& f = new_from->items ();
 		f.push_back (MenuElem (_("Selection..."), sigc::bind (sigc::mem_fun (*this, &GroupTabs::new_from_selection), false)));
@@ -315,7 +315,7 @@ GroupTabs::add_new_from_items (Menu_Helpers::MenuList& items)
 	}
 	items.push_back (MenuElem (_("Create New Group From..."), *new_from));
 
-	new_from = new Menu;
+	new_from = manage (new Menu);
 	{
 		MenuList& f = new_from->items ();
 		f.push_back (MenuElem (_("Selection..."), sigc::bind (sigc::mem_fun (*this, &GroupTabs::new_from_selection), true)));
@@ -355,15 +355,18 @@ GroupTabs::get_menu (RouteGroup* g, bool in_tab_area)
 
 		items.push_back (SeparatorElem());
 
-		vca_menu = new Menu;
-		MenuList& f (vca_menu->items());
-		f.push_back (MenuElem ("New", sigc::bind (sigc::mem_fun (*this, &GroupTabs::assign_group_to_master), 0, g, true)));
+		if (g->has_control_master()) {
+			items.push_back (MenuElem (_("Drop Group from VCA..."), sigc::bind (sigc::mem_fun (*this, &GroupTabs::unassign_group_to_master), g->group_master_number(), g)));
+		} else {
+			vca_menu = manage (new Menu);
+			MenuList& f (vca_menu->items());
+			f.push_back (MenuElem ("New", sigc::bind (sigc::mem_fun (*this, &GroupTabs::assign_group_to_master), 0, g, true)));
 
-		for (VCAList::const_iterator v = vcas.begin(); v != vcas.end(); ++v) {
-			f.push_back (MenuElem (string_compose ("VCA %1", (*v)->number()), sigc::bind (sigc::mem_fun (*this, &GroupTabs::assign_group_to_master), (*v)->number(), g, true)));
+			for (VCAList::const_iterator v = vcas.begin(); v != vcas.end(); ++v) {
+				f.push_back (MenuElem ((*v)->name().empty() ? string_compose ("VCA %1", (*v)->number()) : (*v)->name(), sigc::bind (sigc::mem_fun (*this, &GroupTabs::assign_group_to_master), (*v)->number(), g, true)));
+			}
+			items.push_back (MenuElem (_("Assign Group to VCA..."), *vca_menu));
 		}
-		items.push_back (MenuElem (_("Assign Group to Control Master..."), *vca_menu));
-
 
 		items.push_back (SeparatorElem());
 
@@ -389,38 +392,38 @@ GroupTabs::get_menu (RouteGroup* g, bool in_tab_area)
 
 	items.push_back (SeparatorElem());
 
-	vca_menu = new Menu;
+	vca_menu = manage (new Menu);
 	{
 		MenuList& f (vca_menu->items());
 		f.push_back (MenuElem ("New", sigc::bind (sigc::mem_fun (*this, &GroupTabs::assign_selection_to_master), 0)));
 		for (VCAList::const_iterator v = vcas.begin(); v != vcas.end(); ++v) {
-			f.push_back (MenuElem (string_compose ("VCA %1", (*v)->number()), sigc::bind (sigc::mem_fun (*this, &GroupTabs::assign_selection_to_master), (*v)->number())));
+			f.push_back (MenuElem ((*v)->name().empty() ? string_compose ("VCA %1", (*v)->number()) : (*v)->name(), sigc::bind (sigc::mem_fun (*this, &GroupTabs::assign_selection_to_master), (*v)->number())));
 		}
 	}
 
-	items.push_back (MenuElem (_("Assign Selection to Control Master..."), *vca_menu));
+	items.push_back (MenuElem (_("Assign Selection to VCA..."), *vca_menu));
 
-	vca_menu = new Menu;
+	vca_menu = manage (new Menu);
 	{
 		MenuList& f (vca_menu->items());
-		f.push_back (MenuElem ("New", sigc::bind (sigc::mem_fun (*this, &GroupTabs::assign_selection_to_master), 0)));
+		f.push_back (MenuElem ("New", sigc::bind (sigc::mem_fun (*this, &GroupTabs::assign_recenabled_to_master), 0)));
 		for (VCAList::const_iterator v = vcas.begin(); v != vcas.end(); ++v) {
-			f.push_back (MenuElem (string_compose ("VCA %1", (*v)->number()), sigc::bind (sigc::mem_fun (*this, &GroupTabs::assign_selection_to_master), (*v)->number())));
+			f.push_back (MenuElem ((*v)->name().empty() ? string_compose ("VCA %1", (*v)->number()) : (*v)->name(), sigc::bind (sigc::mem_fun (*this, &GroupTabs::assign_recenabled_to_master), (*v)->number())));
 		}
 
 	}
-	items.push_back (MenuElem (_("Assign Record Enabled to Control Master..."), *vca_menu));
+	items.push_back (MenuElem (_("Assign Record Enabled to VCA..."), *vca_menu));
 
-	vca_menu = new Menu;
+	vca_menu = manage (new Menu);
 	{
 		MenuList& f (vca_menu->items());
-		f.push_back (MenuElem ("New", sigc::bind (sigc::mem_fun (*this, &GroupTabs::assign_selection_to_master), 0)));
+		f.push_back (MenuElem ("New", sigc::bind (sigc::mem_fun (*this, &GroupTabs::assign_soloed_to_master), 0)));
 		for (VCAList::const_iterator v = vcas.begin(); v != vcas.end(); ++v) {
-			f.push_back (MenuElem (string_compose ("VCA %1", (*v)->number()), sigc::bind (sigc::mem_fun (*this, &GroupTabs::assign_selection_to_master), (*v)->number())));
+			f.push_back (MenuElem ((*v)->name().empty() ? string_compose ("VCA %1", (*v)->number()) : (*v)->name(), sigc::bind (sigc::mem_fun (*this, &GroupTabs::assign_soloed_to_master), (*v)->number())));
 		}
 
 	}
-	items.push_back (MenuElem (_("Assign Soloed to Control Master...")));
+	items.push_back (MenuElem (_("Assign Soloed to VCA..."), *vca_menu));
 
 	items.push_back (SeparatorElem());
 	items.push_back (MenuElem (_("Enable All Groups"), sigc::mem_fun(*this, &GroupTabs::activate_all)));
@@ -444,8 +447,8 @@ GroupTabs::assign_group_to_master (uint32_t which, RouteGroup* group, bool renam
 			return;
 		}
 
-		/* VCAs use 1-based counting. Get most recently created VCA... */
-		which = _session->vca_manager().n_vcas();
+		/* Get most recently created VCA... */
+		which = _session->vca_manager().vcas().back()->number();
 	}
 
 	master = _session->vca_manager().vca_by_number (which);
@@ -466,22 +469,44 @@ GroupTabs::assign_group_to_master (uint32_t which, RouteGroup* group, bool renam
 }
 
 void
-GroupTabs::assign_some_to_master (uint32_t which, RouteList rl)
+GroupTabs::unassign_group_to_master (uint32_t which, RouteGroup* group) const
+{
+	if (!_session || !group) {
+		return;
+	}
+
+	boost::shared_ptr<VCA> master = _session->vca_manager().vca_by_number (which);
+
+	if (!master) {
+		/* should never happen; if it does, basically something deeply
+		   odd happened, no reason to tell user because there's no
+		   sensible explanation.
+		*/
+		return;
+	}
+
+	group->unassign_master (master);
+}
+
+void
+GroupTabs::assign_some_to_master (uint32_t which, RouteList rl, std::string vcaname)
 {
 	if (!_session) {
 		return;
 	}
 
 	boost::shared_ptr<VCA> master;
+	bool set_name = false;
 
 	if (which == 0) {
 		if (_session->vca_manager().create_vca (1)) {
 			/* error */
 			return;
 		}
+		set_name = true;
 
-		/* VCAs use 1-based counting. Get most recently created VCA... */
-		which = _session->vca_manager().n_vcas();
+		/* Get most recently created VCA... */
+		which = _session->vca_manager().vcas().back()->number();
 	}
 
 	master = _session->vca_manager().vca_by_number (which);
@@ -502,6 +527,9 @@ GroupTabs::assign_some_to_master (uint32_t which, RouteList rl)
 	for (RouteList::iterator r = rl.begin(); r != rl.end(); ++r) {
 		(*r)->assign (master);
 	}
+	if (set_name && !vcaname.empty()) {
+		master->set_name (vcaname);
+	}
 }
 
 RouteList
@@ -545,7 +573,7 @@ GroupTabs::get_soloed ()
 void
 GroupTabs::assign_selection_to_master (uint32_t which)
 {
-	assign_some_to_master (which, selected_routes ());
+	assign_some_to_master (which, selected_routes (), _("Selection"));
 }
 
 void
@@ -656,16 +684,19 @@ void
 GroupTabs::collect (RouteGroup* g)
 {
 	boost::shared_ptr<RouteList> group_routes = g->route_list ();
-	group_routes->sort (Stripable::PresentationOrderSorter());
+	group_routes->sort (Stripable::Sorter());
 	int const N = group_routes->size ();
 
 	RouteList::iterator i = group_routes->begin ();
 	boost::shared_ptr<RouteList> routes = _session->get_routes ();
-	routes->sort (Stripable::PresentationOrderSorter());
+	routes->sort (Stripable::Sorter());
 	RouteList::const_iterator j = routes->begin ();
 
 	int diff = 0;
 	int coll = -1;
+
+	PresentationInfo::ChangeSuspender cs;
+
 	while (i != group_routes->end() && j != routes->end()) {
 
 		PresentationInfo::order_t const k = (*j)->presentation_info ().order();
@@ -679,21 +710,19 @@ GroupTabs::collect (RouteGroup* g)
 				--diff;
 			}
 
-			(*j)->set_presentation_order (coll, false);
+			(*j)->set_presentation_order (coll);
 
 			++coll;
 			++i;
 
 		} else {
 
-			(*j)->set_presentation_order (k + diff, false);
+			(*j)->set_presentation_order (k + diff);
 
 		}
 
 		++j;
 	}
-
-	_session->notify_presentation_info_change ();
 }
 
 void
@@ -721,7 +750,10 @@ GroupTabs::set_activation (RouteGroup* g, bool a)
 void
 GroupTabs::remove_group (RouteGroup* g)
 {
+	boost::shared_ptr<RouteList> rl (g->route_list ());
 	_session->remove_route_group (*g);
+
+	emit_gui_changed_for_members (rl);
 }
 
 /** Set the color of the tab of a route group */
@@ -729,42 +761,8 @@ void
 GroupTabs::set_group_color (RouteGroup* group, uint32_t color)
 {
 	assert (group);
-	uint32_t r, g, b, a;
-
-	UINT_TO_RGBA (color, &r, &g, &b, &a);
-
-	/* Hack to disallow black route groups; force a dark grey instead */
-	const uint32_t dark_gray = 25;
-
-	if (r < dark_gray && g < dark_gray && b < dark_gray) {
-		r = dark_gray;
-		g = dark_gray;
-		b = dark_gray;
-	}
-
-	GUIObjectState& gui_state = *ARDOUR_UI::instance()->gui_object_state;
-
-	char buf[64];
-
-	/* for historical reasons the colors must be stored as 16 bit color
-	 * values. Ugh.
-	 */
-
-	snprintf (buf, sizeof (buf), "%d:%d:%d", (r<<8), (g<<8), (b<<8));
-	gui_state.set_property (group_gui_id (group), "color", buf);
-
-	/* the group color change notification */
-
-	PBD::PropertyChange change;
-	change.add (Properties::color);
-	group->PropertyChanged (change);
-
-	/* This is a bit of a hack, but this might change
-	   our route's effective color, so emit gui_changed
-	   for our routes.
-	*/
-
-	emit_gui_changed_for_members (group);
+	PresentationInfo::ChangeSuspender cs;
+	group->set_rgba (color);
 }
 
 /** @return the ID string to use for the GUI state of a route group */
@@ -785,6 +783,14 @@ GroupTabs::group_color (RouteGroup* group)
 {
 	assert (group);
 
+	/* prefer libardour color, if set */
+	uint32_t rgba = group->rgba ();
+	if (rgba != 0) {
+		return rgba;
+	}
+
+	/* backwards compatibility, load old color */
+
 	GUIObjectState& gui_state = *ARDOUR_UI::instance()->gui_object_state;
 	string const gui_id = group_gui_id (group);
 	bool empty;
@@ -799,8 +805,7 @@ GroupTabs::group_color (RouteGroup* group)
 
 	int r, g, b;
 
-	/* for historical reasons, colors are stored as 16 bit values.
-	 */
+	/* for historical reasons, colors are stored as 16 bit values.  */
 
 	sscanf (color.c_str(), "%d:%d:%d", &r, &g, &b);
 
@@ -808,6 +813,9 @@ GroupTabs::group_color (RouteGroup* group)
 	g /= 256;
 	b /= 256;
 
+	group->migrate_rgba (RGBA_TO_UINT (r, g, b, 255));
+	gui_state.remove_node (gui_id);
+
 	return RGBA_TO_UINT (r, g, b, 255);
 }
 
@@ -819,7 +827,7 @@ GroupTabs::route_group_property_changed (RouteGroup* rg)
 	   for our routes.
 	*/
 
-	emit_gui_changed_for_members (rg);
+	emit_gui_changed_for_members (rg->route_list ());
 
 	set_dirty ();
 }
@@ -855,9 +863,11 @@ GroupTabs::route_removed_from_route_group (RouteGroup*, boost::weak_ptr<Route> w
 }
 
 void
-GroupTabs::emit_gui_changed_for_members (RouteGroup* rg)
+GroupTabs::emit_gui_changed_for_members (boost::shared_ptr<RouteList> rl)
 {
-	for (RouteList::iterator i = rg->route_list()->begin(); i != rg->route_list()->end(); ++i) {
-		(*i)->gui_changed (X_("color"), 0);
+	PresentationInfo::ChangeSuspender cs;
+
+	for (RouteList::iterator i = rl->begin(); i != rl->end(); ++i) {
+		(*i)->presentation_info().PropertyChanged (Properties::color);
 	}
 }
