@@ -368,6 +368,7 @@ namespace LuaMixer {
 	}
 
 };
+
 ////////////////////////////////////////////////////////////////////////////////
 
 static PBD::ScopedConnectionList _luaexecs;
@@ -422,6 +423,14 @@ lua_exec (std::string cmd)
 	return 0;
 }
 #endif
+////////////////////////////////////////////////////////////////////////////////
+
+// ARDOUR_UI and instance() are not exposed.
+ARDOUR::PresentationInfo::order_t
+lua_translate_order (RouteDialogs::InsertAt place)
+{
+	return ARDOUR_UI::instance()->translate_order (place);
+}
 
 ////////////////////////////////////////////////////////////////////////////////
 
@@ -908,7 +917,16 @@ LuaInstance::register_classes (lua_State* L)
 		.addFunction ("set_toggleaction", &PublicEditor::set_toggleaction)
 		.endClass ()
 
+		.addFunction ("translate_order", &lua_translate_order)
+
 		/* ArdourUI enums */
+		.beginNamespace ("InsertAt")
+		.addConst ("BeforeSelection", RouteDialogs::InsertAt(RouteDialogs::BeforeSelection))
+		.addConst ("AfterSelection", RouteDialogs::InsertAt(RouteDialogs::AfterSelection))
+		.addConst ("First", RouteDialogs::InsertAt(RouteDialogs::First))
+		.addConst ("Last", RouteDialogs::InsertAt(RouteDialogs::Last))
+		.endNamespace ()
+
 		.beginNamespace ("MarkerType")
 		.addConst ("Mark", ArdourMarker::Type(ArdourMarker::Mark))
 		.addConst ("Tempo", ArdourMarker::Type(ArdourMarker::Tempo))
@@ -1043,7 +1061,7 @@ LuaInstance::init ()
 			"   assert(type(f) == 'function', 'Factory is a not a function')"
 			"   assert(type(a) == 'table' or type(a) == 'nil', 'Given argument is invalid')"
 			"   self.scripts[i] = { ['n'] = n, ['s'] = s, ['f'] = f, ['a'] = a, ['c'] = c }"
-			"   local env = _ENV; env.f = nil env.io = nil"
+			"   local env = _ENV; env.f = nil"
 			"   self.instances[i] = load (string.dump(f, true), nil, nil, env)(a)"
 			"   if type(c) == 'function' then"
 			"     self.icons[i] = load (string.dump(c, true), nil, nil, env)(a)"
@@ -1161,6 +1179,11 @@ LuaInstance::init ()
 		_lua_clear = new luabridge::LuaRef(lua_mgr["clear"]);
 
 	} catch (luabridge::LuaException const& e) {
+		fatal << string_compose (_("programming error: %1"),
+				std::string ("Failed to setup Lua action interpreter") + e.what ())
+			<< endmsg;
+		abort(); /*NOTREACHED*/
+	} catch (...) {
 		fatal << string_compose (_("programming error: %1"),
 				X_("Failed to setup Lua action interpreter"))
 			<< endmsg;
@@ -1229,7 +1252,7 @@ LuaInstance::set_state (const XMLNode& node)
 				(*_lua_load)(std::string ((const char*)buf, size));
 			} catch (luabridge::LuaException const& e) {
 				cerr << "LuaException:" << e.what () << endl;
-			}
+			} catch (...) { }
 			for (int i = 0; i < 9; ++i) {
 				std::string name;
 				if (lua_action_name (i, name)) {
@@ -1249,7 +1272,7 @@ LuaInstance::set_state (const XMLNode& node)
 				SlotChanged (p->id(), p->name(), p->signals()); /* EMIT SIGNAL */
 			} catch (luabridge::LuaException const& e) {
 				cerr << "LuaException:" << e.what () << endl;
-			}
+			} catch (...) { }
 		}
 	}
 
@@ -1340,6 +1363,10 @@ LuaInstance::interactive_add (LuaScriptInfo::ScriptType type, int id)
 				string msg = string_compose (_("Loading Session script '%1' failed: %2"), spd.name(), e.what ());
 				Gtk::MessageDialog am (msg);
 				am.run ();
+			} catch (...) {
+				string msg = string_compose (_("Loading Session script '%1' failed: %2"), spd.name(), "Unknown Exception");
+				Gtk::MessageDialog am (msg);
+				am.run ();
 			}
 		default:
 			break;
@@ -1386,7 +1413,7 @@ LuaInstance::call_action (const int id)
 		lua.collect_garbage_step ();
 	} catch (luabridge::LuaException const& e) {
 		cerr << "LuaException:" << e.what () << endl;
-	}
+	} catch (...) { }
 }
 
 void
@@ -1403,7 +1430,7 @@ LuaInstance::render_icon (int i, cairo_t* cr, int w, int h, uint32_t clr)
 		 (*_lua_render_icon)(i + 1, (Cairo::Context *)&ctx, w, h, clr);
 	 } catch (luabridge::LuaException const& e) {
 		 cerr << "LuaException:" << e.what () << endl;
-	 }
+	 } catch (...) { }
 }
 
 bool
@@ -1429,6 +1456,8 @@ LuaInstance::set_lua_action (
 	} catch (luabridge::LuaException const& e) {
 		cerr << "LuaException:" << e.what () << endl;
 		return false;
+	} catch (...) {
+		return false;
 	}
 	_session->set_dirty ();
 	return true;
@@ -1442,6 +1471,8 @@ LuaInstance::remove_lua_action (const int id)
 	} catch (luabridge::LuaException const& e) {
 		cerr << "LuaException:" << e.what () << endl;
 		return false;
+	} catch (...) {
+		return false;
 	}
 	ActionChanged (id, ""); /* EMIT SIGNAL */
 	_session->set_dirty ();
@@ -1463,8 +1494,7 @@ LuaInstance::lua_action_name (const int id, std::string& rv)
 		return true;
 	} catch (luabridge::LuaException const& e) {
 		cerr << "LuaException:" << e.what () << endl;
-		return false;
-	}
+	} catch (...) { }
 	return false;
 }
 
@@ -1494,7 +1524,7 @@ LuaInstance::lua_action_has_icon (const int id)
 		}
 	} catch (luabridge::LuaException const& e) {
 		cerr << "LuaException:" << e.what () << endl;
-	}
+	} catch (...) { }
 	return false;
 }
 
@@ -1529,8 +1559,7 @@ LuaInstance::lua_action (const int id, std::string& name, std::string& script, L
 		return true;
 	} catch (luabridge::LuaException const& e) {
 		cerr << "LuaException:" << e.what () << endl;
-		return false;
-	}
+	} catch (...) { }
 	return false;
 }
 
@@ -1553,7 +1582,7 @@ LuaInstance::register_lua_slot (const std::string& name, const std::string& scri
 		}
 	} catch (luabridge::LuaException const& e) {
 		cerr << "LuaException:" << e.what () << endl;
-	}
+	} catch (...) { }
 
 	if (ah.none ()) {
 		cerr << "Script registered no hooks." << endl;
@@ -1570,7 +1599,7 @@ LuaInstance::register_lua_slot (const std::string& name, const std::string& scri
 		return true;
 	} catch (luabridge::LuaException const& e) {
 		cerr << "LuaException:" << e.what () << endl;
-	}
+	} catch (...) { }
 	_session->set_dirty ();
 	return false;
 }
@@ -1656,11 +1685,13 @@ LuaCallback::LuaCallback (Session *s,
 	}
 
 	try {
-	const std::string& bytecode = LuaScripting::get_factory_bytecode (script);
-	(*_lua_add)(name, script, bytecode, tbl_arg);
+		const std::string& bytecode = LuaScripting::get_factory_bytecode (script);
+		(*_lua_add)(name, script, bytecode, tbl_arg);
 	} catch (luabridge::LuaException const& e) {
 		cerr << "LuaException:" << e.what () << endl;
 		throw failed_constructor ();
+	} catch (...) {
+		throw failed_constructor ();
 	}
 
 	_id.reset ();
@@ -1699,7 +1730,7 @@ LuaCallback::LuaCallback (Session *s, XMLNode & node)
 		(*_lua_load)(std::string ((const char*)buf, size));
 	} catch (luabridge::LuaException const& e) {
 		cerr << "LuaException:" << e.what () << endl;
-	}
+	} catch (...) { }
 	g_free (buf);
 
 	set_session (s);
@@ -1761,7 +1792,7 @@ LuaCallback::init (void)
 			"   assert(type(f) == 'function', 'Factory is a not a function')"
 			"   assert(type(a) == 'table' or type(a) == 'nil', 'Given argument is invalid')"
 			"   self.script = { ['n'] = n, ['s'] = s, ['f'] = f, ['a'] = a }"
-			"   local env = _ENV; env.f = nil env.io = nil"
+			"   local env = _ENV; env.f = nil"
 			"   self.instance = load (string.dump(f, true), nil, nil, env)(a)"
 			"  end"
 			""
@@ -1859,6 +1890,11 @@ LuaCallback::init (void)
 		_lua_load = new luabridge::LuaRef(lua_mgr["restore"]);
 
 	} catch (luabridge::LuaException const& e) {
+		fatal << string_compose (_("programming error: %1"),
+				std::string ("Failed to setup Lua callback interpreter: ") + e.what ())
+			<< endmsg;
+		abort(); /*NOTREACHED*/
+	} catch (...) {
 		fatal << string_compose (_("programming error: %1"),
 				X_("Failed to setup Lua callback interpreter"))
 			<< endmsg;
@@ -1907,7 +1943,7 @@ LuaCallback::lua_slot (std::string& name, std::string& script, ActionHook& ah, A
 	} catch (luabridge::LuaException const& e) {
 		cerr << "LuaException:" << e.what () << endl;
 		return false;
-	}
+	} catch (...) { }
 	return false;
 }
 
