@@ -20,7 +20,7 @@
 #ifndef __lxvst_plugin_ui_h__
 #define __lxvst_plugin_ui_h__
 
-#include <sigc++/signal.h>
+#include "pbd/signals.h"
 #include "vst_plugin_ui.h"
 
 #ifdef LXVST_SUPPORT
@@ -32,14 +32,14 @@ namespace ARDOUR {
 
 class LXVSTPluginUI : public VSTPluginUI
 {
-  public:
+public:
 	LXVSTPluginUI (boost::shared_ptr<ARDOUR::PluginInsert>, boost::shared_ptr<ARDOUR::VSTPlugin>);
 	~LXVSTPluginUI ();
 
 	int get_preferred_height ();
 
-	bool start_updating (GdkEventAny *);
-	bool stop_updating (GdkEventAny *);
+	bool start_updating (GdkEventAny*) { return false; }
+	bool stop_updating (GdkEventAny*) { return false; }
 
 	int package (Gtk::Window&);
 	void forward_key_event (GdkEventKey *);
@@ -49,7 +49,7 @@ private:
 	void resize_callback ();
 	int get_XID ();
 
-	sigc::connection _screen_update_connection;
+	PBD::ScopedConnection _resize_connection;
 };
 
 #endif //LXVST_SUPPORT
