@@ -26,6 +26,7 @@
 #include <sigc++/bind.h>
 
 #include <gtkmm/accelmap.h>
+#include <gtkmm/comboboxtext.h>
 
 #include <glibmm/threads.h>
 
@@ -408,22 +409,6 @@ Meterbridge::on_scroll()
 	metrics_right.set_metric_mode(mm_right, mt_right);
 }
 
-struct PresentationInfoRouteSorter
-{
-	bool operator() (boost::shared_ptr<Route> a, boost::shared_ptr<Route> b) {
-		if (a->is_master() || a->is_monitor()) {
-			/* "a" is a special route (master, monitor, etc), and comes
-			 * last in the mixer ordering
-			 */
-			return false;
-		} else if (b->is_master() || b->is_monitor()) {
-			/* everything comes before b */
-			return true;
-		}
-		return a->presentation_info().order() < b->presentation_info().order();
-	}
-};
-
 void
 Meterbridge::set_session (Session* s)
 {
@@ -449,7 +434,7 @@ Meterbridge::set_session (Session* s)
 	boost::shared_ptr<RouteList> routes = _session->get_routes();
 
 	RouteList copy (*routes);
-	copy.sort (PresentationInfoRouteSorter());
+	copy.sort (Stripable::Sorter (true));
 	add_strips (copy);
 
 	_session->RouteAdded.connect (_session_connections, invalidator (*this), boost::bind (&Meterbridge::add_strips, this, _1), gui_context());
@@ -486,7 +471,6 @@ Meterbridge::session_going_away ()
 int
 Meterbridge::set_state (const XMLNode& node)
 {
-	XMLProperty const * prop;
 	XMLNode* geometry;
 
 	m_width = default_width;
@@ -495,44 +479,15 @@ Meterbridge::set_state (const XMLNode& node)
 	m_root_y = 1;
 
 	if ((geometry = find_named_node (node, "geometry")) != 0) {
-
-		XMLProperty const * prop;
-
-		if ((prop = geometry->property("x_size")) == 0) {
-			prop = geometry->property ("x-size");
-		}
-		if (prop) {
-			m_width = atoi(prop->value());
-		}
-		if ((prop = geometry->property("y_size")) == 0) {
-			prop = geometry->property ("y-size");
-		}
-		if (prop) {
-			m_height = atoi(prop->value());
-		}
-
-		if ((prop = geometry->property ("x_pos")) == 0) {
-			prop = geometry->property ("x-pos");
-		}
-		if (prop) {
-			m_root_x = atoi (prop->value());
-
-		}
-		if ((prop = geometry->property ("y_pos")) == 0) {
-			prop = geometry->property ("y-pos");
-		}
-		if (prop) {
-			m_root_y = atoi (prop->value());
-		}
+		geometry->get_property ("x-size", m_width);
+		geometry->get_property ("y-size", m_height);
+		geometry->get_property ("x-pos", m_root_x);
+		geometry->get_property ("y-pos", m_root_y);
 	}
 
 	set_window_pos_and_size ();
 
-	if ((prop = node.property ("show-meterbridge"))) {
-		if (string_is_affirmative (prop->value())) {
-		       _visible = true;
-		}
-	}
+	node.get_property ("show-meterbridge", _visible);
 
 	return 0;
 }
@@ -540,7 +495,6 @@ Meterbridge::set_state (const XMLNode& node)
 XMLNode&
 Meterbridge::get_state (void)
 {
-	char buf[32];
 	XMLNode* node = new XMLNode ("Meterbridge");
 
 	if (is_realized() && _visible) {
@@ -548,17 +502,13 @@ Meterbridge::get_state (void)
 	}
 
 	XMLNode* geometry = new XMLNode ("geometry");
-	snprintf(buf, sizeof(buf), "%d", m_width);
-	geometry->add_property(X_("x_size"), string(buf));
-	snprintf(buf, sizeof(buf), "%d", m_height);
-	geometry->add_property(X_("y_size"), string(buf));
-	snprintf(buf, sizeof(buf), "%d", m_root_x);
-	geometry->add_property(X_("x_pos"), string(buf));
-	snprintf(buf, sizeof(buf), "%d", m_root_y);
-	geometry->add_property(X_("y_pos"), string(buf));
+	geometry->set_property(X_("x-size"), m_width);
+	geometry->set_property(X_("y-size"), m_height);
+	geometry->set_property(X_("x-pos"), m_root_x);
+	geometry->set_property(X_("y-pos"), m_root_y);
 	node->add_child_nocopy (*geometry);
 
-	node->add_property ("show-meterbridge", _visible ? "yes" : "no");
+	node->set_property ("show-meterbridge", _visible);
 	return *node;
 }
 
