@@ -77,6 +77,7 @@
 #include "midi_region_view.h"
 #include "midi_time_axis.h"
 #include "patch_change_dialog.h"
+#include "patch_change_widget.h"
 #include "piano_roll_header.h"
 #include "playlist_selector.h"
 #include "plugin_selector.h"
@@ -191,10 +192,6 @@ MidiTimeAxisView::set_route (boost::shared_ptr<Route> rt)
 	update_control_names();
 	processors_changed (RouteProcessorChange ());
 
-	_route->processors_changed.connect (*this, invalidator (*this),
-	                                    boost::bind (&MidiTimeAxisView::processors_changed, this, _1),
-	                                    gui_context());
-
 	if (is_track()) {
 		_piano_roll_header->SetNoteSelection.connect (
 			sigc::mem_fun (*this, &MidiTimeAxisView::set_note_selection));
@@ -386,11 +383,6 @@ MidiTimeAxisView::setup_midnam_patches ()
 	}
 }
 
-void
-MidiTimeAxisView::drop_instrument_ref ()
-{
-	midnam_connection.drop_connections ();
-}
 void
 MidiTimeAxisView::start_scroomer_update ()
 {
@@ -412,17 +404,8 @@ MidiTimeAxisView::update_patch_selector ()
 
 	bool pluginprovided = false;
 	if (_route) {
-		boost::shared_ptr<Processor> the_instrument (_route->the_instrument());
-		boost::shared_ptr<PluginInsert> pi = boost::dynamic_pointer_cast<PluginInsert>(the_instrument);
+		boost::shared_ptr<PluginInsert> pi = boost::dynamic_pointer_cast<PluginInsert> (_route->the_instrument ());
 		if (pi && pi->plugin ()->has_midnam ()) {
-			midnam_connection.drop_connections ();
-			the_instrument->DropReferences.connect (midnam_connection, invalidator (*this),
-					boost::bind (&MidiTimeAxisView::drop_instrument_ref, this),
-					gui_context());
-			pi->plugin()->UpdateMidnam.connect (midnam_connection, invalidator (*this),
-					boost::bind (&Plugin::read_midnam, pi->plugin ()),
-					gui_context());
-
 			pluginprovided = true;
 			std::string model_name = pi->plugin ()->midnam_model ();
 			if (gui_property (X_("midnam-model-name")) != model_name) {
@@ -440,6 +423,7 @@ MidiTimeAxisView::update_patch_selector ()
 	}
 }
 
+
 void
 MidiTimeAxisView::model_changed(const std::string& model)
 {
@@ -481,6 +465,10 @@ MidiTimeAxisView::model_changed(const std::string& model)
 	delete controller_menu;
 	controller_menu = 0;
 	build_automation_action_menu(false);
+
+	if (patch_change_dialog ()) {
+		patch_change_dialog ()->refresh ();
+	}
 }
 
 void
@@ -556,11 +544,11 @@ MidiTimeAxisView::append_extra_display_menu_items ()
 
 	items.push_back (MenuElem (_("Note Range"), *range_menu));
 	items.push_back (MenuElem (_("Note Mode"), *build_note_mode_menu()));
-	items.push_back (MenuElem (_("Channel Selector"),
+	items.push_back (MenuElem (_("Channel Selector..."),
 				   sigc::mem_fun(*this, &MidiTimeAxisView::toggle_channel_selector)));
 
-	items.push_back (MenuElem (_("Select Patch..."),
-				sigc::mem_fun(*this, &MidiTimeAxisView::send_patch_change)));
+	items.push_back (MenuElem (_("Patch Selector..."),
+				sigc::mem_fun(*this, &RouteUI::select_midi_patch)));
 
 	color_mode_menu = build_color_mode_menu();
 	if (color_mode_menu) {
@@ -1077,36 +1065,6 @@ MidiTimeAxisView::build_color_mode_menu()
 	return mode_menu;
 }
 
-void
-MidiTimeAxisView::send_patch_change ()
-{
-	if (!_route) {
-		return;
-	}
-
-	Evoral::PatchChange<Evoral::Beats> empty (Evoral::Beats(), 0, 0, 0);
-	PatchChangeDialog d (0, 0, empty, _route->instrument_info(), Gtk::Stock::OK);
-
-	if (d.run() == RESPONSE_CANCEL) {
-		return;
-	}
-	Evoral::PatchChange<Evoral::Beats> p (d.patch ());
-
-	uint8_t chn = p.channel();
-
-	boost::shared_ptr<AutomationControl> bank_msb = _route->automation_control(Evoral::Parameter (MidiCCAutomation, chn, MIDI_CTL_MSB_BANK), true);
-	boost::shared_ptr<AutomationControl> bank_lsb = _route->automation_control(Evoral::Parameter (MidiCCAutomation, chn, MIDI_CTL_LSB_BANK), true);
-	boost::shared_ptr<AutomationControl> program = _route->automation_control(Evoral::Parameter (MidiPgmChangeAutomation, chn), true);
-
-	if (!bank_msb || ! bank_lsb || !program) {
-		return;
-	}
-
-	bank_msb->set_value (p.bank_msb (), Controllable::NoGroup);
-	bank_lsb->set_value (p.bank_lsb (), Controllable::NoGroup);
-	program->set_value  (p.program () , Controllable::NoGroup);
-}
-
 void
 MidiTimeAxisView::set_note_mode(NoteMode mode, bool apply_to_selection)
 {
