@@ -120,8 +120,6 @@ private:
 
 	void setup_midnam_patches ();
 	void update_patch_selector ();
-	void drop_instrument_ref ();
-	PBD::ScopedConnectionList midnam_connection;
 
 	void start_scroomer_update ();
 	void stop_scroomer_update ();
@@ -138,8 +136,6 @@ private:
 	void set_note_mode (ARDOUR::NoteMode mode, bool apply_to_selection = false);
 	void set_color_mode (ARDOUR::ColorMode, bool force = false, bool redisplay = true, bool apply_to_selection = false);
 	void set_note_range (MidiStreamView::VisibleNoteRange range, bool apply_to_selection = false);
-	void send_patch_change ();
-
 	void route_active_changed ();
 	void note_range_changed ();
 	void contents_height_changed ();
@@ -196,6 +192,8 @@ private:
 	ParameterMenuMap _controller_menu_map;
 
 	StepEditor* _step_editor;
+
+	void immediate_patch_chnage_response (int response);
 };
 
 #endif /* __ardour_midi_time_axis_h__ */
