@@ -23,14 +23,15 @@
 #include "pbd/error.h"
 #include "pbd/replace_all.h"
 
-#include "gtkmm2ext/bindable_button.h"
-#include "gtkmm2ext/tearoff.h"
 #include "gtkmm2ext/actions.h"
 #include "gtkmm2ext/utils.h"
 
 #include <gtkmm/menu.h>
 #include <gtkmm/menuitem.h>
 
+#include "widgets/tearoff.h"
+#include "widgets/tooltips.h"
+
 #include "ardour/amp.h"
 #include "ardour/audioengine.h"
 #include "ardour/monitor_processor.h"
@@ -46,13 +47,13 @@
 #include "monitor_section.h"
 #include "public_editor.h"
 #include "timers.h"
-#include "tooltips.h"
 #include "ui_config.h"
 #include "utils.h"
 
 #include "pbd/i18n.h"
 
 using namespace ARDOUR;
+using namespace ArdourWidgets;
 using namespace ARDOUR_UI_UTILS;
 using namespace Gtk;
 using namespace Gtkmm2ext;
@@ -69,6 +70,7 @@ MonitorSection::MonitorSection (Session* s)
 	: SessionHandlePtr (s)
 	, RouteUI (s)
 	, _tearoff (0)
+	, channel_table (0)
 	, channel_table_viewport (*channel_table_scroller.get_hadjustment()
 	                          , *channel_table_scroller.get_vadjustment ())
 	, gain_control (0)
@@ -87,6 +89,7 @@ MonitorSection::MonitorSection (Session* s)
 	, solo_mute_override_button (ArdourButton::led_default_elements)
 	, toggle_processorbox_button (ArdourButton::default_elements)
 	, _inhibit_solo_model_update (false)
+	, _rr_selection ()
 	, _ui_initialized (false)
 {
 
@@ -99,6 +102,8 @@ MonitorSection::MonitorSection (Session* s)
 		load_bindings ();
 	}
 
+	channel_size_group = SizeGroup::create (SIZE_GROUP_HORIZONTAL);
+
 	set_data ("ardour-bindings", bindings);
 
 	_plugin_selector = new PluginSelector (PluginManager::instance());
@@ -308,10 +313,7 @@ MonitorSection::MonitorSection (Session* s)
 	channel_table_scroller.show ();
 	channel_table_scroller.add (channel_table_viewport);
 
-	channel_size_group  = SizeGroup::create (SIZE_GROUP_HORIZONTAL);
 	channel_size_group->add_widget (channel_table_header);
-	channel_size_group->add_widget (channel_table);
-
 	channel_table_header.resize (1, 5);
 
 	Label* l1 = manage (new Label (X_("  ")));
@@ -384,10 +386,6 @@ MonitorSection::MonitorSection (Session* s)
 	level_tbl->attach (*dim_control,         1, 3, 4, 5, EXPAND|FILL, SHRINK, 1, 2);
 	level_tbl->attach (*dim_display,         1, 3, 5, 6, EXPAND     , SHRINK, 1, 2);
 
-	/* note that we don't pack the table_hpacker till later
-	 * -> top level channel_table_packer */
-	table_hpacker.pack_start (channel_table, true, true);
-
 	// mono, dim
 	HBox* mono_dim_box = manage (new HBox);
 	mono_dim_box->set_spacing (PX_SCALE(4));
@@ -412,6 +410,18 @@ MonitorSection::MonitorSection (Session* s)
 	lower_packer->pack_start (*mono_dim_box,        false, false, PX_SCALE(2));
 	lower_packer->pack_start (cut_all_button,       false, false, PX_SCALE(2));
 
+	// calc height of mixer scrollbar
+	int scrollbar_height = 0;
+	{
+		Gtk::Window window (WINDOW_TOPLEVEL);
+		HScrollbar scrollbar;
+		window.add (scrollbar);
+		scrollbar.set_name ("MixerWindow");
+		scrollbar.ensure_style();
+		Gtk::Requisition requisition(scrollbar.size_request ());
+		scrollbar_height = requisition.height;
+	}
+
 	// output port select
 	VBox* out_packer = manage (new VBox);
 	out_packer->set_spacing (PX_SCALE(2));
@@ -429,7 +439,13 @@ MonitorSection::MonitorSection (Session* s)
 	vpacker.pack_start (*level_tbl,           false, false, PX_SCALE(8));
 	vpacker.pack_start (*lower_packer,        false, false, PX_SCALE(8));
 	vpacker.pack_start (master_packer,        false, false, PX_SCALE(10));
-	vpacker.pack_end   (*out_packer,          false, false, PX_SCALE(3));
+	vpacker.pack_end   (*out_packer,          false, false,
+#ifdef MIXBUS
+			scrollbar_height - 2 /* no outer frame */
+#else
+			scrollbar_height + 2 /* frame borders */
+#endif
+			);
 
 	hpacker.set_spacing (0);
 	hpacker.pack_start (vpacker, true, true);
@@ -446,7 +462,6 @@ MonitorSection::MonitorSection (Session* s)
 	mono_dim_box->show ();
 	spin_packer->show ();
 	master_packer.show ();
-	channel_table.show ();
 
 	rude_box->show();
 	solo_tbl->show_all();
@@ -457,7 +472,6 @@ MonitorSection::MonitorSection (Session* s)
 	vpacker.show ();
 	hpacker.show ();
 
-	populate_buttons ();
 	map_state ();
 	assign_controllables ();
 
@@ -497,7 +511,7 @@ MonitorSection::~MonitorSection ()
 	}
 
 	_channel_buttons.clear ();
-	_output_changed_connection.disconnect ();
+	output_changed_connections.drop_connections ();
 
 	delete insert_box; insert_box = 0;
 	delete output_button; output_button = 0;
@@ -511,6 +525,7 @@ MonitorSection::~MonitorSection ()
 	delete solo_cut_display; solo_cut_display = 0;
 	delete _tearoff; _tearoff = 0;
 	delete _output_selector; _output_selector = 0;
+	delete channel_table; channel_table = 0;
 }
 
 bool
@@ -588,62 +603,30 @@ MonitorSection::set_session (Session* s)
 			/* session with monitor section */
 			_monitor = _route->monitor_control ();
 			assign_controllables ();
-			_route->output()->changed.connect (_output_changed_connection, invalidator (*this),
+			_route->output()->changed.connect (output_changed_connections, invalidator (*this),
 											boost::bind (&MonitorSection::update_output_display, this),
 											gui_context());
 			insert_box->set_route (_route);
 			_route->processors_changed.connect (*this, invalidator (*this), boost::bind (&MonitorSection::processors_changed, this, _1), gui_context());
+			_route->output()->PortCountChanged.connect (output_changed_connections, invalidator (*this), boost::bind (&MonitorSection::populate_buttons, this), gui_context());
 			if (_ui_initialized) {
 				update_processor_box ();
 			}
 		} else {
 			/* session with no monitor section */
-			_output_changed_connection.disconnect();
+			output_changed_connections.drop_connections();
 			_monitor.reset ();
 			_route.reset ();
 			delete _output_selector;
 			_output_selector = 0;
 		}
 
-		if (channel_table_scroller.get_parent()) {
-			/* scroller is packed, so remove it */
-			channel_table_packer.remove (channel_table_scroller);
-		}
-
-		if (table_hpacker.get_parent () == &channel_table_packer) {
-			/* this occurs when the table hpacker is directly
-				 packed, so remove it.
-				 */
-			channel_table_packer.remove (table_hpacker);
-		} else if (table_hpacker.get_parent()) {
-			channel_table_viewport.remove ();
-		}
-
-		if (_monitor->output_streams().n_audio() > 7) {
-			/* put the table into a scrolled window, and then put
-			 * that into the channel vpacker, after the table header
-			 */
-			channel_table_viewport.add (table_hpacker);
-			channel_table_packer.pack_start (channel_table_scroller, true, true);
-			channel_table_viewport.show ();
-			channel_table_scroller.show ();
-
-		} else {
-			/* just put the channel table itself into the channel
-			 * vpacker, after the table header
-			 */
-
-			channel_table_packer.pack_start (table_hpacker, true, true);
-			channel_table_scroller.hide ();
-		}
-
-		table_hpacker.show ();
-		channel_table.show ();
+		populate_buttons ();
 
 	} else {
 		/* no session */
 
-		_output_changed_connection.disconnect();
+		output_changed_connections.drop_connections();
 		_monitor.reset ();
 		_route.reset ();
 		control_connections.drop_connections ();
@@ -669,20 +652,37 @@ MonitorSection::ChannelButtonSet::ChannelButtonSet ()
 	invert.unset_flags (Gtk::CAN_FOCUS);
 }
 
-	void
+void
 MonitorSection::populate_buttons ()
 {
 	if (!_monitor) {
 		return;
 	}
 
+	if (channel_table) {
+		channel_size_group->remove_widget (*channel_table);
+		delete channel_table;
+	}
+
+	channel_table = new Gtk::Table();
+
+	channel_table->set_col_spacings (6);
+	channel_table->set_row_spacings (6);
+	channel_table->set_homogeneous (true);
+
+	channel_size_group->add_widget (*channel_table);
+	channel_table->show ();
+	table_hpacker.pack_start (*channel_table, true, true);
+
+	for (ChannelButtons::iterator i = _channel_buttons.begin(); i != _channel_buttons.end(); ++i) {
+		delete *i;
+	}
+	_channel_buttons.clear ();
+
 	Glib::RefPtr<Action> act;
 	uint32_t nchans = _monitor->output_streams().n_audio();
 
-	channel_table.resize (nchans, 5);
-	channel_table.set_col_spacings (6);
-	channel_table.set_row_spacings (6);
-	channel_table.set_homogeneous (true);
+	channel_table->resize (nchans, 5);
 
 	const uint32_t row_offset = 0;
 
@@ -704,16 +704,16 @@ MonitorSection::populate_buttons ()
 		}
 
 		Label* label = manage (new Label (l));
-		channel_table.attach (*label, 0, 1, i+row_offset, i+row_offset+1, EXPAND|FILL);
+		channel_table->attach (*label, 0, 1, i+row_offset, i+row_offset+1, EXPAND|FILL);
 
 		ChannelButtonSet* cbs = new ChannelButtonSet;
 
 		_channel_buttons.push_back (cbs);
 
-		channel_table.attach (cbs->cut, 1, 2, i+row_offset, i+row_offset+1, EXPAND|FILL);
-		channel_table.attach (cbs->dim, 2, 3, i+row_offset, i+row_offset+1, EXPAND|FILL);
-		channel_table.attach (cbs->solo, 3, 4, i+row_offset, i+row_offset+1, EXPAND|FILL);
-		channel_table.attach (cbs->invert, 4, 5, i+row_offset, i+row_offset+1, EXPAND|FILL);
+		channel_table->attach (cbs->cut, 1, 2, i+row_offset, i+row_offset+1, EXPAND|FILL);
+		channel_table->attach (cbs->dim, 2, 3, i+row_offset, i+row_offset+1, EXPAND|FILL);
+		channel_table->attach (cbs->solo, 3, 4, i+row_offset, i+row_offset+1, EXPAND|FILL);
+		channel_table->attach (cbs->invert, 4, 5, i+row_offset, i+row_offset+1, EXPAND|FILL);
 
 		snprintf (buf, sizeof (buf), "monitor-cut-%u", i);
 		act = ActionManager::get_action (X_("Monitor"), buf);
@@ -740,7 +740,40 @@ MonitorSection::populate_buttons ()
 		}
 	}
 
-	channel_table.show_all ();
+	channel_table->show_all ();
+
+	if (channel_table_scroller.get_parent()) {
+		/* scroller is packed, so remove it */
+		channel_table_packer.remove (channel_table_scroller);
+	}
+
+	if (table_hpacker.get_parent () == &channel_table_packer) {
+		/* this occurs when the table hpacker is directly
+			 packed, so remove it.
+			 */
+		channel_table_packer.remove (table_hpacker);
+	} else if (table_hpacker.get_parent()) {
+		channel_table_viewport.remove ();
+	}
+
+	if (nchans > 7) {
+		/* put the table into a scrolled window, and then put
+		 * that into the channel vpacker, after the table header
+		 */
+		channel_table_viewport.add (table_hpacker);
+		channel_table_packer.pack_start (channel_table_scroller, true, true);
+		channel_table_viewport.show ();
+		channel_table_scroller.show ();
+
+	} else {
+		/* just put the channel table itself into the channel
+		 * vpacker, after the table header
+		 */
+		channel_table_packer.pack_start (table_hpacker, true, true);
+		channel_table_scroller.hide ();
+	}
+	table_hpacker.show ();
+	channel_table->show ();
 }
 
 void
@@ -1147,6 +1180,10 @@ MonitorSection::map_state ()
 void
 MonitorSection::do_blink (bool onoff)
 {
+	if (!UIConfiguration::instance().get_blink_alert_indicators ()) {
+		onoff = true;
+	}
+
 	solo_blink (onoff);
 	audition_blink (onoff);
 }
