@@ -20,7 +20,7 @@
 #ifndef __gtk_ardour_mono_panner_editor_h__
 #define __gtk_ardour_mono_panner_editor_h__
 
-#include <gtkmm.h>
+#include <gtkmm/spinbutton.h>
 #include "panner_editor.h"
 
 class MonoPanner;
