@@ -55,6 +55,7 @@
 
 #include "ardour_window.h"
 #include "automation_controller.h"
+#include "gtk_pianokeyboard.h"
 
 namespace ARDOUR {
 	class PluginInsert;
@@ -237,6 +238,7 @@ private:
 		bool                                    update_pending;
 		bool                                    ignore_change;
 
+
 		/* output */
 
 		Gtk::EventBox* display;
@@ -252,8 +254,12 @@ private:
 		/* layout */
 		Gtk::Table* knobtable;
 		int x0, x1, y0, y1;
+
+		bool short_autostate; // modify with set_short_autostate below
 	};
 
+	void set_short_autostate(ControlUI* cui, bool value);
+
 	std::vector<ControlUI*>   input_controls; // workaround for preset load
 	std::vector<ControlUI*>   input_controls_with_automation;
 	std::vector<ControlUI*>   output_controls;
@@ -296,6 +302,29 @@ private:
 	                        Gtk::FileChooserButton*            widget);
 	void path_property_changed (uint32_t key, const ARDOUR::Variant& value);
 
+	void scroller_size_request (Gtk::Requisition*);
+	Gtk::ScrolledWindow scroller;
+
+	Gtk::Expander   _plugin_pianokeyboard_expander;
+	PianoKeyboard*  _piano;
+	Gtk::Widget*    _pianomm;
+	Gtk::VBox       _pianobox;
+	Gtk::SpinButton _piano_velocity;
+	Gtk::SpinButton _piano_channel;
+
+	static void _note_on_event_handler (GtkWidget*, int, gpointer);
+	static void _note_off_event_handler (GtkWidget*, int, gpointer);
+	void note_on_event_handler (int);
+	void note_off_event_handler (int);
+
+	void toggle_pianokeyboard ();
+	void build_midi_table ();
+	void midi_refill_patches ();
+	void midi_bank_patch_change (uint8_t chn);
+	void midi_bank_patch_select (uint8_t chn, uint32_t bankpgm);
+	std::vector<ArdourWidgets::ArdourDropdown*> midi_pgmsel;
+	PBD::ScopedConnectionList midi_connections;
+	std::map<uint32_t, std::string> pgm_names;
 };
 
 class PluginUIWindow : public ArdourWindow
