@@ -40,13 +40,17 @@
 
 #include "pbd/statefuldestructible.h"
 
+#include "ardour/session_handle.h"
+
 #include "canvas/fwd.h"
 
 #include "gtkmm2ext/actions.h"
 #include "gtkmm2ext/bindings.h"
-#include "gtkmm2ext/tabbable.h"
 #include "gtkmm2ext/visibility_tracker.h"
 
+#include "widgets/tabbable.h"
+
+#include "axis_provider.h"
 #include "editing.h"
 #include "selection.h"
 
@@ -55,9 +59,9 @@ namespace ARDOUR {
 	class Region;
 	class Playlist;
 	class RouteGroup;
-        class Trimmable;
-        class Movable;
-        class Stripable;
+	class Trimmable;
+	class Movable;
+	class Stripable;
 }
 
 namespace Gtk {
@@ -80,6 +84,7 @@ class PluginUIWindow;
 class RegionView;
 class RouteTimeAxisView;
 class Selection;
+class StripableTimeAxisView;
 class TempoCurve;
 class TempoMarker;
 class TimeAxisView;
@@ -105,8 +110,9 @@ using ARDOUR::framecnt_t;
  * of PublicEditor need not be recompiled if private methods or member variables
  * change.
  */
-class PublicEditor : public Gtkmm2ext::Tabbable {
-  public:
+class PublicEditor : public ArdourWidgets::Tabbable,  public ARDOUR::SessionHandlePtr, public AxisViewProvider
+{
+public:
 	PublicEditor (Gtk::Widget& content);
 	virtual ~PublicEditor ();
 
@@ -123,9 +129,6 @@ class PublicEditor : public Gtkmm2ext::Tabbable {
 	 */
 	virtual void set_session (ARDOUR::Session* s) = 0;
 
-	/** @return The Session that we are editing, or 0 */
-	virtual ARDOUR::Session* session () const = 0;
-
 	/** Set the snap type.
 	 * @param t Snap type (defined in editing_syms.h)
 	 */
@@ -148,10 +151,10 @@ class PublicEditor : public Gtkmm2ext::Tabbable {
 	 * Snap a value according to the current snap setting.
 	 * ensure_snap overrides SnapOff and magnetic snap
 	 */
-	virtual void snap_to (framepos_t&       first,
-	                      ARDOUR::RoundMode direction = ARDOUR::RoundNearest,
-	                      bool              for_mark  = false,
-			      bool              ensure_snap = false) = 0;
+	virtual void snap_to (ARDOUR::MusicFrame& first,
+	                      ARDOUR::RoundMode   direction = ARDOUR::RoundNearest,
+	                      bool                for_mark  = false,
+	                      bool                ensure_snap = false) = 0;
 
 	/** Undo some transactions.
 	 * @param n Number of transactions to undo.
@@ -181,8 +184,7 @@ class PublicEditor : public Gtkmm2ext::Tabbable {
 	 */
 	virtual Editing::MouseMode current_mouse_mode () const = 0;
 
-	/** @return Whether the current mouse mode is an "internal" editing mode.
-	 */
+	/** @return Whether the current mouse mode is an "internal" editing mode. */
 	virtual bool internal_editing() const = 0;
 
 	/** Possibly start the audition of a region.  If @param r is 0, or not an AudioRegion
@@ -192,20 +194,30 @@ class PublicEditor : public Gtkmm2ext::Tabbable {
 	 */
 	virtual void consider_auditioning (boost::shared_ptr<ARDOUR::Region> r) = 0;
 
+	/* import dialogs -> ardour-ui ?! */
+	virtual void external_audio_dialog () = 0;
+	virtual void session_import_dialog () = 0;
+
 	virtual void new_region_from_selection () = 0;
 	virtual void separate_region_from_selection () = 0;
 
 	virtual void transition_to_rolling (bool fwd) = 0;
 	virtual framepos_t pixel_to_sample (double pixel) const = 0;
+	virtual framepos_t playhead_cursor_sample () const = 0;
 	virtual double sample_to_pixel (framepos_t frame) const = 0;
 	virtual double sample_to_pixel_unrounded (framepos_t frame) const = 0;
+
 	virtual Selection& get_selection () const = 0;
 	virtual bool get_selection_extents (framepos_t &start, framepos_t &end) const = 0;
 	virtual Selection& get_cut_buffer () const = 0;
-	virtual void track_mixer_selection () = 0;
+
+	virtual void set_selection (std::list<Selectable*>, Selection::Operation) = 0;
+
 	virtual bool extend_selection_to_track (TimeAxisView&) = 0;
 	virtual void play_selection () = 0;
 	virtual void play_with_preroll () = 0;
+	virtual void rec_with_preroll () = 0;
+	virtual void rec_with_count_in () = 0;
 	virtual void maybe_locate_with_edit_preroll (framepos_t location) = 0;
 	virtual void goto_nth_marker (int nth) = 0;
 	virtual void trigger_script (int nth) = 0;
@@ -217,11 +229,37 @@ class PublicEditor : public Gtkmm2ext::Tabbable {
 	virtual void set_loop_range (framepos_t start, framepos_t end, std::string cmd) = 0;
 	virtual void set_punch_range (framepos_t start, framepos_t end, std::string cmd) = 0;
 
+	virtual void jump_forward_to_mark () = 0;
+	virtual void jump_backward_to_mark () = 0;
+
+	virtual void set_session_start_from_playhead () = 0;
+	virtual void set_session_end_from_playhead () = 0;
+
+	virtual void toggle_location_at_playhead_cursor () = 0;
+
+	virtual void nudge_forward (bool next, bool force_playhead) = 0;
+	virtual void nudge_backward (bool next, bool force_playhead) = 0;
+
+	virtual void playhead_forward_to_grid () = 0;
+	virtual void playhead_backward_to_grid () = 0;
+
+	virtual void keyboard_selection_begin ( Editing::EditIgnoreOption = Editing::EDIT_IGNORE_NONE) = 0;
+	virtual void keyboard_selection_finish (bool add, Editing::EditIgnoreOption = Editing::EDIT_IGNORE_NONE) = 0;
+
+	virtual void set_punch_start_from_edit_point () = 0;
+	virtual void set_punch_end_from_edit_point () = 0;
+	virtual void set_loop_start_from_edit_point () = 0;
+	virtual void set_loop_end_from_edit_point () = 0;
+
 	virtual Editing::MouseMode effective_mouse_mode () const = 0;
 
 	/** Import existing media */
-	virtual void do_import (std::vector<std::string> paths, Editing::ImportDisposition, Editing::ImportMode mode, ARDOUR::SrcQuality, framepos_t&, boost::shared_ptr<ARDOUR::PluginInfo> instrument=boost::shared_ptr<ARDOUR::PluginInfo>()) = 0;
-	virtual void do_embed (std::vector<std::string> paths, Editing::ImportDisposition, Editing::ImportMode mode,  framepos_t&, boost::shared_ptr<ARDOUR::PluginInfo> instrument=boost::shared_ptr<ARDOUR::PluginInfo>()) = 0;
+	virtual void do_import (std::vector<std::string> paths, Editing::ImportDisposition, Editing::ImportMode mode, ARDOUR::SrcQuality,
+	                        ARDOUR::MidiTrackNameSource, ARDOUR::MidiTempoMapDisposition, framepos_t&,
+	                        boost::shared_ptr<ARDOUR::PluginInfo> instrument=boost::shared_ptr<ARDOUR::PluginInfo>()) = 0;
+	virtual void do_embed (std::vector<std::string> paths, Editing::ImportDisposition, Editing::ImportMode mode,
+	                       framepos_t&,
+	                       boost::shared_ptr<ARDOUR::PluginInfo> instrument=boost::shared_ptr<ARDOUR::PluginInfo>()) = 0;
 
 	/** Open main export dialog */
 	virtual void export_audio () = 0;
@@ -250,10 +288,11 @@ class PublicEditor : public Gtkmm2ext::Tabbable {
 	virtual void set_selected_track (TimeAxisView&, Selection::Operation op = Selection::Set, bool no_remove = false) = 0;
 	virtual void set_selected_mixer_strip (TimeAxisView&) = 0;
 	virtual void hide_track_in_display (TimeAxisView* tv, bool apply_to_selection = false) = 0;
+	virtual void show_track_in_display (TimeAxisView* tv, bool move_into_view = false) = 0;
 
-        virtual void set_stationary_playhead (bool yn) = 0;
-        virtual void toggle_stationary_playhead () = 0;
-        virtual bool stationary_playhead() const = 0;
+	virtual void set_stationary_playhead (bool yn) = 0;
+	virtual void toggle_stationary_playhead () = 0;
+	virtual bool stationary_playhead() const = 0;
 
 	/** Set whether the editor should follow the playhead.
 	 * @param yn true to follow playhead, otherwise false.
@@ -273,23 +312,26 @@ class PublicEditor : public Gtkmm2ext::Tabbable {
 	virtual framecnt_t current_page_samples() const = 0;
 	virtual double visible_canvas_height () const = 0;
 	virtual void temporal_zoom_step (bool coarser) = 0;
-        virtual void ensure_time_axis_view_is_visible (TimeAxisView const & tav, bool at_top = false) = 0;
-        virtual void override_visible_track_count () = 0;
+	virtual void ensure_time_axis_view_is_visible (TimeAxisView const & tav, bool at_top = false) = 0;
+	virtual void override_visible_track_count () = 0;
 	virtual void scroll_tracks_down_line () = 0;
 	virtual void scroll_tracks_up_line () = 0;
-        virtual bool scroll_down_one_track (bool skip_child_views = false) = 0;
-        virtual bool scroll_up_one_track (bool skip_child_views = false) = 0;
+	virtual bool scroll_down_one_track (bool skip_child_views = false) = 0;
+	virtual bool scroll_up_one_track (bool skip_child_views = false) = 0;
+	virtual void select_topmost_track () = 0;
 	virtual void prepare_for_cleanup () = 0;
 	virtual void finish_cleanup () = 0;
 	virtual void reset_x_origin (framepos_t frame) = 0;
 	virtual double get_y_origin () const = 0;
 	virtual void reset_y_origin (double pos) = 0;
+	virtual void set_visible_track_count (int32_t) = 0;
+	virtual void fit_selection () = 0;
 	virtual void remove_last_capture () = 0;
 	virtual void maximise_editing_space () = 0;
 	virtual void restore_editing_space () = 0;
 	virtual framepos_t get_preferred_edit_position (Editing::EditIgnoreOption = Editing::EDIT_IGNORE_NONE, bool from_context_menu = false, bool from_outside_canvas = false) = 0;
 	virtual void toggle_meter_updating() = 0;
-	virtual void split_regions_at (framepos_t, RegionSelection&, const int32_t sub_num, bool snap) = 0;
+	virtual void split_regions_at (ARDOUR::MusicFrame, RegionSelection&, bool snap) = 0;
 	virtual void split_region_at_points (boost::shared_ptr<ARDOUR::Region>, ARDOUR::AnalysisFeatureList&, bool can_ferret, bool select_new = false) = 0;
 	virtual void mouse_add_new_marker (framepos_t where, bool is_cd=false) = 0;
 	virtual void foreach_time_axis_view (sigc::slot<void,TimeAxisView&>) = 0;
@@ -311,15 +353,31 @@ class PublicEditor : public Gtkmm2ext::Tabbable {
 	virtual void set_video_timeline_height (const int h) = 0;
 	virtual void embed_audio_from_video (std::string, framepos_t n = 0, bool lock_position_to_video = true) = 0;
 
-	virtual RouteTimeAxisView* get_route_view_by_route_id (const PBD::ID& id) const = 0;
+	virtual bool track_selection_change_without_scroll () const = 0;
+
+	virtual StripableTimeAxisView* get_stripable_time_axis_by_id (const PBD::ID& id) const = 0;
+
+	virtual TimeAxisView* time_axis_view_from_stripable (boost::shared_ptr<ARDOUR::Stripable> s) const = 0;
 
 	virtual void get_equivalent_regions (RegionView* rv, std::vector<RegionView*>&, PBD::PropertyID) const = 0;
+	virtual RegionView* regionview_from_region (boost::shared_ptr<ARDOUR::Region>) const = 0;
+	virtual RouteTimeAxisView* rtav_from_route (boost::shared_ptr<ARDOUR::Route>) const = 0;
 
 	sigc::signal<void> ZoomChanged;
 	sigc::signal<void> Realized;
 	sigc::signal<void,framepos_t> UpdateAllTransportClocks;
 
-        static sigc::signal<void> DropDownKeys;
+	static sigc::signal<void> DropDownKeys;
+
+	struct RegionAction {
+		Glib::RefPtr<Gtk::Action> action;
+		Editing::RegionActionTarget target;
+
+		RegionAction (Glib::RefPtr<Gtk::Action> a, Editing::RegionActionTarget tgt)
+			: action (a), target (tgt) {}
+	};
+
+	std::map<std::string,RegionAction> region_action_map;
 
 	Glib::RefPtr<Gtk::ActionGroup> editor_actions;
 	Glib::RefPtr<Gtk::ActionGroup> editor_menu_actions;
@@ -368,22 +426,20 @@ class PublicEditor : public Gtkmm2ext::Tabbable {
 	virtual ArdourCanvas::ScrollGroup* get_hvscroll_group () const = 0;
 	virtual ArdourCanvas::ScrollGroup* get_cursor_scroll_group () const = 0;
 
-        virtual ArdourCanvas::GtkCanvasViewport* get_track_canvas() const = 0;
+	virtual ArdourCanvas::GtkCanvasViewport* get_track_canvas() const = 0;
 
-	virtual TimeAxisView* axis_view_from_stripable (boost::shared_ptr<ARDOUR::Stripable>) const = 0;
-
-        virtual void set_current_trimmable (boost::shared_ptr<ARDOUR::Trimmable>) = 0;
-        virtual void set_current_movable (boost::shared_ptr<ARDOUR::Movable>) = 0;
+	virtual void set_current_trimmable (boost::shared_ptr<ARDOUR::Trimmable>) = 0;
+	virtual void set_current_movable (boost::shared_ptr<ARDOUR::Movable>) = 0;
 
 	virtual void center_screen (framepos_t) = 0;
 
 	virtual TrackViewList axis_views_from_routes (boost::shared_ptr<ARDOUR::RouteList>) const = 0;
-	virtual TrackViewList const & get_track_views () = 0;
+	virtual TrackViewList const & get_track_views () const = 0;
 
 	virtual DragManager* drags () const = 0;
-        virtual void maybe_autoscroll (bool, bool, bool from_headers) = 0;
+	virtual void maybe_autoscroll (bool, bool, bool from_headers) = 0;
 	virtual void stop_canvas_autoscroll () = 0;
-        virtual bool autoscroll_active() const = 0;
+	virtual bool autoscroll_active() const = 0;
 
 	virtual void begin_reversible_selection_op (std::string cmd_name) = 0;
 	virtual void commit_reversible_selection_op () = 0;
@@ -392,7 +448,8 @@ class PublicEditor : public Gtkmm2ext::Tabbable {
 	virtual void abort_reversible_command () = 0;
 	virtual void commit_reversible_command () = 0;
 
-	virtual void access_action (std::string,std::string) = 0;
+	virtual void access_action (const std::string&, const std::string&) = 0;
+	virtual void set_toggleaction (const std::string&, const std::string&, bool) = 0;
 
 	virtual MouseCursors const * cursors () const = 0;
 	virtual VerboseCursor * verbose_cursor () const = 0;
@@ -404,10 +461,10 @@ class PublicEditor : public Gtkmm2ext::Tabbable {
 	virtual ARDOUR::Location* find_location_from_marker (ArdourMarker *, bool &) const = 0;
 	virtual ArdourMarker* find_marker_from_location_id (PBD::ID const &, bool) const = 0;
 
-	virtual void snap_to_with_modifier (framepos_t &      first,
-	                                    GdkEvent const *  ev,
-	                                    ARDOUR::RoundMode direction = ARDOUR::RoundNearest,
-	                                    bool              for_mark  = false) = 0;
+	virtual void snap_to_with_modifier (ARDOUR::MusicFrame& first,
+	                                    GdkEvent const *    ev,
+	                                    ARDOUR::RoundMode   direction = ARDOUR::RoundNearest,
+	                                    bool                for_mark  = false) = 0;
 
 	virtual void get_regions_at (RegionSelection &, framepos_t where, TrackViewList const &) const = 0;
 	virtual void get_regions_after (RegionSelection&, framepos_t where, const TrackViewList& ts) const = 0;
@@ -432,7 +489,7 @@ class PublicEditor : public Gtkmm2ext::Tabbable {
 
 	Gtkmm2ext::Bindings* bindings;
 
-  protected:
+protected:
 	friend class DisplaySuspender;
 	virtual void suspend_route_redisplay () = 0;
 	virtual void resume_route_redisplay () = 0;
