@@ -28,13 +28,13 @@
 #include "ardour/profile.h"
 #include "ardour/session.h"
 
+#include "gtkmm2ext/colors.h"
+
 #include "canvas/polygon.h"
 #include "canvas/debug.h"
 #include "canvas/pixbuf.h"
 #include "canvas/text.h"
 #include "canvas/line.h"
-#include "canvas/utils.h"
-#include "canvas/colors.h"
 
 #include "streamview.h"
 #include "region_view.h"
@@ -203,7 +203,7 @@ RegionView::~RegionView ()
 		delete *i;
 	}
 
-        drop_silent_frames ();
+	drop_silent_frames ();
 
 	delete editor;
 }
@@ -220,16 +220,16 @@ RegionView::canvas_group_event (GdkEvent* event)
 void
 RegionView::set_silent_frames (const AudioIntervalResult& silences, double /*threshold*/)
 {
-        framecnt_t shortest = max_framecnt;
+	framecnt_t shortest = max_framecnt;
 
 	/* remove old silent frames */
-        drop_silent_frames ();
+	drop_silent_frames ();
 
-        if (silences.empty()) {
-                return;
-        }
+	if (silences.empty()) {
+		return;
+	}
 
-        uint32_t const color = UIConfiguration::instance().color_mod ("silence", "silence");
+	uint32_t const color = UIConfiguration::instance().color_mod ("silence", "silence");
 
 	for (AudioIntervalResult::const_iterator i = silences.begin(); i != silences.end(); ++i) {
 
@@ -250,7 +250,7 @@ RegionView::set_silent_frames (const AudioIntervalResult& silences, double /*thr
 	}
 
 	/* Find shortest audible segment */
-        framecnt_t shortest_audible = max_framecnt;
+	framecnt_t shortest_audible = max_framecnt;
 
 	framecnt_t s = _region->start();
 	for (AudioIntervalResult::const_iterator i = silences.begin(); i != silences.end(); ++i) {
@@ -267,50 +267,50 @@ RegionView::set_silent_frames (const AudioIntervalResult& silences, double /*thr
 		shortest_audible = min (shortest_audible, dur);
 	}
 
-        _silence_text = new ArdourCanvas::Text (group);
+	_silence_text = new ArdourCanvas::Text (group);
 	_silence_text->set_ignore_events (true);
-        _silence_text->set_font_description (get_font_for_style (N_("SilenceText")));
-        _silence_text->set_color (UIConfiguration::instance().color ("silence text"));
+	_silence_text->set_font_description (get_font_for_style (N_("SilenceText")));
+	_silence_text->set_color (UIConfiguration::instance().color ("silence text"));
 
-        /* both positions are relative to the region start offset in source */
+	/* both positions are relative to the region start offset in source */
 
-        _silence_text->set_x_position (trackview.editor().sample_to_pixel (silences.front().first - _region->start()) + 10.0);
-        _silence_text->set_y_position (20.0);
+	_silence_text->set_x_position (trackview.editor().sample_to_pixel (silences.front().first - _region->start()) + 10.0);
+	_silence_text->set_y_position (20.0);
 
-        double ms = (float) shortest/_region->session().frame_rate();
+	double ms = (float) shortest/_region->session().frame_rate();
 
-        /* ms are now in seconds */
+	/* ms are now in seconds */
 
-        char const * sunits;
+	char const * sunits;
 
-        if (ms >= 60.0) {
-                sunits = _("minutes");
-                ms /= 60.0;
-        } else if (ms < 1.0) {
-                sunits = _("msecs");
-                ms *= 1000.0;
-        } else {
-                sunits = _("secs");
-        }
+	if (ms >= 60.0) {
+		sunits = _("minutes");
+		ms /= 60.0;
+	} else if (ms < 1.0) {
+		sunits = _("msecs");
+		ms *= 1000.0;
+	} else {
+		sunits = _("secs");
+	}
 
 	string text = string_compose (ngettext ("%1 silent segment", "%1 silent segments", silences.size()), silences.size())
 		+ ", "
 		+ string_compose (_("shortest = %1 %2"), ms, sunits);
 
-        if (shortest_audible != max_framepos) {
-                /* ms are now in seconds */
-                double ma = (float) shortest_audible / _region->session().frame_rate();
-                char const * aunits;
+	if (shortest_audible != max_framepos) {
+		/* ms are now in seconds */
+		double ma = (float) shortest_audible / _region->session().frame_rate();
+		char const * aunits;
 
-                if (ma >= 60.0) {
-                        aunits = _("minutes");
-                        ma /= 60.0;
-                } else if (ma < 1.0) {
-                        aunits = _("msecs");
-                        ma *= 1000.0;
-                } else {
-                        aunits = _("secs");
-                }
+		if (ma >= 60.0) {
+			aunits = _("minutes");
+			ma /= 60.0;
+		} else if (ma < 1.0) {
+			aunits = _("msecs");
+			ma *= 1000.0;
+		} else {
+			aunits = _("secs");
+		}
 
 		text += string_compose (_("\n  (shortest audible segment = %1 %2)"), ma, aunits);
 	}
@@ -322,9 +322,9 @@ void
 RegionView::hide_silent_frames ()
 {
 	for (list<ArdourCanvas::Rectangle*>::iterator i = _silent_frames.begin (); i != _silent_frames.end (); ++i) {
-                (*i)->hide ();
+		(*i)->hide ();
 	}
-        _silence_text->hide();
+	_silence_text->hide();
 }
 
 void
@@ -333,10 +333,10 @@ RegionView::drop_silent_frames ()
 	for (list<ArdourCanvas::Rectangle*>::iterator i = _silent_frames.begin (); i != _silent_frames.end (); ++i) {
 		delete *i;
 	}
-        _silent_frames.clear ();
+	_silent_frames.clear ();
 
-        delete _silence_text;
-        _silence_text = 0;
+	delete _silence_text;
+	_silence_text = 0;
 }
 
 gint
@@ -377,6 +377,9 @@ RegionView::region_changed (const PropertyChange& what_changed)
 	if (what_changed.contains (ARDOUR::Properties::name)) {
 		region_renamed ();
 	}
+	if (what_changed.contains (ARDOUR::Properties::position_lock_style)) {
+		region_renamed ();
+	}
 	if (what_changed.contains (ARDOUR::Properties::sync_position)) {
 		region_sync_changed ();
 	}
@@ -512,7 +515,7 @@ void
 RegionView::set_sync_mark_color ()
 {
 	if (sync_mark) {
-		ArdourCanvas::Color c = UIConfiguration::instance().color ("sync mark");
+		Gtkmm2ext::Color c = UIConfiguration::instance().color ("sync mark");
 		sync_mark->set_fill_color (c);
 		sync_mark->set_outline_color (c);
 		sync_line->set_outline_color (c);
@@ -522,7 +525,7 @@ RegionView::set_sync_mark_color ()
 uint32_t
 RegionView::get_fill_color () const
 {
-	ArdourCanvas::Color f = TimeAxisViewItem::get_fill_color();
+	Gtkmm2ext::Color f = TimeAxisViewItem::get_fill_color();
 	char const *modname;
 
 	if (_region->opaque() && (!ARDOUR::Profile->get_mixbus() || (!_dragging && !_region->muted ()))) {
@@ -531,7 +534,7 @@ RegionView::get_fill_color () const
 		modname = "transparent region base";
 	}
 
-	return HSV(f).mod (UIConfiguration::instance().modifier (modname)).color ();
+	return Gtkmm2ext::HSV(f).mod (UIConfiguration::instance().modifier (modname)).color ();
 }
 
 void
@@ -559,21 +562,22 @@ RegionView::make_name () const
 	std::string str;
 
 	// XXX nice to have some good icons for this
+	if (_region->position_lock_style() == MusicTime) {
+		str += "\u266B"; // BEAMED EIGHTH NOTES
+	}
 
 	if (_region->locked()) {
-		str += '>';
+		str += "\u2629"; // CROSS OF JERUSALEM
 		str += _region->name();
-		str += '<';
 	} else if (_region->position_locked()) {
-		str += '{';
+		str += "\u21B9"; // LEFTWARDS ARROW TO BAR OVER RIGHTWARDS ARROW TO BAR
 		str += _region->name();
-		str += '}';
 	} else if (_region->video_locked()) {
 		str += '[';
 		str += _region->name();
 		str += ']';
 	} else {
-		str = _region->name();
+		str += _region->name();
 	}
 
 	if (_region->muted()) {
@@ -943,24 +947,23 @@ RegionView::move_contents (frameoffset_t distance)
  *  Used when inverting snap mode logic with key modifiers, or snap distance calculation.
  *  @return Snapped frame offset from this region's position.
  */
-frameoffset_t
+MusicFrame
 RegionView::snap_frame_to_frame (frameoffset_t x, bool ensure_snap) const
 {
 	PublicEditor& editor = trackview.editor();
-
 	/* x is region relative, convert it to global absolute frames */
 	framepos_t const session_frame = x + _region->position();
 
 	/* try a snap in either direction */
-	framepos_t frame = session_frame;
+	MusicFrame frame (session_frame, 0);
 	editor.snap_to (frame, RoundNearest, false, ensure_snap);
 
 	/* if we went off the beginning of the region, snap forwards */
-	if (frame < _region->position ()) {
-		frame = session_frame;
+	if (frame.frame < _region->position ()) {
+		frame.frame = session_frame;
 		editor.snap_to (frame, RoundUpAlways, false, ensure_snap);
 	}
 
-	/* back to region relative */
-	return frame - _region->position();
+	/* back to region relative, keeping the relevant divisor */
+	return MusicFrame (frame.frame - _region->position(), frame.division);
 }
