@@ -46,6 +46,7 @@
 #include "ardour/event_type_map.h"
 #include "ardour/pannable.h"
 #include "ardour/panner.h"
+#include "ardour/plugin_insert.h"
 #include "ardour/processor.h"
 #include "ardour/profile.h"
 #include "ardour/route_group.h"
@@ -73,6 +74,7 @@
 #include "item_counts.h"
 #include "keyboard.h"
 #include "paste_context.h"
+#include "patch_change_widget.h"
 #include "playlist_selector.h"
 #include "point_selection.h"
 #include "public_editor.h"
@@ -788,57 +790,6 @@ RouteTimeAxisView::build_display_menu ()
 			/* show nothing */
 		}
 
-#ifdef XXX_OLD_DESTRUCTIVE_API_XXX
-		Menu* mode_menu = manage (new Menu);
-		MenuList& mode_items = mode_menu->items ();
-		mode_menu->set_name ("ArdourContextMenu");
-
-		RadioMenuItem::Group mode_group;
-
-		int normal = 0;
-		int tape = 0;
-		int non_layered = 0;
-
-		for (TrackSelection::const_iterator t = s.begin(); t != s.end(); ++t) {
-			RouteTimeAxisView* r = dynamic_cast<RouteTimeAxisView*> (*t);
-			if (!r || !r->is_track ()) {
-				continue;
-			}
-
-			switch (r->track()->mode()) {
-			case Normal:
-				++normal;
-				break;
-			case Destructive:
-				++tape;
-				break;
-			case NonLayered:
-				++non_layered;
-				break;
-			}
-		}
-
-		mode_items.push_back (RadioMenuElem (mode_group, _("Normal Mode")));
-		i = dynamic_cast<RadioMenuItem*> (&mode_items.back ());
-		i->signal_activate().connect (sigc::bind (sigc::mem_fun (*this, &RouteTimeAxisView::set_track_mode), ARDOUR::Normal, true));
-		i->set_active (normal != 0 && tape == 0 && non_layered == 0);
-		i->set_inconsistent (normal != 0 && (tape != 0 || non_layered != 0));
-
-		mode_items.push_back (RadioMenuElem (mode_group, _("Tape Mode")));
-		i = dynamic_cast<RadioMenuItem*> (&mode_items.back ());
-		i->signal_activate().connect (sigc::bind (sigc::mem_fun (*this, &RouteTimeAxisView::set_track_mode), ARDOUR::Destructive, true));
-		i->set_active (normal == 0 && tape != 0 && non_layered == 0);
-		i->set_inconsistent (tape != 0 && (normal != 0 || non_layered != 0));
-
-		mode_items.push_back (RadioMenuElem (mode_group, _("Non-Layered Mode")));
-		i = dynamic_cast<RadioMenuItem*> (&mode_items.back ());
-		i->signal_activate().connect (sigc::bind (sigc::mem_fun (*this, &RouteTimeAxisView::set_track_mode), ARDOUR::NonLayered, true));
-		i->set_active (normal == 0 && tape == 0 && non_layered != 0);
-		i->set_inconsistent (non_layered != 0 && (normal != 0 || tape != 0));
-
-		items.push_back (MenuElem (_("Record Mode"), *mode_menu));
-#endif
-
 		items.push_back (SeparatorElem());
 
 		build_playlist_menu ();
@@ -846,6 +797,13 @@ RouteTimeAxisView::build_display_menu ()
 		items.back().set_sensitive (_editor.get_selection().tracks.size() <= 1);
 	}
 
+	if (!is_midi_track () && _route->the_instrument ()) {
+		/* MIDI Bus */
+		items.push_back (MenuElem (_("Patch Selector..."),
+					sigc::mem_fun(*this, &RouteUI::select_midi_patch)));
+		items.push_back (SeparatorElem());
+	}
+
 	route_group_menu->detach ();
 
 	WeakRouteList r;
@@ -908,32 +866,6 @@ RouteTimeAxisView::build_display_menu ()
 	items.push_back (MenuElem (_("Remove"), sigc::mem_fun(_editor, &PublicEditor::remove_tracks)));
 }
 
-#ifdef XXX_OLD_DESTRUCTIVE_API_XXX
-void
-RouteTimeAxisView::set_track_mode (TrackMode mode, bool apply_to_selection)
-{
-	if (apply_to_selection) {
-		_editor.get_selection().tracks.foreach_route_time_axis (boost::bind (&RouteTimeAxisView::set_track_mode, _1, mode, false));
-	} else {
-
-		bool needs_bounce = false;
-
-		if (!track()->can_use_mode (mode, needs_bounce)) {
-
-			if (!needs_bounce) {
-				/* cannot be done */
-				return;
-			} else {
-				cerr << "would bounce this one\n";
-				return;
-			}
-		}
-
-		track()->set_mode (mode);
-	}
-}
-#endif
-
 void
 RouteTimeAxisView::show_timestretch (framepos_t start, framepos_t end, int layers, int layer)
 {
@@ -2266,9 +2198,43 @@ RouteTimeAxisView::processor_menu_item_toggled (RouteTimeAxisView::ProcessorAuto
 	}
 }
 
+void
+RouteTimeAxisView::reread_midnam ()
+{
+	boost::shared_ptr<PluginInsert> pi = boost::dynamic_pointer_cast<PluginInsert> (_route->the_instrument ());
+	assert (pi);
+	bool rv = pi->plugin ()->read_midnam();
+
+	if (rv && patch_change_dialog ()) {
+		patch_change_dialog ()->refresh ();
+	}
+}
+
+void
+RouteTimeAxisView::drop_instrument_ref ()
+{
+	midnam_connection.drop_connections ();
+}
+
 void
 RouteTimeAxisView::processors_changed (RouteProcessorChange c)
 {
+	if (_route) {
+		boost::shared_ptr<Processor> the_instrument (_route->the_instrument());
+		boost::shared_ptr<PluginInsert> pi = boost::dynamic_pointer_cast<PluginInsert> (the_instrument);
+		if (pi && pi->plugin ()->has_midnam ()) {
+			midnam_connection.drop_connections ();
+			the_instrument->DropReferences.connect (midnam_connection, invalidator (*this),
+					boost::bind (&RouteTimeAxisView::drop_instrument_ref, this),
+					gui_context());
+			pi->plugin()->UpdateMidnam.connect (midnam_connection, invalidator (*this),
+					boost::bind (&RouteTimeAxisView::reread_midnam, this),
+					gui_context());
+
+			reread_midnam ();
+		}
+	}
+
 	if (c.type == RouteProcessorChange::MeterPointChange) {
 		/* nothing to do if only the meter point has changed */
 		return;
