@@ -176,7 +176,7 @@ protected:
 
 	bool route_group_click  (GdkEventButton *);
 
-	void processors_changed (ARDOUR::RouteProcessorChange);
+	virtual void processors_changed (ARDOUR::RouteProcessorChange);
 
 	virtual void add_processor_to_subplugin_menu (boost::weak_ptr<ARDOUR::Processor>);
 	void remove_processor_automation_node (ProcessorAutomationNode* pan);
@@ -264,10 +264,6 @@ protected:
 
 	ArdourCanvas::Rectangle* timestretch_rect;
 
-#ifdef XXX_OLD_DESTRUCTIVE_API_XXX
-	void set_track_mode (ARDOUR::TrackMode, bool apply_to_selection = false);
-#endif
-
 	/** Information about all automatable processor parameters that apply to
 	 *  this route.  The Amp processor is not included in this list.
 	 */
@@ -310,6 +306,10 @@ private:
 	void update_playlist_tip ();
 	void parameter_changed (std::string const & p);
 	void update_track_number_visibility();
+
+	void drop_instrument_ref ();
+	void reread_midnam ();
+	PBD::ScopedConnectionList midnam_connection;
 };
 
 #endif /* __ardour_route_time_axis_h__ */
