@@ -63,6 +63,8 @@ namespace ArdourWidgets {
 class ArdourWindow;
 class IOSelectorWindow;
 class ControlSlaveUI;
+class PatchChangeGridDialog;
+class SaveTemplateDialog;
 
 class RoutePinWindowProxy : public WM::ProxyBase
 {
@@ -112,6 +114,8 @@ public:
 	Gdk::Color route_color () const;
 	void choose_color ();
 
+	void select_midi_patch ();
+
 	bool ignore_toggle;
 	bool wait_for_release;
 	bool multiple_mute_change;
@@ -243,7 +247,7 @@ public:
 	virtual void map_frozen ();
 
 	void adjust_latency ();
-	bool process_save_template_prompter (ArdourWidgets::Prompter& prompter, const std::string& dir);
+	void save_as_template_dialog_response (int response, SaveTemplateDialog* d);
 	void save_as_template ();
 
 	static Gtkmm2ext::ActiveState solo_active_state (boost::shared_ptr<ARDOUR::Stripable>);
@@ -298,10 +302,13 @@ protected:
 
 	bool mark_hidden (bool yn);
 
+	PatchChangeGridDialog* patch_change_dialog () const;
+
 private:
 	void parameter_changed (std::string const&);
 	void relabel_solo_button ();
 	void track_mode_changed ();
+	void delete_patch_change_dialog ();
 
 	std::string route_state_id () const;
 
