@@ -40,15 +40,18 @@ TempoDialog::TempoDialog (TempoMap& map, framepos_t frame, const string&)
 	, _section (0)
 	, bpm_adjustment (60.0, 1.0, 999.9, 0.1, 1.0)
 	, bpm_spinner (bpm_adjustment)
-	, when_bar_label (_("bar:"), ALIGN_LEFT, ALIGN_CENTER)
-	, when_beat_label (_("beat:"), ALIGN_LEFT, ALIGN_CENTER)
-	, pulse_selector_label (_("Pulse note"), ALIGN_LEFT, ALIGN_CENTER)
+	, end_bpm_adjustment (60.0, 1.0, 999.9, 0.1, 1.0)
+	, end_bpm_spinner (end_bpm_adjustment)
+	, _end_bpm_label (_("End Beats per Minute:"), ALIGN_LEFT, ALIGN_CENTER)
+	, when_bar_label (_("bar:"), ALIGN_RIGHT, ALIGN_CENTER)
+	, when_beat_label (_("beat:"), ALIGN_RIGHT, ALIGN_CENTER)
+	, pulse_selector_label (_("Pulse:"), ALIGN_LEFT, ALIGN_CENTER)
 	, tap_tempo_button (_("Tap tempo"))
 {
 	Tempo tempo (map.tempo_at_frame (frame));
 	Timecode::BBT_Time when (map.bbt_at_frame (frame));
 
-	init (when, tempo.note_types_per_minute(), tempo.note_type(), TempoSection::Constant, true, MusicTime);
+	init (when, tempo.note_types_per_minute(), tempo.end_note_types_per_minute(), tempo.note_type(), TempoSection::Constant, true, MusicTime);
 }
 
 TempoDialog::TempoDialog (TempoMap& map, TempoSection& section, const string&)
@@ -57,27 +60,39 @@ TempoDialog::TempoDialog (TempoMap& map, TempoSection& section, const string&)
 	, _section (&section)
 	, bpm_adjustment (60.0, 1.0, 999.9, 0.1, 1.0)
 	, bpm_spinner (bpm_adjustment)
-	, when_bar_label (_("bar:"), ALIGN_LEFT, ALIGN_CENTER)
-	, when_beat_label (_("beat:"), ALIGN_LEFT, ALIGN_CENTER)
-	, pulse_selector_label (_("Pulse note"), ALIGN_LEFT, ALIGN_CENTER)
+	, end_bpm_adjustment (60.0, 1.0, 999.9, 0.1, 1.0)
+	, end_bpm_spinner (end_bpm_adjustment)
+	, _end_bpm_label (_("End Beats per Minute:"), ALIGN_LEFT, ALIGN_CENTER)
+	, when_bar_label (_("bar:"), ALIGN_RIGHT, ALIGN_CENTER)
+	, when_beat_label (_("beat:"), ALIGN_RIGHT, ALIGN_CENTER)
+	, pulse_selector_label (_("Pulse:"), ALIGN_LEFT, ALIGN_CENTER)
 	, tap_tempo_button (_("Tap tempo"))
 {
 	Timecode::BBT_Time when (map.bbt_at_frame (section.frame()));
-	init (when, section.note_types_per_minute(), section.note_type(), section.type(), section.movable(), section.position_lock_style());
+	init (when, section.note_types_per_minute(), section.end_note_types_per_minute(), section.note_type(), section.type()
+	      , section.initial() || section.locked_to_meter(), section.position_lock_style());
 }
 
 void
-TempoDialog::init (const Timecode::BBT_Time& when, double bpm, double note_type, TempoSection::Type type, bool movable, PositionLockStyle style)
+TempoDialog::init (const Timecode::BBT_Time& when, double bpm, double end_bpm, double note_type, TempoSection::Type type, bool initial, PositionLockStyle style)
 {
 	vector<string> strings;
 	NoteTypes::iterator x;
 
 	bpm_spinner.set_numeric (true);
-	bpm_spinner.set_digits (2);
+	bpm_spinner.set_digits (3);
 	bpm_spinner.set_wrap (true);
 	bpm_spinner.set_value (bpm);
 	bpm_spinner.set_alignment (1.0);
 
+	end_bpm_spinner.set_numeric (true);
+	end_bpm_spinner.set_digits (3);
+	end_bpm_spinner.set_wrap (true);
+	end_bpm_spinner.set_value (end_bpm);
+	end_bpm_spinner.set_alignment (1.0);
+
+	Gtkmm2ext::set_size_request_to_display_given_text (pulse_selector, _("one-hundred-twenty-eighth"), 3, 6);
+
 	note_types.insert (make_pair (_("whole"), 1.0));
 	strings.push_back (_("whole"));
 	note_types.insert (make_pair (_("second"), 2.0));
@@ -157,19 +172,30 @@ TempoDialog::init (const Timecode::BBT_Time& when, double bpm, double note_type,
 	table->set_spacings (6);
 	table->set_homogeneous (false);
 
-	int row;
-	Label* bpm_label = manage (new Label(_("Beats per minute:"), ALIGN_LEFT, ALIGN_CENTER));
-	table->attach (*bpm_label, 0, 1, 0, 1);
-	table->attach (bpm_spinner, 1, 5, 0, 1);
+	int row = 0;
 
 	if (UIConfiguration::instance().get_allow_non_quarter_pulse()) {
-		table->attach (pulse_selector_label, 0, 1, 1, 2);
-		table->attach (pulse_selector, 1, 5, 1, 2);
-		row = 2;
-	} else {
-		row = 1;
+		table->attach (pulse_selector_label, 0, 1, row, row + 1);
+		table->attach (pulse_selector, 1, 5, row, row + 1);
+
+		++row;
 	}
 
+	Label* bpm_label = manage (new Label(_("Start Beats per Minute:"), ALIGN_LEFT, ALIGN_CENTER));
+	table->attach (*bpm_label, 0, 1, row, row + 1);
+	table->attach (bpm_spinner, 1, 5, row, row + 1);
+	++row;
+
+	table->attach (_end_bpm_label, 0, 1, row, row + 1);
+	table->attach (end_bpm_spinner, 1, 5, row, row + 1);
+	++row;
+
+	Label* tempo_type_label = manage (new Label(_("Tempo Type:"), ALIGN_LEFT, ALIGN_CENTER));
+	table->attach (*tempo_type_label, 0, 1, row, row + 1);
+	table->attach (tempo_type, 1, 5, row, row + 1);
+
+	++row;
+
 	char buf[64];
 
 	snprintf (buf, sizeof (buf), "%" PRIu32, when.bars);
@@ -177,7 +203,7 @@ TempoDialog::init (const Timecode::BBT_Time& when, double bpm, double note_type,
 	snprintf (buf, sizeof (buf), "%" PRIu32, when.beats);
 	when_beat_entry.set_text (buf);
 
-	if (movable) {
+	if (!initial) {
 		when_bar_entry.set_width_chars(4);
 		when_beat_entry.set_width_chars (4);
 		when_bar_entry.set_alignment (1.0);
@@ -205,13 +231,6 @@ TempoDialog::init (const Timecode::BBT_Time& when, double bpm, double note_type,
 		--row;
 	}
 
-
-	Label* tempo_type_label = manage (new Label(_("Tempo Type:"), ALIGN_LEFT, ALIGN_CENTER));
-	table->attach (*tempo_type_label, 0, 1, row, row + 1);
-	table->attach (tempo_type, 1, 5, row, row + 1);
-
-	++row;
-
 	get_vbox()->set_border_width (12);
 	get_vbox()->pack_end (*table);
 
@@ -223,9 +242,12 @@ TempoDialog::init (const Timecode::BBT_Time& when, double bpm, double note_type,
 	set_default_response (RESPONSE_ACCEPT);
 
 	bpm_spinner.show ();
+	end_bpm_spinner.show ();
 	tap_tempo_button.show ();
+	get_vbox()->set_spacing (6);
 	get_vbox()->pack_end (tap_tempo_button);
-	bpm_spinner.grab_focus ();
+	tap_tempo_button.can_focus ();
+	tap_tempo_button.grab_focus ();
 
 	set_name ("MetricDialog");
 
@@ -233,6 +255,7 @@ TempoDialog::init (const Timecode::BBT_Time& when, double bpm, double note_type,
 	bpm_spinner.signal_button_press_event().connect (sigc::mem_fun (*this, &TempoDialog::bpm_button_press), false);
 	bpm_spinner.signal_button_release_event().connect (sigc::mem_fun (*this, &TempoDialog::bpm_button_release), false);
 	bpm_spinner.signal_changed().connect (sigc::mem_fun (*this, &TempoDialog::bpm_changed));
+	end_bpm_spinner.signal_changed().connect (sigc::mem_fun (*this, &TempoDialog::bpm_changed));
 	when_bar_entry.signal_activate().connect (sigc::bind (sigc::mem_fun (*this, &TempoDialog::response), RESPONSE_ACCEPT));
 	when_bar_entry.signal_key_release_event().connect (sigc::mem_fun (*this, &TempoDialog::entry_key_release), false);
 	when_beat_entry.signal_activate().connect (sigc::bind (sigc::mem_fun (*this, &TempoDialog::response), RESPONSE_ACCEPT));
@@ -241,8 +264,11 @@ TempoDialog::init (const Timecode::BBT_Time& when, double bpm, double note_type,
 	tempo_type.signal_changed().connect (sigc::mem_fun (*this, &TempoDialog::tempo_type_change));
 	lock_style.signal_changed().connect (sigc::mem_fun (*this, &TempoDialog::lock_style_change));
 	tap_tempo_button.signal_button_press_event().connect (sigc::mem_fun (*this, &TempoDialog::tap_tempo_button_press), false);
+	tap_tempo_button.signal_key_press_event().connect (sigc::mem_fun (*this, &TempoDialog::tap_tempo_key_press), false);
 	tap_tempo_button.signal_focus_out_event().connect (sigc::mem_fun (*this, &TempoDialog::tap_tempo_focus_out));
 
+	tempo_type_change();
+
 	tapped = false;
 }
 
@@ -296,6 +322,16 @@ TempoDialog::get_bpm ()
 	return bpm_spinner.get_value ();
 }
 
+double
+TempoDialog::get_end_bpm ()
+{
+	if (get_tempo_type() == TempoSection::Constant) {
+		return bpm_spinner.get_value ();
+	}
+
+	return end_bpm_spinner.get_value ();
+}
+
 bool
 TempoDialog::get_bbt_time (Timecode::BBT_Time& requested)
 {
@@ -360,6 +396,14 @@ TempoDialog::pulse_change ()
 void
 TempoDialog::tempo_type_change ()
 {
+	if (get_tempo_type() == TempoSection::Constant) {
+		end_bpm_spinner.hide ();
+		_end_bpm_label.hide();
+	} else {
+		end_bpm_spinner.show ();
+		_end_bpm_label.show();
+	}
+
 	set_response_sensitive (RESPONSE_ACCEPT, is_user_input_valid());
 }
 
@@ -370,7 +414,27 @@ TempoDialog::lock_style_change ()
 }
 
 bool
-TempoDialog::tap_tempo_button_press (GdkEventButton *ev)
+TempoDialog::tap_tempo_key_press (GdkEventKey*)
+{
+	tap_tempo ();
+	return false;
+}
+
+bool
+TempoDialog::tap_tempo_button_press (GdkEventButton* ev)
+{
+	if (ev->type == GDK_2BUTTON_PRESS || ev->type == GDK_3BUTTON_PRESS) {
+		return true;
+	}
+	if (ev->button != 1) {
+		return true;
+	}
+	tap_tempo ();
+	return false; // grab focus
+}
+
+void
+TempoDialog::tap_tempo ()
 {
 	double t;
 
@@ -404,8 +468,6 @@ TempoDialog::tap_tempo_button_press (GdkEventButton *ev)
 	}
 	tap_count++;
 	last_t = t;
-
-	return true;
 }
 
 bool
@@ -418,11 +480,11 @@ TempoDialog::tap_tempo_focus_out (GdkEventFocus* )
 MeterDialog::MeterDialog (TempoMap& map, framepos_t frame, const string&)
 	: ArdourDialog (_("New Meter"))
 {
-	frame = map.round_to_bar(frame, RoundNearest);
+	frame = map.round_to_bar(frame, RoundNearest).frame;
 	Timecode::BBT_Time when (map.bbt_at_frame (frame));
 	Meter meter (map.meter_at_frame (frame));
 
-	init (when, meter.divisions_per_bar(), meter.note_divisor(), true, MusicTime);
+	init (when, meter.divisions_per_bar(), meter.note_divisor(), false, MusicTime);
 }
 
 MeterDialog::MeterDialog (TempoMap& map, MeterSection& section, const string&)
@@ -430,11 +492,11 @@ MeterDialog::MeterDialog (TempoMap& map, MeterSection& section, const string&)
 {
 	Timecode::BBT_Time when (map.bbt_at_frame (section.frame()));
 
-	init (when, section.divisions_per_bar(), section.note_divisor(), section.movable(), section.position_lock_style());
+	init (when, section.divisions_per_bar(), section.note_divisor(), section.initial(), section.position_lock_style());
 }
 
 void
-MeterDialog::init (const Timecode::BBT_Time& when, double bpb, double divisor, bool movable, PositionLockStyle style)
+MeterDialog::init (const Timecode::BBT_Time& when, double bpb, double divisor, bool initial, PositionLockStyle style)
 {
 	char buf[64];
 	vector<string> strings;
@@ -495,9 +557,9 @@ MeterDialog::init (const Timecode::BBT_Time& when, double bpb, double divisor, b
 		lock_style.set_active_text (strings[0]); // "music"
 	}
 
-	Label* note_label = manage (new Label (_("Note value:"), ALIGN_LEFT, ALIGN_CENTER));
-	Label* lock_label = manage (new Label (_("Lock style:"), ALIGN_LEFT, ALIGN_CENTER));
-	Label* bpb_label = manage (new Label (_("Beats per bar:"), ALIGN_LEFT, ALIGN_CENTER));
+	Label* note_label = manage (new Label (_("Note value:"), ALIGN_RIGHT, ALIGN_CENTER));
+	Label* lock_label = manage (new Label (_("Lock style:"), ALIGN_RIGHT, ALIGN_CENTER));
+	Label* bpb_label = manage (new Label (_("Beats per bar:"), ALIGN_RIGHT, ALIGN_CENTER));
 	Table* table = manage (new Table (3, 3));
 	table->set_spacings (6);
 
@@ -510,7 +572,7 @@ MeterDialog::init (const Timecode::BBT_Time& when, double bpb, double divisor, b
 	when_bar_entry.set_text (buf);
 	when_bar_entry.set_alignment (1.0);
 
-	if (movable) {
+	if (!initial) {
 		Label* when_label = manage (new Label(_("Meter begins at bar:"), ALIGN_LEFT, ALIGN_CENTER));
 
 		table->attach (*when_label, 0, 1, 2, 3, FILL | EXPAND, FILL | EXPAND);
@@ -608,13 +670,13 @@ MeterDialog::entry_key_release (GdkEventKey*)
 void
 MeterDialog::note_type_change ()
 {
-        set_response_sensitive (RESPONSE_ACCEPT, is_user_input_valid());
+	set_response_sensitive (RESPONSE_ACCEPT, is_user_input_valid());
 }
 
 void
 MeterDialog::lock_style_change ()
 {
-        set_response_sensitive (RESPONSE_ACCEPT, is_user_input_valid());
+	set_response_sensitive (RESPONSE_ACCEPT, is_user_input_valid());
 }
 
 double
