@@ -25,6 +25,7 @@
 #include "track_selection.h"
 #include "time_axis_view.h"
 #include "public_editor.h"
+#include "vca_time_axis.h"
 
 using namespace std;
 
@@ -38,25 +39,3 @@ TrackSelection::~TrackSelection ()
 {
 }
 
-TrackViewList
-TrackSelection::add (TrackViewList const & t)
-{
-	TrackViewList added = TrackViewList::add (t);
-
-	for (TrackSelection::const_iterator i = t.begin(); i != t.end(); ++i) {
-
-		/* select anything in the same select-enabled route group */
-		ARDOUR::RouteGroup* rg = (*i)->route_group ();
-		if (rg && rg->is_active() && rg->is_select ()) {
-			TrackViewList tr = _editor->axis_views_from_routes (rg->route_list ());
-			for (TrackViewList::iterator j = tr.begin(); j != tr.end(); ++j) {
-				if (!contains (*j)) {
-					added.push_back (*j);
-					push_back (*j);
-				}
-			}
-		}
-	}
-
-	return added;
-}
