@@ -430,8 +430,8 @@ VideoMonitor::save_session ()
 
 	for(XJSettings::const_iterator it = xjadeo_settings.begin(); it != xjadeo_settings.end(); ++it) {
 	  XMLNode* child = node->add_child (X_("XJSetting"));
-		child->add_property (X_("k"), it->first);
-		child->add_property (X_("v"), it->second);
+		child->set_property (X_("k"), it->first);
+		child->set_property (X_("v"), it->second);
 	}
 }
 
@@ -441,6 +441,7 @@ VideoMonitor::set_session (ARDOUR::Session *s)
 {
 	SessionHandlePtr::set_session (s);
 	if (!_session) { return; }
+	ARDOUR::Config->ParameterChanged.connect (*this, invalidator (*this), ui_bind (&VideoMonitor::parameter_changed, this, _1), gui_context());
 	_session->config.ParameterChanged.connect (*this, invalidator (*this), ui_bind (&VideoMonitor::parameter_changed, this, _1), gui_context());
 	XMLNode* node = _session->extra_xml (X_("XJSettings"));
 	if (!node) { return;}
