@@ -22,7 +22,7 @@
 #include "ardour/tempo.h"
 
 #include "pbd/file_utils.h"
-#include "pbd/convert.h"
+#include "pbd/types_convert.h"
 #include "ardour/session_directory.h"
 
 #include "ardour_ui.h"
@@ -59,8 +59,6 @@ VideoTimeLine::VideoTimeLine (PublicEditor *ed, ArdourCanvas::Container *vbg, in
 	video_offset_lock = false;
 	video_aspect_ratio = 4.0/3.0;
 	Config->ParameterChanged.connect (*this, invalidator (*this), ui_bind (&VideoTimeLine::parameter_changed, this, _1), gui_context());
-	video_server_url = video_get_server_url(Config);
-	server_docroot   = video_get_docroot(Config);
 	video_filename = "";
 	local_file = true;
 	video_file_fps = 25.0;
@@ -68,6 +66,9 @@ VideoTimeLine::VideoTimeLine (PublicEditor *ed, ArdourCanvas::Container *vbg, in
 	vmonitor=0;
 	reopen_vmonitor=false;
 	find_xjadeo();
+	find_harvid();
+	video_server_url = video_get_server_url(Config);
+	server_docroot   = video_get_docroot(Config);
 
 	VtlUpdate.connect (*this, invalidator (*this), boost::bind (&PublicEditor::queue_visual_videotimeline_update, editor), gui_context());
 	GuiUpdate.connect (*this, invalidator (*this), boost::bind (&VideoTimeLine::gui_update, this, _1), gui_context());
@@ -86,11 +87,9 @@ VideoTimeLine::save_session ()
 		return;
 	}
 
-	LocaleGuard lg;
-
 	XMLNode* node = new XMLNode(X_("Videomonitor"));
 	if (!node) return;
-	node->add_property (X_("active"), (vmonitor && vmonitor->is_started())?"yes":"no");
+	node->set_property (X_("active"), (vmonitor && vmonitor->is_started()));
 	_session->add_extra_xml (*node);
 
 	if (vmonitor) {
@@ -103,11 +102,11 @@ VideoTimeLine::save_session ()
 	/* VTL settings */
 	node = _session->extra_xml (X_("Videotimeline"));
 	if (!node) return;
-	node->add_property (X_("id"), id().to_s());
-	node->add_property (X_("Height"), editor->get_videotl_bar_height());
-	node->add_property (X_("VideoOffsetLock"), video_offset_lock?X_("1"):X_("0"));
-	node->add_property (X_("VideoOffset"), video_offset);
-	node->add_property (X_("AutoFPS"), auto_set_session_fps?X_("1"):X_("0"));
+	node->set_property (X_("id"), id());
+	node->set_property (X_("Height"), editor->get_videotl_bar_height());
+	node->set_property (X_("VideoOffsetLock"), video_offset_lock);
+	node->set_property (X_("VideoOffset"), video_offset);
+	node->set_property (X_("AutoFPS"), auto_set_session_fps);
 }
 
 /* close and save settings */
@@ -144,7 +143,6 @@ VideoTimeLine::set_session (ARDOUR::Session *s)
 	if (!_session) { return ; }
 
 	_session->SessionSaveUnderway.connect_same_thread (sessionsave, boost::bind (&VideoTimeLine::save_session, this));
-	LocaleGuard lg;
 
 	XMLNode* node = _session->extra_xml (X_("Videotimeline"));
 
@@ -156,9 +154,9 @@ VideoTimeLine::set_session (ARDOUR::Session *s)
 
 	set_id(*node);
 
-	XMLProperty const * proph = node->property (X_("Height"));
-	if (proph) {
-		editor->set_video_timeline_height(atoi(proph->value()));
+	int height;
+	if (node->get_property (X_("Height"), height)) {
+		editor->set_video_timeline_height(height);
 	}
 #if 0 /* TODO THINK: set FPS first time only ?! */
 	XMLProperty const * propasfps = node->property (X_("AutoFPS"));
@@ -167,28 +165,22 @@ VideoTimeLine::set_session (ARDOUR::Session *s)
 	}
 #endif
 
-	XMLProperty const * propoffset = node->property (X_("VideoOffset"));
-	if (propoffset) {
-		video_offset = atoll(propoffset->value());
+	if (node->get_property (X_("VideoOffset"), video_offset)) {
 		video_offset_p = video_offset;
 	}
 
-	XMLProperty const * proplock = node->property (X_("VideoOffsetLock"));
-	if (proplock) {
-		video_offset_lock = atoi(proplock->value())?true:false;
-	}
+	node->get_property (X_("VideoOffsetLock"), video_offset_lock);
+	node->get_property (X_("LocalFile"), local_file);
 
-	XMLProperty const * localfile = node->property (X_("LocalFile"));
-	if (localfile) {
-		local_file = atoi(localfile->value())?true:false;
+	std::string filename;
+	if (node->get_property (X_("Filename"), filename)) {
+		video_file_info (filename, local_file);
 	}
 
-	XMLProperty const * propf = node->property (X_("Filename"));
-	video_file_info(propf->value(), local_file);
-
 	if ((node = _session->extra_xml (X_("Videomonitor")))) {
-		XMLProperty const * prop = node->property (X_("active"));
-		if (prop && prop->value() == "yes" && found_xjadeo() && !video_filename.empty() && local_file) {
+		bool active;
+		if (node->get_property (X_("active"), active) && active && found_xjadeo () &&
+		    !video_filename.empty () && local_file) {
 			open_video_monitor();
 		}
 	}
@@ -225,11 +217,7 @@ VideoTimeLine::save_undo ()
 int
 VideoTimeLine::set_state (const XMLNode& node, int /*version*/)
 {
-	LocaleGuard lg;
-	XMLProperty const * propoffset = node.property (X_("VideoOffset"));
-	if (propoffset) {
-		video_offset = atoll(propoffset->value());
-	}
+	node.get_property (X_("VideoOffset"), video_offset);
 	ARDOUR_UI::instance()->flush_videotimeline_cache(true);
 	return 0;
 }
@@ -238,8 +226,7 @@ XMLNode&
 VideoTimeLine::get_state ()
 {
 	XMLNode* node = new XMLNode (X_("Videotimeline"));
-	LocaleGuard lg;
-	node->add_property (X_("VideoOffset"), video_offset_p);
+	node->set_property (X_("VideoOffset"), video_offset_p);
 	return *node;
 }
 
@@ -780,6 +767,61 @@ VideoTimeLine::find_xjadeo () {
 	}
 }
 
+void
+VideoTimeLine::harvid_readversion (std::string d, size_t /* s */) {
+	harvid_version += d;
+}
+
+void
+VideoTimeLine::find_harvid () {
+	/* This is mainly for the benefit of the windows version:
+	 * harvid >= 0.8.2 allows an empty docroot and ardour can
+	 * pass the drive-letter along.
+	 *
+	 * It is a chicken/egg w.r.t. the video-server dialog
+	 * but needed for default preferences and initial settings.
+	 */
+	std::string harvid_bin;
+	if (VideoUtils::harvid_version != 0x0) {
+		return;
+	}
+	if (!ArdourVideoToolPaths::harvid_exe(harvid_bin)) {
+		return;
+	}
+	if (harvid_bin.empty ()) {
+		return;
+	}
+	ARDOUR::SystemExec version_check(harvid_bin, X_("--version"));
+	harvid_version = "";
+	version_check.ReadStdout.connect_same_thread (*this, boost::bind (&VideoTimeLine::harvid_readversion, this, _1 ,_2));
+	version_check.Terminated.connect_same_thread (*this, boost::bind (&VideoTimeLine::harvid_readversion, this, "\n" ,1));
+	if (version_check.start(2)) {
+		return;
+	}
+
+#ifdef PLATFORM_WINDOWS
+	version_check.wait (); // 40ms timeout
+#else
+	version_check.wait (WNOHANG);
+#endif
+
+	int timeout = 300;
+	while (harvid_version.empty() && --timeout) {
+		Glib::usleep(10000);
+	}
+
+	size_t vo = harvid_version.find("harvid v");
+	if (vo != string::npos) {
+		int v_major, v_minor, v_micro;
+		if(sscanf(harvid_version.substr(vo + 8, string::npos).c_str(),"%d.%d.%d",
+					&v_major, &v_minor, &v_micro) == 3)
+		{
+			VideoUtils::harvid_version = (v_major << 16) | (v_minor << 8) | v_micro;
+			info << "harvid version: "<< hex << VideoUtils::harvid_version << endmsg;
+		}
+	}
+}
+
 void
 VideoTimeLine::open_video_monitor() {
 	if (!found_xjadeo()) return;
@@ -826,8 +868,10 @@ VideoTimeLine::open_video_monitor() {
 		if (_session) {
 			XMLNode* node = _session->extra_xml (X_("Videomonitor"));
 			if (node) {
-				XMLProperty const * prop = node->property (X_("active"));
-				if (prop && prop->value() != "yes") _session->set_dirty ();
+				bool active;
+				if (node->get_property (X_("active"), active) && !active) {
+					_session->set_dirty ();
+				}
 			} else {
 				_session->set_dirty ();
 			}
