@@ -33,6 +33,7 @@
 #include "ardour/cycles.h"
 #include "ardour/latent.h"
 #include "ardour/libardour_visibility.h"
+#include "ardour/midi_ring_buffer.h"
 #include "ardour/midi_state_tracker.h"
 #include "ardour/parameter_descriptor.h"
 #include "ardour/types.h"
@@ -159,6 +160,8 @@ class LIBARDOUR_API Plugin : public PBD::StatefulDestructible, public Latent
 		return boost::shared_ptr<ScalePoints>();
 	}
 
+	bool write_immediate_event (size_t size, const uint8_t* buf);
+
 	void realtime_handle_transport_stopped ();
 	void realtime_locate ();
 	void monitoring_changed ();
@@ -179,6 +182,11 @@ class LIBARDOUR_API Plugin : public PBD::StatefulDestructible, public Latent
 	virtual bool read_midnam () { return false; }
 	virtual std::string midnam_model () { return ""; }
 	PBD::Signal0<void> UpdateMidnam;
+	PBD::Signal0<void> UpdatedMidnam;
+
+	virtual bool knows_bank_patch () { return false; }
+	virtual uint32_t bank_patch (uint8_t chn) { return UINT32_MAX; }
+	PBD::Signal1<void, uint8_t> BankPatchChange;
 
 	struct PresetRecord {
 	    PresetRecord () : valid (false) {}
@@ -375,6 +383,8 @@ private:
 
 	PBD::ScopedConnection _preset_connection;
 
+	MidiRingBuffer<framepos_t> _immediate_events;
+
 	void resolve_midi ();
 };
 
