@@ -59,6 +59,7 @@
 #include "ardour/slavable.h"
 
 class RoutePinWindowProxy;
+class PatchChangeGridDialog;
 
 namespace ARDOUR {
 
@@ -220,6 +221,9 @@ public:
 	RoutePinWindowProxy * pinmgr_proxy () const { return _pinmgr_proxy; }
 	void set_pingmgr_proxy (RoutePinWindowProxy* wp) { _pinmgr_proxy = wp ; }
 
+	PatchChangeGridDialog* patch_selector_dialog () const { return _patch_selector_dialog; }
+	void set_patch_selector_dialog  (PatchChangeGridDialog* d) { _patch_selector_dialog = d; }
+
 	boost::shared_ptr<AutomationControl> automation_control_recurse (PBD::ID const & id) const;
 
 	/* special processors */
@@ -381,7 +385,7 @@ public:
 
 	boost::weak_ptr<Route> weakroute ();
 
-	int save_as_template (const std::string& path, const std::string& name);
+	int save_as_template (const std::string& path, const std::string& name, const std::string& description );
 
 	PBD::Signal1<void,void*> SelectedChanged;
 
@@ -773,6 +777,7 @@ private:
 	*/
 	boost::weak_ptr<Processor> _processor_after_last_custom_meter;
 	RoutePinWindowProxy *_pinmgr_proxy;
+	PatchChangeGridDialog* _patch_selector_dialog;
 
 	void reset_instrument_info ();
 
