@@ -259,7 +259,7 @@ class LIBARDOUR_API Session : public PBD::StatefulDestructible, public PBD::Scop
 	 */
 	RouteList new_route_from_template (uint32_t how_many, PresentationInfo::order_t insert_at, const std::string& template_path, const std::string& name, PlaylistDisposition pd = NewPlaylist);
 	RouteList new_route_from_template (uint32_t how_many, PresentationInfo::order_t insert_at, XMLNode&, const std::string& name, PlaylistDisposition pd = NewPlaylist);
-	std::vector<std::string> get_paths_for_new_sources (bool allow_replacing, const std::string& import_file_path, 
+	std::vector<std::string> get_paths_for_new_sources (bool allow_replacing, const std::string& import_file_path,
 	                                                    uint32_t channels, std::vector<std::string> const & smf_track_names);
 
 	int bring_all_sources_into_session (boost::function<void(uint32_t,uint32_t,std::string)> callback);
@@ -531,7 +531,7 @@ class LIBARDOUR_API Session : public PBD::StatefulDestructible, public PBD::Scop
 	int archive_session (const std::string&, const std::string&, ArchiveEncode compress_audio = FLAC_16BIT, bool only_used_sources = false, Progress* p = 0);
 
 	int restore_state (std::string snapshot_name);
-	int save_template (std::string template_name, bool replace_existing = false);
+	int save_template (const std::string& template_name, const std::string& description = "", bool replace_existing = false);
 	int save_history (std::string snapshot_name = "");
 	int restore_history (std::string snapshot_name);
 	void remove_state (std::string snapshot_name);
@@ -1203,6 +1203,7 @@ class LIBARDOUR_API Session : public PBD::StatefulDestructible, public PBD::Scop
 	void reconnect_ltc_output ();
 
 	VCAManager& vca_manager() { return *_vca_manager; }
+	VCAManager* vca_manager_ptr() { return _vca_manager; }
 
 	void auto_connect_thread_wakeup ();
 
@@ -1421,8 +1422,9 @@ class LIBARDOUR_API Session : public PBD::StatefulDestructible, public PBD::Scop
 	Glib::Threads::Mutex save_state_lock;
 	Glib::Threads::Mutex peak_cleanup_lock;
 
-	int      load_options (const XMLNode&);
-	int      load_state (std::string snapshot_name);
+	int        load_options (const XMLNode&);
+	int        load_state (std::string snapshot_name);
+	static int parse_stateful_loading_version (const std::string&);
 
 	framepos_t _last_roll_location;
 	/** the session frame time at which we last rolled, located, or changed transport direction */
@@ -1940,8 +1942,6 @@ class LIBARDOUR_API Session : public PBD::StatefulDestructible, public PBD::Scop
 	void   add_click (framepos_t pos, bool emphasis);
 	framecnt_t _count_in_samples;
 
-	std::vector<Route*> master_outs;
-
 	/* range playback */
 
 	std::list<AudioRange> current_audio_range;
