@@ -133,7 +133,7 @@ AudioSource::get_state ()
 	XMLNode& node (Source::get_state());
 
 	if (_captured_for.length()) {
-		node.add_property ("captured-for", _captured_for);
+		node.set_property ("captured-for", _captured_for);
 	}
 
 	return node;
@@ -142,12 +142,7 @@ AudioSource::get_state ()
 int
 AudioSource::set_state (const XMLNode& node, int /*version*/)
 {
-	XMLProperty const * prop;
-
-	if ((prop = node.property ("captured-for")) != 0) {
-		_captured_for = prop->value();
-	}
-
+	node.get_property ("captured-for", _captured_for);
 	return 0;
 }
 
@@ -387,7 +382,9 @@ AudioSource::read_peaks_with_fpp (PeakData *peaks, framecnt_t npeaks, framepos_t
 
 		if (statbuf.st_size < expected_file_size) {
 			warning << string_compose (_("peak file %1 is truncated from %2 to %3"), _peakpath, expected_file_size, statbuf.st_size) << endmsg;
+			lm.release(); // build_peaks_from_scratch() takes _lock
 			const_cast<AudioSource*>(this)->build_peaks_from_scratch ();
+			lm.acquire ();
 			if (g_stat (_peakpath.c_str(), &statbuf) != 0) {
 				error << string_compose (_("Cannot open peakfile @ %1 for size check (%2) after rebuild"), _peakpath, strerror (errno)) << endmsg;
 			}
