@@ -37,7 +37,9 @@
 #include "ardour/plugin_insert.h"
 #include "ardour/record_enable_control.h"
 #include "ardour/session.h"
+#ifdef LV2_SUPPORT
 #include "ardour/uri_map.h"
+#endif
 #include "ardour/value_as_string.h"
 
 #include "pbd/i18n.h"
@@ -55,6 +57,7 @@ Automatable::Automatable(Session& session)
 
 Automatable::Automatable (const Automatable& other)
         : ControlSet (other)
+        , Slavable ()
         , _a_session (other._a_session)
 {
         Glib::Threads::Mutex::Lock lm (other._control_lock);
@@ -67,12 +70,9 @@ Automatable::Automatable (const Automatable& other)
 
 Automatable::~Automatable ()
 {
-	{
-		Glib::Threads::Mutex::Lock lm (_control_lock);
-
-		for (Controls::const_iterator li = _controls.begin(); li != _controls.end(); ++li) {
-			boost::dynamic_pointer_cast<AutomationControl>(li->second)->drop_references ();
-		}
+	Glib::Threads::Mutex::Lock lm (_control_lock);
+	for (Controls::const_iterator li = _controls.begin(); li != _controls.end(); ++li) {
+		boost::dynamic_pointer_cast<AutomationControl>(li->second)->drop_references ();
 	}
 }
 
@@ -278,7 +278,7 @@ Automatable::get_automation_xml_state ()
 
 	for (Controls::iterator li = controls().begin(); li != controls().end(); ++li) {
 		boost::shared_ptr<AutomationList> l = boost::dynamic_pointer_cast<AutomationList>(li->second->list());
-		if (l && !l->empty()) {
+		if (l) {
 			node->add_child_nocopy (l->get_state ());
 		}
 	}
@@ -314,34 +314,6 @@ Automatable::get_parameter_automation_state (Evoral::Parameter param)
 	return result;
 }
 
-void
-Automatable::set_parameter_automation_style (Evoral::Parameter param, AutoStyle s)
-{
-	Glib::Threads::Mutex::Lock lm (control_lock());
-
-	boost::shared_ptr<AutomationControl> c = automation_control(param, true);
-
-	if (c && (s != c->automation_style())) {
-		c->set_automation_style (s);
-		_a_session.set_dirty ();
-	}
-}
-
-AutoStyle
-Automatable::get_parameter_automation_style (Evoral::Parameter param)
-{
-	Glib::Threads::Mutex::Lock lm (control_lock());
-
-	boost::shared_ptr<Evoral::Control> c = control(param);
-	boost::shared_ptr<AutomationList> l = boost::dynamic_pointer_cast<AutomationList>(c->list());
-
-	if (c) {
-		return l->automation_style();
-	} else {
-		return Absolute; // whatever
-	}
-}
-
 void
 Automatable::protect_automation ()
 {
@@ -367,25 +339,53 @@ Automatable::protect_automation ()
 }
 
 void
-Automatable::transport_located (framepos_t now)
+Automatable::non_realtime_locate (framepos_t now)
 {
+	bool rolling = _a_session.transport_rolling ();
+
 	for (Controls::iterator li = controls().begin(); li != controls().end(); ++li) {
 
 		boost::shared_ptr<AutomationControl> c
 				= boost::dynamic_pointer_cast<AutomationControl>(li->second);
 		if (c) {
-                        boost::shared_ptr<AutomationList> l
+			boost::shared_ptr<AutomationList> l
 				= boost::dynamic_pointer_cast<AutomationList>(c->list());
 
-			if (l) {
-				l->start_write_pass (now);
+			if (!l) {
+				continue;
+			}
+
+			bool am_touching = c->touching ();
+			if (rolling && am_touching) {
+			/* when locating while rolling, and writing automation,
+			 * start a new write pass.
+			 * compare to compare to non_realtime_transport_stop()
+			 */
+				const bool list_did_write = !l->in_new_write_pass ();
+				c->stop_touch (-1); // time is irrelevant
+				l->stop_touch (-1);
+				c->commit_transaction (list_did_write);
+				l->write_pass_finished (now, Config->get_automation_thinning_factor ());
+
+				if (l->automation_state () == Write) {
+					l->set_automation_state (Touch);
+				}
+				if (l->automation_playback ()) {
+					c->set_value_unchecked (c->list ()->eval (now));
+				}
+			}
+
+			l->start_write_pass (now);
+
+			if (rolling && am_touching) {
+				c->start_touch (now);
 			}
 		}
 	}
 }
 
 void
-Automatable::transport_stopped (framepos_t now)
+Automatable::non_realtime_transport_stop (framepos_t now, bool /*flush_processors*/)
 {
 	for (Controls::iterator li = controls().begin(); li != controls().end(); ++li) {
 		boost::shared_ptr<AutomationControl> c =
@@ -408,7 +408,7 @@ Automatable::transport_stopped (framepos_t now)
 		*/
 		const bool list_did_write = !l->in_new_write_pass ();
 
-		l->stop_touch (true, now);
+		l->stop_touch (now);
 
 		c->commit_transaction (list_did_write);
 
@@ -424,6 +424,19 @@ Automatable::transport_stopped (framepos_t now)
 	}
 }
 
+void
+Automatable::automation_run (framepos_t start, pframes_t nframes)
+{
+	for (Controls::iterator li = controls().begin(); li != controls().end(); ++li) {
+		boost::shared_ptr<AutomationControl> c =
+			boost::dynamic_pointer_cast<AutomationControl>(li->second);
+		if (!c) {
+			continue;
+		}
+		c->automation_run (start, nframes);
+	}
+}
+
 boost::shared_ptr<Evoral::Control>
 Automatable::control_factory(const Evoral::Parameter& param)
 {
@@ -506,6 +519,21 @@ Automatable::control_factory(const Evoral::Parameter& param)
 	return boost::shared_ptr<Evoral::Control>(control);
 }
 
+boost::shared_ptr<AutomationControl>
+Automatable::automation_control (PBD::ID const & id) const
+{
+	Controls::const_iterator li;
+
+	for (li = _controls.begin(); li != _controls.end(); ++li) {
+		boost::shared_ptr<AutomationControl> ac = boost::dynamic_pointer_cast<AutomationControl> (li->second);
+		if (ac && (ac->id() == id)) {
+			return ac;
+		}
+	}
+
+	return boost::shared_ptr<AutomationControl>();
+}
+
 boost::shared_ptr<AutomationControl>
 Automatable::automation_control (const Evoral::Parameter& id, bool create)
 {
@@ -525,12 +553,6 @@ Automatable::clear_controls ()
 	ControlSet::clear_controls ();
 }
 
-string
-Automatable::value_as_string (boost::shared_ptr<const AutomationControl> ac) const
-{
-	return ARDOUR::value_as_string(ac->desc(), ac->get_value());
-}
-
 bool
 Automatable::find_next_event (double now, double end, Evoral::ControlEvent& next_event, bool only_active) const
 {
@@ -546,6 +568,13 @@ Automatable::find_next_event (double now, double end, Evoral::ControlEvent& next
 			continue;
 		}
 
+		boost::shared_ptr<SlavableAutomationControl> sc
+			= boost::dynamic_pointer_cast<SlavableAutomationControl>(li->second);
+
+		if (sc) {
+			sc->find_next_event (now, end, next_event);
+		}
+
 		Evoral::ControlList::const_iterator i;
 		boost::shared_ptr<const Evoral::ControlList> alist (li->second->list());
 		Evoral::ControlEvent cp (now, 0.0f);
