@@ -96,7 +96,6 @@ setup_enum_writer ()
 	IOChange _IOChange;
 	AutomationType _AutomationType;
 	AutoState _AutoState;
-	AutoStyle _AutoStyle;
 	AutoConnectOption _AutoConnectOption;
 	TracksAutoNamingRule _TracksAutoNamingRule;
 	Session::StateOfTheState _Session_StateOfTheState;
@@ -132,7 +131,7 @@ setup_enum_writer ()
 	Session::SlaveState _Session_SlaveState;
 	MTC_Status _MIDI_MTC_Status;
 	Evoral::OverlapType _OverlapType;
-        BufferingPreset _BufferingPreset;
+    BufferingPreset _BufferingPreset;
 	AutoReturnTarget _AutoReturnTarget;
 	PresentationInfo::Flag _PresentationInfo_Flag;
 	MusicalMode::Type mode;
@@ -170,18 +169,6 @@ setup_enum_writer ()
 	REGISTER_ENUM (TrimAutomation);
 	REGISTER_ENUM (PhaseAutomation);
 	REGISTER_ENUM (MonitoringAutomation);
-	REGISTER_ENUM (EQGain);
-	REGISTER_ENUM (EQFrequency);
-	REGISTER_ENUM (EQQ);
-	REGISTER_ENUM (EQShape);
-	REGISTER_ENUM (EQHPF);
-	REGISTER_ENUM (EQEnable);
-	REGISTER_ENUM (CompThreshold);
-	REGISTER_ENUM (CompSpeed);
-	REGISTER_ENUM (CompMode);
-	REGISTER_ENUM (CompMakeup);
-	REGISTER_ENUM (CompRedux);
-	REGISTER_ENUM (CompEnable);
 	REGISTER_ENUM (BusSendLevel);
 	REGISTER_ENUM (BusSendEnable);
 	REGISTER (_AutomationType);
@@ -192,10 +179,6 @@ setup_enum_writer ()
 	REGISTER_ENUM (Play);
 	REGISTER_BITS (_AutoState);
 
-	REGISTER_ENUM (Absolute);
-	REGISTER_ENUM (Trim);
-	REGISTER_BITS (_AutoStyle);
-
 	REGISTER_ENUM (CaptureTime);
 	REGISTER_ENUM (ExistingMaterial);
 	REGISTER (_AlignStyle);
@@ -550,6 +533,8 @@ setup_enum_writer ()
 	REGISTER_CLASS_ENUM (AutomationList, Discrete);
 	REGISTER_CLASS_ENUM (AutomationList, Linear);
 	REGISTER_CLASS_ENUM (AutomationList, Curved);
+	REGISTER_CLASS_ENUM (AutomationList, Logarithmic);
+	REGISTER_CLASS_ENUM (AutomationList, Exponential);
 	REGISTER (_AutomationList_InterpolationStyle);
 
 	REGISTER_CLASS_ENUM (AnyTime, Timecode);
@@ -722,7 +707,6 @@ setup_enum_writer ()
 	REGISTER_CLASS_ENUM (PresentationInfo, MasterOut);
 	REGISTER_CLASS_ENUM (PresentationInfo, MonitorOut);
 	REGISTER_CLASS_ENUM (PresentationInfo, Auditioner);
-	REGISTER_CLASS_ENUM (PresentationInfo, Selected);
 	REGISTER_CLASS_ENUM (PresentationInfo, Hidden);
 	REGISTER_CLASS_ENUM (PresentationInfo, OrderSet);
 	REGISTER_BITS (_PresentationInfo_Flag);
@@ -768,374 +752,3 @@ setup_enum_writer ()
 }
 
 } /* namespace ARDOUR */
-
-/* deserializing types from ardour/types.h */
-
-std::istream& operator>>(std::istream& o, HeaderFormat& var)
-{
-	std::string s;
-	o >> s;
-	var = (HeaderFormat) string_2_enum (s, var);
-	return o;
-}
-
-std::ostream& operator<<(std::ostream& o, const HeaderFormat& var)
-{
-	std::string s = enum_2_string (var);
-	return o << s;
-}
-
-std::istream& operator>>(std::istream& o, SampleFormat& var)
-{
-	std::string s;
-	o >> s;
-	var = (SampleFormat) string_2_enum (s, var);
-	return o;
-}
-
-std::ostream& operator<<(std::ostream& o, const SampleFormat& var)
-{
-	std::string s = enum_2_string (var);
-	return o << s;
-}
-
-std::istream& operator>>(std::istream& o, AutoConnectOption& var)
-{
-	std::string s;
-	o >> s;
-	var = (AutoConnectOption) string_2_enum (s, var);
-	return o;
-}
-
-std::ostream& operator<<(std::ostream& o, const AutoConnectOption& var)
-{
-	std::string s = enum_2_string (var);
-	return o << s;
-}
-
-std::istream& operator>>(std::istream& o, TracksAutoNamingRule& var)
-{
-	std::string s;
-	o >> s;
-	var = (TracksAutoNamingRule) string_2_enum (s, var);
-	return o;
-}
-
-std::ostream& operator<<(std::ostream& o, const TracksAutoNamingRule& var)
-{
-	std::string s = enum_2_string (var);
-	return o << s;
-}
-
-std::istream& operator>>(std::istream& o, MonitorModel& var)
-{
-	std::string s;
-	o >> s;
-	var = (MonitorModel) string_2_enum (s, var);
-	return o;
-}
-
-std::ostream& operator<<(std::ostream& o, const MonitorModel& var)
-{
-	std::string s = enum_2_string (var);
-	return o << s;
-}
-
-std::istream& operator>>(std::istream& o, VUMeterStandard& var)
-{
-	std::string s;
-	o >> s;
-	var = (VUMeterStandard) string_2_enum (s, var);
-	return o;
-}
-
-std::ostream& operator<<(std::ostream& o, const VUMeterStandard& var)
-{
-	std::string s = enum_2_string (var);
-	return o << s;
-}
-
-std::istream& operator>>(std::istream& o, MeterLineUp& var)
-{
-	std::string s;
-	o >> s;
-	var = (MeterLineUp) string_2_enum (s, var);
-	return o;
-}
-
-std::ostream& operator<<(std::ostream& o, const MeterLineUp& var)
-{
-	std::string s = enum_2_string (var);
-	return o << s;
-}
-
-std::istream& operator>>(std::istream& o, PFLPosition& var)
-{
-	std::string s;
-	o >> s;
-	var = (PFLPosition) string_2_enum (s, var);
-	return o;
-}
-
-std::ostream& operator<<(std::ostream& o, const PFLPosition& var)
-{
-	std::string s = enum_2_string (var);
-	return o << s;
-}
-
-std::istream& operator>>(std::istream& o, AFLPosition& var)
-{
-	std::string s;
-	o >> s;
-	var = (AFLPosition) string_2_enum (s, var);
-	return o;
-}
-
-std::ostream& operator<<(std::ostream& o, const AFLPosition& var)
-{
-	std::string s = enum_2_string (var);
-	return o << s;
-}
-
-std::istream& operator>>(std::istream& o, EditMode& var)
-{
-	std::string s;
-	o >> s;
-	var = (EditMode) string_2_enum (s, var);
-	return o;
-}
-
-std::ostream& operator<<(std::ostream& o, const EditMode& var)
-{
-	std::string s = enum_2_string (var);
-	return o << s;
-}
-std::istream& operator>>(std::istream& o, ListenPosition& var)
-{
-	std::string s;
-	o >> s;
-	var = (ListenPosition) string_2_enum (s, var);
-	return o;
-}
-
-std::ostream& operator<<(std::ostream& o, const ListenPosition& var)
-{
-	std::string s = enum_2_string (var);
-	return o << s;
-}
-std::istream& operator>>(std::istream& o, LayerModel& var)
-{
-	std::string s;
-	o >> s;
-	var = (LayerModel) string_2_enum (s, var);
-	return o;
-}
-
-std::ostream& operator<<(std::ostream& o, const LayerModel& var)
-{
-	std::string s = enum_2_string (var);
-	return o << s;
-}
-
-std::istream& operator>>(std::istream& o, InsertMergePolicy& var)
-{
-	std::string s;
-	o >> s;
-	var = (InsertMergePolicy) string_2_enum (s, var);
-	return o;
-}
-std::ostream& operator<<(std::ostream& o, const InsertMergePolicy& var)
-{
-	std::string s = enum_2_string (var);
-	return o << s;
-}
-
-std::istream& operator>>(std::istream& o, SyncSource& var)
-{
-	std::string s;
-	o >> s;
-	var = (SyncSource) string_2_enum (s, var);
-	return o;
-}
-
-std::ostream& operator<<(std::ostream& o, const SyncSource& var)
-{
-	std::string s = enum_2_string (var);
-	return o << s;
-}
-std::istream& operator>>(std::istream& o, ShuttleBehaviour& var)
-{
-	std::string s;
-	o >> s;
-	var = (ShuttleBehaviour) string_2_enum (s, var);
-	return o;
-}
-
-std::ostream& operator<<(std::ostream& o, const ShuttleBehaviour& var)
-{
-	std::string s = enum_2_string (var);
-	return o << s;
-}
-std::istream& operator>>(std::istream& o, ShuttleUnits& var)
-{
-	std::string s;
-	o >> s;
-	var = (ShuttleUnits) string_2_enum (s, var);
-	return o;
-}
-
-std::ostream& operator<<(std::ostream& o, const ShuttleUnits& var)
-{
-	std::string s = enum_2_string (var);
-	return o << s;
-}
-std::istream& operator>>(std::istream& o, TimecodeFormat& var)
-{
-	std::string s;
-	o >> s;
-	var = (TimecodeFormat) string_2_enum (s, var);
-	return o;
-}
-
-std::ostream& operator<<(std::ostream& o, const TimecodeFormat& var)
-{
-	std::string s = enum_2_string (var);
-	return o << s;
-}
-std::istream& operator>>(std::istream& o, DenormalModel& var)
-{
-	std::string s;
-	o >> s;
-	var = (DenormalModel) string_2_enum (s, var);
-	return o;
-}
-
-std::ostream& operator<<(std::ostream& o, const DenormalModel& var)
-{
-	std::string s = enum_2_string (var);
-	return o << s;
-}
-std::istream& operator>>(std::istream& o, WaveformScale& var)
-{
-	std::string s;
-	o >> s;
-	var = (WaveformScale) string_2_enum (s, var);
-	return o;
-}
-
-std::ostream& operator<<(std::ostream& o, const WaveformScale& var)
-{
-	std::string s = enum_2_string (var);
-	return o << s;
-}
-std::istream& operator>>(std::istream& o, WaveformShape& var)
-{
-	std::string s;
-	o >> s;
-	var = (WaveformShape) string_2_enum (s, var);
-	return o;
-}
-
-std::ostream& operator<<(std::ostream& o, const WaveformShape& var)
-{
-	std::string s = enum_2_string (var);
-	return o << s;
-}
-
-std::istream& operator>>(std::istream& o, PositionLockStyle& var)
-{
-	std::string s;
-	o >> s;
-	var = (PositionLockStyle) string_2_enum (s, var);
-	return o;
-}
-
-std::ostream& operator<<(std::ostream& o, const PositionLockStyle& var)
-{
-	std::string s = enum_2_string (var);
-	return o << s;
-}
-
-std::istream& operator>>(std::istream& o, Evoral::OverlapType& var)
-{
-	std::string s;
-	o >> s;
-	var = (Evoral::OverlapType) string_2_enum (s, var);
-	return o;
-}
-
-std::ostream& operator<<(std::ostream& o, const Evoral::OverlapType& var)
-{
-	std::string s = enum_2_string (var);
-	return o << s;
-}
-
-std::istream& operator>>(std::istream& o, FadeShape& var)
-{
-	std::string s;
-	o >> s;
-	var = (FadeShape) string_2_enum (s, var);
-	return o;
-}
-
-std::ostream& operator<<(std::ostream& o, const FadeShape& var)
-{
-	std::string s = enum_2_string (var);
-	return o << s;
-}
-
-std::istream& operator>>(std::istream& o, RegionSelectionAfterSplit& var)
-{
-	std::string s;
-	o >> s;
-	var = (RegionSelectionAfterSplit) string_2_enum (s, var);
-	return o;
-}
-
-std::ostream& operator<<(std::ostream& o, const RegionSelectionAfterSplit& var)
-{
-	std::string s = enum_2_string (var);
-	return o << s;
-}
-
-std::istream& operator>>(std::istream& o, ARDOUR::BufferingPreset& var)
-{
-	std::string s;
-	o >> s;
-	var = (ARDOUR::BufferingPreset) string_2_enum (s, var);
-	return o;
-}
-
-std::ostream& operator<<(std::ostream& o, const ARDOUR::BufferingPreset& var)
-{
-	std::string s = enum_2_string (var);
-	return o << s;
-}
-
-std::istream& operator>>(std::istream& o, AutoReturnTarget& var)
-{
-	std::string s;
-	o >> s;
-	var = (AutoReturnTarget) string_2_enum (s, var);
-	return o;
-}
-
-std::ostream& operator<<(std::ostream& o, const AutoReturnTarget& var)
-{
-	std::string s = enum_2_string (var);
-	return o << s;
-}
-
-std::istream& operator>>(std::istream& o, MeterType& var)
-{
-	std::string s;
-	o >> s;
-	var = (MeterType) string_2_enum (s, var);
-	return o;
-}
-
-std::ostream& operator<<(std::ostream& o, const MeterType& var)
-{
-	std::string s = enum_2_string (var);
-	return o << s;
-}
