@@ -84,6 +84,8 @@
 #include "midi++/port.h"
 #include "midi++/mmc.h"
 
+#include "LuaBridge/LuaBridge.h"
+
 #include "ardour/analyser.h"
 #include "ardour/audio_library.h"
 #include "ardour/audio_backend.h"
@@ -111,8 +113,9 @@
 #include "ardour/runtime_functions.h"
 #include "ardour/session_event.h"
 #include "ardour/source_factory.h"
+#ifdef LV2_SUPPORT
 #include "ardour/uri_map.h"
-
+#endif
 #include "audiographer/routines.h"
 
 #if defined (__APPLE__)
@@ -144,7 +147,7 @@ PBD::Signal1<void,int> ARDOUR::PluginScanTimeout;
 PBD::Signal0<void> ARDOUR::GUIIdle;
 PBD::Signal3<bool,std::string,std::string,int> ARDOUR::CopyConfigurationFiles;
 
-std::vector<std::string> ARDOUR::reserved_io_names;
+std::map<std::string, bool> ARDOUR::reserved_io_names;
 
 static bool have_old_configuration_files = false;
 
@@ -284,7 +287,7 @@ lotsa_files_please ()
 	if (newmax > 0) {
 		info << string_compose (_("Your system is configured to limit %1 to only %2 open files"), PROGRAM_NAME, newmax) << endmsg;
 	} else {
-		error << string_compose (_("Could not set system open files limit. Current limit is %1 open files"), _getmaxstdio)  << endmsg;
+		error << string_compose (_("Could not set system open files limit. Current limit is %1 open files"), _getmaxstdio())  << endmsg;
 	}
 #endif
 }
@@ -422,6 +425,12 @@ ARDOUR::init (bool use_windows_vst, bool try_optimization, const char* localedir
 		return true;
 	}
 
+#ifndef NDEBUG
+	if (getenv("LUA_METATABLES")) {
+		luabridge::Security::setHideMetatables (false);
+	}
+#endif
+
 	if (!PBD::init()) return false;
 
 #ifdef ENABLE_NLS
@@ -544,20 +553,24 @@ ARDOUR::init (bool use_windows_vst, bool try_optimization, const char* localedir
 	   support may not even be active. Without adding an API to control
 	   surface support that would list their port names, we do have to
 	   list them here.
+
+	   We also need to know if the given I/O is an actual route.
+	   For routes (e.g. "master"), bus creation needs to be allowed the first time,
+	   while for pure I/O (e.g. "Click") track/bus creation must always fail.
 	*/
 
-	char const * const reserved[] = {
-		_("Monitor"),
-		_("Master"),
-		_("Control"),
-		_("Click"),
-		_("Mackie"),
-		0
-	};
+	reserved_io_names[_("Monitor")] = true;
+	reserved_io_names[_("Master")] = true;
+	reserved_io_names["auditioner"] = true; // auditioner.cc  Track (s, "auditioner",...)
 
-	for (int n = 0; reserved[n]; ++n) {
-		reserved_io_names.push_back (reserved[n]);
-	}
+	/* pure I/O */
+	reserved_io_names[X_("Click")] = false; // session.cc ClickIO (*this, X_("Click")
+	reserved_io_names[_("Control")] = false;
+	reserved_io_names[_("Mackie")] = false;
+	reserved_io_names[_("FaderPort Recv")] = false;
+	reserved_io_names[_("FaderPort Send")] = false;
+	reserved_io_names[_("FaderPort8 Recv")] = false;
+	reserved_io_names[_("FaderPort8 Send")] = false;
 
 	libardour_initialized = true;
 
@@ -584,13 +597,13 @@ ARDOUR::cleanup ()
 		return;
 	}
 
+	delete &ControlProtocolManager::instance();
 	ARDOUR::AudioEngine::destroy ();
 
 	delete Library;
 #ifdef HAVE_LRDF
 	lrdf_cleanup ();
 #endif
-	delete &ControlProtocolManager::instance();
 #ifdef WINDOWS_VST_SUPPORT
 	fst_exit ();
 #endif
@@ -724,6 +737,7 @@ ARDOUR::set_translations_enabled (bool yn)
 	(void) ::write (fd, &c, 1);
 	(void) ::close (fd);
 
+	Config->ParameterChanged ("enable-translation");
 	return true;
 }
 
