@@ -25,7 +25,7 @@
 #include <ctime>
 #include <list>
 
-#include "pbd/convert.h"
+#include "pbd/types_convert.h"
 #include "pbd/stl_delete.h"
 #include "pbd/xml++.h"
 #include "pbd/enumwriter.h"
@@ -35,9 +35,14 @@
 #include "ardour/session.h"
 #include "ardour/audiofilesource.h"
 #include "ardour/tempo.h"
+#include "ardour/types_convert.h"
 
 #include "pbd/i18n.h"
 
+namespace PBD {
+	DEFINE_ENUM_CONVERT(ARDOUR::Location::Flags);
+}
+
 using namespace std;
 using namespace ARDOUR;
 using namespace PBD;
@@ -54,7 +59,9 @@ PBD::Signal1<void,Location*> Location::changed;
 Location::Location (Session& s)
 	: SessionHandleRef (s)
 	, _start (0)
+	, _start_beat (0.0)
 	, _end (0)
+	, _end_beat (0.0)
 	, _flags (Flags (0))
 	, _locked (false)
 	, _position_lock_style (AudioTime)
@@ -64,7 +71,7 @@ Location::Location (Session& s)
 }
 
 /** Construct a new Location, giving it the position lock style determined by glue-new-markers-to-bars-and-beats */
-Location::Location (Session& s, framepos_t sample_start, framepos_t sample_end, const std::string &name, Flags bits)
+Location::Location (Session& s, framepos_t sample_start, framepos_t sample_end, const std::string &name, Flags bits, const uint32_t sub_num)
 	: SessionHandleRef (s)
 	, _name (name)
 	, _start (sample_start)
@@ -74,7 +81,7 @@ Location::Location (Session& s, framepos_t sample_start, framepos_t sample_end,
 	, _position_lock_style (s.config.get_glue_new_markers_to_bars_and_beats() ? MusicTime : AudioTime)
 
 {
-	recompute_bbt_from_frames ();
+	recompute_beat_from_frames (sub_num);
 
 	assert (_start >= 0);
 	assert (_end >= 0);
@@ -85,9 +92,9 @@ Location::Location (const Location& other)
 	, StatefulDestructible()
 	, _name (other._name)
 	, _start (other._start)
-	, _bbt_start (other._bbt_start)
+	, _start_beat (other._start_beat)
 	, _end (other._end)
-	, _bbt_end (other._bbt_end)
+	, _end_beat (other._end_beat)
 	, _flags (other._flags)
 	, _position_lock_style (other._position_lock_style)
 
@@ -125,8 +132,8 @@ Location::operator== (const Location& other)
 	if (_name != other._name ||
 	    _start != other._start ||
 	    _end != other._end ||
-	    _bbt_start != other._bbt_start ||
-	    _bbt_end != other._bbt_end ||
+	    _start_beat != other._start_beat ||
+	    _end_beat != other._end_beat ||
 	    _flags != other._flags ||
 	    _position_lock_style != other._position_lock_style) {
 		return false;
@@ -143,9 +150,9 @@ Location::operator= (const Location& other)
 
 	_name = other._name;
 	_start = other._start;
-	_bbt_start = other._bbt_start;
+	_start_beat = other._start_beat;
 	_end = other._end;
-	_bbt_end = other._bbt_end;
+	_end_beat = other._end_beat;
 	_flags = other._flags;
 	_position_lock_style = other._position_lock_style;
 
@@ -178,10 +185,10 @@ Location::set_name (const std::string& str)
 /** Set start position.
  *  @param s New start.
  *  @param force true to force setting, even if the given new start is after the current end.
- *  @param allow_bbt_recompute True to recompute BBT start time from the new given start time.
+ *  @param allow_beat_recompute True to recompute BEAT start time from the new given start time.
  */
 int
-Location::set_start (framepos_t s, bool force, bool allow_bbt_recompute)
+Location::set_start (framepos_t s, bool force, bool allow_beat_recompute, const uint32_t sub_num)
 {
 	if (s < 0) {
 		return -1;
@@ -201,8 +208,8 @@ Location::set_start (framepos_t s, bool force, bool allow_bbt_recompute)
 		if (_start != s) {
 			_start = s;
 			_end = s;
-			if (allow_bbt_recompute) {
-				recompute_bbt_from_frames ();
+			if (allow_beat_recompute) {
+				recompute_beat_from_frames (sub_num);
 			}
 
 			start_changed (this); /* EMIT SIGNAL */
@@ -235,8 +242,8 @@ Location::set_start (framepos_t s, bool force, bool allow_bbt_recompute)
 		framepos_t const old = _start;
 
 		_start = s;
-		if (allow_bbt_recompute) {
-			recompute_bbt_from_frames ();
+		if (allow_beat_recompute) {
+			recompute_beat_from_frames (sub_num);
 		}
 		start_changed (this); /* EMIT SIGNAL */
 		StartChanged (); /* EMIT SIGNAL */
@@ -255,10 +262,10 @@ Location::set_start (framepos_t s, bool force, bool allow_bbt_recompute)
 /** Set end position.
  *  @param s New end.
  *  @param force true to force setting, even if the given new end is before the current start.
- *  @param allow_bbt_recompute True to recompute BBT end time from the new given end time.
+ *  @param allow_beat_recompute True to recompute BEAT end time from the new given end time.
  */
 int
-Location::set_end (framepos_t e, bool force, bool allow_bbt_recompute)
+Location::set_end (framepos_t e, bool force, bool allow_beat_recompute, const uint32_t sub_num)
 {
 	if (e < 0) {
 		return -1;
@@ -278,8 +285,8 @@ Location::set_end (framepos_t e, bool force, bool allow_bbt_recompute)
 		if (_start != e) {
 			_start = e;
 			_end = e;
-			if (allow_bbt_recompute) {
-				recompute_bbt_from_frames ();
+			if (allow_beat_recompute) {
+				recompute_beat_from_frames (sub_num);
 			}
 			//start_changed (this); /* EMIT SIGNAL */
 			//StartChanged (); /* EMIT SIGNAL */
@@ -303,8 +310,8 @@ Location::set_end (framepos_t e, bool force, bool allow_bbt_recompute)
 		framepos_t const old = _end;
 
 		_end = e;
-		if (allow_bbt_recompute) {
-			recompute_bbt_from_frames ();
+		if (allow_beat_recompute) {
+			recompute_beat_from_frames (sub_num);
 		}
 
 		end_changed(this); /* EMIT SIGNAL */
@@ -321,7 +328,7 @@ Location::set_end (framepos_t e, bool force, bool allow_bbt_recompute)
 }
 
 int
-Location::set (framepos_t s, framepos_t e, bool allow_bbt_recompute)
+Location::set (framepos_t s, framepos_t e, bool allow_beat_recompute, const uint32_t sub_num)
 {
 	if (s < 0 || e < 0) {
 		return -1;
@@ -341,8 +348,8 @@ Location::set (framepos_t s, framepos_t e, bool allow_bbt_recompute)
 			_start = s;
 			_end = s;
 
-			if (allow_bbt_recompute) {
-				recompute_bbt_from_frames ();
+			if (allow_beat_recompute) {
+				recompute_beat_from_frames (sub_num);
 			}
 
 			start_change = true;
@@ -364,8 +371,8 @@ Location::set (framepos_t s, framepos_t e, bool allow_bbt_recompute)
 			framepos_t const old = _start;
 			_start = s;
 
-			if (allow_bbt_recompute) {
-				recompute_bbt_from_frames ();
+			if (allow_beat_recompute) {
+				recompute_beat_from_frames (sub_num);
 			}
 
 			start_change = true;
@@ -382,8 +389,8 @@ Location::set (framepos_t s, framepos_t e, bool allow_bbt_recompute)
 			framepos_t const old = _end;
 			_end = e;
 
-			if (allow_bbt_recompute) {
-				recompute_bbt_from_frames ();
+			if (allow_beat_recompute) {
+				recompute_beat_from_frames (sub_num);
 			}
 
 			end_change = true;
@@ -411,7 +418,7 @@ Location::set (framepos_t s, framepos_t e, bool allow_bbt_recompute)
 }
 
 int
-Location::move_to (framepos_t pos)
+Location::move_to (framepos_t pos, const uint32_t sub_num)
 {
 	if (pos < 0) {
 		return -1;
@@ -424,7 +431,7 @@ Location::move_to (framepos_t pos)
 	if (_start != pos) {
 		_start = pos;
 		_end = _start + length();
-		recompute_bbt_from_frames ();
+		recompute_beat_from_frames (sub_num);
 
 		changed (this); /* EMIT SIGNAL */
 		Changed (); /* EMIT SIGNAL */
@@ -554,8 +561,8 @@ Location::cd_info_node(const string & name, const string & value)
 {
 	XMLNode* root = new XMLNode("CD-Info");
 
-	root->add_property("name", name);
-	root->add_property("value", value);
+	root->set_property("name", name);
+	root->set_property("value", value);
 
 	return *root;
 }
@@ -565,7 +572,6 @@ XMLNode&
 Location::get_state ()
 {
 	XMLNode *node = new XMLNode ("Location");
-	char buf[64];
 
 	typedef map<string, string>::const_iterator CI;
 
@@ -573,16 +579,17 @@ Location::get_state ()
 		node->add_child_nocopy(cd_info_node(m->first, m->second));
 	}
 
-	id().print (buf, sizeof (buf));
-	node->add_property("id", buf);
-	node->add_property ("name", name());
-	snprintf (buf, sizeof (buf), "%" PRId64, start());
-	node->add_property ("start", buf);
-	snprintf (buf, sizeof (buf), "%" PRId64, end());
-	node->add_property ("end", buf);
-	node->add_property ("flags", enum_2_string (_flags));
-	node->add_property ("locked", (_locked ? "yes" : "no"));
-	node->add_property ("position-lock-style", enum_2_string (_position_lock_style));
+	node->set_property ("id", id ());
+	node->set_property ("name", name());
+	node->set_property ("start", start());
+	node->set_property ("end", end());
+	if (position_lock_style() == MusicTime) {
+		node->set_property ("start-beat", _start_beat);
+		node->set_property ("end-beat", _end_beat);
+	}
+	node->set_property ("flags", _flags);
+	node->set_property ("locked", _locked);
+	node->set_property ("position-lock-style", _position_lock_style);
 
 	if (_scene_change) {
 		node->add_child_nocopy (_scene_change->get_state());
@@ -594,8 +601,6 @@ Location::get_state ()
 int
 Location::set_state (const XMLNode& node, int version)
 {
-	XMLProperty const * prop;
-
 	XMLNodeList cd_list = node.children();
 	XMLNodeConstIterator cd_iter;
 	XMLNode *cd_node;
@@ -612,46 +617,40 @@ Location::set_state (const XMLNode& node, int version)
 		warning << _("XML node for Location has no ID information") << endmsg;
 	}
 
-	if ((prop = node.property ("name")) == 0) {
+	std::string str;
+	if (!node.get_property ("name", str)) {
 		error << _("XML node for Location has no name information") << endmsg;
 		return -1;
 	}
 
-	set_name (prop->value());
-
-	if ((prop = node.property ("start")) == 0) {
-		error << _("XML node for Location has no start information") << endmsg;
-		return -1;
-	}
+	set_name (str);
 
 	/* can't use set_start() here, because _end
 	   may make the value of _start illegal.
 	*/
 
-	sscanf (prop->value().c_str(), "%" PRId64, &_start);
+	if (!node.get_property ("start", _start)) {
+		error << _("XML node for Location has no start information") << endmsg;
+		return -1;
+	}
 
-	if ((prop = node.property ("end")) == 0) {
+	if (!node.get_property ("end", _end)) {
 		error << _("XML node for Location has no end information") << endmsg;
 		return -1;
 	}
 
-	sscanf (prop->value().c_str(), "%" PRId64, &_end);
+	Flags old_flags (_flags);
 
-	if ((prop = node.property ("flags")) == 0) {
+	if (!node.get_property ("flags", _flags)) {
 		error << _("XML node for Location has no flags information") << endmsg;
 		return -1;
 	}
 
-	Flags old_flags (_flags);
-	_flags = Flags (string_2_enum (prop->value(), _flags));
-
 	if (old_flags != _flags) {
 		FlagsChanged ();
 	}
 
-	if ((prop = node.property ("locked")) != 0) {
-		_locked = string_is_affirmative (prop->value());
-	} else {
+	if (!node.get_property ("locked", _locked)) {
 		_locked = false;
 	}
 
@@ -663,25 +662,18 @@ Location::set_state (const XMLNode& node, int version)
 			continue;
 		}
 
-		if ((prop = cd_node->property ("name")) != 0) {
-			cd_name = prop->value();
-		} else {
+		if (!cd_node->get_property ("name", cd_name)) {
 			throw failed_constructor ();
 		}
 
-		if ((prop = cd_node->property ("value")) != 0) {
-			cd_value = prop->value();
-		} else {
+		if (!cd_node->get_property ("value", cd_value)) {
 			throw failed_constructor ();
 		}
 
-
 		cd_info[cd_name] = cd_value;
 	}
 
-	if ((prop = node.property ("position-lock-style")) != 0) {
-		_position_lock_style = PositionLockStyle (string_2_enum (prop->value(), _position_lock_style));
-	}
+	node.get_property ("position-lock-style", _position_lock_style);
 
 	XMLNode* scene_child = find_named_node (node, SceneChange::xml_node_name);
 
@@ -689,7 +681,16 @@ Location::set_state (const XMLNode& node, int version)
 		_scene_change = SceneChange::factory (*scene_child, version);
 	}
 
-	recompute_bbt_from_frames ();
+	if (position_lock_style() == AudioTime) {
+		recompute_beat_from_frames (0);
+	} else{
+		/* music */
+		if (!node.get_property ("start-beat", _start_beat) ||
+		    !node.get_property ("end-beat", _end_beat)) {
+			recompute_beat_from_frames (0);
+		}
+	}
+
 
 	changed (this); /* EMIT SIGNAL */
 	Changed (); /* EMIT SIGNAL */
@@ -709,32 +710,30 @@ Location::set_position_lock_style (PositionLockStyle ps)
 
 	_position_lock_style = ps;
 
-	recompute_bbt_from_frames ();
+	if (ps == MusicTime) {
+		recompute_beat_from_frames (0);
+	}
 
 	position_lock_style_changed (this); /* EMIT SIGNAL */
 	PositionLockStyleChanged (); /* EMIT SIGNAL */
 }
 
 void
-Location::recompute_bbt_from_frames ()
+Location::recompute_beat_from_frames (const uint32_t sub_num)
 {
-	if (_position_lock_style != MusicTime) {
-		return;
-	}
-
-	_bbt_start = _session.tempo_map().beat_at_frame (_start);
-	_bbt_end = _session.tempo_map().beat_at_frame (_end);
+	_start_beat = _session.tempo_map().exact_beat_at_frame (_start, sub_num);
+	_end_beat = _session.tempo_map().exact_beat_at_frame (_end, sub_num);
 }
 
 void
-Location::recompute_frames_from_bbt ()
+Location::recompute_frames_from_beat ()
 {
 	if (_position_lock_style != MusicTime) {
 		return;
 	}
 
 	TempoMap& map (_session.tempo_map());
-	set (map.frame_at_beat (_bbt_start), map.frame_at_beat (_bbt_end), false);
+	set (map.frame_at_beat (_start_beat), map.frame_at_beat (_end_beat), false);
 }
 
 void
@@ -1007,6 +1006,7 @@ Locations::remove (Location *loc)
 
 		for (i = locations.begin(); i != locations.end(); ++i) {
 			if ((*i) == loc) {
+				bool was_loop = (*i)->is_auto_loop();
 				delete *i;
 				locations.erase (i);
 				was_removed = true;
@@ -1014,6 +1014,12 @@ Locations::remove (Location *loc)
 					current_location = 0;
 					was_current = true;
 				}
+				if (was_loop) {
+					if (_session.get_play_loop()) {
+						_session.request_play_loop (false, false);
+					}
+					_session.auto_loop_location_changed (0);
+				}
 				break;
 			}
 		}
@@ -1060,7 +1066,7 @@ Locations::set_state (const XMLNode& node, int version)
 
 	Location* session_range_location = 0;
 	if (version < 3000) {
-		session_range_location = new Location (_session, 0, 0, _("session"), Location::IsSessionRange);
+		session_range_location = new Location (_session, 0, 0, _("session"), Location::IsSessionRange, 0);
 		new_locations.push_back (session_range_location);
 	}
 
