@@ -238,8 +238,7 @@ LuaProc::load_script ()
 		try {
 			lua_dsp_init (_session.nominal_frame_rate ());
 		} catch (luabridge::LuaException const& e) {
-			;
-		}
+		} catch (...) { }
 	}
 
 	_ctrl_params.clear ();
@@ -354,6 +353,8 @@ LuaProc::can_support_io_configuration (const ChanCount& in, ChanCount& out, Chan
 			}
 		} catch (luabridge::LuaException const& e) {
 			_iotable = NULL;
+		} catch (...) {
+			_iotable = NULL;
 		}
 	}
 
@@ -584,6 +585,8 @@ LuaProc::configure_io (ChanCount in, ChanCount out)
 				std::cerr << "LuaException: " << e.what () << "\n";
 #endif
 				return false;
+			} catch (...) {
+				return false;
 			}
 		}
 	}
@@ -734,6 +737,8 @@ LuaProc::connect_and_run (BufferSet& bufs,
 		std::cerr << "LuaException: " << e.what () << "\n";
 #endif
 		return -1;
+	} catch (...) {
+		return -1;
 	}
 #ifdef WITH_LUAPROC_STATS
 	int64_t t1 = g_get_monotonic_time ();
