@@ -289,16 +289,6 @@ MidiDiskstream::use_copy_playlist ()
 	}
 }
 
-#ifdef XXX_OLD_DESTRUCTIVE_API_XXX
-/** Overloaded from parent to die horribly
- */
-int
-MidiDiskstream::set_destructive (bool yn)
-{
-	return yn ? -1 : 0;
-}
-#endif
-
 void
 MidiDiskstream::set_note_mode (NoteMode m)
 {
