@@ -140,13 +140,6 @@ MidiTrack::set_diskstream (boost::shared_ptr<Diskstream> ds)
 	mds->reset_tracker ();
 
 	_diskstream->set_track (this);
-#ifdef XXX_OLD_DESTRUCTIVE_API_XXX
-	if (Profile->get_trx()) {
-		_diskstream->set_destructive (false);
-	} else {
-		_diskstream->set_destructive (_mode == Destructive);
-	}
-#endif
 	_diskstream->set_record_enabled (false);
 
 	_diskstream_data_recorded_connection.disconnect ();
@@ -356,9 +349,10 @@ MidiTrack::update_controls(const BufferSet& bufs)
 	for (MidiBuffer::const_iterator e = buf.begin(); e != buf.end(); ++e) {
 		const Evoral::Event<framepos_t>&         ev      = *e;
 		const Evoral::Parameter                  param   = midi_parameter(ev.buffer(), ev.size());
-		const boost::shared_ptr<Evoral::Control> control = this->control(param);
+		const boost::shared_ptr<AutomationControl> control = automation_control (param);
 		if (control) {
 			control->set_double(ev.value(), _session.transport_frame(), false);
+			control->Changed (false, Controllable::NoGroup);
 		}
 	}
 }
