@@ -492,7 +492,11 @@ PluginInsert::create_automatable_parameters ()
 			if (Variant::type_is_numeric(desc.datatype)) {
 				list = boost::shared_ptr<AutomationList>(new AutomationList(param, desc));
 			}
-			add_control (boost::shared_ptr<AutomationControl> (new PluginPropertyControl(this, param, desc, list)));
+			boost::shared_ptr<AutomationControl> c (new PluginPropertyControl(this, param, desc, list));
+			if (!Variant::type_is_numeric(desc.datatype)) {
+				c->set_flags (Controllable::Flag ((int)c->flags() | Controllable::NotAutomatable));
+			}
+			add_control (c);
 		}
 	}
 
@@ -701,6 +705,18 @@ PluginInsert::bypassable_changed ()
 	BypassableChanged ();
 }
 
+bool
+PluginInsert::write_immediate_event (size_t size, const uint8_t* buf)
+{
+	bool rv = true;
+	for (Plugins::iterator i = _plugins.begin(); i != _plugins.end(); ++i) {
+		if (!(*i)->write_immediate_event (size, buf)) {
+			rv = false;
+		}
+	}
+	return rv;
+}
+
 void
 PluginInsert::preset_load_set_value (uint32_t p, float v)
 {
