@@ -35,14 +35,3 @@ RouteGroupMember::set_route_group (RouteGroup *rg)
 	_route_group = rg;
 	route_group_changed (); /* EMIT SIGNAL */
 }
-
-bool
-RouteGroupMember::use_group (PBD::Controllable::GroupControlDisposition gcd, bool (RouteGroup::*predicate)(void) const) const
-{
-	bool active_for_predicate = _route_group && (_route_group->*predicate)() && _route_group->is_active();
-
-	return ((gcd != PBD::Controllable::NoGroup) && (gcd != PBD::Controllable::ForGroup)) &&
-		_route_group && 
-		((active_for_predicate && (gcd != PBD::Controllable::InverseGroup)) ||
-		 (!active_for_predicate && (gcd == PBD::Controllable::InverseGroup)));
-}
