@@ -64,6 +64,7 @@ using namespace Timecode;
 void
 Session::ltc_tx_initialize()
 {
+	assert (!ltc_encoder && !ltc_enc_buf);
 	ltc_enc_tcformat = config.get_timecode_format();
 
 	ltc_tx_parse_offset();
@@ -84,8 +85,8 @@ Session::ltc_tx_initialize()
 	ltc_prev_cycle = -1;
 	ltc_tx_reset();
 	ltc_tx_resync_latency();
-	Xrun.connect_same_thread (*this, boost::bind (&Session::ltc_tx_reset, this));
-	engine().GraphReordered.connect_same_thread (*this, boost::bind (&Session::ltc_tx_resync_latency, this));
+	Xrun.connect_same_thread (ltc_tx_connections, boost::bind (&Session::ltc_tx_reset, this));
+	engine().GraphReordered.connect_same_thread (ltc_tx_connections, boost::bind (&Session::ltc_tx_resync_latency, this));
 	restarting = false;
 }
 
@@ -93,6 +94,7 @@ void
 Session::ltc_tx_cleanup()
 {
 	DEBUG_TRACE (DEBUG::LTC, "LTC TX cleanup\n");
+	ltc_tx_connections.drop_connections ();
 	free(ltc_enc_buf);
 	ltc_enc_buf = NULL;
 	ltc_encoder_free(ltc_encoder);
@@ -115,6 +117,7 @@ void
 Session::ltc_tx_reset()
 {
 	DEBUG_TRACE (DEBUG::LTC, "LTC TX reset\n");
+	assert (ltc_encoder);
 	ltc_enc_pos = -9999; // force re-start
 	ltc_buf_len = 0;
 	ltc_buf_off = 0;
@@ -168,6 +171,12 @@ Session::ltc_tx_send_time_code_for_cycle (framepos_t start_frame, framepos_t end
 	pframes_t txf = 0;
 	boost::shared_ptr<Port> ltcport = ltc_output_port();
 
+	if (!ltcport) {
+		assert (deletion_in_progress ());
+		return;
+	}
+
+	/* marks buffer as not written */
 	Buffer& buf (ltcport->get_buffer (nframes));
 
 	if (!ltc_encoder || !ltc_enc_buf) {
@@ -275,7 +284,7 @@ Session::ltc_tx_send_time_code_for_cycle (framepos_t start_frame, framepos_t end
 	/* LTC TV standard offset */
 	if (current_speed != 0) {
 		/* ditto - send "NOW" if not rolling */
-		cycle_start_frame -= ltc_frame_alignment(frames_per_timecode_frame(), TV_STANDARD(cur_timecode));
+		cycle_start_frame -= ltc_frame_alignment(samples_per_timecode_frame(), TV_STANDARD(cur_timecode));
 	}
 
 	/* cycle-start may become negative due to latency compensation */
@@ -448,7 +457,7 @@ Session::ltc_tx_send_time_code_for_cycle (framepos_t start_frame, framepos_t end
 
 
 	// (4) check if alignment matches
-	const double fptcf = frames_per_timecode_frame();
+	const double fptcf = samples_per_timecode_frame();
 
 	/* maximum difference of bit alignment in audio-samples.
 	 *
