@@ -40,6 +40,8 @@
 #include "ardour/slave.h"
 #include "ardour/ticker.h"
 #include "ardour/types.h"
+#include "ardour/vca.h"
+#include "ardour/vca_manager.h"
 
 #include "midi++/mmc.h"
 
@@ -142,6 +144,11 @@ Session::no_roll (pframes_t nframes)
 
 	ltc_tx_send_time_code_for_cycle (_transport_frame, end_frame, _target_transport_speed, _transport_speed, nframes);
 
+	VCAList v = _vca_manager->vcas ();
+	for (VCAList::const_iterator i = v.begin(); i != v.end(); ++i) {
+		(*i)->automation_run (_transport_frame, nframes);
+	}
+
 	if (_process_graph) {
 		DEBUG_TRACE(DEBUG::ProcessThreads,"calling graph/no-roll\n");
 		_process_graph->routes_no_roll( nframes, _transport_frame, end_frame, non_realtime_work_pending(), declick);
@@ -180,6 +187,11 @@ Session::process_routes (pframes_t nframes, bool& need_butler)
 	const framepos_t start_frame = _transport_frame;
 	const framepos_t end_frame = _transport_frame + floor (nframes * _transport_speed);
 
+	VCAList v = _vca_manager->vcas ();
+	for (VCAList::const_iterator i = v.begin(); i != v.end(); ++i) {
+		(*i)->automation_run (start_frame, nframes);
+	}
+
 	if (_process_graph) {
 		DEBUG_TRACE(DEBUG::ProcessThreads,"calling graph/process-routes\n");
 		if (_process_graph->process_routes (nframes, start_frame, end_frame, declick, need_butler) < 0) {
@@ -225,6 +237,11 @@ Session::silent_process_routes (pframes_t nframes, bool& need_butler)
 	const framepos_t start_frame = _transport_frame;
 	const framepos_t end_frame = _transport_frame + lrintf(nframes * _transport_speed);
 
+	VCAList v = _vca_manager->vcas ();
+	for (VCAList::const_iterator i = v.begin(); i != v.end(); ++i) {
+		(*i)->automation_run (start_frame, nframes);
+	}
+
 	if (_process_graph) {
 		_process_graph->silent_process_routes (nframes, start_frame, end_frame, need_butler);
 	} else {
@@ -314,6 +331,47 @@ Session::process_with_events (pframes_t nframes)
 		process_event (ev);
 	}
 
+	/* count in */
+	if (_transport_speed != 1.0 && _count_in_samples > 0) {
+		_count_in_samples = 0;
+	}
+
+	if (_count_in_samples > 0) {
+		framecnt_t ns = std::min ((framecnt_t)nframes, _count_in_samples);
+
+		no_roll (ns);
+		run_click (_transport_frame - _count_in_samples, ns);
+
+		_count_in_samples -= ns;
+		nframes -= ns;
+
+		/* process events.. */
+		if (!events.empty() && next_event != events.end()) {
+			SessionEvent* this_event = *next_event;
+			Events::iterator the_next_one = next_event;
+			++the_next_one;
+
+			while (this_event && this_event->action_frame == _transport_frame) {
+				process_event (this_event);
+				if (the_next_one == events.end()) {
+					this_event = 0;
+				} else {
+					this_event = *the_next_one;
+					++the_next_one;
+				}
+			}
+			set_next_event ();
+		}
+
+		check_declick_out ();
+
+		if (nframes == 0) {
+			return;
+		} else {
+			_engine.split_cycle (ns);
+		}
+	}
+
 	/* Decide on what to do with quarter-frame MTC during this cycle */
 
 	bool const was_sending_qf_mtc = _send_qf_mtc;
@@ -359,8 +417,14 @@ Session::process_with_events (pframes_t nframes)
 
 	if (events.empty() || next_event == events.end()) {
 		try_run_lua (nframes); // also during export ?? ->move to process_without_events()
-		process_without_events (nframes);
-		return;
+		/* lua scripts may inject events */
+		while (_n_lua_scripts > 0 && pending_events.read (&ev, 1) == 1) {
+			merge_event (ev);
+		}
+		if (events.empty() || next_event == events.end()) {
+			process_without_events (nframes);
+			return;
+		}
 	}
 
 	if (_transport_speed == 1.0) {
@@ -627,7 +691,7 @@ Session::follow_slave (pframes_t nframes)
 	}
 
 
-	if (_slave_state == Running && !non_realtime_work_pending()) {
+	if (_slave_state == Running && 0 == (post_transport_work () & ~PostTransportSpeed)) {
 		/* speed is set, we're locked, and good to go */
 		return true;
 	}
@@ -986,6 +1050,7 @@ Session::queue_event (SessionEvent* ev)
 	} else if (_state_of_the_state & Loading) {
 		merge_event (ev);
 	} else {
+		Glib::Threads::Mutex::Lock lm (rb_write_lock);
 		pending_events.write (&ev, 1);
 	}
 }
@@ -1109,7 +1174,7 @@ Session::process_event (SessionEvent* ev)
 
 	case SessionEvent::PunchIn:
 		// cerr << "PunchIN at " << transport_frame() << endl;
-		if (config.get_punch_in() && record_status() == Enabled) {
+		if (config.get_punch_in() && record_status() == Enabled && !preroll_record_punch_enabled()) {
 			enable_record ();
 		}
 		remove = false;
@@ -1118,13 +1183,21 @@ Session::process_event (SessionEvent* ev)
 
 	case SessionEvent::PunchOut:
 		// cerr << "PunchOUT at " << transport_frame() << endl;
-		if (config.get_punch_out()) {
+		if (config.get_punch_out() && !preroll_record_punch_enabled()) {
 			step_back_from_record ();
 		}
 		remove = false;
 		del = false;
 		break;
 
+	case SessionEvent::RecordStart:
+		if (preroll_record_punch_enabled() && record_status() == Enabled) {
+			enable_record ();
+		}
+		remove = false;
+		del = false;
+		break;
+
 	case SessionEvent::StopOnce:
 		if (!non_realtime_work_pending()) {
 			_clear_event_type (SessionEvent::StopOnce);
@@ -1224,6 +1297,9 @@ Session::compute_stop_limit () const
 		return max_framepos;
 	}
 
+	if (preroll_record_punch_enabled ()) {
+		return max_framepos;
+	}
 
 	bool const punching_in = (config.get_punch_in () && _locations->auto_punch_location());
 	bool const punching_out = (config.get_punch_out () && _locations->auto_punch_location());
