@@ -17,7 +17,7 @@ BBTTest::addTest ()
 	Meter    meter(4.0, 4.0);
 
 	/* no need to supply the frame for a new music-locked meter */
-	map.add_meter (meter, 4.0, BBT_Time(2, 1, 0), MusicTime);
+	map.add_meter (meter, BBT_Time(2, 1, 0), 0, MusicTime);
 
 	/* add some good stuff here */
 }
