@@ -47,10 +47,12 @@
 #include <glibmm/fileutils.h>
 
 #include "pbd/cpus.h"
+#include "pbd/control_math.h"
 #include "pbd/error.h"
 #include "pbd/stacktrace.h"
 #include "pbd/xml++.h"
 #include "pbd/basename.h"
+#include "pbd/scoped_file_descriptor.h"
 #include "pbd/strsplit.h"
 #include "pbd/replace_all.h"
 
@@ -475,27 +477,27 @@ ARDOUR::sync_source_to_string (SyncSource src, bool sh)
 		/* no other backends offer sync for now ... deal with this if we
 		 * ever have to.
 		 */
-		return _("JACK");
+		return S_("SyncSource|JACK");
 
 	case MTC:
 		if (sh) {
-			return _("MTC");
+			return S_("SyncSource|MTC");
 		} else {
 			return _("MIDI Timecode");
 		}
 
 	case MIDIClock:
 		if (sh) {
-			return _("M-Clock");
+			return S_("SyncSource|M-Clk");
 		} else {
 			return _("MIDI Clock");
 		}
 
 	case LTC:
-		return _("LTC");
+		return S_("SyncSource|LTC");
 	}
 	/* GRRRR .... stupid, stupid gcc - you can't get here from there, all enum values are handled */
-	return _("JACK");
+	return S_("SyncSource|JACK");
 }
 
 float
@@ -590,39 +592,6 @@ ARDOUR::auto_state_to_string (AutoState as)
 	return "";
 }
 
-AutoStyle
-ARDOUR::string_to_auto_style (std::string str)
-{
-	if (str == X_("Absolute")) {
-		return Absolute;
-	} else if (str == X_("Trim")) {
-		return Trim;
-	}
-
-	fatal << string_compose (_("programming error: %1 %2"), "illegal AutoStyle string: ", str) << endmsg;
-	abort(); /*NOTREACHED*/
-	return Trim;
-}
-
-string
-ARDOUR::auto_style_to_string (AutoStyle as)
-{
-	/* to be used only for XML serialization, no i18n done */
-
-	switch (as) {
-	case Absolute:
-		return X_("Absolute");
-		break;
-	case Trim:
-		return X_("Trim");
-		break;
-	}
-
-	fatal << string_compose (_("programming error: %1 %2"), "illegal AutoStyle type: ", as) << endmsg;
-	abort(); /*NOTREACHED*/
-	return "";
-}
-
 std::string
 bool_as_string (bool yn)
 {
@@ -742,13 +711,36 @@ ARDOUR::how_many_dsp_threads ()
 double
 ARDOUR::gain_to_slider_position_with_max (double g, double max_gain)
 {
-        return gain_to_slider_position (g * 2.0/max_gain);
+	return gain_to_position (g * 2.0 / max_gain);
 }
 
 double
 ARDOUR::slider_position_to_gain_with_max (double g, double max_gain)
 {
-	return slider_position_to_gain (g * max_gain/2.0);
+	return position_to_gain (g) * max_gain / 2.0;
+}
+
+#include "sha1.c"
+
+std::string
+ARDOUR::compute_sha1_of_file (std::string path)
+{
+	PBD::ScopedFileDescriptor fd (g_open (path.c_str(), O_RDONLY, 0444));
+	if (fd < 0) {
+		return std::string ();
+	}
+	char buf[4096];
+	ssize_t n_read;
+	char hash[41];
+	Sha1Digest s;
+	sha1_init (&s);
+
+	while ((n_read = ::read(fd, buf, sizeof(buf))) > 0) {
+		sha1_write (&s, (const uint8_t*) buf, n_read);
+	}
+
+	sha1_result_hash (&s, hash);
+	return std::string (hash);
 }
 
 extern "C" {
