@@ -649,10 +649,10 @@ vstfx_parse_vst_state (VSTState* vstfx)
 	switch (plugin->dispatcher (plugin, effGetPlugCategory, 0, 0, 0, 0))
 	{
 		case kPlugCategEffect:         info->Category = strdup ("Effect"); break;
-		case kPlugCategSynth:          info->Category = strdup ("Synth"); break;
-		case kPlugCategAnalysis:       info->Category = strdup ("Anaylsis"); break;
+		case kPlugCategSynth:          info->Category = strdup ("Instrument"); break;
+		case kPlugCategAnalysis:       info->Category = strdup ("Analyser"); break;
 		case kPlugCategMastering:      info->Category = strdup ("Mastering"); break;
-		case kPlugCategSpacializer:    info->Category = strdup ("Spacializer"); break;
+		case kPlugCategSpacializer:    info->Category = strdup ("Spatial"); break;
 		case kPlugCategRoomFx:         info->Category = strdup ("RoomFx"); break;
 		case kPlugSurroundFx:          info->Category = strdup ("SurroundFx"); break;
 		case kPlugCategRestoration:    info->Category = strdup ("Restoration"); break;
@@ -682,8 +682,8 @@ vstfx_parse_vst_state (VSTState* vstfx)
 #endif
 
 	for (int i = 0; i < info->numParams; ++i) {
-		char name[64];
-		char label[64];
+		char name[VestigeMaxLabelLen];
+		char label[VestigeMaxLabelLen];
 
 		/* Not all plugins give parameters labels as well as names */
 
