@@ -339,10 +339,23 @@ expected_message_length(unsigned char status, const unsigned char *second_byte,
 		}
 
 		/*
-		 * Format of this kind of messages is like this: 0xFF 0xwhatever 0xlength and then "length" bytes.
-		 * Second byte points to this:                        ^^^^^^^^^^
+		 * Format of this kind of messages is like this: 0xFF 0xTYPE 0xlength and then "length" bytes.
+		 * TYPE is < 127, length may be 0
+		 *
+		 * "lenght" is a 7bit value, the 8th bit is used to extend the length.
+		 * eg.  ff02 8266  <0x166 byte (C) message follows>
 		 */
-		return (*(second_byte + 1) + 3);
+		int32_t mlen = 0;
+		int32_t off;
+		for (off = 1; off < 4; ++off) {
+			uint8_t val = *(second_byte + off);
+			mlen = mlen << 7 | (val & 0x7f);
+			if (0 == (val & 0x80)) {
+				mlen += 2 + off;  // 2 byte "header" 0xff <type> + <length of length>
+				break;
+			}
+		}
+		return mlen;
 	}
 
 	if ((status & 0xF0) == 0xF0) {
@@ -653,14 +666,17 @@ make_string(const unsigned char *buf, const size_t buffer_length, uint32_t len)
 int
 smf_event_is_textual(const smf_event_t *event)
 {
-	if (!smf_event_is_metadata(event))
+	if (!smf_event_is_metadata(event)) {
 		return (0);
+	}
 
-	if (event->midi_buffer_length < 4)
+	if (event->midi_buffer_length < 4) {
 		return (0);
+	}
 
-	if (event->midi_buffer[3] < 1 || event->midi_buffer[3] > 9)
+	if (event->midi_buffer[1] < 1 || event->midi_buffer[1] > 7) {
 		return (0);
+	}
 
 	return (1);
 }
@@ -676,11 +692,8 @@ smf_event_extract_text(const smf_event_t *event)
 	uint32_t string_length = 0;
 	uint32_t length_length = 0;
 
-	if (!smf_event_is_textual(event))
-		return (NULL);
-
-	if (event->midi_buffer_length < 3) {
-		g_critical("smf_event_extract_text: truncated MIDI message.");
+	if (!smf_event_is_textual(event)) {
+		g_warning ("smf_event_extract_text: event is not textual.");
 		return (NULL);
 	}
 
@@ -812,8 +825,22 @@ parse_mtrk_chunk(smf_track_t *track)
 			break;
 		}
 
-		if (event_is_end_of_track(event))
+		if (event_is_end_of_track(event)) {
 			break;
+		}
+
+		if (smf_event_is_metadata (event)) {
+			switch (event->midi_buffer[1]) {
+			case 0x03:
+				track->name = smf_event_extract_text (event);
+				break;
+			case 0x04:
+				track->instrument = smf_event_extract_text (event);
+				break;
+			default:
+				break;
+			}
+		}
 	}
 
 	track->file_buffer = NULL;
