@@ -786,7 +786,7 @@ Bindings::save_all_bindings_as_html (ostream& ostr)
 
 	ostr << "<table border=\"2\" cellpadding=\"6\"><tbody>\n\n";
 	ostr << "<tr>\n\n";
-	
+
 	/* first column: separate by group */
 	ostr << "<td>\n\n";
 	for (list<Bindings*>::const_iterator b = bindings.begin(); b != bindings.end(); ++b) {
@@ -825,17 +825,21 @@ Bindings::save_all_bindings_as_html (ostream& ostr)
 
 		for (p = paths.begin(), k = keys.begin(), l = labels.begin(); p != paths.end(); ++k, ++p, ++l) {
 
+			string print_path = *p;
+			/* strip <Actions>/ from the start */
+			print_path = print_path.substr (10);
+
 			if ((*k).empty()) {
-				ostr << *p  << " ( " << *l << " ) "  << "</br>" << endl;
+				ostr << print_path  << " ( " << *l << " ) "  << "</br>" << endl;
 			} else {
-				ostr << *p << " ( " << *l << " ) " << " => " << *k << "</br>" << endl;
+				ostr << print_path << " ( " << *l << " ) " << " => " << *k << "</br>" << endl;
 			}
 		}
 	}
 	ostr << "</td>\n\n";
 	ostr << "</tr>\n\n";
 	ostr << "</tbody></table>\n\n";
-	
+
 	ostr << "</body>\n";
 	ostr << "</html>\n";
 }
@@ -857,7 +861,7 @@ Bindings::save_as_html (ostream& ostr, bool categorize) const
 		GroupMap group_map;
 
 		for (KeybindingMap::const_iterator k = press_bindings.begin(); k != press_bindings.end(); ++k) {
-			
+
 			if (k->first.name().empty()) {
 				continue;
 			}
@@ -879,13 +883,13 @@ Bindings::save_as_html (ostream& ostr, bool categorize) const
 			}
 		}
 
-		
+
 		for (GroupMap::const_iterator gm = group_map.begin(); gm != group_map.end(); ++gm) {
 
 			if (categorize) {
 				ostr << "<h3>" << gm->first << "</h3>\n";
 			}
-			
+
 			for (vector<KeybindingMap::const_iterator>::const_iterator k = gm->second.begin(); k != gm->second.end(); ++k) {
 
 				if ((*k)->first.name().empty()) {
@@ -934,7 +938,7 @@ Bindings::save_as_html (ostream& ostr, bool categorize) const
 						key_name.replace (pos, strlen (targets[n]), replacements[n]);
 					}
 				}
-				
+
 				key_name.append(" ");
 
 				while (key_name.length()<28)
@@ -946,7 +950,7 @@ Bindings::save_as_html (ostream& ostr, bool categorize) const
 			ostr << "\n\n";
 
 		}
-		
+
 		ostr << "\n";
 	}
 }
