@@ -99,7 +99,10 @@ typedef struct {
 	float v_knee;
 	float v_ratio;
 	float v_thresdb;
+	float v_gainr;
+	float v_makeup;
 	float v_lvl;
+	float v_lv1;
 	float v_lvl_in;
 	float v_lvl_out;
 #endif
@@ -126,6 +129,7 @@ instantiate(const LV2_Descriptor* descriptor,
 	acomp->tau = (1.0 - exp (-2.f * M_PI * 25.f / acomp->srate));
 #ifdef LV2_EXTENDED
 	acomp->need_expose = true;
+	acomp->v_lvl_out = -70.f;
 #endif
 
 	return (LV2_Handle)acomp;
@@ -283,14 +287,16 @@ run_mono(LV2_Handle instance, uint32_t n_samples)
 
 	float ratio = *acomp->ratio;
 	float thresdb = *acomp->thresdb;
-	float makeup_target = from_dB(*acomp->makeup);
+	float makeup = *acomp->makeup;
+	float makeup_target = from_dB(makeup);
 	float makeup_gain = acomp->makeup_gain;
 
-	const const float tau = acomp->tau;
+	const float tau = acomp->tau;
 
 	if (*acomp->enable <= 0) {
 		ratio = 1.f;
 		thresdb = 0.f;
+		makeup = 0.f;
 		makeup_target = 1.f;
 	}
 
@@ -309,9 +315,15 @@ run_mono(LV2_Handle instance, uint32_t n_samples)
 		acomp->v_thresdb = thresdb;
 		acomp->need_expose = true;
 	}
+
+	if (acomp->v_makeup != makeup) {
+		acomp->v_makeup = makeup;
+		acomp->need_expose = true;
+	}
 #endif
 
 	float in_peak = 0;
+	acomp->v_gainr = 0.0;
 
 	for (i = 0; i < n_samples; i++) {
 		in0 = input[i];
@@ -345,6 +357,9 @@ run_mono(LV2_Handle instance, uint32_t n_samples)
 		Lgain = from_dB(cdb);
 
 		*(acomp->gainr) = Lyl;
+		if (Lyl > acomp->v_gainr) {
+			acomp->v_gainr = Lyl;
+		}
 
 		lgaininp = in0 * Lgain;
 
@@ -364,7 +379,13 @@ run_mono(LV2_Handle instance, uint32_t n_samples)
 	acomp->makeup_gain = makeup_gain;
 
 #ifdef LV2_EXTENDED
-	acomp->v_lvl += .1 * (in_peak - acomp->v_lvl) + 1e-12;  // crude LPF TODO use n_samples/rate TC
+	const float old_v_lv1 = acomp->v_lv1;
+	const float old_v_lvl = acomp->v_lvl;
+	const float tot_rel_c = exp(-1000.f/(*(acomp->release) * srate) * n_samples);
+	const float tot_atk_c = exp(-1000.f/(*(acomp->attack) * srate) * n_samples);
+	acomp->v_lv1 = fmaxf (in_peak, tot_rel_c*old_v_lv1 + (1.f-tot_rel_c)*in_peak);
+	acomp->v_lvl = tot_atk_c*old_v_lvl + (1.f-tot_atk_c)*acomp->v_lv1;
+
 	if (!isfinite_local (acomp->v_lvl)) {
 		acomp->v_lvl = 0.f;
 	}
@@ -374,7 +395,8 @@ run_mono(LV2_Handle instance, uint32_t n_samples)
 		// >= 1dB difference
 		acomp->need_expose = true;
 		acomp->v_lvl_in = v_lvl_in;
-		acomp->v_lvl_out = v_lvl_out - to_dB(makeup_gain);
+		const float relax_coef = exp(-(float)n_samples/srate);
+		acomp->v_lvl_out = fmaxf (v_lvl_out, relax_coef*acomp->v_lvl_out + (1.f-relax_coef)*v_lvl_out);
 	}
 	if (acomp->need_expose && acomp->queue_draw) {
 		acomp->need_expose = false;
@@ -415,14 +437,16 @@ run_stereo(LV2_Handle instance, uint32_t n_samples)
 
 	float ratio = *acomp->ratio;
 	float thresdb = *acomp->thresdb;
-	float makeup_target = from_dB(*acomp->makeup);
+	float makeup = *acomp->makeup;
+	float makeup_target = from_dB(makeup);
 	float makeup_gain = acomp->makeup_gain;
 
-	const const float tau = acomp->tau;
+	const float tau = acomp->tau;
 
 	if (*acomp->enable <= 0) {
 		ratio = 1.f;
 		thresdb = 0.f;
+		makeup = 0.f;
 		makeup_target = 1.f;
 	}
 
@@ -441,9 +465,15 @@ run_stereo(LV2_Handle instance, uint32_t n_samples)
 		acomp->v_thresdb = thresdb;
 		acomp->need_expose = true;
 	}
+
+	if (acomp->v_makeup != makeup) {
+		acomp->v_makeup = makeup;
+		acomp->need_expose = true;
+	}
 #endif
 
 	float in_peak = 0;
+	acomp->v_gainr = 0.0;
 
 	for (i = 0; i < n_samples; i++) {
 		in0 = input0[i];
@@ -479,6 +509,9 @@ run_stereo(LV2_Handle instance, uint32_t n_samples)
 		Lgain = from_dB(cdb);
 
 		*(acomp->gainr) = Lyl;
+		if (Lyl > acomp->v_gainr) {
+			acomp->v_gainr = Lyl;
+		}
 
 		lgaininp = in0 * Lgain;
 		rgaininp = in1 * Lgain;
@@ -501,7 +534,12 @@ run_stereo(LV2_Handle instance, uint32_t n_samples)
 	acomp->makeup_gain = makeup_gain;
 
 #ifdef LV2_EXTENDED
-	acomp->v_lvl += .1 * (in_peak - acomp->v_lvl) + 1e-12;  // crude LPF TODO use n_samples/rate TC
+	const float old_v_lv1 = acomp->v_lv1;
+	const float old_v_lvl = acomp->v_lvl;
+	const float tot_rel_c = exp(-1000.f/(*(acomp->release) * srate) * n_samples);
+	const float tot_atk_c = exp(-1000.f/(*(acomp->attack) * srate) * n_samples);
+	acomp->v_lv1 = fmaxf (in_peak, tot_rel_c*old_v_lv1 + (1.f-tot_rel_c)*in_peak);
+	acomp->v_lvl = tot_atk_c*old_v_lvl + (1.f-tot_atk_c)*acomp->v_lv1;
 	if (!isfinite_local (acomp->v_lvl)) {
 		acomp->v_lvl = 0.f;
 	}
@@ -511,7 +549,8 @@ run_stereo(LV2_Handle instance, uint32_t n_samples)
 		// >= 1dB difference
 		acomp->need_expose = true;
 		acomp->v_lvl_in = v_lvl_in;
-		acomp->v_lvl_out = v_lvl_out - to_dB(makeup_gain);
+		const float relax_coef = exp(-2.0*n_samples/srate);
+		acomp->v_lvl_out = fmaxf (v_lvl_out, relax_coef*acomp->v_lvl_out + (1.f-relax_coef)*v_lvl_out);
 	}
 	if (acomp->need_expose && acomp->queue_draw) {
 		acomp->need_expose = false;
@@ -546,10 +585,11 @@ cleanup(LV2_Handle instance)
 
 #ifdef LV2_EXTENDED
 static float
-comp_curve (AComp* self, float xg) {
+comp_curve (const AComp* self, float xg) {
 	const float knee = self->v_knee;
 	const float ratio = self->v_ratio;
 	const float thresdb = self->v_thresdb;
+	const float makeup = self->v_makeup;
 
 	const float width = 6.f * knee + 0.01f;
 	float yg = 0.f;
@@ -561,23 +601,19 @@ comp_curve (AComp* self, float xg) {
 	} else {
 		yg = xg + (1.f / ratio - 1.f ) * (xg - thresdb + width / 2.f) * (xg - thresdb + width / 2.f) / (2.f * width);
 	}
+
+	yg += makeup;
+
 	return yg;
 }
 
-static LV2_Inline_Display_Image_Surface *
-render_inline (LV2_Handle instance, uint32_t w, uint32_t max_h)
+static void
+render_inline_full (cairo_t* cr, const AComp* self)
 {
-	AComp* self = (AComp*)instance;
-	uint32_t h = MIN (w, max_h);
+	const float w = self->w;
+	const float h = self->h;
 
-	if (!self->display || self->w != w || self->h != h) {
-		if (self->display) cairo_surface_destroy(self->display);
-		self->display = cairo_image_surface_create (CAIRO_FORMAT_ARGB32, w, h);
-		self->w = w;
-		self->h = h;
-	}
-
-	cairo_t* cr = cairo_create (self->display);
+	const float makeup_thres = self->v_thresdb + self->v_makeup;
 
 	// clear background
 	cairo_rectangle (cr, 0, 0, w, h);
@@ -594,9 +630,9 @@ render_inline (LV2_Handle instance, uint32_t w, uint32_t max_h)
 	cairo_set_dash(cr, dash2, 2, 2);
 	cairo_set_source_rgba (cr, 0.5, 0.5, 0.5, 0.5);
 
-	for (uint32_t d = 1; d < 6; ++d) {
-		const float x = -.5 + floorf (w * (d * 10.f / 60.f));
-		const float y = -.5 + floorf (h * (d * 10.f / 60.f));
+	for (uint32_t d = 1; d < 7; ++d) {
+		const float x = -.5 + floorf (w * (d * 10.f / 70.f));
+		const float y = -.5 + floorf (h * (d * 10.f / 70.f));
 
 		cairo_move_to (cr, x, 0);
 		cairo_line_to (cr, x, h);
@@ -606,29 +642,59 @@ render_inline (LV2_Handle instance, uint32_t w, uint32_t max_h)
 		cairo_line_to (cr, w, y);
 		cairo_stroke (cr);
 	}
+	cairo_set_source_rgba (cr, 0.5, 0.5, 0.5, 1.0);
+	cairo_set_dash(cr, dash1, 2, 2);
 	if (self->v_thresdb < 0) {
-		cairo_set_source_rgba (cr, 0.5, 0.5, 0.5, 1.0);
-		const float y = -.5 + floorf (h * (self->v_thresdb / -60.f));
-		cairo_set_dash(cr, dash1, 2, 2);
+		const float y = -.5 + floorf (h * ((makeup_thres - 10.f) / -70.f));
 		cairo_move_to (cr, 0, y);
 		cairo_line_to (cr, w, y);
 		cairo_stroke (cr);
-		cairo_move_to (cr, 0, h);
-		cairo_line_to (cr, w, 0);
-		cairo_stroke (cr);
 	}
+	// diagonal unity
+	cairo_move_to (cr, 0, h);
+	cairo_line_to (cr, w, 0);
+	cairo_stroke (cr);
 	cairo_restore (cr);
 
+	{ // 0, 0
+		cairo_set_source_rgba (cr, 0.5, 0.5, 0.5, 0.5);
+		const float x = -.5 + floorf (w * (60.f / 70.f));
+		const float y = -.5 + floorf (h * (10.f / 70.f));
+		cairo_move_to (cr, x, 0);
+		cairo_line_to (cr, x, h);
+		cairo_stroke (cr);
+		cairo_move_to (cr, 0, y);
+		cairo_line_to (cr, w, y);
+		cairo_stroke (cr);
+	}
+
+	{ // GR
+		const float x = -.5 + floorf (w * (62.5f / 70.f));
+		const float y = -.5 + floorf (h * (10.0f / 70.f));
+		const float wd = floorf (w * (5.f / 70.f));
+		const float ht = floorf (h * (55.f / 70.f));
+		cairo_rectangle (cr, x, y, wd, ht);
+		cairo_fill (cr);
+
+		const float h_gr = fminf (ht, floorf (h * self->v_gainr / 70.f));
+		cairo_set_source_rgba (cr, 0.95, 0.0, 0.0, 1.0);
+		cairo_rectangle (cr, x, y, wd, h_gr);
+		cairo_fill (cr);
+		cairo_set_source_rgba (cr, 0.5, 0.5, 0.5, 0.5);
+		cairo_rectangle (cr, x, y, wd, ht);
+		cairo_set_source_rgba (cr, 0.75, 0.75, 0.75, 1.0);
+		cairo_stroke (cr);
+	}
 
 	// draw curve
 	cairo_set_source_rgba (cr, .8, .8, .8, 1.0);
 	cairo_move_to (cr, 0, h);
 
 	for (uint32_t x = 0; x < w; ++x) {
-		// plot -60..0  dB
-		const float x_db = 60.f * (-1.f + x / (float)w);
-		const float y_db = comp_curve (self, x_db);
-		const float y = h * (y_db / -60.f);
+		// plot -60..+10  dB
+		const float x_db = 70.f * (-1.f + x / (float)w) + 10.f;
+		const float y_db = comp_curve (self, x_db) - 10.f;
+		const float y = h * (y_db / -70.f);
 		cairo_line_to (cr, x, y);
 	}
 	cairo_stroke_preserve (cr);
@@ -638,25 +704,25 @@ render_inline (LV2_Handle instance, uint32_t w, uint32_t max_h)
 	cairo_clip (cr);
 
 	// draw signal level & reduction/gradient
-	const float top = comp_curve (self, 0);
+	const float top = comp_curve (self, 0) - 10.f;
 	cairo_pattern_t* pat = cairo_pattern_create_linear (0.0, 0.0, 0.0, h);
-	if (top > self->v_thresdb) {
+	if (top > makeup_thres - 10.f) {
 		cairo_pattern_add_color_stop_rgba (pat, 0.0, 0.8, 0.1, 0.1, 0.5);
-		cairo_pattern_add_color_stop_rgba (pat, top / -60.f, 0.8, 0.1, 0.1, 0.5);
+		cairo_pattern_add_color_stop_rgba (pat, top / -70.f, 0.8, 0.1, 0.1, 0.5);
 	}
 	if (self->v_knee > 0) {
-		cairo_pattern_add_color_stop_rgba (pat, (self->v_thresdb / -60.f), 0.7, 0.7, 0.2, 0.5);
-		cairo_pattern_add_color_stop_rgba (pat, ((self->v_thresdb - self->v_knee) / -60.f), 0.5, 0.5, 0.5, 0.5);
+		cairo_pattern_add_color_stop_rgba (pat, ((makeup_thres -10.f) / -70.f), 0.7, 0.7, 0.2, 0.5);
+		cairo_pattern_add_color_stop_rgba (pat, ((makeup_thres - self->v_knee - 10.f) / -70.f), 0.5, 0.5, 0.5, 0.5);
 	} else {
-		cairo_pattern_add_color_stop_rgba (pat, (self->v_thresdb / -60.f), 0.7, 0.7, 0.2, 0.5);
-		cairo_pattern_add_color_stop_rgba (pat, ((self->v_thresdb - .01) / -60.f), 0.5, 0.5, 0.5, 0.5);
+		cairo_pattern_add_color_stop_rgba (pat, ((makeup_thres - 10.f)/ -70.f), 0.7, 0.7, 0.2, 0.5);
+		cairo_pattern_add_color_stop_rgba (pat, ((makeup_thres - 10.01f) / -70.f), 0.5, 0.5, 0.5, 0.5);
 	}
 	cairo_pattern_add_color_stop_rgba (pat, 1.0, 0.5, 0.5, 0.5, 0.5);
 
 	// maybe cut off at x-position?
-	const float x = w * (self->v_lvl_in + 60) / 60.f;
-	//const float y = h * (self->v_lvl_out + 60) / 60.f;
-	cairo_rectangle (cr, 0, h - x, x, h);
+	const float x = w * (self->v_lvl_in + 60) / 70.f;
+	const float y = x + h*self->v_makeup;
+	cairo_rectangle (cr, 0, h - y, x, y);
 	if (self->v_ratio > 1.0) {
 		cairo_set_source (cr, pat);
 	} else {
@@ -665,10 +731,133 @@ render_inline (LV2_Handle instance, uint32_t w, uint32_t max_h)
 	cairo_fill (cr);
 
 	cairo_pattern_destroy (pat); // TODO cache pattern
+}
+
+static void
+render_inline_only_bars (cairo_t* cr, const AComp* self)
+{
+	const float w = self->w;
+	const float h = self->h;
+
+	cairo_rectangle (cr, 0, 0, w, h);
+	cairo_set_source_rgba (cr, .2, .2, .2, 1.0);
+	cairo_fill (cr);
 
 
-	// create RGBA surface
+	cairo_save (cr);
+
+	const float ht = 0.25f * h;
+
+	const float x1 = w*0.05;
+	const float wd = w - 2.0f*x1;
+
+	const float y1 = 0.17*h;
+	const float y2 = h - y1 - ht;
+
+	cairo_set_source_rgba (cr, 0.5, 0.5, 0.5, 0.5);
+
+	cairo_rectangle (cr, x1, y1, wd, ht);
+	cairo_fill (cr);
+
+	cairo_rectangle (cr, x1, y2, wd, ht);
+	cairo_fill (cr);
+
+	cairo_set_source_rgba (cr, 0.75, 0.0, 0.0, 1.0);
+	const float w_gr = (self->v_gainr > 60.f) ? wd : wd * self->v_gainr * (1.f/60.f);
+	cairo_rectangle (cr, x1+wd-w_gr, y2, w_gr, ht);
+	cairo_fill (cr);
+
+	if (self->v_lvl_out > -60.f) {
+		if (self->v_lvl_out > 10.f) {
+			cairo_set_source_rgba (cr, 0.75, 0.0, 0.0, 1.0);
+		} else if (self->v_lvl_out > 0.f) {
+			cairo_set_source_rgba (cr, 0.66, 0.66, 0.0, 1.0);
+		} else {
+			cairo_set_source_rgba (cr, 0.0, 0.66, 0.0, 1.0);
+		}
+		const float w_g = (self->v_lvl_out > 10.f) ? wd : wd * (60.f+self->v_lvl_out) / 70.f;
+		cairo_rectangle (cr, x1, y1, w_g, ht);
+		cairo_fill (cr);
+	}
+
+	cairo_set_source_rgba (cr, 1.0, 1.0, 1.0, 1.0);
+
+	const float tck = 0.33*ht;
+
+	cairo_set_line_width (cr, .5);
+
+	for (uint32_t d = 1; d < 7; ++d) {
+		const float x = x1 + (d * wd * (10.f / 70.f));
+
+		cairo_move_to (cr, x, y1);
+		cairo_line_to (cr, x, y1+tck);
+
+		cairo_move_to (cr, x, y1+ht);
+		cairo_line_to (cr, x, y1+ht-tck);
+
+		cairo_move_to (cr, x, y2);
+		cairo_line_to (cr, x, y2+tck);
+
+		cairo_move_to (cr, x, y2+ht);
+		cairo_line_to (cr, x, y2+ht-tck);
+	}
+
+	cairo_stroke (cr);
+
+	const float x_0dB = x1 + wd*(60.f/70.f);
+
+	cairo_move_to (cr, x_0dB, y1);
+	cairo_line_to (cr, x_0dB, y1+ht);
+
+	cairo_rectangle (cr, x1, y1, wd, ht);
+	cairo_rectangle (cr, x1, y2, wd, ht);
+	cairo_stroke (cr);
+
+	cairo_set_line_width (cr, 2.0);
+
+	// visualize threshold
+	const float tr = x1 + wd * (60.f+self->v_thresdb) / 70.f;
+	cairo_set_source_rgba (cr, 0.95, 0.95, 0.0, 1.0);
+	cairo_move_to (cr, tr, y1);
+	cairo_line_to (cr, tr, y1+ht);
+	cairo_stroke (cr);
+
+	// visualize ratio
+	const float reduced_0dB = self->v_thresdb * (1.f - 1.f/self->v_ratio);
+	const float rt = x1 + wd * (60.f+reduced_0dB) / 70.f;
+	cairo_set_source_rgba (cr, 0.95, 0.0, 0.0, 1.0);
+	cairo_move_to (cr, rt, y1);
+	cairo_line_to (cr, rt, y1+ht);
+	cairo_stroke (cr);
+}
+
+static LV2_Inline_Display_Image_Surface *
+render_inline (LV2_Handle instance, uint32_t w, uint32_t max_h)
+{
+	AComp* self = (AComp*)instance;
+
+	uint32_t h = MIN (w, max_h);
+	if (w < 200) {
+		h = 40;
+	}
+
+	if (!self->display || self->w != w || self->h != h) {
+		if (self->display) cairo_surface_destroy(self->display);
+		self->display = cairo_image_surface_create (CAIRO_FORMAT_ARGB32, w, h);
+		self->w = w;
+		self->h = h;
+	}
+
+	cairo_t* cr = cairo_create (self->display);
+
+	if (w >= 200) {
+		render_inline_full (cr, self);
+	} else {
+		render_inline_only_bars (cr, self);
+	}
+
 	cairo_destroy (cr);
+
 	cairo_surface_flush (self->display);
 	self->surf.width = cairo_image_surface_get_width (self->display);
 	self->surf.height = cairo_image_surface_get_height (self->display);
