@@ -1633,6 +1633,18 @@ FaderPort8::select_strip (boost::weak_ptr<Stripable> ws)
 		return;
 	}
 #if 1 /* single exclusive selection by default, toggle via shift */
+
+# if 1 /* selecting a selected strip -> move fader to unity */
+	if (s == first_selected_stripable () && !shift_mod ()) {
+		if (_ctrls.fader_mode () == ModeTrack) {
+			boost::shared_ptr<AutomationControl> ac = s->gain_control ();
+			ac->start_touch (ac->session().transport_frame());
+			ac->set_value (ac->normal (), PBD::Controllable::UseGroup);
+		}
+		return;
+	}
+# endif
+
 	if (shift_mod ()) {
 		ToggleStripableSelection (s);
 	} else {
