@@ -28,6 +28,7 @@
 #include "pbd/error.h"
 #include "pbd/failed_constructor.h"
 #include "pbd/file_utils.h"
+#include "pbd/types_convert.h"
 #include "pbd/xml++.h"
 #include "pbd/compose.h"
 
@@ -198,13 +199,12 @@ GenericMidiControlProtocol::reload_maps ()
 
 		MapInfo mi;
 
-		XMLProperty const * prop = tree.root()->property ("name");
-
-		if (!prop) {
+		std::string str;
+		if (!tree.root()->get_property ("name", str)) {
 			continue;
 		}
 
-		mi.name = prop->value ();
+		mi.name = str;
 		mi.path = fullpath;
 
 		map_info.push_back (mi);
@@ -224,6 +224,10 @@ GenericMidiControlProtocol::drop_all ()
 	controllables.clear ();
 
 	for (MIDIPendingControllables::iterator i = pending_controllables.begin(); i != pending_controllables.end(); ++i) {
+		(*i)->connection.disconnect();
+		if ((*i)->own_mc) {
+			delete (*i)->mc;
+		}
 		delete *i;
 	}
 	pending_controllables.clear ();
@@ -341,6 +345,10 @@ GenericMidiControlProtocol::start_learning (Controllable* c)
 	Glib::Threads::Mutex::Lock lm2 (controllables_lock);
 	DEBUG_TRACE (DEBUG::GenericMidi, string_compose ("Learn binding: Controlable number: %1\n", c));
 
+	/* drop any existing mappings for the same controllable for which
+	 * learning has just started.
+	 */
+
 	MIDIControllables::iterator tmp;
 	for (MIDIControllables::iterator i = controllables.begin(); i != controllables.end(); ) {
 		tmp = i;
@@ -352,22 +360,24 @@ GenericMidiControlProtocol::start_learning (Controllable* c)
 		i = tmp;
 	}
 
+	/* check pending controllables (those for which a learn is underway) to
+	 * see if it is for the same one for which learning has just started.
+	 */
+
 	{
 		Glib::Threads::Mutex::Lock lm (pending_lock);
 
-		MIDIPendingControllables::iterator ptmp;
 		for (MIDIPendingControllables::iterator i = pending_controllables.begin(); i != pending_controllables.end(); ) {
-			ptmp = i;
-			++ptmp;
 			if (((*i)->mc)->get_controllable() == c) {
+				(*i)->connection.disconnect();
 				if ((*i)->own_mc) {
 					delete (*i)->mc;
 				}
-				(*i)->connection.disconnect();
 				delete *i;
-				pending_controllables.erase (i);
+				i = pending_controllables.erase (i);
+			} else {
+				++i;
 			}
-			i = ptmp;
 		}
 	}
 
@@ -386,6 +396,8 @@ GenericMidiControlProtocol::start_learning (Controllable* c)
 		own_mc = true;
 	}
 
+	/* stuff the new controllable into pending */
+
 	{
 		Glib::Threads::Mutex::Lock lm (pending_lock);
 
@@ -404,21 +416,20 @@ GenericMidiControlProtocol::learning_stopped (MIDIControllable* mc)
 	Glib::Threads::Mutex::Lock lm (pending_lock);
 	Glib::Threads::Mutex::Lock lm2 (controllables_lock);
 
-	MIDIPendingControllables::iterator tmp;
-
 	for (MIDIPendingControllables::iterator i = pending_controllables.begin(); i != pending_controllables.end(); ) {
-		tmp = i;
-		++tmp;
-
 		if ( (*i)->mc == mc) {
 			(*i)->connection.disconnect();
 			delete *i;
-			pending_controllables.erase(i);
+			i = pending_controllables.erase(i);
+		} else {
+			++i;
 		}
-
-		i = tmp;
 	}
 
+	/* add the controllable for which learning stopped to our list of
+	 * controllables
+	 */
+
 	controllables.push_back (mc);
 }
 
@@ -569,17 +580,13 @@ XMLNode&
 GenericMidiControlProtocol::get_state ()
 {
 	XMLNode& node (ControlProtocol::get_state());
-	char buf[32];
 
-	snprintf (buf, sizeof (buf), "%" PRIu64, _feedback_interval);
-	node.add_property (X_("feedback_interval"), buf);
-	snprintf (buf, sizeof (buf), "%d", _threshold);
-	node.add_property (X_("threshold"), buf);
-
-	node.add_property (X_("motorized"), _motorised ? "yes" : "no");
+	node.set_property (X_("feedback-interval"), _feedback_interval);
+	node.set_property (X_("threshold"), _threshold);
+	node.set_property (X_("motorized"), _motorised);
 
 	if (!_current_binding.empty()) {
-		node.add_property ("binding", _current_binding);
+		node.set_property ("binding", _current_binding);
 	}
 
 	XMLNode* children = new XMLNode (X_("Controls"));
@@ -607,31 +614,20 @@ GenericMidiControlProtocol::set_state (const XMLNode& node, int version)
 {
 	XMLNodeList nlist;
 	XMLNodeConstIterator niter;
-	const XMLProperty* prop;
 
 	if (ControlProtocol::set_state (node, version)) {
 		return -1;
 	}
 
-	if ((prop = node.property ("feedback_interval")) != 0) {
-		if (sscanf (prop->value().c_str(), "%" PRIu64, &_feedback_interval) != 1) {
-			_feedback_interval = 10000;
-		}
-	} else {
+	if (!node.get_property ("feedback-interval", _feedback_interval)) {
 		_feedback_interval = 10000;
 	}
 
-	if ((prop = node.property ("threshold")) != 0) {
-		if (sscanf (prop->value().c_str(), "%d", &_threshold) != 1) {
-			_threshold = 10;
-		}
-	} else {
+	if (!node.get_property ("threshold", _threshold)) {
 		_threshold = 10;
 	}
 
-	if ((prop = node.property ("motorized")) != 0) {
-		_motorised = string_is_affirmative (prop->value ());
-	} else {
+	if (!node.get_property ("motorized", _motorised)) {
 		_motorised = false;
 	}
 
@@ -640,15 +636,20 @@ GenericMidiControlProtocol::set_state (const XMLNode& node, int version)
 	{
 		Glib::Threads::Mutex::Lock lm (pending_lock);
 		for (MIDIPendingControllables::iterator i = pending_controllables.begin(); i != pending_controllables.end(); ++i) {
+			(*i)->connection.disconnect();
+			if ((*i)->own_mc) {
+				delete (*i)->mc;
+			}
 			delete *i;
 		}
 		pending_controllables.clear ();
 	}
 
+	std::string str;
 	// midi map has to be loaded first so learned binding can go on top
-	if ((prop = node.property ("binding")) != 0) {
+	if (node.get_property ("binding", str)) {
 		for (list<MapInfo>::iterator x = map_info.begin(); x != map_info.end(); ++x) {
-			if (prop->value() == (*x).name) {
+			if (str == (*x).name) {
 				load_bindings ((*x).path);
 				break;
 			}
@@ -669,9 +670,9 @@ GenericMidiControlProtocol::set_state (const XMLNode& node, int version)
 			if (!nlist.empty()) {
 				for (niter = nlist.begin(); niter != nlist.end(); ++niter) {
 
-					if ((prop = (*niter)->property ("id")) != 0) {
+					PBD::ID id;
+					if ((*niter)->get_property ("id", id)) {
 
-						ID id = prop->value ();
 						DEBUG_TRACE (DEBUG::GenericMidi, string_compose ("Relearned binding for session: Control ID: %1\n", id.to_s()));
 						Controllable* c = Controllable::by_id (id);
 
@@ -680,6 +681,9 @@ GenericMidiControlProtocol::set_state (const XMLNode& node, int version)
 
 							if (mc->set_state (**niter, version) == 0) {
 								controllables.push_back (mc);
+							} else {
+								warning << string_compose ("Generic MIDI control: Failed to set state for Control ID: %1\n", id.to_s());
+								delete mc;
 							}
 
 						} else {
@@ -732,13 +736,6 @@ GenericMidiControlProtocol::load_bindings (const string& xmlpath)
 
 	if ((prop = root->property ("version")) == 0) {
 		return -1;
-	} else {
-		int major;
-		int minor;
-		int micro;
-
-		sscanf (prop->value().c_str(), "%d.%d.%d", &major, &minor, &micro);
-		Stateful::loading_state_version = (major * 1000) + minor;
 	}
 
 	const XMLNodeList& children (root->children());
@@ -753,25 +750,18 @@ GenericMidiControlProtocol::load_bindings (const string& xmlpath)
 	for (citer = children.begin(); citer != children.end(); ++citer) {
 
 		if ((*citer)->name() == "DeviceInfo") {
-			const XMLProperty* prop;
 
-			if ((prop = (*citer)->property ("bank-size")) != 0) {
-				_bank_size = atoi (prop->value());
+			if ((*citer)->get_property ("bank-size", _bank_size)) {
 				_current_bank = 0;
 			}
 
-			if ((prop = (*citer)->property ("motorized")) != 0) {
-				_motorised = string_is_affirmative (prop->value ());
-			} else {
+			if (!(*citer)->get_property ("motorized", _motorised)) {
 				_motorised = false;
 			}
 
-			if ((prop = (*citer)->property ("threshold")) != 0) {
-				_threshold = atoi (prop->value ());
-			} else {
+			if (!(*citer)->get_property ("threshold", _threshold)) {
 				_threshold = 10;
 			}
-
 		}
 
 		if ((*citer)->name() == "Binding") {
@@ -780,8 +770,8 @@ GenericMidiControlProtocol::load_bindings (const string& xmlpath)
 			if (child->property ("uri")) {
 				/* controllable */
 
+				Glib::Threads::Mutex::Lock lm2 (controllables_lock);
 				if ((mc = create_binding (*child)) != 0) {
-					Glib::Threads::Mutex::Lock lm2 (controllables_lock);
 					controllables.push_back (mc);
 				}
 
@@ -795,12 +785,12 @@ GenericMidiControlProtocol::load_bindings (const string& xmlpath)
 				}
 
 			} else if (child->property ("action")) {
-                                MIDIAction* ma;
+				MIDIAction* ma;
 
 				if ((ma = create_action (*child)) != 0) {
 					actions.push_back (ma);
 				}
-                        }
+			}
 		}
 	}
 
@@ -881,7 +871,7 @@ GenericMidiControlProtocol::create_binding (const XMLNode& node)
 	}
 
 	if ((prop = node.property (X_("momentary"))) != 0) {
-		momentary = string_is_affirmative (prop->value());
+		momentary = string_to<bool> (prop->value());
 	} else {
 		momentary = false;
 	}
@@ -1244,3 +1234,13 @@ GenericMidiControlProtocol::input_port() const
 {
 	return _input_port;
 }
+
+void
+GenericMidiControlProtocol::maybe_start_touch (Controllable* controllable)
+{
+	AutomationControl *actl = dynamic_cast<AutomationControl*> (controllable);
+	if (actl) {
+		actl->start_touch (session->audible_frame ());
+	}
+}
+
