@@ -94,6 +94,7 @@ MackieControlProtocolGUI::MackieControlProtocolGUI (MackieControlProtocol& p)
 	, ipmidi_base_port_adjustment (_cp.ipmidi_base(), 0, 32767, 1, 1000)
 	, discover_button (_("Discover Mackie Devices"))
 	, _device_dependent_widget (0)
+	, _ignore_profile_changed (false)
 	, ignore_active_change (false)
 {
 	Gtk::Label* l;
@@ -333,6 +334,7 @@ MackieControlProtocolGUI::device_dependent_widget ()
 	int row = 0;
 
 	uint32_t n_surfaces = 1 + _cp.device_info().extenders();
+	uint32_t main_pos = _cp.device_info().master_position();
 
 	if (!_cp.device_info().uses_ipmidi()) {
 		dd_table = Gtk::manage (new Gtk::Table (n_surfaces, 2));
@@ -387,12 +389,12 @@ MackieControlProtocolGUI::device_dependent_widget ()
 			string receive_string;
 
 			if (n_surfaces > 1) {
-				if (n == 0) {
-					send_string = _("Main surface sends via:");
-					receive_string = _("Main surface receives via:");
+				if (n == main_pos) {
+					send_string = string_compose(_("Main surface at position %1 sends via:"), n + 1);
+					receive_string = string_compose(_("Main surface at position %1 receives via:"), n + 1);
 				} else {
-					send_string = string_compose (_("Extender %1 sends via:"), n);
-					receive_string = string_compose (_("Extender %1 receives via:"), n);
+					send_string = string_compose (_("Extender at position %1 sends via:"), n + 1);
+					receive_string = string_compose (_("Extender at position %1 receives via:"), n + 1);
 				}
 			} else {
 				send_string = _("Surface sends via:");
@@ -804,6 +806,10 @@ MackieControlProtocolGUI::action_changed (const Glib::ustring &sPath, const Glib
 				_cp.device_profile().set_button_action ((*row)[function_key_columns.id], modifier, i->second);
 			}
 
+			_ignore_profile_changed = true;
+			_profile_combo.set_active_text ( _cp.device_profile().name() );
+			_ignore_profile_changed = false;
+
 		} else {
 			std::cerr << "no such action\n";
 		}
@@ -833,11 +839,13 @@ MackieControlProtocolGUI::device_changed ()
 void
 MackieControlProtocolGUI::profile_combo_changed ()
 {
-	string profile = _profile_combo.get_active_text();
+	if (!_ignore_profile_changed) {
+		string profile = _profile_combo.get_active_text();
 
-	_cp.set_profile (profile);
+		_cp.set_profile (profile);
 
-	refresh_function_key_editor ();
+		refresh_function_key_editor ();
+	}
 }
 
 void
