@@ -138,7 +138,7 @@ MackieControlProtocol::MackieControlProtocol (Session& session)
 		_last_bank[i] = 0;
 	}
 
-	PresentationInfo::Change.connect (gui_connections, MISSING_INVALIDATOR, boost::bind (&MackieControlProtocol::notify_presentation_info_changed, this), this);
+	PresentationInfo::Change.connect (gui_connections, MISSING_INVALIDATOR, boost::bind (&MackieControlProtocol::notify_presentation_info_changed, this, _1), this);
 
 	_instance = this;
 
@@ -184,19 +184,12 @@ MackieControlProtocol::~MackieControlProtocol()
 void
 MackieControlProtocol::thread_init ()
 {
-	struct sched_param rtparam;
-
 	pthread_set_name (event_loop_name().c_str());
 
 	PBD::notify_event_loops_about_thread_creation (pthread_self(), event_loop_name(), 2048);
 	ARDOUR::SessionEvent::create_per_thread_pool (event_loop_name(), 128);
 
-	memset (&rtparam, 0, sizeof (rtparam));
-	rtparam.sched_priority = 9; /* XXX should be relative to audio (JACK) thread */
-
-	if (pthread_setschedparam (pthread_self(), SCHED_FIFO, &rtparam) != 0) {
-		// do we care? not particularly.
-	}
+	set_thread_priority ();
 }
 
 void
@@ -335,7 +328,7 @@ MackieControlProtocol::get_sorted_stripables()
 			}
 			break;
 		case Selected: // For example: a group (this is USER)
-			if (s->presentation_info().selected() && !s->presentation_info().hidden()) {
+			if (s->is_selected() && !s->presentation_info().hidden()) {
 				sorted.push_back (s);
 			}
 			break;
@@ -975,7 +968,7 @@ MackieControlProtocol::update_configuration_state ()
 	}
 
 	XMLNode* devnode = new XMLNode (X_("Configuration"));
-	devnode->add_property (X_("name"), _device_info.name());
+	devnode->set_property (X_("name"), _device_info.name());
 
 	configuration_state->remove_nodes_and_delete (X_("name"), _device_info.name());
 	configuration_state->add_child_nocopy (*devnode);
@@ -995,18 +988,15 @@ MackieControlProtocol::get_state()
 	XMLNode& node (ControlProtocol::get_state());
 
 	DEBUG_TRACE (DEBUG::MackieControl, "MackieControlProtocol::get_state init\n");
-	char buf[16];
 
 	// add current bank
-	snprintf (buf, sizeof (buf), "%d", _current_initial_bank);
-	node.add_property (X_("bank"), buf);
+	node.set_property (X_("bank"), _current_initial_bank);
 
 	// ipMIDI base port (possibly not used)
-	snprintf (buf, sizeof (buf), "%d", _ipmidi_base);
-	node.add_property (X_("ipmidi-base"), buf);
+	node.set_property (X_("ipmidi-base"), _ipmidi_base);
 
-	node.add_property (X_("device-profile"), _device_profile.name());
-	node.add_property (X_("device-name"), _device_info.name());
+	node.set_property (X_("device-profile"), _device_profile.name());
+	node.set_property (X_("device-name"), _device_info.name());
 
 	{
 		Glib::Threads::Mutex::Lock lm (surfaces_lock);
@@ -1032,29 +1022,27 @@ MackieControlProtocol::set_state (const XMLNode & node, int version)
 {
 	DEBUG_TRACE (DEBUG::MackieControl, string_compose ("MackieControlProtocol::set_state: active %1\n", active()));
 
-	int retval = 0;
-	const XMLProperty* prop;
-	uint32_t bank = 0;
-
 	if (ControlProtocol::set_state (node, version)) {
 		return -1;
 	}
 
-	if ((prop = node.property (X_("ipmidi-base"))) != 0) {
-		set_ipmidi_base (atoi (prop->value()));
+	uint16_t ipmidi_base;
+	if (node.get_property (X_("ipmidi-base"), ipmidi_base)) {
+		set_ipmidi_base (ipmidi_base);
 	}
 
+	uint32_t bank = 0;
 	// fetch current bank
-	if ((prop = node.property (X_("bank"))) != 0) {
-		bank = atoi (prop->value());
+	node.get_property (X_("bank"), bank);
+
+	std::string device_name;
+	if (node.get_property (X_("device-name"), device_name)) {
+		set_device_info (device_name);
 	}
 
-	if ((prop = node.property (X_("device-name"))) != 0) {
-		set_device_info (prop->value());
-	}
-
-	if ((prop = node.property (X_("device-profile"))) != 0) {
-		if (prop->value().empty()) {
+	std::string device_profile_name;
+	if (node.get_property (X_("device-profile"), device_profile_name)) {
+		if (device_profile_name.empty()) {
 			string default_profile_name;
 
 			/* start by looking for a user-edited profile for the current device name */
@@ -1084,8 +1072,8 @@ MackieControlProtocol::set_state (const XMLNode & node, int version)
 			set_profile (default_profile_name);
 
 		} else {
-			if (profile_exists (prop->value())) {
-				set_profile (prop->value());
+			if (profile_exists (device_profile_name)) {
+				set_profile (device_profile_name);
 			} else {
 				set_profile (DeviceProfile::default_profile_name);
 			}
@@ -1106,7 +1094,7 @@ MackieControlProtocol::set_state (const XMLNode & node, int version)
 
 	DEBUG_TRACE (DEBUG::MackieControl, "MackieControlProtocol::set_state done\n");
 
-	return retval;
+	return 0;
 }
 
 string
@@ -1297,8 +1285,17 @@ MackieControlProtocol::notify_solo_active_changed (bool active)
 }
 
 void
-MackieControlProtocol::notify_presentation_info_changed ()
+MackieControlProtocol::notify_presentation_info_changed (PBD::PropertyChange const & what_changed)
 {
+	PBD::PropertyChange order_or_hidden;
+
+	order_or_hidden.add (Properties::hidden);
+	order_or_hidden.add (Properties::order);
+
+	if (!what_changed.contains (order_or_hidden)) {
+		return;
+	}
+
 	{
 		Glib::Threads::Mutex::Lock lm (surfaces_lock);
 
@@ -1307,23 +1304,7 @@ MackieControlProtocol::notify_presentation_info_changed ()
 		}
 	}
 
-	Sorted sorted = get_sorted_stripables();
-	uint32_t sz = n_strips();
-
-	// if a remote id has been moved off the end, we need to shift
-	// the current bank backwards.
-
-	if (sorted.size() - _current_initial_bank < sz) {
-		// but don't shift backwards past the zeroth channel
-		if (sorted.size() < sz) {  // avoid unsigned math mistake below
-			(void) switch_banks(0, true);
-		} else {
-			(void) switch_banks (max((Sorted::size_type) 0, sorted.size() - sz), true);
-		}
-	} else {
-		// Otherwise just refresh the current bank
-		refresh_current_bank();
-	}
+	refresh_current_bank();
 }
 
 ///////////////////////////////////////////
@@ -1476,14 +1457,14 @@ MackieControlProtocol::build_button_map ()
 	DEFINE_BUTTON_HANDLER (Button::View, &MackieControlProtocol::view_press, &MackieControlProtocol::view_release);
 	DEFINE_BUTTON_HANDLER (Button::NameValue, &MackieControlProtocol::name_value_press, &MackieControlProtocol::name_value_release);
 	DEFINE_BUTTON_HANDLER (Button::TimecodeBeats, &MackieControlProtocol::timecode_beats_press, &MackieControlProtocol::timecode_beats_release);
-	DEFINE_BUTTON_HANDLER (Button::F1, &MackieControlProtocol::F1_press, &MackieControlProtocol::F1_release);
-	DEFINE_BUTTON_HANDLER (Button::F2, &MackieControlProtocol::F2_press, &MackieControlProtocol::F2_release);
-	DEFINE_BUTTON_HANDLER (Button::F3, &MackieControlProtocol::F3_press, &MackieControlProtocol::F3_release);
-	DEFINE_BUTTON_HANDLER (Button::F4, &MackieControlProtocol::F4_press, &MackieControlProtocol::F4_release);
-	DEFINE_BUTTON_HANDLER (Button::F5, &MackieControlProtocol::F5_press, &MackieControlProtocol::F5_release);
-	DEFINE_BUTTON_HANDLER (Button::F6, &MackieControlProtocol::F6_press, &MackieControlProtocol::F6_release);
-	DEFINE_BUTTON_HANDLER (Button::F7, &MackieControlProtocol::F7_press, &MackieControlProtocol::F7_release);
-	DEFINE_BUTTON_HANDLER (Button::F8, &MackieControlProtocol::F8_press, &MackieControlProtocol::F8_release);
+//	DEFINE_BUTTON_HANDLER (Button::F1, &MackieControlProtocol::F1_press, &MackieControlProtocol::F1_release);
+//	DEFINE_BUTTON_HANDLER (Button::F2, &MackieControlProtocol::F2_press, &MackieControlProtocol::F2_release);
+//	DEFINE_BUTTON_HANDLER (Button::F3, &MackieControlProtocol::F3_press, &MackieControlProtocol::F3_release);
+//	DEFINE_BUTTON_HANDLER (Button::F4, &MackieControlProtocol::F4_press, &MackieControlProtocol::F4_release);
+//	DEFINE_BUTTON_HANDLER (Button::F5, &MackieControlProtocol::F5_press, &MackieControlProtocol::F5_release);
+//	DEFINE_BUTTON_HANDLER (Button::F6, &MackieControlProtocol::F6_press, &MackieControlProtocol::F6_release);
+//	DEFINE_BUTTON_HANDLER (Button::F7, &MackieControlProtocol::F7_press, &MackieControlProtocol::F7_release);
+//	DEFINE_BUTTON_HANDLER (Button::F8, &MackieControlProtocol::F8_press, &MackieControlProtocol::F8_release);
 	DEFINE_BUTTON_HANDLER (Button::MidiTracks, &MackieControlProtocol::miditracks_press, &MackieControlProtocol::miditracks_release);
 	DEFINE_BUTTON_HANDLER (Button::Inputs, &MackieControlProtocol::inputs_press, &MackieControlProtocol::inputs_release);
 	DEFINE_BUTTON_HANDLER (Button::AudioTracks, &MackieControlProtocol::audiotracks_press, &MackieControlProtocol::audiotracks_release);
@@ -1539,14 +1520,6 @@ MackieControlProtocol::handle_button_event (Surface& surface, Button& button, Bu
 		return;
 	}
 
-	if ((button_id != Button::Marker) && (modifier_state() & MODIFIER_MARKER)) {
-		marker_modifier_consumed_by_button = true;
-	}
-
-	if ((button_id != Button::Nudge) && (modifier_state() & MODIFIER_NUDGE)) {
-		nudge_modifier_consumed_by_button = true;
-	}
-
 	DEBUG_TRACE (DEBUG::MackieControl, string_compose ("Handling %1 for button %2 (%3)\n", (bs == press ? "press" : "release"), button.id(),
 							   Button::id_to_name (button.bid())));
 
@@ -1554,6 +1527,8 @@ MackieControlProtocol::handle_button_event (Surface& surface, Button& button, Bu
 
 	string action = _device_profile.get_button_action (button.bid(), _modifier_state);
 
+	DEBUG_TRACE (DEBUG::MackieControl, string_compose ("device profile returned [%1] for that button\n", action));
+
 	if (!action.empty()) {
 
 		if (action.find ('/') != string::npos) { /* good chance that this is really an action */
@@ -1567,10 +1542,12 @@ MackieControlProtocol::handle_button_event (Surface& surface, Button& button, Bu
 			   occur either.
 			*/
 			if (bs == press) {
+				update_led (surface, button, on);
 				DEBUG_TRACE (DEBUG::MackieControl, string_compose ("executing action %1\n", action));
 				access_action (action);
+			} else {
+				update_led (surface, button, off);
 			}
-
 			return;
 
 		} else {
@@ -1594,14 +1571,30 @@ MackieControlProtocol::handle_button_event (Surface& surface, Button& button, Bu
 		}
 	}
 
+	/* Now that we have the correct (maybe remapped) button ID, do these
+	 * checks on it.
+	 */
+
+	if ((button_id != Button::Marker) && (modifier_state() & MODIFIER_MARKER)) {
+		marker_modifier_consumed_by_button = true;
+	}
+
+	if ((button_id != Button::Nudge) && (modifier_state() & MODIFIER_NUDGE)) {
+		nudge_modifier_consumed_by_button = true;
+	}
+
 	/* lookup using the device-INDEPENDENT button ID */
 
+	DEBUG_TRACE (DEBUG::MackieControl, string_compose ("now looking up button ID %1\n", button_id));
+
 	ButtonMap::iterator b = button_map.find (button_id);
 
 	if (b != button_map.end()) {
 
 		ButtonHandlers& bh (b->second);
 
+		DEBUG_TRACE (DEBUG::MackieControl, string_compose ("button found in map, now invoking %1\n", (bs == press ? "press" : "release")));
+
 		switch  (bs) {
 		case press:
 			surface.write (button.set_state ((this->*(bh.press)) (button)));
@@ -2033,13 +2026,14 @@ MackieControlProtocol::select_range (uint32_t pressed)
 
 	pull_stripable_range (_down_select_buttons, stripables, pressed);
 
-	DEBUG_TRACE (DEBUG::MackieControl, string_compose ("select range: found %1 stripables, first = %2\n", stripables.size(), stripables.front()->name()));
+	DEBUG_TRACE (DEBUG::MackieControl, string_compose ("select range: found %1 stripables, first = %2\n", stripables.size(),
+	                                                   (stripables.empty() ? "null" : stripables.front()->name())));
 
 	if (stripables.empty()) {
 		return;
 	}
 
-	if (stripables.size() == 1 && ControlProtocol::last_selected().size() == 1 && stripables.front()->presentation_info().selected()) {
+	if (stripables.size() == 1 && ControlProtocol::last_selected().size() == 1 && stripables.front()->is_selected()) {
 		/* cancel selection for one and only selected stripable */
 		ToggleStripableSelection (stripables.front());
 	} else {
@@ -2368,10 +2362,15 @@ MackieControlProtocol::is_mapped (boost::shared_ptr<Stripable> r) const
 }
 
 void
-MackieControlProtocol::update_selected (boost::shared_ptr<Stripable> s, bool became_selected)
+MackieControlProtocol::stripable_selection_changed ()
 {
-	if (became_selected) {
+	//this function is called after the stripable selection is "stable", so this is the place to check surface selection state
+	for (Surfaces::iterator si = surfaces.begin(); si != surfaces.end(); ++si) {
+		(*si)->update_strip_selection ();
+	}
 
+	boost::shared_ptr<Stripable> s = first_selected_stripable ();
+	if (s) {
 		check_fader_automation_state ();
 
 		/* It is possible that first_selected_route() may return null if we
@@ -2381,7 +2380,7 @@ MackieControlProtocol::update_selected (boost::shared_ptr<Stripable> s, bool bec
 		 * set_subview_mode() will fail, and we will reset to None.
 		 */
 
-		if (set_subview_mode (_subview_mode, first_selected_stripable())) {
+		if (set_subview_mode (_subview_mode, s)) {
 			set_subview_mode (None, boost::shared_ptr<Stripable>());
 		}
 
