@@ -140,7 +140,6 @@ class MackieControlProtocol
 	bool is_midi_track (boost::shared_ptr<ARDOUR::Stripable>) const;
 	bool is_mapped (boost::shared_ptr<ARDOUR::Stripable>) const;
 	boost::shared_ptr<ARDOUR::Stripable> first_selected_stripable () const;
-	void update_selected (boost::shared_ptr<ARDOUR::Stripable>, bool selected);
 
 	void check_fader_automation_state ();
 	void update_fader_automation_state ();
@@ -190,7 +189,7 @@ class MackieControlProtocol
 	void notify_routes_added (ARDOUR::RouteList &);
 	void notify_vca_added (ARDOUR::VCAList &);
 
-	void notify_presentation_info_changed();
+	void notify_presentation_info_changed(PBD::PropertyChange const &);
 
 	void recalibrate_faders ();
 	void toggle_backlight ();
@@ -360,8 +359,7 @@ class MackieControlProtocol
 	void clear_surfaces ();
 	void force_special_stripable_to_strip (boost::shared_ptr<ARDOUR::Stripable> r, uint32_t surface, uint32_t strip_number);
 	void build_button_map ();
-	void gui_track_selection_changed (ARDOUR::StripableNotificationListPtr, bool save_list);
-	void _gui_track_selection_changed (ARDOUR::StripableNotificationList*, bool save_list, bool gui_did_change);
+	void stripable_selection_changed ();
 	int ipmidi_restart ();
         void initialize ();
         int set_device_info (const std::string& device_name);
@@ -443,22 +441,22 @@ class MackieControlProtocol
 	Mackie::LedState flip_release (Mackie::Button &);
 	Mackie::LedState name_value_press (Mackie::Button &);
 	Mackie::LedState name_value_release (Mackie::Button &);
-	Mackie::LedState F1_press (Mackie::Button &);
-	Mackie::LedState F1_release (Mackie::Button &);
-	Mackie::LedState F2_press (Mackie::Button &);
-	Mackie::LedState F2_release (Mackie::Button &);
-	Mackie::LedState F3_press (Mackie::Button &);
-	Mackie::LedState F3_release (Mackie::Button &);
-	Mackie::LedState F4_press (Mackie::Button &);
-	Mackie::LedState F4_release (Mackie::Button &);
-	Mackie::LedState F5_press (Mackie::Button &);
-	Mackie::LedState F5_release (Mackie::Button &);
-	Mackie::LedState F6_press (Mackie::Button &);
-	Mackie::LedState F6_release (Mackie::Button &);
-	Mackie::LedState F7_press (Mackie::Button &);
-	Mackie::LedState F7_release (Mackie::Button &);
-	Mackie::LedState F8_press (Mackie::Button &);
-	Mackie::LedState F8_release (Mackie::Button &);
+//	Mackie::LedState F1_press (Mackie::Button &);
+//	Mackie::LedState F1_release (Mackie::Button &);
+//	Mackie::LedState F2_press (Mackie::Button &);
+//	Mackie::LedState F2_release (Mackie::Button &);
+//	Mackie::LedState F3_press (Mackie::Button &);
+//	Mackie::LedState F3_release (Mackie::Button &);
+//	Mackie::LedState F4_press (Mackie::Button &);
+//	Mackie::LedState F4_release (Mackie::Button &);
+//	Mackie::LedState F5_press (Mackie::Button &);
+//	Mackie::LedState F5_release (Mackie::Button &);
+//	Mackie::LedState F6_press (Mackie::Button &);
+//	Mackie::LedState F6_release (Mackie::Button &);
+//	Mackie::LedState F7_press (Mackie::Button &);
+//	Mackie::LedState F7_release (Mackie::Button &);
+//	Mackie::LedState F8_press (Mackie::Button &);
+//	Mackie::LedState F8_release (Mackie::Button &);
 	Mackie::LedState touch_press (Mackie::Button &);
 	Mackie::LedState touch_release (Mackie::Button &);
 	Mackie::LedState enter_press (Mackie::Button &);
