@@ -37,6 +37,8 @@
 #include "pbd/abstract_ui.h"
 
 #include "ardour/types.h"
+#include "ardour/send.h"
+#include "ardour/plugin.h"
 #include "control_protocol/control_protocol.h"
 
 #include "pbd/i18n.h"
@@ -45,6 +47,7 @@ class OSCControllable;
 class OSCRouteObserver;
 class OSCGlobalObserver;
 class OSCSelectObserver;
+class OSCCueObserver;
 
 namespace ARDOUR {
 class Session;
@@ -75,15 +78,14 @@ class OSC : public ARDOUR::ControlProtocol, public AbstractUI<OSCUIRequest>
 	XMLNode& get_state ();
 	int set_state (const XMLNode&, int version);
 
+	void stripable_selection_changed () {}
+
 	bool has_editor () const { return true; }
 	void* get_gui () const;
 	void  tear_down_gui ();
 
 	int set_active (bool yn);
 	bool get_active () const;
-	int set_feedback (bool yn);
-	bool get_feedback () const;
-
 
 	int start ();
 	int stop ();
@@ -96,25 +98,50 @@ class OSC : public ARDOUR::ControlProtocol, public AbstractUI<OSCUIRequest>
 		All
 	};
 
+	enum JogMode {
+		JOG,
+		NUDGE,
+		SCRUB,
+		SHUTTLE,
+		MARKER,
+		SCROLL,
+		TRACK,
+		BANK
+	};
+
 	typedef std::vector<boost::shared_ptr<ARDOUR::Stripable> > Sorted;
-	Sorted get_sorted_stripables(std::bitset<32> types);
-	// cue
-	//Sorted cue_get_sorted_stripables(boost::shared_ptr<Route> aux);
+	Sorted get_sorted_stripables(std::bitset<32> types, bool cue);
+	typedef std::map<boost::shared_ptr<ARDOUR::AutomationControl>, uint32_t> FakeTouchMap;
+	FakeTouchMap _touch_timeout;
 
 // keep a surface's global setup by remote server url
 	struct OSCSurface {
 	public:
 		std::string remote_url;		// the url these setting belong to
+		bool no_clear;				// don't send osc clear messages on strip change
+		JogMode jogmode;			// current jogmode
 		uint32_t bank;				// current bank
 		uint32_t bank_size;			// size of banks for this surface
+		int plug_page;				// current plugin page
+		uint32_t plug_page_size;	// plugin page size (number of controls)
+		int plugin_id;			// id of current plugin
+		std::vector<int> plug_params; // vector to store ports that are controls
+		std::vector<int> plugins;	// stores allowable plugins with index (work around MB strip PIs)
+		int send_page;				// current send page
+		uint32_t send_page_size;	// send page size in channels
+		PBD::ScopedConnection proc_connection; // for processor signal monitoring
 		std::bitset<32> strip_types;// what strip types are a part of this bank
 		uint32_t nstrips;			// how many strips are there for strip_types
 		std::bitset<32> feedback;	// What is fed back? strips/meters/timecode/bar_beat/global
 		int gainmode;				// what kind of faders do we have Gain db or position 0 to 1?
+		PBD::Controllable::GroupControlDisposition usegroup;	// current group disposition
 		uint32_t expand;			// Used by /select/select
 		bool expand_enable;			// use expand instead of select
 		OSCSelectObserver* sel_obs;	// So we can sync select feedback with selected channel
 		Sorted strips;				// list of stripables for this surface
+		bool cue;					// is this a cue surface
+		uint32_t aux;				// aux index for this cue surface
+		Sorted sends;				// list of sends for cue aux
 	};
 		/*
 		 * feedback bits:
@@ -132,10 +159,12 @@ class OSC : public ARDOUR::ControlProtocol, public AbstractUI<OSCUIRequest>
 		 * [11] - Send Playhead position as minutes seconds
 		 * [12]	- Send Playhead position like primary/secondary GUI clocks
 		 * [13] - Send well known feedback (for /select/command
+		 * [14] - use OSC 1.0 only (#reply -> /reply)
 		 */
 
 
 // storage for  each surface's settings
+	mutable Glib::Threads::Mutex surfaces_lock;
 	typedef std::vector<OSCSurface> Surface;
 	Surface _surface;
 
@@ -152,6 +181,10 @@ class OSC : public ARDOUR::ControlProtocol, public AbstractUI<OSCUIRequest>
 	void set_defaultstrip (int st) { default_strip = st; }
 	int get_defaultfeedback() { return default_feedback; }
 	void set_defaultfeedback (int fb) { default_feedback = fb; }
+	int get_send_size() { return default_send_size; }
+	void set_send_size (int ss) { default_send_size = ss; }
+	int get_plugin_size() { return default_plugin_size; }
+	void set_plugin_size (int ps) { default_plugin_size = ps; }
 	void clear_devices ();
 	void gui_changed ();
 	std::string get_remote_port () { return remote_port; }
@@ -174,7 +207,6 @@ class OSC : public ARDOUR::ControlProtocol, public AbstractUI<OSCUIRequest>
 	lo_server _osc_unix_server;
 	std::string _osc_unix_socket_path;
 	std::string _osc_url_file;
-	bool _send_route_changes;
 	OSCDebugMode _debugmode;
 	bool address_only;
 	std::string remote_port;
@@ -182,8 +214,13 @@ class OSC : public ARDOUR::ControlProtocol, public AbstractUI<OSCUIRequest>
 	uint32_t default_strip;
 	uint32_t default_feedback;
 	uint32_t default_gainmode;
+	uint32_t default_send_size;
+	uint32_t default_plugin_size;
 	bool tick;
 	bool bank_dirty;
+	float scrub_speed;		// Current scrub speed
+	double scrub_place;		// place of play head at latest jog/scrub wheel tick
+	int64_t scrub_time;		// when did the wheel move last?
 	bool global_init;
 	boost::shared_ptr<ARDOUR::Stripable> _select;	// which stripable out of /surface/stripables is gui selected
 
@@ -200,9 +237,10 @@ class OSC : public ARDOUR::ControlProtocol, public AbstractUI<OSCUIRequest>
 	std::string get_unix_server_url ();
 	lo_address get_address (lo_message msg);
 	OSCSurface * get_surface (lo_address addr);
+	int check_surface (lo_message msg);
 	uint32_t get_sid (boost::shared_ptr<ARDOUR::Stripable> strip, lo_address addr);
 	boost::shared_ptr<ARDOUR::Stripable> get_strip (uint32_t ssid, lo_address addr);
-	void global_feedback (std::bitset<32> feedback, lo_address addr, uint32_t gainmode);
+	void global_feedback (OSCSurface sur, lo_address addr);
 
 	void send_current_value (const char* path, lo_arg** argv, int argc, lo_message msg);
 	void current_value_query (const char* path, size_t len, lo_arg **argv, int argc, lo_message msg);
@@ -212,6 +250,10 @@ class OSC : public ARDOUR::ControlProtocol, public AbstractUI<OSCUIRequest>
 	int catchall (const char *path, const char *types, lo_arg **argv, int argc, void *data);
 	static int _catchall (const char *path, const char *types, lo_arg **argv, int argc, void *data, void *user_data);
 
+	int set_automation (const char *path, const char* types, lo_arg **argv, int argc, lo_message msg);
+	int touch_detect (const char *path, const char* types, lo_arg **argv, int argc, lo_message msg);
+	int fake_touch (boost::shared_ptr<ARDOUR::AutomationControl> ctrl);
+
 	int route_get_sends (lo_message msg);
 	int route_get_receives(lo_message msg);
 	void routes_list (lo_message msg);
@@ -219,6 +261,26 @@ class OSC : public ARDOUR::ControlProtocol, public AbstractUI<OSCUIRequest>
 	void transport_speed (lo_message msg);
 	void record_enabled (lo_message msg);
 
+	// cue
+	Sorted cue_get_sorted_stripables(boost::shared_ptr<ARDOUR::Stripable> aux, uint32_t id, lo_message msg);
+	int cue_parse (const char *path, const char* types, lo_arg **argv, int argc, lo_message msg);
+	int cue_set (uint32_t aux, lo_message msg);
+	int _cue_set (uint32_t aux, lo_address addr);
+	int cue_next (lo_message msg);
+	int cue_previous (lo_message msg);
+	int cue_send_fader (uint32_t id, float position, lo_message msg);
+	int cue_send_enable (uint32_t id, float state, lo_message msg);
+	int cue_aux_fader (float position, lo_message msg);
+	int cue_aux_mute (float state, lo_message msg);
+	void cue_set_aux (uint32_t aux, lo_message msg);
+	int cue_float_message (std::string, float value, lo_address addr);
+	int text_message (std::string path, std::string val, lo_address addr);
+	boost::shared_ptr<ARDOUR::Send> cue_get_send (uint32_t id, lo_address addr);
+	// end cue
+
+	int select_plugin_parameter (const char *path, const char* types, lo_arg **argv, int argc, lo_message msg);
+	int surface_parse (const char *path, const char* types, lo_arg **argv, int argc, lo_message msg);
+
 #define OSC_DEBUG \
 	if (_debugmode == All) { \
 		debugmsg (dgettext(PACKAGE, "OSC"), path, types, argv, argc); \
@@ -244,13 +306,15 @@ class OSC : public ARDOUR::ControlProtocol, public AbstractUI<OSCUIRequest>
 	PATH_CALLBACK_MSG(refresh_surface);
 	PATH_CALLBACK_MSG(bank_up);
 	PATH_CALLBACK_MSG(bank_down);
+	PATH_CALLBACK_MSG(master_select);
 
 #define PATH_CALLBACK(name) \
 	static int _ ## name (const char *path, const char *types, lo_arg **argv, int argc, void *data, void *user_data) { \
 		return static_cast<OSC*>(user_data)->cb_ ## name (path, types, argv, argc, data); \
 	} \
-	int cb_ ## name (const char *path, const char *types, lo_arg ** argv, int argc, void *) { \
+	int cb_ ## name (const char *path, const char *types, lo_arg ** argv, int argc, void *data) { \
 		OSC_DEBUG; \
+		check_surface (data); \
 		if (argc > 0 && !strcmp (types, "f") && argv[0]->f != 1.0) { return 0; } \
 		name (); \
 		return 0; \
@@ -316,8 +380,9 @@ class OSC : public ARDOUR::ControlProtocol, public AbstractUI<OSCUIRequest>
 	static int _ ## name (const char *path, const char *types, lo_arg **argv, int argc, void *data, void *user_data) { \
 		return static_cast<OSC*>(user_data)->cb_ ## name (path, types, argv, argc, data); \
 	} \
-	int cb_ ## name (const char *path, const char *types, lo_arg **argv, int argc, void *) { \
+	int cb_ ## name (const char *path, const char *types, lo_arg **argv, int argc, void *data) { \
 		OSC_DEBUG; \
+		check_surface (data); \
 		if (argc > 0) { \
 			name (optional argv[0]->type); \
 		} \
@@ -331,10 +396,15 @@ class OSC : public ARDOUR::ControlProtocol, public AbstractUI<OSCUIRequest>
 	PATH_CALLBACK1(jump_by_seconds,f,);
 	PATH_CALLBACK1(master_set_gain,f,);
 	PATH_CALLBACK1(master_set_fader,f,);
+	PATH_CALLBACK1(master_delta_gain,f,);
 	PATH_CALLBACK1(master_set_trim,f,);
 	PATH_CALLBACK1(master_set_mute,i,);
 	PATH_CALLBACK1(monitor_set_gain,f,);
 	PATH_CALLBACK1(monitor_set_fader,f,);
+	PATH_CALLBACK1(monitor_delta_gain,f,);
+	PATH_CALLBACK1(monitor_set_mute,i,);
+	PATH_CALLBACK1(monitor_set_dim,i,);
+	PATH_CALLBACK1(monitor_set_mono,i,);
 
 #define PATH_CALLBACK1_MSG(name,arg1type) \
 	static int _ ## name (const char *path, const char *types, lo_arg **argv, int argc, void *data, void *user_data) { \
@@ -351,13 +421,15 @@ class OSC : public ARDOUR::ControlProtocol, public AbstractUI<OSCUIRequest>
 	// pan position needs message info to send feedback
 	PATH_CALLBACK1_MSG(master_set_pan_stereo_position,f);
 
-	PATH_CALLBACK1_MSG(set_surface_bank_size,i);
-	PATH_CALLBACK1_MSG(set_surface_strip_types,i);
-	PATH_CALLBACK1_MSG(set_surface_feedback,i);
-	PATH_CALLBACK1_MSG(set_surface_gainmode,i);
+	PATH_CALLBACK1_MSG(scrub,f);
+	PATH_CALLBACK1_MSG(jog,f);
+	PATH_CALLBACK1_MSG(jog_mode,f);
+	PATH_CALLBACK1_MSG(bank_delta,f);
+	PATH_CALLBACK1_MSG(use_group,f);
 	PATH_CALLBACK1_MSG(sel_recenable,i);
 	PATH_CALLBACK1_MSG(sel_recsafe,i);
 	PATH_CALLBACK1_MSG(sel_mute,i);
+	PATH_CALLBACK1_MSG(sel_master_send_enable,i);
 	PATH_CALLBACK1_MSG(sel_solo,i);
 	PATH_CALLBACK1_MSG(sel_solo_iso,i);
 	PATH_CALLBACK1_MSG(sel_solo_safe,i);
@@ -366,27 +438,37 @@ class OSC : public ARDOUR::ControlProtocol, public AbstractUI<OSCUIRequest>
 	PATH_CALLBACK1_MSG(sel_phase,i);
 	PATH_CALLBACK1_MSG(sel_gain,f);
 	PATH_CALLBACK1_MSG(sel_fader,f);
+	PATH_CALLBACK1_MSG(sel_dB_delta,f);
 	PATH_CALLBACK1_MSG(sel_trim,f);
 	PATH_CALLBACK1_MSG(sel_pan_position,f);
 	PATH_CALLBACK1_MSG(sel_pan_width,f);
 	PATH_CALLBACK1_MSG(sel_pan_elevation,f);
 	PATH_CALLBACK1_MSG(sel_pan_frontback,f);
 	PATH_CALLBACK1_MSG(sel_pan_lfe,f);
+	PATH_CALLBACK1_MSG(sel_send_page,f);
+	PATH_CALLBACK1_MSG(sel_plug_page,f);
+	PATH_CALLBACK1_MSG(sel_plugin,f);
 	PATH_CALLBACK1_MSG(sel_comp_enable,f);
 	PATH_CALLBACK1_MSG(sel_comp_threshold,f);
 	PATH_CALLBACK1_MSG(sel_comp_speed,f);
 	PATH_CALLBACK1_MSG(sel_comp_mode,f);
 	PATH_CALLBACK1_MSG(sel_comp_makeup,f);
 	PATH_CALLBACK1_MSG(sel_eq_enable,f);
-	PATH_CALLBACK1_MSG(sel_eq_hpf,f);
+	PATH_CALLBACK1_MSG(sel_eq_hpf_freq,f);
+	PATH_CALLBACK1_MSG(sel_eq_hpf_enable,f);
+	PATH_CALLBACK1_MSG(sel_eq_hpf_slope,f);
+	PATH_CALLBACK1_MSG(sel_eq_lpf_freq,f);
+	PATH_CALLBACK1_MSG(sel_eq_lpf_enable,f);
+	PATH_CALLBACK1_MSG(sel_eq_lpf_slope,f);
 	PATH_CALLBACK1_MSG(sel_expand,i);
 
 #define PATH_CALLBACK2(name,arg1type,arg2type) \
 	static int _ ## name (const char *path, const char *types, lo_arg **argv, int argc, void *data, void *user_data) { \
 		return static_cast<OSC*>(user_data)->cb_ ## name (path, types, argv, argc, data); \
 	} \
-	int cb_ ## name (const char *path, const char *types, lo_arg **argv, int argc, void *) { \
+	int cb_ ## name (const char *path, const char *types, lo_arg **argv, int argc, void *data) { \
 		OSC_DEBUG; \
+		check_surface (data); \
 		if (argc > 1) { \
 			name (argv[0]->arg1type, argv[1]->arg2type); \
 		} \
@@ -449,7 +531,6 @@ class OSC : public ARDOUR::ControlProtocol, public AbstractUI<OSCUIRequest>
 	PATH_CALLBACK2_MSG(sel_eq_q,i,f);
 	PATH_CALLBACK2_MSG(sel_eq_shape,i,f);
 
-	PATH_CALLBACK4(set_surface,i,i,i,i);
 	PATH_CALLBACK2(locate,i,i);
 	PATH_CALLBACK2(loop_location,i,i);
 	PATH_CALLBACK2_MSG_s(route_rename,i,s);
@@ -496,6 +577,7 @@ class OSC : public ARDOUR::ControlProtocol, public AbstractUI<OSCUIRequest>
 	int route_set_gain_abs (int rid, float level, lo_message msg);
 	int route_set_gain_dB (int rid, float dB, lo_message msg);
 	int route_set_gain_fader (int rid, float pos, lo_message msg);
+	int strip_db_delta (int ssid, float delta, lo_message msg);
 	int route_set_trim_abs (int rid, float level, lo_message msg);
 	int route_set_trim_dB (int rid, float dB, lo_message msg);
 	int route_set_pan_stereo_position (int rid, float left_right_fraction, lo_message msg);
@@ -515,21 +597,39 @@ class OSC : public ARDOUR::ControlProtocol, public AbstractUI<OSCUIRequest>
 	int set_bank (uint32_t bank_start, lo_message msg);
 	int _set_bank (uint32_t bank_start, lo_address addr);
 	int bank_up (lo_message msg);
+	int bank_delta (float delta, lo_message msg);
+	int use_group (float value, lo_message msg);
 	int bank_down (lo_message msg);
-	int set_surface (uint32_t b_size, uint32_t strips, uint32_t fb, uint32_t gmode, lo_message msg);
+	int set_surface (uint32_t b_size, uint32_t strips, uint32_t fb, uint32_t gmode, uint32_t se_size, uint32_t pi_size, lo_message msg);
 	int set_surface_bank_size (uint32_t bs, lo_message msg);
 	int set_surface_strip_types (uint32_t st, lo_message msg);
 	int set_surface_feedback (uint32_t fb, lo_message msg);
 	int set_surface_gainmode (uint32_t gm, lo_message msg);
 	int refresh_surface (lo_message msg);
+	int sel_send_pagesize (uint32_t size, lo_message msg);
+	int sel_send_page (int page, lo_message msg);
+	int sel_plug_pagesize (uint32_t size, lo_message msg);
+	int sel_plug_page (int page, lo_message msg);
+	int sel_plugin (int delta, lo_message msg);
+	int _sel_plugin (int id, lo_address addr);
+	void processor_changed (lo_address addr);
 
+	int scrub (float delta, lo_message msg);
+	int jog (float delta, lo_message msg);
+	int jog_mode (float mode, lo_message msg);
 	int master_set_gain (float dB);
 	int master_set_fader (float position);
+	int master_delta_gain (float delta);
 	int master_set_trim (float dB);
 	int master_set_pan_stereo_position (float position, lo_message msg);
 	int master_set_mute (uint32_t state);
+	int master_select (lo_message msg);
 	int monitor_set_gain (float dB);
 	int monitor_set_fader (float position);
+	int monitor_delta_gain (float delta);
+	int monitor_set_mute (uint32_t state);
+	int monitor_set_dim (uint32_t state);
+	int monitor_set_mono (uint32_t state);
 	int sel_recenable (uint32_t state, lo_message msg);
 	int sel_recsafe (uint32_t state, lo_message msg);
 	int sel_mute (uint32_t state, lo_message msg);
@@ -541,12 +641,14 @@ class OSC : public ARDOUR::ControlProtocol, public AbstractUI<OSCUIRequest>
 	int sel_phase (uint32_t state, lo_message msg);
 	int sel_gain (float state, lo_message msg);
 	int sel_fader (float state, lo_message msg);
+	int sel_dB_delta (float delta, lo_message msg);
 	int sel_trim (float val, lo_message msg);
 	int sel_pan_position (float val, lo_message msg);
 	int sel_pan_width (float val, lo_message msg);
 	int sel_sendgain (int id, float dB, lo_message msg);
 	int sel_sendfader (int id, float pos, lo_message msg);
 	int sel_sendenable (int id, float pos, lo_message msg);
+	int sel_master_send_enable (int state, lo_message msg);
 	int sel_expand (uint32_t state, lo_message msg);
 	int sel_pan_elevation (float val, lo_message msg);
 	int sel_pan_frontback (float val, lo_message msg);
@@ -557,7 +659,12 @@ class OSC : public ARDOUR::ControlProtocol, public AbstractUI<OSCUIRequest>
 	int sel_comp_mode (float val, lo_message msg);
 	int sel_comp_makeup (float val, lo_message msg);
 	int sel_eq_enable (float val, lo_message msg);
-	int sel_eq_hpf (float val, lo_message msg);
+	int sel_eq_hpf_freq (float val, lo_message msg);
+	int sel_eq_hpf_enable (float val, lo_message msg);
+	int sel_eq_hpf_slope (float val, lo_message msg);
+	int sel_eq_lpf_freq (float val, lo_message msg);
+	int sel_eq_lpf_enable (float val, lo_message msg);
+	int sel_eq_lpf_slope (float val, lo_message msg);
 	int sel_eq_gain (int id, float val, lo_message msg);
 	int sel_eq_freq (int id, float val, lo_message msg);
 	int sel_eq_q (int id, float val, lo_message msg);
@@ -575,11 +682,11 @@ class OSC : public ARDOUR::ControlProtocol, public AbstractUI<OSCUIRequest>
 	void notify_routes_added (ARDOUR::RouteList &);
 	void notify_vca_added (ARDOUR::VCAList &);
 
-	void update_clock ();
 	int cancel_all_solos ();
 	bool periodic (void);
 	sigc::connection periodic_connection;
 	PBD::ScopedConnectionList session_connections;
+	PBD::ScopedConnectionList cueobserver_connections;
 
 	int route_send_fail (std::string path, uint32_t ssid, float val, lo_address addr);
 	int sel_send_fail (std::string path, uint32_t id, float val, lo_address addr);
@@ -592,6 +699,9 @@ class OSC : public ARDOUR::ControlProtocol, public AbstractUI<OSCUIRequest>
 	typedef std::list<OSCGlobalObserver*> GlobalObservers;
 	GlobalObservers global_observers;
 
+	typedef std::list<OSCCueObserver*> CueObservers;
+	CueObservers cue_observers;
+
 	void debugmsg (const char *prefix, const char *path, const char* types, lo_arg **argv, int argc);
 
 	static OSC* _instance;
