@@ -38,7 +38,7 @@ function dsp_run (ins, outs, n_samples)
 	local gain = ARDOUR.DSP.dB_to_coefficient (ctrl[1])
 	assert (#ins == #outs) -- ensure that we can run in-place (channel count matches)
 	for c = 1,#ins do
-		assert (ins[c]:sameinstance(outs[c])) -- check in-place
+		assert (ins[c] == outs[c]) -- check in-place
 		ARDOUR.DSP.apply_gain_to_buffer (ins[c], n_samples, gain); -- process in-place
 	end
 end
